
/*
 * bs_config_impl.h:
 *      this defines the classes which were only
 *      declared in bs_config.h.
 *
 *@@include #define INCL_WINWORKPLACE
 *@@include #include <os2.h>
 *@@include #include <list>
 *@@include #include "helpers\configsys.h"
 *@@include #include "base\bs_errors.h"
 *@@include #include "base\bs_config.h"
 */

/*
 *      This file Copyright (C) 1999-2000 Ulrich Mller.
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, in version 2 as it comes in the COPYING
 *      file of this distribution.
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 */

#ifndef WARPIN_CONFIG_IMPL_HEADER_INCLUDED
    #define WARPIN_CONFIG_IMPL_HEADER_INCLUDED

    /* ******************************************************************
     *
     *  BSConfigExcpt error codes
     *
     ********************************************************************/

    // error codes for BSConfigExcpt thrown by BSConfigSys class
    const int CFGEXCPT_OPEN                 = 1;
                    // error opening CONFIG.SYS (BSConfigSys);
                    // BSConfigExcpt.iData has the APIRET
    const int CFGEXCPT_DOUBLEREPLACEMODE    = 2;
                    // error parsing CONFIGSYS attribute (BSCfgSysManip):
                    // replace mode specified twice,
                    // BSConfigExcpt.pszSubstring has the failing part
    const int CFGEXCPT_INVALIDVERTICAL      = 3;
                    // error parsing CONFIGSYS attribute (BSCfgSysManip):
                    // vertical modifier not allowed here
                    // BSConfigExcpt.pszSubstring has the failing part
    const int CFGEXCPT_INVALIDSEARCH        = 4;
                    // invalid search string (BSCfgSysManip)
                    // BSConfigExcpt.pszSubstring has the failing part
    const int CFGEXCPT_MANIPULATING         = 5;
                    // error in BSConfigSys::Manipulate()
                    // (should not happen)
    const int CFGEXCPT_WRITE                = 6;
                    // error in BSConfigSys::WriteBack();
                    // BSConfigExcpt.iData has the APIRET
    const int CFGEXCPT_NOTIMPLEMENTED       = 7;
                    // not implemented yet
    const int CFGEXCPT_NOSEPARATOR          = 8;
                    // this pops up while modifying CONFIG.SYS
                    // and ADDLEFT or ADDRIGHT have been specified,
                    // but the line in CONFIG.SYS has no "=" character
    const int CFGEXCPT_SYNTAX               = 9;
                    // invalid attribute passed to BSCfgSysManip::BSCfgSysManip
    const int CFGEXCPT_PARSELOG             = 10;
                    // syntax error parsing log entry in BSCfgSysManip::AddToUndoList

    // error codes for BSConfigExcpt thrown by the BSRegisterClass, BSDeregisterClass,
    // BSReplaceClass classes
    const int REGEXCPT_SYNTAX               = 60;
                    // syntax error in constructor (invalid pszRegisterClass);
                    // BSConfigExcpt.pszSubstring has the whole failing pszRegisterClass.
                    // This is also used by BSReplaceClass::BSReplaceClass.
    const int REGEXCPT_QUERYCLASSLIST       = 61;
                    // error querying WPS class list
    const int REGEXCPT_ALREADYREGISTERED    = 62;
                    // BSRegisterClass::Register(FALSE) has determined that the
                    // class is already registered;
                    // BSConfigExcpt.pszSubstring has the currently registered DLL
    const int REGEXCPT_REGISTER             = 63;
                    // BSRegisterClass::Register failed;
                    // BSConfigExcpt.iData has the APIRET of winhRegisterClass
    const int REGEXCPT_DEREGISTER           = 64;
                    // BSDeregisterClass::Deregister failed
    const int REGEXCPT_REPLACE              = 65;
                    // BSReplaceClass::Replace failed (WinReplaceObjectClass returned FALSE)
    const int REGEXCPT_UNREPLACE            = 66;
                    // BSUnreplaceClass::Unreplace failed (WinReplaceObjectClass returned FALSE)

    // error codes for BSConfigExcpt thrown by BSCreateWPSObject class
    const int WPOEXCPT_NOLOCATION           = 100;
                    // no location given
    const int WPOEXCPT_NOTITLE              = 101;
                    // no title given
    const int WPOEXCPT_NOCLASS              = 102;
                    // no class given
    const int WPOEXCPT_CREATE               = 103;
                    // error in BSCreateWPSObject::CreateObject
    const int WPOEXCPT_DELETEOBJECT         = 104;
                    // error deleting object (BSDeleteWPSObject::Delete)

    // error codes for BSConfigExcpt thrown by BSExecute class
    const int EXEEXCPT_SYNTAX               = 200;
                    // syntax error parsing EXECUTE tag
    const int EXEEXCPT_DOSERROR             = 201;
                    // DOS error starting executable;
                    // BSConfigExcpt.iData has the APIRET

    const int PRFEXCPT_SYNTAX               = 300;
                    // syntax error in CLEARPROFILE or WRITEPROFILE tag
    const int PRFEXCPT_PRFERROR             = 301;
                    // error clearing/writing to specified profile

    // error codes for BSConfigExcpt thrown by BSKillProcess class
    const int KILLEXCPT_SYNTAX              = 400;
                    // syntax error parsing KILLPROCESS tag

    /* ******************************************************************
     *
     *  CONFIG.SYS class declarations
     *
     ********************************************************************/

    // flags for BSCfgSysManip (below)

    /*
     *@@ BSCfgSysManip:
     *      the "CONFIG.SYS manipulator" class.
     *      An instance of this must be created
     *      with a suitable attribute string and
     *      passed to BSConfigSys::Manipulate.
     *
     *      See BSCfgSysManip::BSCfgSysManip for details.
     *
     *@@changed V0.9.0 (99-11-06) [umoeller]: renamed from CfgSysManip
     *@@changed V0.9.1 (2000-01-06) [umoeller]: added UNIQUE(xxx) support
     */

    class BSCfgSysManip : public BSConfigBase
    {
        // allow BSConfigSys access to private members
        friend class BSConfigSys;

        public:
            CONFIGMANIP _ConfigManip;

            BSCfgSysManip(const char *pcszConfigSys);
            virtual ~BSCfgSysManip();

            virtual const char* DescribeType();
            virtual BSString DescribeData();

            // override matching BSConfigBase identifier method
            virtual BSCfgSysManip* IsCfgSysManip() { return this; }
    };

    /*
     *@@ BSConfigSys:
     *      this is the BSConfigSys class for CONFIG.SYS manipulation.
     *      When an instance of this is created, CONFIG.SYS is read in
     *      by the constructor, so only create an instance of this
     *      if you're actually to modify the file.
     *
     *      Note that this is not a subclass of BSConfigBase since
     *      this does not contain any configuration data. Instead,
     *      this represents a CONFIG.SYS file on which BSCfgSysManip
     *      instances are to be invoked.
     *
     *      The destructor does _not_ write the file. Explicitly call
     *      BSConfigSys::WriteBack() for that.
     *
     *@@changed V0.9.0 (99-11-06) [umoeller]: renamed from ConfigSys
     *@@changed V0.9.6 (2000-10-27) [umoeller]: added _fDirty
     */

    class BSConfigSys
    {
        protected:
            char        *_pszContent;
            // char        _szFilename[30];         // "?:\CONFIG.SYS"
            BOOL        _fDirty;                 // TRUE if _pszContent has changed

        public:
            BSConfigSys();
            ~BSConfigSys();

            int Manipulate(BSCfgSysManip& Manip,
                           BSCfgSysDoneLogger *pLogger,
                           BSFileLogger *pLogFile);
            int Flush(PSZ pszBackup,
                      BSFileLogger *pLogFile)
                const;
    };

    /* ******************************************************************
     *
     *  Register-class class declarations
     *
     ********************************************************************/

    /*
     *@@ BSRegisterClass:
     *      this is the BSRegisterClass class for registering WPS classes.
     *
     *@@changed V0.9.0 (99-11-06) [umoeller]: renamed from RegisterClass
     */

    class BSRegisterClass : public BSConfigBase
    {
        public:
            // all these must be public because otherwise
            // WarpIN cannot resolve macros
            char        *_pszClassName;
            char       *_pszModule;

            virtual const char* DescribeType();
            virtual BSString DescribeData();

            BSRegisterClass(const char *pcszRegisterClass);
            virtual ~BSRegisterClass();

            // override matching BSConfigBase identifier method
            virtual BSRegisterClass* IsRegisterClass() { return this; }

            bool IsRegistered(char *pszBuffer)
                 const;

            int Register(bool fReplace,
                         BSRegisterDoneLogger *pLogger,
                         BSFileLogger *pLogFile)
                         const;
    };

    /*
     *@@ BSDeregisterClass:
     *      the reverse to the above. This is used to deregister
     *      classes that were previously registered using the
     *      BSRegisterClass class.
     *
     *@@changed V0.9.0 (99-11-06) [umoeller]: renamed from BSDeregisterClass
     */

    class BSDeregisterClass : public BSConfigBase
    {
        public:
            char        *_pszClassName;

            virtual const char* DescribeType();
            virtual BSString DescribeData();

            BSDeregisterClass(const char *pcszDeregisterClass);
            virtual ~BSDeregisterClass();

            // override matching BSConfigBase identifier method
            virtual BSDeregisterClass* IsDeregisterClass() { return this; }

            int Deregister(BSFileLogger *pLogFile)
                const;
    };

    /*
     *@@ BSReplaceClass:
     *      this is the BSReplaceClass class for replacing WPS classes.
     *
     *@@changed V0.9.0 (99-11-06) [umoeller]: renamed from ReplaceClass
     */

    class BSReplaceClass : public BSConfigBase
    {
        public:
            // all these must be public because otherwise
            // WarpIN cannot resolve macros
            char        *_pszOldClassName;
            char        *_pszNewClassName;

            BSReplaceClass(const char *pcszReplaceClass);
            virtual ~BSReplaceClass();

            virtual const char* DescribeType();
            virtual BSString DescribeData();

            // override matching BSConfigBase identifier method
            virtual BSReplaceClass* IsReplaceClass() { return this; }

            int Replace(BSReplaceDoneLogger *pLogger,
                        BSFileLogger *pLogFile)
                const;
    };

    /*
     *@@ BSUnreplaceClass:
     *      the reverse to the above. This is used to un-replace
     *      classes that were previously replaced using the
     *      BSRegisterClass class.
     *
     *      Since the BSReplaceDoneLogger class stores items just
     *      like the input to BSReplaceClass was, we inherit from
     *      that class to use the same parsing routines.
     *
     *@@changed V0.9.0 (99-11-06) [umoeller]: renamed from UnreplaceClass
     */

    class BSUnreplaceClass : public BSReplaceClass
    {
        public:
            // inherit constructor from BSReplaceClass
            BSUnreplaceClass(const char *pcszUnreplaceClass)
                    : BSReplaceClass(pcszUnreplaceClass)
            {};

            virtual const char* DescribeType();
            virtual BSString DescribeData();

            // override matching BSConfigBase identifier method
            virtual BSUnreplaceClass* IsUnreplaceClass() { return this; }

            int Unreplace(BSFileLogger *pLogFile)
                const;
    };

    /* ******************************************************************
     *
     *  Create-object class declarations
     *
     ********************************************************************/

    /*
     *@@ BSCreateWPSObject:
     *      this has configuration info for creating
     *      objects.
     *
     *@@changed V0.9.0 (99-11-06) [umoeller]: renamed from CreateWPSObject
     */

    class BSCreateWPSObject : public BSConfigBase
    {
        public:
            // the following should always be set
            PSZ             _pszClassName,
                            _pszTitle,
                            _pszSetupString,     // this one might be NULL
                            _pszLocation;
            BOOL            _fReplace;

            // the following is set by BSConfigExcpt::CreateObject
            // if an object has been created successfully
            HOBJECT         _hobj;

            BSCreateWPSObject(const char *pcszCreateObject);
            virtual ~BSCreateWPSObject();

            virtual const char* DescribeType();
            virtual BSString DescribeData();

            // override matching BSConfigBase identifier method
            virtual BSCreateWPSObject* IsCreateWPSObject() { return this; }

            void CreateObject(BSWPSObjectsDoneLogger *pLogger,
                              BSFileLogger *pLogFile);
            bool DoesObjectExist();
            void DestroyObject(BSFileLogger *pLogFile);
    };

    /*
     *@@ BSDeleteWPSObject:
     *      the reverse to the above. This is used to delete
     *      objects that were previously created using the
     *      BSCreateWPSObject class.
     */

    class BSDeleteWPSObject : public BSConfigBase
    {
        public:
            PSZ             _pszObjectID;

            BSDeleteWPSObject(const char *pcszID2Delete);
            virtual ~BSDeleteWPSObject();

            virtual const char* DescribeType();
            virtual BSString DescribeData();

            // override matching BSConfigBase identifier method
            virtual BSDeleteWPSObject* IsDeleteWPSObject() { return this; }

            int Delete(BSFileLogger *pLogFile)
                const;
    };

    /* ******************************************************************
     *
     *  Profile classes
     *
     ********************************************************************/

    /*
     *@@ BSClearProfile:
     *      this has information to clear data
     *      from a profile.
     *
     *@@added V0.9.1 (2000-02-07) [umoeller]
     */

    class BSClearProfile : public BSConfigBase
    {
        public:
            PSZ             _pszProfile;
            PSZ             _pszApplication;
            PSZ             _pszKey;

            BSClearProfile(const char *pcszClearProfile);
            virtual ~BSClearProfile();

            virtual const char* DescribeType();
            virtual BSString DescribeData();

            // override matching BSConfigBase identifier method
            virtual BSClearProfile* IsClearProfile() { return this; }

            int Clear(HAB hab,
                      BSFileLogger *pLogFile)
                const;

            BSString DescribePrfKey() const;

        protected:
            // protected constructor because otherwise we
            // cannot derive from this
            BSClearProfile(ULONG cfgt) : BSConfigBase(cfgt)
            {
                _pszProfile = 0;
                _pszApplication = 0;
                _pszKey = 0;
            }
    };

    /*
     *@@ BSWriteProfile:
     *      this has information to write a string
     *      to a profile. This is derived from
     *      BSClearProfile to inherit the
     *      profile/app/key members.
     *
     *@@added V0.9.1 (2000-02-07) [umoeller]
     */

    class BSWriteProfile : public BSClearProfile
    {
        public:
            PSZ             _pszWriteString;

            BSWriteProfile(const char *pcszWriteProfile);
            virtual ~BSWriteProfile();

            // override matching BSConfigBase identifier method
            virtual BSWriteProfile* IsWriteProfile() { return this; }

            int Write(HAB hab,
                      BSClearPrfAttrsLogger *pLogger,
                      BSFileLogger *pLogFile)
                const;
    };

    /* ******************************************************************
     *
     *  Execute application program class
     *
     ********************************************************************/

    /*
     *@@ BSExecute:
     *      representation of EXECUTE.
     *
     *@@added V0.9.1 (2000-02-07) [umoeller]
     *@@changed V0.9.9 (2001-03-27) [umoeller]: separated executable and parameters
     */

    class BSExecute : public BSConfigBase
    {
        public:
            ULONG       _ulExecType;
                            // ORed CFGT_* flags as specified with BSConfigBase;
                            // these are the flags on the "Configure" page this
                            // EXECUTE thing is dependent on. If this is 0,
                            // the execute will always be executed.

            PSZ         _pszExecutable,     // V0.9.9 (2001-03-27) [umoeller]
                        _pszParams;

            ULONG       _arc;   // error code of Execute()

            BSExecute(const char *pcszExecute);
            virtual ~BSExecute();

            virtual const char* DescribeType();
            virtual BSString DescribeData();

            // override matching BSConfigBase identifier method
            virtual BSExecute* IsExecute() { return this; }

            int Execute(BSFileLogger *pLogFile);
    };

    /*
     *@@ BSDeExecute:
     *      representation of DEEXECUTE.
     *
     *      This is derived from BSExecute, so run
     *      the parent's Execute() on this.
     *
     *@@added V0.9.9 (2001-03-27) [umoeller]
     */

    class BSDeExecute : public BSExecute
    {
        public:
            BSDeExecute(const char *pcszDeExecute)
                : BSExecute(pcszDeExecute)
            {};

            virtual const char* DescribeType();
            virtual BSString DescribeData();

            // override IsExecute again to return null
            virtual BSExecute* IsExecute() { return NULL; }
            // override matching BSConfigBase identifier method
            virtual BSDeExecute* IsDeExecute() { return this; }
    };

    /* ******************************************************************
     *
     *  Kill process class
     *
     ********************************************************************/

    /*
     *@@ BSKillProcess:
     *
     *@@added V0.9.1 (2000-02-07) [umoeller]
     */

    class BSKillProcess : public BSConfigBase
    {
        public:
            PSZ         _pszKillProcess;

            BSKillProcess(const char *pcszKillProcess);
            virtual ~BSKillProcess();

            virtual const char* DescribeType();
            virtual BSString DescribeData();

            // override matching BSConfigBase identifier method
            virtual BSKillProcess* IsKillProcess() { return this; }

            int KillProcess(BSFileLogger *pLogFile)
                const;
    };

#endif

