
/*
 *@@sourcefile bs_rexx.h:
 *      header for bs_rexx.c. See remarks there.
 *
 *@@include #define INCL_REXXSAA
 *@@include #include <os2.h>
 *@@include #include <rexxsaa.h>    // if using Toolkit headers; with EMX, this is in os2.h
 *@@include #include "base\bs_rexx.h"
 */

/*
 *      Copyright (C) 2000 Cornelis Bockemhl.
 *      Copyright (C) 2000-2001 Ulrich Mller.
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, in version 2 as it comes in the COPYING
 *      file of this distribution.
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 */

#ifndef BSREXX_INCLUDED_
    #define BSREXX_INCLUDED_

    /*
     *@@ FNREXX:
     *      function prototype for REXX function handlers.
     *      Even though this is defined in the OS/2 headers
     *      as "RexxFunctionHandler",
     *      we cannot use that definition, because EMX defines
     *      it differently.
     *
     *@@added V0.9.2 (2000-03-11) [umoeller]
     */

    typedef ULONG APIENTRY FNREXX(PCSZ,
                                  ULONG,
                                  PRXSTRING,
                                  PCSZ,
                                  PRXSTRING);

    /*
     *@@ WIREXXFUNCTION:
     *      representation of a WarpIN REXX function which
     *      is implemented in this file and can be called
     *      from script REXX code.
     *      A static global array of these exists in
     *      G_aFERexxFunctions.
     *
     *@@added V0.9.2 (2000-03-10) [umoeller]
     */

    typedef struct _WIREXXFUNCTION
    {
        const char  *pcszFunctionName;  // function name
        FNREXX      *pfn;               // function address
    } WIREXXFUNCTION, *PWIREXXFUNCTION;

    /*
     *@@ FERexx:
     *      this class is supposed to be instantiated as one global object
     *      in WarpIN in order to enable storing and calling REXX scripts.
     *
     *@@added V0.9.2 (2000-03-10) [cbo]
     */

    class FERexx
    {
        private:
            WIREXXFUNCTION  *_paFunctions;
            ULONG           _cFunctions;

        public:
            FERexx(WIREXXFUNCTION *paFunctions,
                   ULONG cFunctions);

            virtual ~FERexx();

            // Exported REXX related functions
            void StoreRexxCode(const char* pcszFuncName,
                               const char* pcszCode);

            // return codes:
            #define WIREXX_NOERROR          0
            #define WIREXX_INVALIDNAME      1
            #define WIREXX_NOCODE           2
            #define WIREXX_REXXFAILED       3

            int ExecuteRexxCode(const char *pcszFuncName,
                                const char *pcszArgs,
                                char **ppszRet);

            int StoreExecuteRexxCode(const char *pcszName,
                                     const char *pcszCode,
                                     const char *pcszArgs,
                                     char **ppszRet);

            // Exported WarpIN interface functions
            ULONG ParseRexxTags(const char* hdrstring);

            BOOL ExecuteRexxMacros(char **ppszStr,
                                   PULONG pulErrorOffset,
                                   char **ppszErrorCall);

            BOOL ExecuteRexxMacros(BSString& str,
                                   PULONG pulErrorOffset,
                                   BSString& strErrorCall);
    };

    // REXX support: We need access to the global G_FERexx object
    // which is defined in warpin.cpp (cbo 2000-03-01)
    // we use a pointer instead of a static object now V0.9.2 (2000-03-10) [umoeller]
    // V0.9.9 (2001-02-28) [umoeller]: the variable is now in bs_rexx.cpp
    extern FERexx *G_pFERexx;

#endif // _WIREXX_INCLUDED_

