
/*
 * bs_script.h:
 *      header file for bs_script.cpp. See remarks there.
 *
 *@@include #include "base\bs_string.h"
 *@@include #include "base\bs_logger.h"
 *@@include #include "base\bs_config.h"
 *@@include #include "base\bs_script.h"
 */

/*
 *      This file Copyright (C) 1999-2001 Ulrich Mller.
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, in version 2 as it comes in the COPYING
 *      file of this distribution.
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 */

#ifndef WARPIN_SCRIPT_HEADER_INCLUDED
    #define WARPIN_SCRIPT_HEADER_INCLUDED

    /* ******************************************************************
     *
     *  Forward declarations
     *
     ********************************************************************/

    class BSConfigBase;
    class BSScriptBase;
    class BSGroupDeclBase;

    extern CHAR     G_szMessageFile[2*CCHMAXPATH];

    /* ******************************************************************
     *
     *  BSScriptExcpt declaration
     *
     ********************************************************************/

    /*
     *@@ BSScriptExcpt:
     *      exception thrown by the various install script parsers.
     *
     *V0.9.9 (2001-02-28) [umoeller]
     */

    class BSScriptExcpt : public BSExcptBase
    {
        public:
            unsigned long       _ulLine,
                                _ulColumn;

            BSScriptExcpt(const BSString &str,
                          unsigned long ulLine = 0,
                          unsigned long ulColumn = 0);

            BSScriptExcpt(int iMessage,
                          char** apszTable = NULL,
                          int iCount = 0);
    };

    /* ******************************************************************
     *
     *  BSPageInfo declaration
     *
     ********************************************************************/

    /*
     *@@ BSPageInfo:
     *      one of these structures is created for each PAGE block
     *      found in the install script by FEArchive::ParsePages.
     *
     *      A list of all the pages in an archive is stored in
     *      each archive's FEArchive.PageInfoList. That list is
     *      valid after a call to FEArchive::ParsePages.
     *
     *@@changed V0.9.0 (99-10-31) [umoeller]: this used to be WPIPAGEINFO
     */

    class BSPageInfo
    {
        public:
            LONG        _lPageIndex;    // index of this page

            enum { MODE_TEXT,           // this page is plain text
                   MODE_README,         // this page is a readme (MLE)
                   MODE_CONTAINER,      // this is the "Container" page
                   MODE_CONFIGURE       // this is the "Configure" page
                 }      _PageMode;

            BSString    _strInfoText;    // has text found in <TEXT> ... </TEXT> block

            BSString    _strReadmeSrc;   // for the README page, has the <README> ... </README> block

            ULONG       _ulExtractFromPck;
                                         // if 0, _strReadmeSrc specifies the text to
                                         // be displayed directly; otherwise it is a
                                         // file name in the package with this index,
                                         // which must be extracted.
                                         // Set to 0 by the constructor.
                                         // V0.9.9 (2001-04-04) [umoeller]

            // <README> attribute, one of the following:
            enum {  README_PLAIN,        // plain text
                    README_FLOW,         // reformat line breaks
                    README_HTML          // limited HTML support
                 }
                        _ReadmeFormat;   // set to README_PLAIN by the constructor

            LONG        _lNextButton;
                    // the "Next" button:
                    // -- if > 0, the button is visible and enabled
                    //    and has the index of the next page;
                    // -- if == 0 --> "Install" button, start install
            BSString     _strNextButtonTitle;
                    // button text

            /*
             *@@ BSPageInfo:
             *      default constructor.
             */

            BSPageInfo()
            {
                _lPageIndex = 0;
                _PageMode = MODE_TEXT;

                _ulExtractFromPck = 0;
                _ReadmeFormat = README_PLAIN;

                _lNextButton = 0;
            }
    };

    /* ******************************************************************
     *
     *  BSVarPrompt declaration
     *
     ********************************************************************/

    #define VPT_NUM         1
    #define VPT_ALPHANUM    2
    #define VPT_PATH        3
    #define VPT_FAIL        4

    /*
     *@@ BSVarPrompt:
     *      class representing a VARPROMPT declaration.
     *      This is passed to guiVarPrompt if a variable
     *      is not defined.
     *
     *      BSScriptBase has a static BSVarPromptsMap of
     *      all defined BSVarPrompt's.
     *
     *@@added V0.9.2 (2000-03-11) [umoeller]
     */

    class BSVarPrompt
    {
        public:
            BSString    _strVarName;
                    // variable name; e.g. "ETC";
                    // specified with NAME attribute
            BSString    _strDescription;
                    // description; block within <VARPROMPT> ... </VARPROMPT>
            ULONG       _ulType,
                        _ulMin,
                        _ulMax;

            /*
             *@@ BSVarPrompt:
             *      default constructor.
             */

            BSVarPrompt(BSString strVarName = "")
            {
                _strVarName = strVarName;
                _strDescription = "";
                _ulType = VPT_ALPHANUM;
                _ulMin = 0;
                _ulMax = 0;
            }

            bool Validate(const char *pcszValue)
                 const;
    };

    int operator==(const BSVarPrompt &v1, const BSVarPrompt &v2);
    int operator<(const BSVarPrompt &v1, const BSVarPrompt &v2);

    typedef map< BSString,          // key class
                 BSVarPrompt,       // data class
                 less<BSString>
               > BSVarPromptsMap;

    /* ******************************************************************
     *
     *  BSPckDeclBase declaration
     *
     ********************************************************************/

    // flags for FEPackage.ulPathType
    #define PATH_DISABLED   0x00        // path cannot be changed
    #define PATH_VARIABLE   0x01        // path can be freely changed
    #define PATH_BASE       0x02        // path is base for all packages

    // flags for FEPackage.ulConfigureFlags
    #define PCK_CONFIGSYS       0x0001    // CONFIG.SYS can be updated
    #define PCK_WPSCLASSES      0x0002    // WPS classes can be installed
    #define PCK_WPSOBJECTS      0x0004    // WPS objects can be created
    #define PCK_CLEARPROFILES   0x0008    // has clear-profile data -- V0.9.1 (2000-02-07) [umoeller]
    #define PCK_WRITEPROFILES   0x0010    // has write-profile data -- V0.9.1 (2000-02-07) [umoeller]
    #define PCK_EXECUTES        0x0020    // needs user program to be executed -- V0.9.1 (2000-02-03) [umoeller]
    #define PCK_KILLPROCESS     0x0040    // pck has KILLPROCESS tags -- V0.9.1 (2000-02-12) [umoeller]

    #define PCK_FILES           0x0080    // has files; this is only used
                                          // in database mode

    #define PCK_DEEXECUTES      0x0100    // has DEEXECUTE data

    /*
     *@@ BSPckDeclBase:
     *      abstract base class for all kinds of package
     *      and group declarations.
     *
     *      Class hierarchy:
     *
     +      BSDeclRoot
     +        |
     +        +--- BSPckDeclBase
     +        |
     +        +--- BSGroupDeclBase
     *
     *
     *@@added V0.9.9 (2001-02-19) [umoeller]
     */

    class BSDeclRoot
    {
        public:
            BOOL                    _fIsGroup;
                    // TRUE if this represents a group

            BSGroupDeclBase         *_pGroup;
                    // the group this belongs to or NULL if root

            BSString                _strTitle;
                    // TITLE attribute V0.9.9 (2001-02-28) [umoeller];
                    // required for both packages and groups

        protected:
            BSDeclRoot(BSGroupDeclBase *pGroup,
                       BOOL fIsGroup);
    };

    /*
     *@@ BSPckDeclBase:
     *      BSDeclRoot subclass for package declarations.
     *      There are two subclasses of this for old-style
     *      (BSOldPckDecl) and new-style scripts, but
     *      the data fields are mostly identical for the two.
     *
     *@@added V0.9.9 (2001-02-28) [umoeller]
     */

    class BSPckDeclBase : public BSDeclRoot
    {
        public:

            enum { INTERNAL,        // package is internal (no EXTERNAL attrib)
                   EXTERNAL,        // package is external, but optional
                   REQEXTERNAL }    // package is external and required
                                    _External;

            BSString                _strExternalArchive;
                    // if external, the external archive name

            ULONG                   _ulIndexInArchive;
                    // index of this package in the archive (>= 1);
                    // 0 for groups

            BSString                _strDescription,
                    // old style: stuff between PCK and /PCK
                    // new style: contents of DESCRIPTION element
                    // empty for groups
                                    _strTargetPath,
                    // TARGET attribute V0.9.9 (2001-02-28) [umoeller]
                    // empty for groups
                                    _strID;
                    // PACKAGEID attribute V0.9.9 (2001-02-28) [umoeller]
                    // empty for groups

            enum { UNKNOWN,
                   SELECTED,        // package is initially selected
                   DESELECTED,      // package is initially deselected
                   NODESELECT }     // package must always be selected
                                    _Selected;

            ULONG                   _ulPathType;
                    // any combination of the following:
                    // -- PATH_DISABLED:   path cannot be changed
                    // -- PATH_VARIABLE: path can be freely changed
                    // -- PATH_BASE: path is base for all packages
                    // 0 for groups

            // this holds all configuration items, which
            // can be of a whole lotta subclasses actually
            // (BSCfgSysManip, BSRegisterClass, BSReplaceClass,
            // BSCreateWPSObject, BSClearProfile, BSWriteProfile,
            // BSExecute);
            // we use a list of the shared base class to avoid
            // template bloat which we had by creating one list
            // for each of these subclasses... saved about 100 KB code!
            list<BSConfigBase*>     _listConfigObjects;

            ULONG                   _ulConfigData;
                    // any combination of the following:
                    // -- PCK_CONFIGSYS: CONFIG.SYS can be updated;
                    // -- PCK_WPSCLASSES: WPS classes can be installed;
                    // -- PCK_WPSOBJECTS: WPS objects can be created.

            BSRequiresStringsLogger _logRequiresStrings;
                    // "REQUIRES=" attributes, as in script.
                    // Each of these can either be a decimal string
                    // (which is resolved later) or a five- or
                    // six-part package ID; V0.9.1 (2000-01-07) [umoeller]

            // configuration attributes from install script:
            // either set by FEArchive or loaded from database
            BSCfgSysAttrsLogger     _logCfgSysAttrs;
            BSRegisterAttrsLogger   _logRegisterClassAttrs;
            BSReplaceAttrsLogger    _logReplaceClassAttrs;
            BSWPSObjectAttrsLogger  _logWPSObjectAttrs;
            BSExecuteAttrsLogger    _logExecuteAttrs; // added V0.9.1 (2000-02-03) [umoeller]
            BSDeExecuteAttrsLogger  _logDeExecuteAttrs; // added V0.9.9 (2001-03-27) [umoeller]
            BSClearPrfAttrsLogger   _logClearPrfAttrs; // added V0.9.1 (2000-02-07) [umoeller]
            BSWritePrfAttrsLogger   _logWritePrfAttrs; // added V0.9.1 (2000-02-07) [umoeller]
            BSKillProcessAttrsLogger _logKillProcessAttrs; // added V0.9.1 (2000-02-07) [umoeller]

        protected:
            BSPckDeclBase(BSGroupDeclBase *pGroup);
    };

    /*
     *@@ BSDBPckDecl:
     *      pseudo-declaration for packages from a database.
     *
     *      Yes, this isn't pretty, but since FEPackageBase has
     *      a pDecl member pointing to a BSPckDeclBase instance
     *      and many of the package data members have now been
     *      encapsulated in BSPckDeclBase, we must use this for
     *      database packages.
     *
     *@@added V0.9.9 (2001-02-28) [umoeller]
     */

    class BSDBPckDecl : public BSPckDeclBase
    {
        public:
            BSDBPckDecl()
                : BSPckDeclBase(NULL)       // group: none here
            {
            };
    };

    /*
     *@@ BSGroupDeclBase:
     *      BSDeclRoot subclass for group declarations.
     *
     *      Note that this class does not parse itself
     *      since we only have a couple of data members.
     *
     *@@added V0.9.9 (2001-02-28) [umoeller]
     */

    class BSGroupDeclBase : public BSDeclRoot
    {
        public:
            list<BSDeclRoot*>       _DeclarationsList;

            BOOL        _fInitiallyExpanded;

            /*
             *@@ BSGroupDeclBase:
             *      constructor. This expects the group title
             *      and the EXPANDED boolean attribute directly,
             *      which must have been parsed by the caller.
             *
             *@@added V0.9.9 (2001-02-28) [umoeller]
             */

            BSGroupDeclBase(BSGroupDeclBase *pGroup,
                            const char *pcszTitle,
                            BOOL fInitiallyExpanded)
                : BSDeclRoot(pGroup,
                             TRUE)      // is group
            {
                _strTitle = pcszTitle;
                _fInitiallyExpanded = fInitiallyExpanded;
            }
    };

    /* ******************************************************************
     *
     *  BSScriptBase and subclasses
     *
     ********************************************************************/

    typedef BOOL XWPENTRY FNCONFIRMREXXALLOWED(PCSZ);
    typedef FNCONFIRMREXXALLOWED *PFNCONFIRMREXXALLOWED;

    /*
     *@@ BSScriptBase:
     *      base class for WarpIN scripts. This is an abstract
     *      base class and must be implemented by one of the
     *      two subclasses:
     *
     *      -- BSOldScript for "old-style" scripts;
     *
     *      -- BSXmlScript for XML scripts.
     *
     *      See bs_script.cpp for an introduction.
     */

    class BSScriptBase
    {
        public:

            ULONG                   _ulCodepage;
                    // CODEPAGE tag found with WARPIN tag;
                    // defaults to 850

            static BSVarPromptsMap  VarPromptsMap;
                    // string map with all VARPROMPTS ever found

            BSString                _strTitle;
                    // contents of optional <TITLE> tag;
                    // this can be empty if not specified

            list<BSDeclRoot*>       _DeclarationsList;
                    // list of all root package and group declarations...
                    // if this is an old-style script, these are
                    // really instances of BSOldPckDecl.
                    // NOTE: This does not contain declarations which
                    // are non-root, i.e. members of a group package.
                    // Instead, those declarations are in the group only.

            list<BSPageInfo*>       _PageInfoList;
                    // list of pages in the install script;
                    // this is passed to the GUI
                    // (valid only after explicit call to ParsePages).

            BSScriptBase()
            {
                _ulCodepage = 850;
            }

            // virtual parse method... implemented by subclasses
            virtual VOID Parse(const BSString &strScript,
                               PFNCONFIRMREXXALLOWED pfnConfirmRexxAllowed) = 0;

            static BSScriptBase* CreateScript(const BSString &strScript,
                                              PFNCONFIRMREXXALLOWED pfnConfirmRexxAllowed);
    };

#endif

