/* $Id: calbacks.h,v 1.21 2001/05/31 18:13:42 umoeller Exp $ */

/*
 * calbacks.h:
 *      this defines all the callbacks that the GUI must
 *      implement for WarpIN to work.
 *
 *@@include #include <os2.h>
 *@@include #include "wiarchive\wiarchive.h"
 *@@include #include "frontend\calbacks.h"
 */

/*
 *      Copyright (C) 1998-2000 Ulrich Mller.
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, in version 2 as it comes in the COPYING
 *      file of this distribution.
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 */

#ifndef WARPIN_CALLBACKS_HEADER_INCLUDED
    #define WARPIN_CALLBACKS_HEADER_INCLUDED

    /* ******************************************************************
     *
     *  Forward declarations
     *
     ********************************************************************/

    class FEArcPackageBase;
    class FEArcPackagePck;
    class FEDBPackage;
    class FEJobBase;
    class FEInstallJob;
    class FEDatabaseThreadStatus;
    class BSVarPrompt;
    class BSExecute;
    class BSFileLogger;

    /* ******************************************************************
     *
     *  Required GUI callbacks on main thread (gui.cpp)
     *
     ********************************************************************/

    BOOL guiInitialize();

    #define GUICP_SET           0
    #define GUICP_TRANSLATED    1
    #define GUICP_FAILED        2

    ULONG guiSetCodepage(ULONG ulCodepage,
                         BSString &strScript);

    VOID guiCleanupBeforeExit();

    ULONG guiShowMessage(const BSString &strTitle,
                         const BSString &strMessage,
                         ULONG ulFlags);

    VOID guiSetupGlobalStatus(BOOL fFullInstall);

    #define GSTAT_COLLECTING            1
    #define GSTAT_REMOVING              2
    #define GSTAT_UNPACKING             3
    #define GSTAT_CONFIGSYS             4
    #define GSTAT_WPSCLASSES            5
    #define GSTAT_WPSOBJECTS            6
    #define GSTAT_DONEWITHALL           7

    VOID guiUpdateGlobalStatus(BYTE bStatus);

    BOOL guiBeginInstallMode();

    VOID guiOnJobSelectionChanged(FEJobBase *pJob);

    PSZ guiVarPrompt(BSVarPrompt &VarPrompt);

    BOOL guiExecute(BSExecute *pExec,
                    BSFileLogger *pLogFile);

    VOID guiProductInfo(HWND hwndOwner, BOOL fLongVersion);

    VOID guiSleep(ULONG ulSleep);

    BOOL XWPENTRY guiConfirmRexxAllowed(const char *pcszScript);

    /* ******************************************************************
     *
     *  Required GUI callbacks for Install thread (gui_installing.cpp)
     *
     ********************************************************************/

    BOOL guiStartInstall(BSFileLogger *pLogFile);

    void guiUpdatePackage(FEInstallJob *pJob, ULONG ulPckThis, ULONG ulPcksTotal);

    void guiUpdateFile(WIFileHeader *pwifh, ULONG ulFileThis, ULONG ulFilesTotal);

    void guiUpdateBytes(double dBytesDone, double dBytesTotal);

    /*
     *@@ GUIFILEERROR:
     *      structure passed to guiFileExists
     *      if a file exists and prompts are needed.
     *
     *@@changed V0.9.1 (2000-02-06) [umoeller]: renamed from FILEEXISTS
     */

    typedef struct _GUIFILEERROR
    {
        CHAR            szFilename[2*CCHMAXPATH];
                                    // fully q'd pathname of existing file
        FILESTATUS3     fs3Existing;    // file status of existing file

        WIFileHeader*   pwifh;          // file header in archive
        FDATE           fDateHeader;    // date
        FTIME           fTimeHeader;    // and time of *pwifh

        int             iComp;          // < 0:  existing file is newer
                                        // == 0: existing file is same write date
                                        // > 0:  existing file is older

        // constructor
        _GUIFILEERROR()
        {
            memset(this, 0, sizeof(*this));
        }

    } GUIFILEERROR, *PGUIFILEERROR;

    int guiFileExists(PGUIFILEERROR pfi);

    VOID wpiPrepareGUIFILEERROR(PGUIFILEERROR pfi,
                                const char *pcszBasePath,
                                WIFileHeader* pwifh);

    #define ECLS_DOS_WRITE      1       // OS/2 DOS error writing (99-10-26) [umoeller]
    #define ECLS_DOS_READ       2       // OS/2 DOS error reading (99-10-26) [umoeller]
    #define ECLS_DOS_CREATEDIR  3       // OS/2 DOS error creating directory (99-10-26) [umoeller]
    #define ECLS_WIARCHIVE      4       // back-end error (99-10-26) [umoeller]
    #define ECLS_EXCEPTION      5       // exception occured
    #define ECLS_DECOMP         6       // zlib decompression error
    #define ECLS_UNKNOWN        7       // unknown error (shouldn't happen)

    int guiFileError(FEInstallJob *pJob, // FEArcPackagePck *pCurrentArcPck,
                     PVOID pData,
                     int iErrorClass,
                     int iErrorCode,
                     int iErrorRsp);

    // error codes in wiarchive.h:
    // const int CBRC_ABORT = 1;
    // const int CBRC_RETRY = 2;
    // const int CBRC_IGNORE = 3;

    // return codes for guiFileLockedError (99-10-26) [umoeller]
    /* #define CBRC_SKIP         1       // skip locked file
    #define CBRC_RETRY        2       // retry same file
    #define CBRC_UNLOCK       3       // DosReplaceModule
    #define CBRC_DEFER        4       // defer thru CONFIG.SYS
    #define CBRC_ABORT        5       // abort install
       */

    int guiFileLockedError(PGUIFILEERROR pfi,
                           int iErrorCode);

    void guiDoneWithInstall(void);

    /* ******************************************************************
     *
     *  Required GUI callbacks for database mode (gui_db.cpp)
     *
     ********************************************************************/

    BOOL guiBeginDatabaseMode();

    BOOL guiStartVerify(list<FEDBPackage*>* pPackagesList,
                        BOOL fModal,
                        BSFileLogger *pLogFile);

    BOOL guiStartRemove(list<FEDBPackage*>* pPackagesList,
                        ULONG ulConfigFlags,
                        BOOL fModal,
                        BSFileLogger *pLogFile);

    // database callback codes (ulReportCode)
    #define DBC_NEXTPACKAGE                 1
    #define DBC_NEXTFILE                    2

    #define DBC_FILEERROR                   3
    #define DBC_EXISTINGFILENEWER           4
    #define DBC_EXISTINGFILEOLDER           5
    #define DBC_DELETEERROR                 6
    #define DBC_DELETEDIRERROR              7
    #define DBC_PACKAGEREMOVED              8
    #define DBC_TARGETDIRNOTFOUND           9
    #define DBC_CLASSDEREGISTERED           10
    #define DBC_DEREGISTERCLASSERROR        11
    #define DBC_CLASSUNREPLACED             12
    #define DBC_UNREPLACECLASSERROR         13
    #define DBC_OBJECTDELETED               14
    #define DBC_OBJECTDELETEERROR           15
    #define DBC_CFGSYSUNDONE                16
    #define DBC_UNDOCFGSYSERROR             17
    #define DBC_PROFILECLEARED              18  // V0.9.5 (2000-08-26) [umoeller]
    #define DBC_CLEARPROFILEERROR           19  // V0.9.5 (2000-08-26) [umoeller]

    #define DBC_FILELOCKED                  20  // V0.9.4 (2000-07-01) [umoeller]

    #define DBC_DEEXECUTING                 21  // V0.9.9 (2001-03-30) [umoeller]
    #define DBC_DEEXECUTE_DONE              22  // V0.9.9 (2001-03-30) [umoeller]
    #define DBC_DEEXECUTE_ERROR             23  // V0.9.9 (2001-03-30) [umoeller]

    #define DBC_DELETE_EXISTINGFILENEWER    24  // V0.9.9 (2001-04-06) [umoeller]

    ULONG guiDatabaseCallback(ULONG ulReportCode,
                              FEDatabaseThreadStatus *pStatus,
                              ULONG ulExtra,
                              ULONG ulUser);

#endif

