
/*
 * fe_archive.h:
 *      header file for fe_archive.cpp.
 *
 *@@added V0.9.0 (99-10-31) [umoeller]
 *@@include #include "base\bs_string.h"
 *@@include #include "base\bs_script.h"
 */

/*
 *      Copyright (C) 1999-2000 Ulrich Mller.
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, in version 2 as it comes in the COPYING
 *      file of this distribution.
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 */

#ifndef WARPIN_FEARCHIVE_HEADER_INCLUDED
    #define WARPIN_FEARCHIVE_HEADER_INCLUDED

    /* ******************************************************************
     *
     *  Forward declarations
     *
     ********************************************************************/

    class FEArcPackageBase;
    class FEArcPackageGroup;
    class FEArcPackagePck;
    class FEDBPackage;
    class BSScriptBase;
    class BSDeclRoot;

    /* ******************************************************************
     *
     *  FEArchive declaration
     *
     ********************************************************************/

    /*
     *@@ FEArchive:
     *      front-end archive class. This is a wrapper around
     *      the back-end WIArchive (which is contained as a member)
     *      and implements script parsing and package management.
     *
     *      See fe_archive.cpp for details.
     *
     *@@added V0.9.0 (99-10-31) [umoeller]
     */

    class FEArchive
    {
        public:
            BSString                _strArcFilename;
                    // archive filename as passed to constructor

            FEArchive               *_pReferencedFrom;
                    // NULL if "root" archive; otherwise this is
                    // an external archive, and this points to
                    // the "mother" archive with the script
                    // V0.9.9 (2001-02-28) [umoeller]

            list<WIPackHeader*>     *_pPackHeadersList;
                    // list of packages in the archive (returned by back-end)

            WIArchive               _Arc;
                    // the back end archive instance
            const WIArcHeader       *_pArcHeader;
                    // pointer to archive header (back end)

            PFNWICALLBACK           _pWICallback2Use;
                    // as set from FEArchive constructor

            BSScriptBase            *_pScript;
                    // pointer to script V0.9.9 (2001-02-19) [umoeller]

            list<FEArcPackageBase*> _PackagesList;
                    // list of package infos (created by FEArchive::ParseScript;
                    // valid after constructor call)

            FEArchive(const char *pcszArcFilename,
                      FEArchive *pReferencedFrom = NULL);

            VOID ParseScript(PFNWICALLBACK pWICallback);

            FEArcPackagePck* FindArcPackage(ULONG ulIndex);

            FEArcPackagePck* IsInArchive(const char *pszPckID,
                                         PULONG pulComp);

        private:
            BSString                _strProfile;
                    // copy of the installation profile found in the archive

            static BSString         _strArchivePath;
                    // path for all archives (cbo 2000-05-23)
            static BSString         _strArchivePathEnv;
                    // WI_ARCHIVE_PATH definition in environment

            VOID CreatePackages(list<BSDeclRoot*> &DeclarationsList,
                                FEArcPackageGroup *pParentGroup,
                                PFNWICALLBACK pWICallback);

            VOID ExtractArchivePath(BSString& strArchiveName);
            VOID PrependArchivePath(BSString& strArchiveName);
                    // archive path handling functions (cbo 2000-05-23)

            VOID ResolveRequirements();
    };

#endif
