
/*
 * fe_base.h:
 *      header file for fe_base.cpp. See remarks there.
 */

/*
 *      Copyright (C) 1998-2000 Ulrich Mller.
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, in version 2 as it comes in the COPYING
 *      file of this distribution.
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 */

#ifndef WARPIN_FEBASE_HEADER_INCLUDED
    #define WARPIN_FEBASE_HEADER_INCLUDED

    /* ******************************************************************
     *
     *  Exception declarations
     *
     ********************************************************************/

    /*
     *@@ FEConstructorExcpt:
     *      exception used when errors occur in constructors.
     *
     *@@changed V0.9.0 (99-11-06) [umoeller]: renamed from ConstructorExcpt
     */

    class FEConstructorExcpt : public BSExcptBase
    {
        public:
            FEConstructorExcpt(const BSString &strClass,
                               const char *pszSourceFile,
                               int iSourceLine,
                               int iMessage);
    };

    /*
     *@@ FEFatalErrorExcpt:
     *      if this is thrown, WarpIN will display an error message
     *      and exit.
     *
     *@@changed V0.9.0 (99-11-06) [umoeller]: renamed from FatalErrorExcpt
     */

    class FEFatalErrorExcpt : public BSExcptBase
    {
        public:
            FEFatalErrorExcpt(const BSString &strMessage);
            FEFatalErrorExcpt(int iMessage,
                              char** apszTable = 0,
                              int iCount = 0);
    };

    /*
     *@@ FEEnvironmentExcept:
     *      exception thrown by GetEnvironment (warpin.cpp)
     *      if an invalid environment variable value
     *      has been encountered.
     */

    class FEEnvironmentExcept : public BSExcptBase
    {
        public:
            FEEnvironmentExcept(const BSString &strCommand,
                                const BSString &strValue);
    };

    APIRET wpiGetMessage(BSString &strBuf,
                         ULONG ulMsgNumber,
                         PCHAR *pTable = NULL,
                         ULONG ulTable = 0);

    ULONG wpiMessageBox(ULONG ulTitleMsg,
                        ULONG ulMsgMsg,
                        ULONG ulFlags,
                        PCHAR *pTable = NULL,
                        ULONG cTable = 0);

    void wpiCTime2FTime(time_t *pCTime,
                        FDATE *pfDate,
                        FTIME *pfTime);

    int wpiCompareFileDates(WIFileHeader* pHeader,
                            FILESTATUS3 *pfs3,
                            FDATE *pfdateLastWrite,
                            FTIME *pftimeLastWrite);

#endif
