/* $Id: fe_database.h,v 1.18 2001/03/31 21:05:49 umoeller Exp $ */


/*
 * fe_database.h:
 *      header file for fe_database.cpp.
 *
 *@@changed V0.9.0 (99-10-31) [umoeller]: renamed from database.h
 *@@include #include "frontend\fe_base.h"
 */

/*
 *      Copyright (C) 1999-2000 Ulrich Mller.
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, in version 2 as it comes in the COPYING
 *      file of this distribution.
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 */

#ifndef WARPIN_FEDATABASE_HEADER_INCLUDED
    #define WARPIN_FEDATABASE_HEADER_INCLUDED

    /* ******************************************************************
     *
     *  Forward declarations
     *
     ********************************************************************/

    class FEDBPackage;
    class FEPackageID;

    // INI keys in database
    #define WPIKEY_FILESLIST        "Files"
    #define WPIKEY_TARGETPATH       "TargetPath"
    #define WPIKEY_REQUIRES         "Requires"
    #define WPIKEY_CONFIGATTRS      "ConfigSysAttrs"
    #define WPIKEY_CONFIGDONE       "ConfigSysDone"
    #define WPIKEY_REGISTERATTRS    "RegisterAttrs"
    #define WPIKEY_REGISTERDONE     "RegisterDone"
    #define WPIKEY_REPLACEATTRS     "ReplaceAttrs"
    #define WPIKEY_REPLACEDONE      "ReplaceDone"
    #define WPIKEY_WPSOBJECTATTRS   "WPSObjectAttrs"
    #define WPIKEY_WPSOBJECTDONE    "WPSObjectDone"
    #define WPIKEY_CLEARPRFATTRS    "ClearPrfAttrs"  // added V0.9.1 (2000-02-07) [umoeller]
    #define WPIKEY_WRITEPRFATTRS    "WritePrfAttrs"  // added V0.9.1 (2000-02-07) [umoeller]
    #define WPIKEY_WRITEPRFDONE     "WritePrfDone"   // added V0.9.1 (2000-02-07) [umoeller]
    #define WPIKEY_EXECUTEATTRS     "ExecuteAttrs"   // added V0.9.1 (2000-02-03) [umoeller]
    #define WPIKEY_EXECUTEDONE      "ExecuteDone"    // added V0.9.1 (2000-02-03) [umoeller]
    #define WPIKEY_DEEXECUTEATTRS   "DeExecuteAttrs" // added V0.9.9 (2001-03-27) [umoeller]
    #define WPIKEY_DEEXECUTERESOLVED "DeExecuteResolved" // added V0.9.9 (2001-03-27) [umoeller]
    #define WPIKEY_KILLPROCESSATTRS "KillProcessAttrs" // added V0.9.1 (2000-02-12) [umoeller]
    #define WPIKEY_WIPACKHEADER     "WIPackHeader"
    #define WPIKEY_INSTALLDATETIME  "InstallDateTime"
    #define WPIKEY_DESCRIPTION      "Description"   // added V0.9.2 (2000-03-10) [umoeller]
    #define WPIKEY_CODEPAGE         "Codepage"      // added V0.9.2 (2000-03-10) [umoeller]

    /*
     *@@ FEDatabaseExcpt:
     *      exception thrown by the database when
     *      it appears to be corrupt.
     *
     *@@added V0.9.1 (2000-01-08) [umoeller]
     */

    class FEDatabaseExcpt : public BSExcptBase
    {
        public:
            BOOL        _fRecover;
            BSString    _strFailingPckID;

            FEDatabaseExcpt(const char *pszPckID,
                            ULONG ulErrNo,
                            BOOL fRecover);
            FEDatabaseExcpt(const char *pszPckID,
                            const char *pszErrMsg,
                            BOOL fRecover);
    };

    /* ******************************************************************
     *
     *  FEDatabase declaration
     *
     ********************************************************************/

    #define DBT_VERIFY          1
    #define DBT_DEINSTALL       2

    #define DBT_DELETEFILES     0x01
    #define DBT_UNDOCONFIG      0x02

    /*
     *@@ FEDatabaseThreadStatus:
     *      structure stored in FEDatabaseThread
     *      which gets updated while the database thread
     *      is running. A pointer to this structure is
     *      always passed to guiDatabaseCallback so
     *      the GUI knows what's going on.
     *
     *@@added V0.9.0 (99-11-02) [umoeller]
     */

    class FEDatabaseThreadStatus
    {
        public:
            ULONG               cPackages;
                            // packages count (in list), initialized by DB thread

            ULONG               cFiles;
                            // total files count (all packages)

            ULONG               ulCurrentPackage;
                            // package being worked on
                            // (updated while DB thread is working)

            ULONG               ulCurrentFile;
                            // total file index being worked on
                            // (updated while DB thread is working)

            FEDBPackage         *pPackage;
                            // current package

            WIFileHeader        *pwifh;
                            // current file (back-end file header)

            ULONG               cDirsInDir,
                                cFilesInDir;
                            // with DBC_DELETEDIRERROR, this tells the
                            // number of directories and files which
                            // are left in a directory that cannot be
                            // deleted

            /*
             *@@ FEDatabaseThreadStatus:
             *      default constructor. Sets all fields to 0.
             */

            FEDatabaseThreadStatus()
            {
                memset(this, 0, sizeof(*this));
            }
    };

    /*
     *@@ FEDatabase:
     *      the database class. An instance (and only
     *      one instance) is created by main().
     */

    class FEDatabase
    {
        // allow only FEPackage class to access our private members
        friend class FEDBPackage;
        friend class FEInstallJob;

        private:
            // profile handle of DATABASE.INI (opened by constructor)
            HINI                hiniDatabase;

        public:
            PSZ EnumPackages(PSZ pszPackageIDSearch);

            // list of all packages in the Global Database;
            // this is only valid after LoadPackagesList has been
            // called
            list<FEDBPackage*>  DBPackagesList;

            FEDatabase(HAB habThread1,
                       const char *pszWarpINPath,
                       const char *pszDeleteID = 0);

            ~FEDatabase();

            FEPackageID* FindPackageID(const char *pcszAuthor,
                                       const char *pcszApplication,
                                       const char *pcszPackageName)
                         const;

            const FEDBPackage* FindDBPackage(const char *pcszAuthor,
                                             const char *pcszApplication,
                                             const char *pcszPackageName);

            const FEDBPackage* FindDBPackage(const FEPackageID &PckID);

            BOOL RemovePackage(const FEDBPackage *pPackage);

            ULONG LoadPackagesList(VOID);

            ULONG IsPackageRequired(FEPackageID& PckIDThis,
                                    list<FEDBPackage*> &RequiredByList,
                                    list<FEDBPackage*> *pIgnoreList);

            BOOL VerifyPackages(list<FEDBPackage*>* pPackagesList,
                                ULONG ulUser,
                                BSFileLogger *pLogFile);

            BOOL DeinstallPackages(list<FEDBPackage*>* pPackagesList,
                                   ULONG ulFlags,
                                   ULONG ulUser,
                                   BSFileLogger *pLogFile);
    };

#endif
