
/*
 * fe_db_thread.h:
 *      header file for fe_db_thread.cpp.
 *
 *@@added V0.9.1 (2000-02-06) [umoeller]
 *@@include #include "frontend\fe_base.h"
 */

/*
 *      Copyright (C) 1999-2000 Ulrich Mller.
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, in version 2 as it comes in the COPYING
 *      file of this distribution.
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 */

#ifndef WARPIN_FEDATABASETHREAD_HEADER_INCLUDED
    #define WARPIN_FEDATABASETHREAD_HEADER_INCLUDED

    /* ******************************************************************
     *
     *  Database thread
     *
     ********************************************************************/

    /*
     *@@ FEDatabaseThread:
     *      this class represents the database thread.
     *
     *      After constructing an instance of this,
     *      the database thread is automatically
     *      started.
     *
     *      This data is stored in PTHREADINFO->ulData
     *      to be passed to the database thread.
     *
     *@@changed V0.9.0 (99-11-02) [umoeller]: renamed from DATABASETHREADINFO
     *@@changed V0.9.0 (99-11-03) [umoeller]: thread automatically started now
     *@@changed V0.9.9 (2001-03-30) [umoeller]: added log file
     */

    struct FEDatabaseThread
    {
        // HAB                     _hab;
                // removed V0.9.12 (2001-05-22) [umoeller]

        list<FEDBPackage*>      _PackagesList;
                        // package list to work on

        ULONG                   _ulTask;
                        // one of the following:
                        // -- DBT_VERIFY: do verify on PackagesList
                        // -- DBT_DEINSTALL: deinstall PackagesList


        ULONG                   _flDeinstall;
                        // for DBT_DEINSTALL: any or none of the following:
                        // -- DBT_DELETEFILES
                        // -- DBT_UNDOCONFIG

        ULONG                   _ulUser;
                        // user parameter passed to callback (HWND)

        FEDatabase              *_pDatabase;
                        // pointer to database

        FEDatabaseThreadStatus  _Status;
                        // files and packages count passed to callback
                        // (99-11-02) [umoeller]

        BSFileLogger            *_pLogFile; // V0.9.9 (2001-03-30) [umoeller]

        FEDatabaseThread(FEDatabase* pDatabase_,
                         list<FEDBPackage*> *pPackagesList_,
                         ULONG ulTask_,
                         ULONG flDeinstall_,
                         ULONG ulUser_,
                         BSFileLogger *pLogFile);

    };

    #ifdef THREADS_HEADER_INCLUDED
        void _Optlink datDatabaseThread(PTHREADINFO pti);
    #endif
#endif
