/* $Id: fe_package.h,v 1.15 2001/05/22 17:21:49 umoeller Exp $ */


/*
 * fe_package.h:
 *      header file for fe_package.cpp.
 *
 *@@include #define INCL_WINSHELLDATA
 *@@include #define INCL_WINWORKPLACE
 *@@include #include <os2.h>
 *@@include #include <string.h>
 *@@include #include "base\bs_errors.h"
 *@@include #include "base\bs_logger.h"
 *@@include #include "base\bs_config.h"
 *@@include #include "frontend\fe_package.h"
 */

/*
 *      Copyright (C) 1998-2000 Ulrich Mller.
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, in version 2 as it comes in the COPYING
 *      file of this distribution.
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 */

#ifndef WARPIN_FEPACKAGE_HEADER_INCLUDED
    #define WARPIN_FEPACKAGE_HEADER_INCLUDED

    /* ******************************************************************
     *
     *  Forward declarations
     *
     ********************************************************************/

    class FEArchive;
    class FEDatabase;
    class FEPackageBase;
    class FEArcPackagePck;
    class FEArcPackageGroup;
    class FEDBPackage;
    class BSPckDeclBase;

    /* ******************************************************************
     *
     *  FEPackageID declaration
     *
     ********************************************************************/

    #define IDCOMP_SAMEPCK      0x10000
    #define IDCOMP_SAMEVERSION  0x00001
    #define IDCOMP_THISNEWER    0x00002
    #define IDCOMP_THISOLDER    0x00004
    #define IDCOMP_VERSIONMASK  0x0000F

    /*
     *@@ FEPackageID:
     *      helper class for splitting a six-part package ID
     *      into more meaningful information, which is done
     *      by the constructor.
     *
     *@@added V0.9.0 (99-10-26) [umoeller]
     */

    class FEPackageID
    {
        public:
            BSString    _strAuthor,
                        _strApplication,
                        _strPackageName;
            ULONG       _ulVersionMajor,
                        _ulVersionMinor,
                        _ulVersionRevision;

            FEPackageID(const char *pszPackageID,
                        const char *pszParentSource,
                        int iParentLine,
                        BOOL fAllowShort);

            FEPackageID(const FEArcPackagePck *pPackage,
                        const char *pszParentSource,
                        int iParentLine,
                        BOOL fAllowShort);

            FEPackageID(const FEDBPackage *pPackage,
                        const char *pszParentSource,
                        int iParentLine,
                        BOOL fAllowShort);

            ~FEPackageID();

            VOID CreateStringSix(BSString &str) const;

            ULONG Compare(FEPackageID &id) const;

        private:
            ULONG Init(const char *pszPackageID,
                       BOOL fAllowShort);
    };

    /* ******************************************************************
     *
     *  FEPackageBase definition
     *
     ********************************************************************/

    // flags for FEPackage.ulType
    #define PCK_PCK     1               // package is FEArcPackagePck
    #define PCK_GROUP   2               // package is FEArcPackageGroup
    #define PCK_DBPCK   3               // package is FEDBPackage

    /*
     *@@ FEPackageBase:
     *      FEPackageBase is the base class used by the front end for
     *      package maintenance. FEPackageBase is NEVER used directly
     *      (that's why the constructor is protected), but only the
     *      subclasses of it:
     *
     *      --  FEArcPackageBase is for packages which are constructed from
     *          an archive (in install mode). This has two subclasses in
     *          turn, FEArcPackagePck and FEArcPackageGroup.
     *
     *      --  FEDBPackage takes a HINI and a six-part package ID to
     *          create a ready-made package info from database information
     *          which has previously been stored using FEInstallJob::Store.
     *
     *@@changed V0.9.0 (99-10-31) [umoeller]: renamed from "PackageInfo".
     *@@changed V0.9.0 (99-11-06) [umoeller]: moved lotsa stuff to the subclasses
     *@@changed V0.9.12 (2001-05-17) [umoeller]: moved _pDecl to subclasses to prevent crashes with group packages
     */

    class FEPackageBase
    {
        // All the following fields are set by the constructors:

        protected:
            ULONG           _ulType;
                    // one of the following:
                    // -- PCK_PCK:     this is a package (PCK tag)
                    //                 --> FEArcPackagePck instance
                    // -- PCK_GROUP:   this is a group of packages (GROUP tag)
                    //                 --> FEArcPackageGroup instance
                    // -- PCK_DBPCK:   this is a database package
                    //                 --> FEDBPackage instance

        public:
            // BSPckDeclBase       *_pDecl;
                    // package declaration, as in script...
                    // -- for FEArcPackagePck, this points to the
                    //    package declaration from the script;
                    // -- for FEArcPackageGroup, this is NULL
                    //    always (FEArcPackageGroup has a special
                    //    member for this);
                    // -- for FEDBPackage, this points to a pseudo
                    //    declaration which is created to hold package
                    //    data.
               // removed this, moved the member to the respective
               // subclasses to avoid the crashes with group jobs...
               // this was too unsafe. V0.9.12 (2001-05-17) [umoeller]

            BSRequiresIDsLogger _logRequiresIDs;
                    // list of five- or six-part package IDs which
                    // specify the packages required by this package.
                    // This is either resolved from the REQUIRES
                    // attributes in the script or retrieved directly
                    // from the database.
                    // added V0.9.1 (2000-01-07) [umoeller]:

            WIPackHeader    _PackHeader;
                    // copy of package header in WIArchive

            // All the following fields are updated while this
            // package is being installed:

            PSZ             _pszFilesList;
                    // list of files that were installed,
                    // each of which is null-terminated;
                    // added to during install or loaded from database
            ULONG           _cbFilesList;
                    // sizeof(*pszFilesList)

            // actual configuration done;
            // added to during wpiConfigure (after install)
            // or loaded from database (in db mode)
            BSCfgSysDoneLogger          _logCfgSysDone;
            BSRegisterDoneLogger        _logRegisterClassesDone;
            BSReplaceDoneLogger         _logReplaceClassDone;
            BSWPSObjectsDoneLogger      _logWPSObjectsDone;
            BSExecuteDoneLogger         _logExecutesDone; // added V0.9.1 (2000-02-03) [umoeller]
            BSDeExecuteResolvedLogger   _logDeExecutesResolved; // V0.9.9 (2001-03-27) [umoeller]
            BSClearPrfAttrsLogger       _logWritePrfsDone; // added V0.9.1 (2000-02-07) [umoeller]
                        // note that there's no BSWritePrfsDoneLogger;
                        // this uses a "clear profiles" logger for undoing

            const char* QueryTargetPath() const;
            const char* QueryPackageID() const;
            const char* QueryTitle() const;

        protected:
            FEPackageBase(ULONG ulType);
            virtual ~FEPackageBase();
    };

#endif
