
/*
 * fe_package_db.h:
 *      header file for fe_package_db.cpp.
 *
 *@@include #define INCL_WINSHELLDATA
 *@@include #define INCL_WINWORKPLACE
 *@@include #include <os2.h>
 *@@include #include <string.h>
 *@@include #include "base\bs_errors.h"
 *@@include #include "base\bs_logger.h"
 *@@include #include "base\bs_config.h"
 *@@include #include "frontend\fe_package_arc.h"
 *@@include #include "frontend\warpin.h"
 */

/*
 *      Copyright (C) 1998-2000 Ulrich Mller.
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, in version 2 as it comes in the COPYING
 *      file of this distribution.
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 */

#ifndef WARPIN_FEPACKAGEDB_HEADER_INCLUDED
    #define WARPIN_FEPACKAGEDB_HEADER_INCLUDED

    class FEDatabaseExcpt;
    class BSConfigBase;

    /*
     *@@ FEDBPackage:
     *      FEPackageBase subclass for packages from the database.
     *      Instances of this class can ONLY be created from a
     *      given database, that is, you'll only get instances
     *      of this if a package has been previously installed.
     *
     *      The constructor reads the package information in
     *      exactly the same format which has been used by
     *      FEInstallJob::Store.
     *
     *@@added V0.9.0 (99-11-03) [umoeller]
     */

    class FEDBPackage : public FEPackageBase
    {
        public:
            BSPckDeclBase       *_pDecl;
                    // package declaration from the script
                    // moved this here from FEPackageBase
                    // V0.9.12 (2001-05-17) [umoeller]

            // the following is for the Database thread (Database mode only)
            ULONG           _ulStatus;      // 0: status unknown
                                            // 1: errors found
                                            // 2: no errors found
            PVOID           _pvGuiData;

            // this holds all configuration items to be undone,
            // which can be of a whole lotta subclasses actually.
            // I have taken one single list which takes all config
            // objects of all subclasses. This replaces the several
            // lists for each of the subclasses, which lead to an
            // incredible code bloat due to template expansion.
            list<BSConfigBase*>         _listUndoConfig;

            DATETIME        _InstallDateTime;

            ULONG           _ulCodepage;
                        // codepage which was used in the original archive
                        // when this package was stored in the database

            FEDBPackage(FEDatabase* pDatabase,
                        const char *pszPckID);

            FEDBPackage(FEDatabase* pDatabase,
                        FEPackageID &PckID);

            list<WIFileHeader*>* GetFileHeaders(PULONG pulHeaderCount);

        private:
            BOOL Load(FEDatabase *pDatabase,
                      FEPackageID &PckID);
    };

#endif
