/* $Id: gui.h,v 1.25 2001/05/22 17:21:52 umoeller Exp $ */


/*
 * gui.h:
 *      header file for gui.c.
 *
 *      This contains only declarations which are platform-specific,
 *      in this case specific to OS/2 Presentation Manager (PM).
 *      warpin.cpp does not see this file. Required GUI callbacks
 *      are therefore declared in calbacks.h instead.
 *
 *      Dialog ID's shared with the resource compiler are now in
 *      dlgids.h (99-10-24) [umoeller].
 *
 *      Copyright (C) 1999-2000 Ulrich Mller.
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, in version 2 as it comes in the COPYING
 *      file of this distribution.
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 */

#ifndef WARPIN_GUI_HEADER_INCLUDED     // added (99-10-24) [umoeller]
    #define WARPIN_GUI_HEADER_INCLUDED

    /* ******************************************************************
     *
     *  Forward declarations
     *
     ********************************************************************/

    class FEDatabase;
    class FEArcPackageBase;
    class FEDBPackage;
    class FEInstallJob;
    class BSConfigBase;

    /* ******************************************************************
     *
     *  WarpIN setup
     *
     ********************************************************************/

    #define WPOBJID_WARPINFOLDER    "<WARPIN_FOLDER>"   // (99-10-22) [umoeller]
    #define WPOBJID_WARPINEXE       "<WARPIN_EXE>"      // (99-10-22) [umoeller]
    #define WPOBJID_WARPINUSERGUIDE "<WARPIN_USERGUIDE>" // V0.9.1 (2000-02-08) [umoeller]
    #define WPOBJID_WARPINPROGGUIDE "<WARPIN_PROGGUIDE>" // V0.9.1 (2000-02-08) [umoeller]

    // OS2.INI application for WarpIN
    #define INIAPP_WARPIN           "WarpIN"            // (99-10-24) [umoeller]
    #define INIKEY_WARPINPATH       "Path"              // (99-10-24) [umoeller]

    /* ******************************************************************
     *
     *  Other declarations
     *
     ********************************************************************/

    // extra window messages used by WarpIN
    #define WPIM_TURNTOPAGE                    (WM_USER+1)
    #define WPIM_QUERYDRIVES                   (WM_USER+2)
    #define WPIM_UPDATE                        (WM_USER+3)
    #define WPIM_FILEEXISTS                    (WM_USER+4)
    #define WPIM_UPDATEPACKAGE                 (WM_USER+5)
    #define WPIM_UPDATEFILE                    (WM_USER+6)
    #define WPIM_UPDATEBYTES                   (WM_USER+7)
    #define WPIM_DONEWITHINSTALL               (WM_USER+8)
    #define WPIM_INSTALLERROR                  (WM_USER+9)
    #define WPIM_FILELOCKEDERROR               (WM_USER+10)
    #define WPIM_PACKAGEREMOVED                (WM_USER+11)
    #define WPIM_ADDSTRING                     (WM_USER+12)
    #define WPIM_DELETE_EXISTINGFILENEWER      (WM_USER+13)

    /*
     *@@ GUISETTINGS:
     *      global structure for GUI settings, which
     *      are stored in WARPIN.INI.
     */

    struct GUISETTINGS
    {
        ULONG           ulLeftView;
                            // one of the following:
                            // -- ID_WIMI_DATABASE_TREEVIEW: applications/packages tree
                            // -- ID_WIMI_DATABASE_DETAILSVIEW: packages details
        ULONG           ulRightView;
                            // one of the following:
                            // -- ID_WIMI_VIEW_FILES
                            // -- ID_WIMI_VIEW_CONFIG
    };

    /*
     *@@ NLSSTRINGS:
     *      this has all the PSZs which are loaded from
     *      the string resources. This is stored in the
     *      global G_NLSStrings variable.
     */

    struct NLSSTRINGS
    {
        // Database thread report strings
        PSZ     pszFileError,
                pszExistingFileNewer,
                pszExistingFileOlder,
                pszDeleteError,
                pszDeleteDirError,
                pszPackageRemoved,
                pszClassDeregistered,
                pszErrorDeregisteringClass,
                pszClassUnreplaced,
                pszErrorUnreplacingClass,
                pszObjectDeleted,
                pszErrorDeletingObject,
                pszCfgSysUndone,
                pszErrorUndoingCfgSys,
                pszProfileCleared,
                pszErrorClearingProfile,

                pszDeExecuting,
                pszDeExecuteDone,
                pszDeExecuteError,

                pszWarningExistingNewer,

                pszUpdatingConfigSys,
                pszRegisteringClasses,
                pszCreatingObjects,

                pszDriveXFree,
                pszElapsed,
                pszToGo,
                pszPackageX,
                pszFileX,

                pszDrive,
                pszFreeKB,
                pszRequiredKB,

                pszDBMainWindowTitle,
                pszDBDeinstallingArchive,
                pszDBDeinstallingPackages,
                pszDBVerifyingPackages,

                pszFileName,
                pszSize,
                pszCreationDate,
                pszLastWriteDate,

                pszApplication,
                pszPackageName,
                pszVersion,
                pszAuthor,
                pszTargetPath,
                pszFiles,
                pszInstallDate,
                pszInstallTime,

                pszVerifying,
                pszDeleting,
                pszDone,

                pszNextButton;      // V0.9.2 (2000-03-11) [umoeller]

        // constructor: loads all the stuff
        NLSSTRINGS()
        {
            memset(this, 0, sizeof(*this));
        }
    };

    /*
     *@@ PACKAGERECORD:
     *      extended record core. This is used on the "Container"
     *      page in "Archive" mode. pJob points to the
     *      corresponding install job, from which the FEPackage
     *      can be retrieved.
     */

    typedef struct _PACKAGERECORD
    {
        RECORDCORE      recc;               // standard record core
        // FEPackage*   pPckInfo;           // linkage to PACKAGEINFO
        FEJobBase       *pJob;              // linkage to corresponding install job (99-11-01) [umoeller]
    } PACKAGERECORD, *PPACKAGERECORD;

    /*
     *@@ DRIVERECORD:
     *      extended record core for "Drive Info" window.
     *      This is only used in "Archive" mode with the
     *      "Container" page.
     *
     *@@changed V0.9.2 (2000-03-04) [umoeller]: made drive bytes double values
     */

    typedef struct _DRIVERECORD
    {
        RECORDCORE     recc;                // standard record core
        ULONG          ulDrive;             // drive no. this recc represents
        double         dFree;               // free space on drive
        double         dRequired;           // required space on drive
        CHAR           szFree[100],
                       szRequired[100];
        PSZ            pszFree,             // points to szFree
                       pszRequired;         // points to szRequired
    } DRIVERECORD, *PDRIVERECORD;

    /*
     *@@ DATABASERECORD:
     *      this is used for the "Database" container,
     *      both for applications (then pFEPackage is NULL)
     *      and packages (then pPackageinfo points to the
     *      package info).
     */

    typedef struct _DATABASERECORD
    {
        RECORDCORE      recc;                // standard record core

        FEDBPackage*    pPckInfo;            // pointer to package;
                                             // for applications, this is NULL
        // the following are used for both applications and packages
        PSZ             pszAuthor;
        PSZ             pszApplication;

        // the following are used for packages only
        PSZ             pszPackageName;
        ULONG           ulVersionMajor;
        ULONG           ulVersionMinor;
        ULONG           ulVersionRevision;
        CHAR            szVersion[30];      // string composed of version number
        PSZ             pszVersion;         // points to szVersion
        ULONG           ulFiles;            // copy of pFEPackage->PackHeader.files
        ULONG           ulTotalSize;        // copy of pFEPackage->PackHeader.origsize
        PSZ             pszTargetPath;      // points to pFEPackage->szTargetPath
        CDATE           cdateInstall;       // date of install
        CTIME           ctimeInstall;       // time of install

        // status window
        HWND            hwndStatus;         // if != NULLHANDLE, this has the open
                                            // database status window for this package
                                            // (verification or de-installation)
    } DATABASERECORD, *PDATABASERECORD;

    /*
     *@@ FILERECORDCORE:
     *      extended record core for "File List" window
     *      in "Data base" mode.
     */

    typedef struct _FILERECORDCORE
    {
        RECORDCORE     recc;                // standard record core
        CHAR           szFilename[CCHMAXPATH];
        ULONG          ulSize;
        CDATE          cdateCreation;
        CDATE          cdateLastWrite;
    } FILERECORDCORE, *PFILERECORDCORE;

    /*
     *@@ DBCONFIGRECORD:
     *
     *@@added V0.9.1 (2000-01-05) [umoeller]:
     */

    typedef struct _DBCONFIGRECORD
    {
        RECORDCORE      recc;               // standard reecord core
        const char*     pszConfigType;      // static string
        PSZ             pszConfigData;      // dynamic buffer, must be freed
        BSConfigBase    *pConfig;           // corresponding configuration object
                                            // V0.9.4 (2000-07-01) [umoeller]
    } DBCONFIGRECORD, *PDBCONFIGRECORD;

    /*
     *@@ DATABASESTATUSINFO:
     *      database status window structure.
     *      This has two purposes:
     *
     *      1)   this must be passed to fnwpDatabaseStatus
     *           as pCreate in WinLoadDlg (mp2 in WM_INITDLG)
     *           to tell the dialog what it should do (ulCmd below);
     *
     *      2)   then, a copy of this struct is allocated and
     *           stored in the dialog's QWL_USER.
     *
     *      Note that the handle of the database status
     *      window is also stored in the DATABASERECORD.
     */

    typedef struct _DATABASESTATUSINFO
    {
        // what the dialog should do
        ULONG           _ulCmd;              // DBACTION_* flags (above)
        ULONG           _ulFlags;
                        // only for DBACTION_DEINSTALL:
                        // null or any of the following:
                        // DBDEINST_CONFIGSYS
                        // DBDEINST_WPSCLASSES
                        // DBDEINST_WPSOBJECTS

        // window title
        PSZ             _pszWindowTitle;

        // the following is updated while the Database thread is running
        BOOL            _fErrorsFound;

        BSFileLogger    *_pNonModalLogFile;
                // if this is != NULL, the status window is operating
                // in non-modal mode, and this is the log file instance
                // that was created from the database gui; this must
                // be closed by fnwpDatabaseStatus then.
                // If this is NULL, we're operating modally, and the
                // logger has been opened by warpin.cpp, and therefore
                // is still used.

        // constructor
        _DATABASESTATUSINFO(ULONG ulCmd_,       // in: DBACTION_* flags
                            ULONG ulFlags_,     // in: for deinstall, DBDEINST_* flags
                            BSFileLogger *pNonModalLogFile, // in: log file or NULL
                            PSZ pszWindowTitle_)    // in: window title
        {
            _ulCmd = ulCmd_;
            _ulFlags = ulFlags_;
            _pszWindowTitle = strdup(pszWindowTitle_);

            _pNonModalLogFile = pNonModalLogFile;

            _fErrorsFound = FALSE;
        }

        // copy constructor
        _DATABASESTATUSINFO(_DATABASESTATUSINFO &X)
        {
            _ulCmd = X._ulCmd;
            _ulFlags = X._ulFlags;
            _pszWindowTitle = strdup(X._pszWindowTitle);

            _pNonModalLogFile = X._pNonModalLogFile;

            _fErrorsFound = FALSE;
        }

        // destructor
        ~_DATABASESTATUSINFO()
        {
            free(_pszWindowTitle);
        }
    } DATABASESTATUSINFO, *PDATABASESTATUSINFO;

    /* ******************************************************************
     *
     *  Shared global variables
     *
     ********************************************************************/

    extern WPIGLOBALS   G_WpiGlobals;
        // in warpin.cpp

    extern FEDatabase*  G_pDatabase;
        // in warpin.cpp

    extern GUISETTINGS  G_GuiSettings;
        // in gui_shared.cpp

    extern HPOINTER     G_hptrMain,           // added (99-10-26) [umoeller]
                        G_hptrSelect,
                        G_hptrDeselect,
                        G_hptrSomeselect,
                        G_hptrError,
                        G_hptrQMark,
                        G_hptrOK,
                        G_hptrApp;
        // in gui.cpp

    // extern HEV          G_hevGUIDone;
        // in gui.cpp
        // removed V0.9.12 (2001-05-22) [umoeller]

    extern NLSSTRINGS   G_NLSStrings;
        // in gui.cpp

    extern HWND         G_hwndHelpInstance;
        // in gui.cpp

    extern HWND         G_hwndMain,
                        G_hwndPackagesCnr,
                        G_hmenuMain;
        // in gui.cpp

    extern ULONG        G_ulDateFormat,
                        G_ulTimeFormat;
    extern CHAR         G_szDateSep[10],
                        G_szTimeSep[10],
                        G_szThousand[10];
        // in gui.cpp

    /* ******************************************************************
     *
     *  Prototypes for funcs in gui_shared.c
     *
     ********************************************************************/

    VOID gshrCreateSplash(VOID);

    VOID gshrDestroySplash(VOID);

    HWND gshrLoadDlg(HWND hwndParent,
                     HWND hwndOwner,
                     PFNWP pfnDlgProc,
                     HMODULE hmod,
                     ULONG idDlg,
                     PVOID pCreateParams);

    VOID gshrLoadSettings(VOID);

    #define SETFL_SAVELOGFILEPATH           0x0001

    VOID gshrSaveSettings(ULONG fl);

    VOID gshrSetControlsFont(HWND hwnd, SHORT sIDMin, SHORT sIDMax);

    HWND gshrLoadMLE(ULONG ulDlgTemplateID,
                     BSString &strText,
                     BOOL fHTML);

    ULONG gshrMLEMessageBox(ULONG ulTitleID,
                            ULONG ulMsgID,
                            BOOL fShowCancel,
                            PSZ* ppTable = NULL,
                            ULONG cbTable = 0);

    MRESULT EXPENTRY gshr_fnwpDialogWithHelp(HWND hwndDlg,
                                             ULONG msg,
                                             MPARAM mp1,
                                             MPARAM mp2);

    VOID gshrPreferencesDlg(HWND hwndOwner);

    HWND gshrLoadConfirmDlg(HWND hwndOwner,
                            ULONG ulDlgID,
                            PFNWP pfnwp,
                            PGUIFILEERROR pfi);

    BOOL gshrDisplayHelp(ULONG ulHelpPanel);

    BOOL gshrDefWindowProc(HWND hwnd,
                           ULONG msg,
                           MPARAM mp1,
                           MPARAM mp2,
                           ULONG ulDefHelpPanel);

    /* ******************************************************************
     *
     *  Help panel IDs in warpin.hlp
     *
     ********************************************************************/

    #define IDHI_MAIN                       1
    #define IDHI_PAGE_TEXTONLY              2       // for MODE_TEXT and MODE_README
    #define IDHI_PAGE_CONTAINER             3
    #define IDHI_PAGE_CONFIGURE             4
    #define IDHI_DATABASE                   5
    #define IDHI_CONFIRM_DEINSTALL          6
    #define IDHI_CONFIRMATIONS_DLG          7
    #define IDHI_DEFAULTPATHS_DLG           8
    #define IDHI_VARPROMPT_DLG              9

#endif
