/* $Id: warpin.h,v 1.27 2001/06/04 07:26:58 umoeller Exp $ */


/*
 * warpin.h:
 *      header file for both warpin.cpp and gui.cpp.
 *
 *      This contains only declarations which are independent of the
 *      implemented GUI, i.e. for the platform-independent frontend
 *      stuff (warpin.cpp).
 *
 *      See warpin.cpp for more about this concept.
 */

/*
 *      Copyright (C) 1998-2000 Ulrich Mller.
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, in version 2 as it comes in the COPYING
 *      file of this distribution.
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 */

#ifndef WARPIN_HEADER_INCLUDED
    #define WARPIN_HEADER_INCLUDED

    /* ******************************************************************
     *
     *  Forward declarations
     *
     ********************************************************************/

    // forward declarations
    class FEArchive;

    class FEArcPackageGroup;
    class FEArcPackagePck;

    class FEJobBase;
    class FEInstallJob;
    class FEGroupJob;

    class BSPageInfo;

    class BSKillProcess;

    class BSFileLogger;

    /* ******************************************************************
     *
     *  Declarations
     *
     ********************************************************************/

    // named mutex semaphore for one-instance check
    #define IDMUTEX_ONEINSTANCE     "\\SEM32\\WARPIN\\ONEINST.SEM"

    // flags for the "File exists" fields in WPIGLOBALS:
    #define FE_PROMPT      0        // 0: prompt
    #define FE_SKIP        1        // 1: skip
    #define FE_OVERWRITE   2        // 2: overwrite

    /*
     *@@ WPIGLOBALS:
     *      global structure in warpin.cpp which gets
     *      passed to guiBeginInstallMode. This has
     *      all the info you'll ever need.
     */

    typedef struct _WPIGLOBALS
    {
        /*
         * Global data (always set by main()):
         *
         */

        // anchor block of this (opened by main());
        // the anchor block is created by the "naked
        // front end" because it's needed for all Prf*
        // functions (the database)
        HAB                 habThread1;

        // fully qualified file name of WARPIN.EXE
        CHAR                szWarpINFile[2*CCHMAXPATH];

        // path extracted from that (without trailing "\")
        CHAR                szWarpINPath[CCHMAXPATH];   // (99-10-24) [umoeller]

        // profile handle of WARPIN.INI (opened by main())
        HINI                hiniWarpIN;

        // application title; copied from <TITLE> tag in archive
        BSString            strAppTitle;

        // file name of log file; defaults to INSTALL.LOG in the
        // WarpIN directory.
        // This may be changed by the user in the GUI; if the
        // first byte is \0, logging is disabled.
        CHAR                szLogFilename[CCHMAXPATH];

        /*
         * Data for "Install" mode:
         *
         */

        BOOL                fDisplayPages;
                    // whether to display pages; defaults to TRUE unless
                    // the user has set WARPIN_DISPLAYPAGES=NO
                    // V0.9.6 (2000-11-23) [umoeller]

        list<FEArchive*>    ArchivesList;
                    // list of all archives opened by WarpIN
        FEArchive*          pFirstArchive;
                    // first archive on that list

        // list of jobs
        list<FEJobBase*>    AllJobsList;        // install jobs and group jobs

        BOOL                fAnyAlreadyInstalled;
                    // TRUE if any packages from the archive are already
                    // installed (checked from database)

        list<BSPageInfo*>*  pPageInfoList;
                    // pointer to the list in a FEArchive instance
                    // (which will be used for displaying the pages)

        list<LONG>          lPagesList;
                    // list of indices to the pages that were visited;
                    // initially empty. This gets updated by the GUI.

        ULONG               ulAllowConfig;
                    // a combination of the following (fe_package.h):
                    // -- PCK_CONFIGSYS: CONFIG.SYS can be updated;
                    // -- PCK_WPSCLASSES: WPS classes can be installed;
                    // -- PCK_WPSOBJECTS: WPS objects can be created.
                    // This gets set initially to the sum of the
                    // package information.
                    // The GUI is responsible for updating this
                    // when leaving the CONFIGURE page so that
                    // warpin.cpp knows whether to do config.

        BOOL                fInstallComplete,
                    // in install mode: TRUE after all files have been
                    //      successfully unpacked
                            fDeinstallComplete,
                    // in database mode: TRUE if the database thread
                    //      has de-installed anything
                            fConfigSysChanged,
                    // TRUE after CONFIG.SYS has been changed
                            fWPSClassesChanged,
                    // TRUE after WPS class list has been changed
                            fWPSObjectsCreated;
                    // TRUE after WPS objects have been created

        CHAR                szConfigSysBackup[CCHMAXPATH];
                    // backup file name of original CONFIG.SYS;
                    // made this a global variable V0.9.6 (2000-10-27) [umoeller]

        string              strTempFiles;
                    // list of all temp files ever created while
                    // WarpIN was running, separated by '|'

        // this flag is set by the "Drive info" window and
        // set to FALSE if there's not enough space on a
        // drive to hold the installation
        BOOL                fPlentyOfSpace;

        // the following flags are for when a file exists:
                                    // 0: prompt
                                    // 1: skip
                                    // 2: overwrite
        ULONG               ulIfSameDate,       // default: 2
                            ulIfExistingOlder,  // default: 2
                            ulIfExistingNewer;  // default: 0

        ULONG               fSuppressREXXConfirm;
                // V0.9.12 (2001-05-31) [umoeller]

        // install thread
        THREADINFO          tiInstallThread;

        // two buffers for specifiying the default apps and tools
        // directories; these must be global, or putenv won't work
        // V0.9.4 (2000-07-26) [umoeller]
        CHAR                szDefaultAppsPath[400],
                                        // WARPIN_DEFAULTAPPSPATH=[path]
                            szDefaultToolsPath[400];
                                        // WARPIN_DEFAULTTOOLSPATH=[path]

        /*
         *@@ _WPIGLOBALS:
         *      constructor.
         */

        _WPIGLOBALS()
        {
            habThread1 = NULLHANDLE;
            szWarpINFile[0] = 0;
            hiniWarpIN = NULLHANDLE;

            pFirstArchive = NULL;

            fDisplayPages = TRUE;   // V0.9.6 (2000-11-23) [umoeller]

            fAnyAlreadyInstalled = FALSE;
            pPageInfoList = 0;

            fInstallComplete = FALSE;
            fDeinstallComplete = FALSE;
            fConfigSysChanged = FALSE;
            fWPSClassesChanged = FALSE;
            fWPSObjectsCreated = FALSE;

            szConfigSysBackup[0] = 0;

            ulAllowConfig = 0;

            fPlentyOfSpace = TRUE;

            // file exists
            ulIfSameDate = 2;           // overwrite
            ulIfExistingOlder = 2;      // overwrite
            ulIfExistingNewer = 0;      // prompt

            fSuppressREXXConfirm = FALSE;

            memset(&tiInstallThread, 0, sizeof(tiInstallThread));

            szDefaultAppsPath[0] = 0;
            szDefaultToolsPath[0] = 0;
        }

    } WPIGLOBALS, *PWPIGLOBALS;

    /* #if __cplusplus
    extern "C" {
    #endif

    APIRET APIENTRY DosReplaceModule(PSZ pszOldModule,
                                     PSZ pszNewModule,
                                     PSZ pszBackupModule);
            // undocumented, but most well-known OS/2 API for
            // replacing an executable which is in use

    #if __cplusplus
    }
    #endif */

    /* ******************************************************************
     *
     *  Prototypes for funcs in warpin.cpp
     *
     ********************************************************************/

    ULONG wpiResolveMacros(BSString &str);

    ULONG wpiResolveMacros(char **ppsz);

    FEArcPackagePck* wpiFindPackageInArchives(const char *pszID,
                                              PULONG pulComp);

    FEGroupJob* wpiFindGroupJob(const FEArcPackageGroup *pGroupPackage);

    VOID wpiCheckJobs(PULONG pulToAdd,
                      PULONG pulToRemove,
                      PULONG pulConfig);

    BOOL wpiCheckDependencies(VOID);

    BOOL wpiVerifySelections(BOOL fPromptCreateDirs);

    APIRET wpiExtractTempFile(ULONG ulPckIndex,
                              const char *pcszFileInPck,
                              BSString *pstrTempFileName);

    BOOL wpiCreateCIDFile(const char *pcszCmdName,
                          const char *pcszArcName);

    /* ******************************************************************
     *
     *  Prototypes for funcs in wpi_configure.cpp
     *
     ********************************************************************/

    BOOL wpiKillOneProcess(BSKillProcess *pKill,
                           BSFileLogger *pLogFile);

    ULONG wpiKillProcesses(BSFileLogger *pLogFile);

    BOOL wpiConfigSys(BSFileLogger *pLogFile);

    BOOL wpiWPSClasses(BSFileLogger *pLogFile);

    BOOL wpiCreateObjects(BSFileLogger *pLogFile);

    BOOL wpiConfigure(BSFileLogger *pLogFile);

    /* ******************************************************************
     *
     *  Install thread
     *
     ********************************************************************/

    int  WICallback(short mode,
                    short s,
                    WIFileHeader* pwifh,
                    void *pvUser);

    #ifdef THREADS_HEADER_INCLUDED
        void _Optlink wpiInstallThread(PTHREADINFO pti);
    #endif

    VOID wpiStartInstallThread(BSFileLogger *pLogFile);

#endif
