/*
 * arcsubsys.h:
 *      header file for asubsys.cpp.
 *
 *@@include <map>
 *@@include "wipengine\fdu.h"
 *@@include "wipengine\threadbase.h"
 *@@include "expat\expat.h"
 *@@include "helpers\linklist.h"
 *@@include "helpers\tree.h"
 *@@include "helpers\xml.h"
 */

/*
 *      Copyright (C) 2000-2001 Teemu Ahola.
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, in version 2 as it comes in the COPYING
 *      file of this distribution.
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 */

#ifndef _ARCSUBSYS_H_
#define _ARCSUBSYS_H_


/*
 *Return values:
 */
#define RET_OK                          0
#define RET_ERR_CALLBACKNOTDEFINED      1
#define RET_ERR_THREADCREATIONFAILED    2
#define RET_ERR_PARAMETERERROR          3
#define RET_ERR_INITIALIZATIONFAILED    4

/*
 *Queue messages:
 */
#define QUE_MSG_QUIT                    1
#define QUE_MSG_ADD_FILES               2
#define QUE_MSG_DELETE_FILES            3
#define QUE_MSG_QUERY_FILES             4

/*
 *Call-back types:
 */
#define CB_TYPE_FILE_ADDINGCOMPLETED    1
#define CB_TYPE_FILE_QUERYINGCOMPLETED  2
#define CB_TYPE_FILE_DELETINGCOMPLETED  3
#define CB_TYPE_QUITCOMPLETED           4


/*
 * Default callback function.
 */
LONG assDefaultCallBack(SHORT p_sCBType, LONG, PVOID p_pData, PVOID);


class ArcSubSystem: public ThreadBase {
    public:
    HQUEUE              _hQReceiver; // Message queue for receiving.
    private:
    FileDatabaseUnit    _Files;
    HQUEUE              _hQMessageLogger; // MLU's message queue handler.
    HMTX                _mtxSemFiles; // For coordination of _Files usage.
    HMTX                _mtxSemQueName; // For coordination of _bssQueueName usage.
    HEV                 _hevSemInit;  // For initialization syncronization.
    HEV                 _hevSemError;  // For error indication.
    PFNWIPECALLBACK     _pfnWipeCallBack; // A call-back for the archive unit.
    PVOID               _pUserData; // A pointer to a user data. This pointer is used as
                                    // a one of the parameters of the call-back function.
    BSString            _bssQueueName; // Message queue name.

    public:
    ArcSubSystem(HQUEUE p_hQMLU,
                 PFNWIPECALLBACK p_pfnWipeCallBack,
                 PVOID p_pUserData
                 ) {
        _hQMessageLogger = p_hQMLU;
        // If no callback function is provided, use default.
        if (p_pfnWipeCallBack)
            _pfnWipeCallBack = p_pfnWipeCallBack;
        else
            _pfnWipeCallBack = &assDefaultCallBack;

        _pUserData = p_pUserData;
        _mtxSemFiles = NULLHANDLE;
        _hQReceiver = NULLHANDLE;
        _hevSemInit = NULLHANDLE;
        _hevSemError = NULLHANDLE;
        _mtxSemQueName = NULLHANDLE;
        InitSemaphores();
        }

    ~ArcSubSystem();

    SetCallBackData(PFNWIPECALLBACK p_pfnWipeCallBack, PVOID p_pUserData) {
        _pfnWipeCallBack = p_pfnWipeCallBack;
        _pUserData = p_pUserData;
        }

    USHORT Start(VOID);

    // Interface helper functions.
    USHORT InsertFiles(FileVector *, HEV = NULLHANDLE);
    USHORT DeleteFiles(FileVector *, HEV = NULLHANDLE);
    USHORT QueryFiles(FileVector *, HEV = NULLHANDLE);
    USHORT CloseArchive(HEV = NULLHANDLE);


    private:
    USHORT InsertFiles(FileVector &);
    USHORT DeleteFiles(FileVector &);
    USHORT QueryFiles(FileVector &);
    VOID _ThdFunction();
    VOID InitSemaphores(VOID);
    };


/*
 *@@ArcSubSystemPtrMap:
 *  typedef for a ArcSubSystem class multimap.
 */
typedef multimap<AID, // key class
                ArcSubSystem*, // data class
                less<AID> // comparsion method
                > ArcSubSystemPtrMap;



#endif

