/*
 * fdu.h:
 *      header file for fdu.cpp.
 *
 *@@include <deque>
 *@@include <vector>
 *@@include "helpers\stringh.h"
 *@@include "helpers\xstring.h"
 *@@include "base\bs_string.h"
 */

/*
 *      Copyright (C) 2000-2001 Teemu Ahola.
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, in version 2 as it comes in the COPYING
 *      file of this distribution.
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 */

#ifndef _FDU_H_
#define _FDU_H_


/*
 *Return values:
 */
#define RET_OK                          0
#define RET_ERR_NOTFOUND                1
#define RET_ERR_PARAMERROR              2


/*
 *@@FileClass:
 *  a class for file related information.
 */
class FileClass {
    public:
    BSString    _bssFileName;
    BSString    _bssFilePath;
    BSString    _bssInstallPath;

    FileClass() {};

    FileClass(BSString p_bssFileName, BSString p_bssFilePath,
              BSString p_bssInstallPath) {
        _bssFileName = p_bssFileName;
        _bssFilePath = p_bssFilePath;
        _bssInstallPath = p_bssInstallPath;
        }

    int compare(const FileClass &) const;
    };

int operator==(const FileClass &, const FileClass &);

int operator<(const FileClass &, const FileClass &);

int operator<=(const FileClass &, const FileClass &);

int operator!=(const FileClass &, const FileClass &);

int operator>(const FileClass &, const FileClass &);

int operator>=(const FileClass &, const FileClass &);


/*
 *@@
 *  typedef for a FileClass deque.
 */
typedef deque<FileClass // data class
             > FileDeque;

/*
 *@@FileVector:
 *  typedef for file vector.
 */
typedef vector<FileClass> FileVector;


class FileDatabaseUnit {
    public:
    FileDeque   _Files;
    public:
    FileDatabaseUnit() {}
    FileDatabaseUnit(ULONG p_ulDefSize) {
        _Files.resize(p_ulDefSize);
        }
    USHORT InsertFile(const FileClass &);
    USHORT InsertFiles(const FileVector &);
    USHORT DeleteFile(const FileClass &);
    USHORT DeleteFile(ULONG);
    USHORT ReplaceFile(const FileClass &, const FileClass &);
    USHORT ReplaceFile(ULONG, const FileClass &);
    USHORT QueryFiles(FileVector &);
    USHORT QueryFile(ULONG, FileClass &);
    ULONG ReturnFileCount() {
        return _Files.size();
        }
    };




#endif


