
/*
 *@@sourcefile gui.cpp:
 *      this implements the graphical user interface (GUI)
 *      of WarpIN, at least the main stuff (there is more
 *      GUI code in the other gui_* source files).
 *
 *      The functions in here get called from main() in warpin.cpp
 *      thru the required callbacks listed in include\frontend\calbacks.h.
 *
 *      See warpin.cpp for details about this concept.
 *
 *      Note that all functions which get called from outside
 *      this file (i.e. either warpin.cpp or database.cpp)
 *      have the gui* prefix. These functions are all prototyped
 *      in warpin.h.
 *
 *@@header "frontend\gui.h"
 */

/*
 *
 *      This file Copyright (C) 1998-2000 Ulrich Mller.
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, in version 2 as it comes in the COPYING
 *      file of this distribution.
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 */

#define OS2EMX_PLAIN_CHAR
    // this is needed for "os2emx.h"; if this is defined,
    // emx will define PSZ as _signed_ char, otherwise
    // as unsigned char

#define INCL_DOS
#define INCL_DOSERRORS
#define INCL_WIN
#define INCL_WINWORKPLACE
#define INCL_GPILOGCOLORTABLE
#define INCL_GPIPRIMITIVES
#include <os2.h>

#include <stdlib.h>
// #include <stdio.h>           // included with wiarchive.h
#ifdef __IBMCPP__
    #include <direct.h>
#endif
#include <string.h>
#include <ctype.h>
#include <setjmp.h>             // needed for except.h
#include <assert.h>             // needed for except.h
#include <io.h>

#include "setup.h"
#include "bldlevel.h"

// include's from helpers
#include "helpers\configsys.h"
#include "helpers\datetime.h"
#include "helpers\dialog.h"
#include "helpers\dosh.h"
#include "helpers\cnrh.h"
#include "helpers\comctl.h"
#include "helpers\gpih.h"
#include "helpers\winh.h"
#include "helpers\standards.h"
#include "helpers\stringh.h"
#include "helpers\textview.h"
#include "helpers\threads.h"
#include "helpers\textv_html.h"
#include "helpers\xstring.h"

// back-end includes
#include "wiarchive\wiarchive.h"

// base includes (99-11-07) [umoeller]
#include "base\bs_string.h"
#include "base\bs_errors.h"
#include "base\bs_logger.h"
#include "base\bs_config.h"
#include "base\bs_config_impl.h"
#include "base\bs_script.h"

// front-end includes
#include "frontend\fe_base.h"
#include "frontend\fe_archive.h"
#include "frontend\fe_database.h"
#include "frontend\fe_job.h"
#include "frontend\fe_package.h"
#include "frontend\fe_package_arc.h"

#include "frontend\warpin.h"
#include "frontend\calbacks.h"

#include "frontend\dlgids.h"
#include "frontend\gui.h"

#include "helpers\except.h"

#pragma hdrstop

/* ******************************************************************
 *
 *  Private declarations
 *
 ********************************************************************/

// forward declarations
MRESULT EXPENTRY fnwpPages(HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2);
MRESULT EXPENTRY fnwpDriveInfo(HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2);
MRESULT EXPENTRY fnwpDatabaseStatus(HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2);
VOID TurnToPage(LONG lSearchPage, BOOL fStoreThis);

/* ******************************************************************
 *
 *  Global variables
 *
 ********************************************************************/

HMQ                 G_hmqThread1;

NLSSTRINGS          G_NLSStrings;

// this always has the main window, in all modes;
// this is used as the owner for message boxes, so this
// better be a valid window at all times (99-11-01) [umoeller]
HWND                G_hwndMain = HWND_DESKTOP;

// handle of main menu bar
HWND                G_hmenuMain = NULLHANDLE;

// subwindows
HWND                G_hwndPageText = NULLHANDLE,
                    G_hwndPageReadme = NULLHANDLE,
                    G_hwndPageContainer = NULLHANDLE,
                    G_hwndPageConfigure = NULLHANDLE,
                    G_hwndDriveInfo = NULLHANDLE;

// tooltip control
HWND                G_hwndCnrTooltip = NULLHANDLE;

// "WarpIN is intializing" window during startup
HWND                G_hwndWarpINStartup = NULLHANDLE;

// "Global status" window
HWND                G_hwndGlobalStatus = NULLHANDLE;
ULONG               G_ulCurrentBoldID = 0;

// help instance
HWND                G_hwndHelpInstance = NULLHANDLE;
ULONG               G_ulPageHelp = 0;

// packages container in various dlgs (there can be only one at a time)
HWND                G_hwndPackagesCnr = NULLHANDLE;

HPOINTER            G_hptrMain = NULLHANDLE,    // added (99-10-26) [umoeller]
                    G_hptrSelect = NULLHANDLE,
                    G_hptrDeselect = NULLHANDLE,
                    G_hptrSomeselect = NULLHANDLE,
                    G_hptrDeinstall = NULLHANDLE, // added (99-11-01) [umoeller]
                    G_hptrError = NULLHANDLE,
                    G_hptrQMark = NULLHANDLE,
                    G_hptrOK    = NULLHANDLE,
                    G_hptrNewer = NULLHANDLE,
                    G_hptrApp   = NULLHANDLE;

// HEV                 G_hevGUIDone = NULLHANDLE;

// system mini icon size (16 or 20)
ULONG               G_ulMiniIconSize;

CHAR                G_szMainWindowTitle[400];

// the following is TRUE after the package record cores
// have been created on the "Container" page
BOOL                G_fContainerPageReady = FALSE;

PPACKAGERECORD      G_preccSelected = NULL;

BSPageInfo*         G_pCurrentPageInfo = NULL;

// NLS settings from OS2.INI
ULONG               G_ulDateFormat;
ULONG               G_ulTimeFormat;
CHAR                G_szDateSep[10],
                    G_szTimeSep[10],
                    G_szThousand[10];

// list of files to copy on WarpIN upgrade
PCSZ G_apcszUpgradeFiles[] =
        {
            "COPYING",
            "warpin.exe",
            "wizilla.exe",
            "wic.exe",
            "wicpm.exe",
            "wpi2exe.exe",
            "wpirtl.dll",
            "wicpm.inf",
            "wpi_prog.inf",
            "wpi_user.inf",
            "warpin.sym",
            "wic.sym",
            "wicpm.sym",
            "warpin.hlp",
            "warpin.tmf",
            "readme.txt",
            "history.txt",
            "test\\apps.cmd"
        };

MSGBOXSTRINGS G_MsgBoxStrings =
        {
            "~Yes",
            "~No",
            "~OK",
            "~Cancel",
            "~Abort",
            "~Retry",
            "~Ignore",
            "~Enter",
            "Yes to ~all"
        };

/* ******************************************************************
 *
 *  Various helper functions
 *
 ********************************************************************/

/*
 *@@ LoadString:
 *      this loads one NLS string from the resources.
 *      Used during guiInitialize.
 */

VOID LoadString(ULONG ulID, PSZ *ppsz)
{
    CHAR    szBuf[255];
    if (*ppsz)
        free(*ppsz);
    if (WinLoadString(G_WpiGlobals.habThread1,
                      NULLHANDLE, ulID ,
                      sizeof(szBuf), szBuf))
        *ppsz = strdup(szBuf);
    else
        *ppsz = strdup("Error: String resource not found");
}

/*
 *@@ CreateWarpINObjects:
 *      this gets called by guiInitialize if the
 *      WarpIN program object does not exist yet
 *      to create the WarpIN objects.
 *
 *      Returns TRUE only if all objects could be
 *      created.
 *
 *@@added (99-10-22) [umoeller]
 *@@changed V0.9.3 (2000-04-28) [umoeller]: added readme shadow
 *@@changed V0.9.3 (2000-05-23) [umoeller]: changed to "replace" objects mode
 */

BOOL CreateWarpINObjects(VOID)
{
    HOBJECT     hobjFolder = NULLHANDLE;
    if ((hobjFolder = WinCreateObject("WPFolder",
                                      "WarpIN",
                                      "OBJECTID="WPOBJID_WARPINFOLDER";",
                                      "<WP_DESKTOP>",
                                      CO_REPLACEIFEXISTS))
                == NULLHANDLE)
        return (FALSE);

    WinSetObjectData(hobjFolder, "OPEN=DEFAULT");

    BSString    strSetupString;

    strSetupString._printf(
            "ASSOCFILTER=*.WPI;ASSOCTYPE=WarpIN Archive;EXENAME=%s;PROGTYPE=PM;STARTUPDIR=%s;"
                "OBJECTID=" WPOBJID_WARPINEXE";",
            G_WpiGlobals.szWarpINFile,        // EXEFILE
            G_WpiGlobals.szWarpINPath);       // startup dir

    if (!WinCreateObject("WPProgram",
                         "WarpIN",
                         strSetupString.c_str(),
                         WPOBJID_WARPINFOLDER,
                         CO_REPLACEIFEXISTS))
        return (FALSE);

    // OK, successfull install:
    // store our path in OS2.INI so that future instances of
    // WarpIN can know if they're not being started from the
    // initial location
    PrfWriteProfileString(HINI_USER,
                          INIAPP_WARPIN,
                          INIKEY_WARPINPATH,
                          G_WpiGlobals.szWarpINPath);

    // WarpIN User's Guide
    strSetupString._printf(
            "EXENAME=VIEW.EXE;PROGTYPE=PM;PARAMETERS=WPI_USER.INF;STARTUPDIR=%s;"
                "OBJECTID=" WPOBJID_WARPINPROGGUIDE";",
            G_WpiGlobals.szWarpINPath);       // startup dir
    WinCreateObject("WPProgram",
                    "WarpIN User's Guide",
                    strSetupString.c_str(),
                    WPOBJID_WARPINFOLDER,
                    CO_REPLACEIFEXISTS);

    // WarpIN Programmer's Guide and Reference
    strSetupString._printf(
            "EXENAME=VIEW.EXE;PROGTYPE=PM;PARAMETERS=WPI_PROG.INF;STARTUPDIR=%s;"
                "OBJECTID=" WPOBJID_WARPINUSERGUIDE";",
            G_WpiGlobals.szWarpINPath);       // startup dir
    WinCreateObject("WPProgram",
                    "WarpIN Programmer's Guide and Reference",
                    strSetupString.c_str(),
                    WPOBJID_WARPINFOLDER,
                    CO_REPLACEIFEXISTS);

    // README (shadow)
    strSetupString._printf("SHADOWID=%s\\readme.txt;",
                        G_WpiGlobals.szWarpINPath);
    WinCreateObject("WPShadow",
                    "readme.txt",
                    strSetupString.c_str(),
                    WPOBJID_WARPINFOLDER,
                    CO_REPLACEIFEXISTS);

    return (TRUE);
}

typedef struct _UPGRADEINFO
{
    PCSZ        pcszPathToUpgrade;
    PCSZ        pcszPathThis;
    PCSZ        pcszFailed;         // out: file that failed.
    APIRET      arc;
} UPGRADEINFO, *PUPGRADEINFO;

/*
 *@@ fntUpgradeExisting:
 *      synchronous thread started by CheckInstall
 *      to copy objects over to the existing installation.
 *
 *@@added V0.9.7 (2001-01-07) [umoeller]
 */

VOID _Optlink fntUpgradeExisting(PTHREADINFO ptiMyself)
{
    BOOL brc = TRUE;
    // copy info
    PUPGRADEINFO pInfo = (PUPGRADEINFO)(ptiMyself->ulData);

    HWND hwndInfo = WinWindowFromID(G_hwndWarpINStartup, ID_WIDI_INFOTEXT);

    WinSetWindowULong(hwndInfo,
                      QWL_STYLE,
                      WS_VISIBLE | SS_TEXT | DT_LEFT | DT_TOP | DT_WORDBREAK);
    winhSetWindowFont(hwndInfo, "8.Helv");

    BSString    strTemplate;
    wpiGetMessage(strTemplate,
                  197);         // "Copying "%s" to "%s""

    ULONG ul;
    for (ul = 0;
         ul < sizeof(G_apcszUpgradeFiles) / sizeof(G_apcszUpgradeFiles[0]);   // array item count
         ul++)
    {
        PCSZ pcszFileThis = G_apcszUpgradeFiles[ul];

        BSString strPathThis = pInfo->pcszPathThis,
                 strPathToUpgrade = pInfo->pcszPathToUpgrade,
                 strFileThis = pcszFileThis;

        BSString strInfo,
                 str1 = strPathThis + "\\" + strFileThis,
                 str2 = strPathToUpgrade + "\\" + strFileThis;
        strInfo._printf(strTemplate.c_str(), // "Copying "%s" to "%s""
                        str1.c_str(),
                        str2.c_str());
        WinSetWindowText(hwndInfo, strInfo.c_str());

        APIRET arc = DosCopy(str1.c_str(),
                             str2.c_str(),
                             DCPY_EXISTING);        // overwrite
        if (arc != NO_ERROR)
        {
            pInfo->pcszFailed = pcszFileThis;
            pInfo->arc = arc;
            // return FALSE
            brc = FALSE;
        }

    }

    WinPostMsg(ptiMyself->hwndNotify, WM_USER, (MPARAM)brc, 0);
}

/*
 *@@ CheckInstall:
 *      called by guiInitialize to check if WarpIN is installed.
 *      If not, CreateWarpINObjects() gets called in turn.
 *
 *@@added V0.9.7 (2001-01-06) [umoeller]
 *@@changed V0.9.9 (2001-04-06) [umoeller]: now creating "test" in target
 */

BOOL CheckInstall(VOID)
{
    BOOL brc = TRUE,
         fForceRecreateObjects = FALSE;

    // check if WARPIN.EXE has been installed in a different
    // directory before
    CHAR szLastPath[CCHMAXPATH];
    if (PrfQueryProfileString(HINI_USER, INIAPP_WARPIN, INIKEY_WARPINPATH,
                              NULL,     // default string
                              szLastPath, sizeof(szLastPath)))
    {
        // OK, WarpIN has been installed already:
        if (stricmp(szLastPath, G_WpiGlobals.szWarpINPath) != 0)
        {
            BOOL    fAllowUpgrade = TRUE;
            CHAR    szOtherVersion[100] = "?";
            PSZ     apszTable[] = {
                        szLastPath,
                        szOtherVersion,
                        BLDLEVEL_VERSION,
                        szLastPath};
            // different: give warning

            // get version number of the other installation
            PEXECUTABLE pOtherExec = {0};
            CHAR        szOtherExec[2*CCHMAXPATH];
            sprintf(szOtherExec, "%s\\WARPIN.EXE", szLastPath);
            APIRET arc = doshExecOpen(szOtherExec, &pOtherExec);
            if (arc == NO_ERROR)
            {
                arc = doshExecQueryBldLevel(pOtherExec);
                if (arc == NO_ERROR)
                {
                    if (pOtherExec->pszVersion)
                    {
                        ULONG   ulMajorThis,
                                ulMinorThis,
                                ulRevThis,
                                ulMajorOther,
                                ulMinorOther,
                                ulRevOther;
                        strcpy(szOtherVersion, pOtherExec->pszVersion);
                        // compare versions...
                        sscanf(szOtherVersion,
                               "%d.%d.%d",
                               &ulMajorOther,
                               &ulMinorOther,
                               &ulRevOther);
                        sscanf(BLDLEVEL_VERSION,
                               "%d.%d.%d",
                               &ulMajorThis,
                               &ulMinorThis,
                               &ulRevThis);

                        if (    (ulMajorOther > ulMajorThis)
                             || (   (ulMajorOther == ulMajorThis)
                                 && (ulMinorOther > ulMinorThis)
                                )
                             || (   (ulMajorOther == ulMajorThis)
                                 && (ulMinorOther == ulMinorThis)
                                 && (ulRevOther > ulRevThis)
                                )
                           )
                            fAllowUpgrade = FALSE;
                        // else: OK, allow upgrade
                    }
                    else
                        fAllowUpgrade = FALSE;
                }
                else
                    fAllowUpgrade = FALSE;
                doshExecClose(pOtherExec);
            } // if (arc == NO_ERROR)
            else
                fAllowUpgrade = FALSE;

            // show msg box
            BSString    strUpgrade;
            wpiGetMessage(strUpgrade,
                          151, // "previously installed -- update?"
                          apszTable, 4); // string replacements: directory
            HWND hwndUpgrade = gshrLoadMLE(ID_WID_UPGRADE,
                                           strUpgrade,
                                           TRUE);       // HTML
            if (!fAllowUpgrade)
                // existing is newer:
                // disable "upgrade existing"
                WinEnableControl(hwndUpgrade, 98, FALSE);

            // set window title
            BSString    strTitle;
            wpiGetMessage(strTitle,
                          108); // "warning"
            WinSetWindowText(hwndUpgrade, strTitle.c_str());

            winhCenterWindow(hwndUpgrade);
            // go!!
            ULONG ulrc = WinProcessDlg(hwndUpgrade);
            switch (ulrc)
            {
                case 98:        // upgrade existing
                {
                    // create "test" subdirectory in target path
                    // V0.9.9 (2001-04-06) [umoeller]
                    string strTemp = szLastPath;
                    if (strTemp[strTemp.length() - 1] != '\\')
                        strTemp += "\\";
                    strTemp += "test";
                    DosCreateDir(strTemp.c_str(), NULL);
                    // end V0.9.9 (2001-04-06) [umoeller]

                    UPGRADEINFO ui;
                    ui.pcszPathToUpgrade = szLastPath;
                    ui.pcszPathThis = G_WpiGlobals.szWarpINPath;
                    if (thrRunSync(G_WpiGlobals.habThread1,
                                   fntUpgradeExisting,
                                   "UpgradeExisting",
                                   (ULONG)&ui)       // no param...
                        == FALSE)
                    {
                        // error copying:
                        CHAR szError[10];
                        PSZ pszDescription = doshQuerySysErrorMsg(ui.arc);
                        sprintf(szError, "%d", ui.arc);
                        PCSZ apszTable[] = {szError,
                                            ui.pcszFailed,
                                            pszDescription};
                        wpiMessageBox(100,
                                      198,      // error %1 copying %2
                                      MB_CANCEL | MB_INFORMATION,
                                      (PCHAR*)apszTable,
                                      3);
                        free(pszDescription);
                    }
                    else
                    {
                        PSZ psz = szLastPath;
                        wpiMessageBox(100,
                                      199,      // success
                                      MB_CANCEL | MB_INFORMATION,
                                      &psz, 1);
                    }

                    brc = FALSE;
                break; }

                case 99:        // disable existing
                {
                    // update profile data
                    PrfWriteProfileString(HINI_USER,
                                          INIAPP_WARPIN,
                                          INIKEY_WARPINPATH,
                                          G_WpiGlobals.szWarpINPath);
                    PSZ psz = szLastPath;
                    if (wpiMessageBox(100,
                                      200,      // update prog objects?
                                      MB_YESNO | MB_INFORMATION,
                                      &psz, 1)
                                == MBID_YES)
                        fForceRecreateObjects = TRUE;
                break; }

                default:
                    // no continue: get outta here
                    brc = FALSE;
            }

            WinDestroyWindow(hwndUpgrade);
        } // end if (stricmp(szLastPath, G_WpiGlobals.szWarpINPath) != 0)
    } // end if (PrfQueryProfileString(HINI_USER, INIAPP_WARPIN, INIKEY_WARPINPATH,

    if (brc)
    {
        if (WinQueryObject("<WP_DESKTOP>"))
        {
            // WPS running:
            // check installation (99-10-22) [umoeller]
            HOBJECT hobjWarpinExe = WinQueryObject(WPOBJID_WARPINEXE);
            if ((!hobjWarpinExe) || (fForceRecreateObjects))
            {
                ULONG ulrc = DID_OK;
                // does not exist yet:
                // prompt with install text,
                // but only if we haven't forced update above
                if (!hobjWarpinExe)
                {
                    PSZ     pszTable = BLDLEVEL_VERSION;
                    ulrc = gshrMLEMessageBox(149,      // title
                                             148,
                                             TRUE,     // show cancel
                                             &pszTable, 1);  // string replacements
                }

                if (ulrc == DID_OK)
                {
                    if (CreateWarpINObjects())
                        // success:
                        wpiMessageBox(100,
                                      152,
                                      MB_OK | MB_INFORMATION);
                    else
                        // error creating objects:
                        guiShowMessage("WarpIN",
                                       "An error occured creating the WarpIN WPS objects.",
                                       MB_OK | MB_ICONHAND);
                }

                brc = FALSE;
                    // in any case, don't continue
            }
        }
    }

    return (brc);
}

/* ******************************************************************
 *
 *  Helpers for "Full install" and "Add/remove" modes
 *
 ********************************************************************/

/*
 *@@ UpdatePackageDisplay:
 *      this updates the display on the "Container"
 *      page.
 *
 *      This is called in two situations: when a
 *      package has been (de)selected or when the
 *      emphasis changes.
 *
 *@@changed V0.9.4 (2000-07-26) [umoeller]: fixed outdated dlg items
 */

VOID UpdatePackageDisplay()
{
    FEInstallJob *pInstallJob = G_preccSelected->pJob->IsInstallJob();

    // update container page
    winhShowDlgItem(G_hwndPageContainer, ID_WIDI_PATHENTRY,
                    (pInstallJob != NULL));
    winhShowDlgItem(G_hwndPageContainer, ID_WIDI_PATHTEXT,
                    (pInstallJob != NULL));
    if (pInstallJob)
    {
        const FEArcPackagePck *pPackage = pInstallJob->_pArcPackage;

        // enable target path entry field if...
        BOOL fEnable = (
                            // package has been selected
                            (pInstallJob->Is2BeInstalled())
                            // and path is not fixed
                         && (pPackage->_pDecl->_ulPathType & PATH_VARIABLE)
                            // and package is not installed yet
                         && (pInstallJob->_pPackageIDInstalled == 0)
                       );
        winhEnableDlgItem(G_hwndPageContainer, ID_WIDI_PATHENTRY,
                          (fEnable));
        winhEnableDlgItem(G_hwndPageContainer, ID_WIDI_PATHTEXT,
                          (fEnable));
        // and set target path
        WinSetDlgItemText(G_hwndPageContainer, ID_WIDI_PATHENTRY,
                          pInstallJob->QueryTargetPath());

        // set installed version
        /* FEPackageID PckID(pInstallJob->_pArcPackage->_pszID, __FILE__, __LINE__,
                          FALSE);    // don't allow short format
            */
        /* sprintf(szVersion, "%d.%d.%d",
                PckID._ulVersionMajor,
                PckID._ulVersionMinor,
                PckID._ulVersionRevision); */
        /* WinSetDlgItemText(G_hwndPageContainer, ID_WIDI_PCKINARCHIVE,
                          szVersion); */

        if (    (pInstallJob->_ulInstallStatus == INSTALLED_NO)
             || (pInstallJob->_pPackageIDInstalled == 0)
           )
        {
            // not installed:
            WinEnableControl(G_hwndPageContainer, ID_WIDI_PCKINSTALLED_TXT, FALSE);
            WinSetDlgItemText(G_hwndPageContainer, ID_WIDI_PCKINSTALLED,
                              "");
        }
        else
        {
            WinEnableControl(G_hwndPageContainer, ID_WIDI_PCKINSTALLED_TXT, TRUE);
            CHAR    szVersion[100 + CCHMAXPATH];
            sprintf(szVersion, "%d.%d.%d",
                    pInstallJob->_pPackageIDInstalled->_ulVersionMajor,
                    pInstallJob->_pPackageIDInstalled->_ulVersionMinor,
                    pInstallJob->_pPackageIDInstalled->_ulVersionRevision);
            WinSetDlgItemText(G_hwndPageContainer, ID_WIDI_PCKINSTALLED,
                              szVersion);
        }
    }

    guiUpdateGlobalStatus(GSTAT_COLLECTING);
}

/*
 *@@ CnrPackageOwnerDraw:
 *      helper function for container owner draw. This
 *      is shared between several window procedures,
 *      used with package record cores when WM_DRAWITEM
 *      comes in.
 *
 *      We need container owner draw to draw the package
 *      icons ourselves.
 *
 *      This returns the BOOL which must be returned
 *      from WM_DRAWITEM.
 *
 *@@added V0.9.0 (99-11-01) [umoeller]
 */

MRESULT CnrPackageOwnerDraw(POWNERITEM poi)
{
    MRESULT mrc = (MPARAM)FALSE;

    // check if we're to draw the text or the icon
    if (poi->idItem == CMA_ICON)
    {
        // icon:

        PCNRDRAWITEMINFO pcdii = (PCNRDRAWITEMINFO)poi->hItem;
        HPOINTER hptr;

        // 1) draw current install status to the left
        FEJobBase *pJob =  ((PPACKAGERECORD)(pcdii->pRecord))->pJob;
        FEInstallJob *pInstallJob = pJob->IsInstallJob();
        if (pInstallJob)
            if (pInstallJob->_ulInstallStatus != INSTALLED_NO)
            {
                HPOINTER hptrDraw;
                if (pInstallJob->_ulInstallStatus == INSTALLED_IS_NEWER)
                    hptrDraw = G_hptrNewer;
                else
                    hptrDraw = G_hptrOK;
                WinDrawPointer(poi->hps,
                               poi->rclItem.xLeft,
                               poi->rclItem.yBottom,
                               hptrDraw,
                               DP_MINI);
            }

        // 2) draw selection status (for installation)
        switch (pJob->QueryJobID())
        {
            case JOB_INSTALL:
                hptr = G_hptrSelect;
            break;

            case JOB_GROUP_MIXED:
                hptr = G_hptrSomeselect;
            break;

            case JOB_DEINSTALL:
                hptr = G_hptrDeinstall;
            break;

            default:    // JOB_INSTALL_IGNORE
                hptr = G_hptrDeselect;
            break;

        }

        WinDrawPointer(poi->hps,
                       poi->rclItem.xLeft + G_ulMiniIconSize + 5,
                       poi->rclItem.yBottom,
                       hptr,
                       DP_MINI);

        // mrc = (MPARAM)TRUE;     // we've drawn the item
    }

    return (mrc);
}

/*
 *@@ Packages2Cnr:
 *      helper function which inserts all packages into
 *      the given container window, which should be in
 *      Tree view to give a meaningful display.
 *
 *@@added V0.9.0 (99-11-01) [umoeller]
 *@@changed V0.9.12 (2001-05-17) [umoeller]: fixed crashes with group jobs
 */

ULONG Packages2Cnr(HWND hwndCnr)
{
    ULONG ulCount;
    PPACKAGERECORD preccThis;
    string strTitleWithSize;            // replaced char[] V0.9.12 (2001-05-17) [umoeller]

    // go thru the list of packages
    // (this has both "groups" and real packages)
    list<FEJobBase*>::iterator JobStart = G_WpiGlobals.AllJobsList.begin(),
                               JobEnd = G_WpiGlobals.AllJobsList.end();
    for (; JobStart != JobEnd; JobStart++)
    {
        FEJobBase *pJobThis = *JobStart;

        // create record core for this package
        preccThis = (PPACKAGERECORD)cnrhAllocRecords(hwndCnr,
                                                     sizeof(PACKAGERECORD),
                                                     1);
        ULONG ulAttrs = CRA_RECORDREADONLY;

        // store job in recc
        preccThis->pJob = pJobThis;
        // and recc in job
        pJobThis->_pvGuiData = (PRECORDCORE)preccThis;

        FEGroupJob *pGroupJob = pJobThis->IsGroupJob();
        const FEArcPackageBase *pPackageThis = NULL;
        if (pGroupJob)
        {
            // is group:
            strTitleWithSize = pGroupJob->_pGroupPackage->_pGroupDecl->_strTitle;
            if (pGroupJob->_pGroupPackage->_pGroupDecl->_fInitiallyExpanded)
                ulAttrs |= CRA_EXPANDED;
            else
                ulAttrs |= CRA_COLLAPSED;
            pPackageThis = pGroupJob->_pGroupPackage;
        }
        else
        {
            // is package:
            FEInstallJob *pInstallJob = pJobThis->IsInstallJob();
            if (pInstallJob)
            {
                const FEArcPackagePck *pPckPckThis = pInstallJob->_pArcPackage;
                pPackageThis = pPckPckThis;
                // only for packages: sum up size
                CHAR szSizeRounded[20];
                strhThousandsULong(szSizeRounded,
                                   pPackageThis->_ulSizeRounded,
                                   G_szThousand[0]);
                FEPackageID PckID(pPckPckThis, __FILE__, __LINE__,
                                  FALSE);    // don't allow short format
                strTitleWithSize._printf("%s\n(V%d.%d.%d, %s KB)",
                        pPckPckThis->_pDecl->_strTitle.c_str(),
                        PckID._ulVersionMajor,
                        PckID._ulVersionMinor,
                        PckID._ulVersionRevision,
                        szSizeRounded);
            }
        }

        PRECORDCORE preccParent = NULL;
        // if the package belongs to a group, find
        // the group to get the parent record
        if (pPackageThis->_pGroup)
        {
            FEGroupJob *pGroupJob = wpiFindGroupJob(pPackageThis->_pGroup);
            if (pGroupJob)
                preccParent = (PRECORDCORE)(pGroupJob->_pvGuiData);
        }

        // insert record core into cnr
        cnrhInsertRecords(hwndCnr,
                          // parent record:
                          preccParent,
                          (PRECORDCORE)preccThis,
                          TRUE,
                          strdup(strTitleWithSize.c_str()),
                          ulAttrs,
                          1);
        ulCount++;
    }

    return (ulCount);
}

/*
 *@@ CreateCnrTooltip:
 *      creates a tooltip control for the packages
 *      container. The tooltip window handle is
 *      stored in G_hwndCnrTooltip.
 *
 *@@added V0.9.2 (2000-03-10) [umoeller]
 */

VOID CreateCnrTooltip(HWND hwndOwner,
                      HWND hwndCnr)
{
    G_hwndCnrTooltip = WinCreateWindow(HWND_DESKTOP,  // parent
                                       COMCTL_TOOLTIP_CLASS, // wnd class
                                       "",            // window text
                                       TTS_SHADOW | TTF_SHYMOUSE | TTS_ALWAYSTIP,
                                            // tooltip window style
                                       10, 10, 10, 10,    // window pos and size, ignored
                                       hwndOwner, // owner window -- important!
                                       HWND_TOP,      // hwndInsertBehind, ignored
                                       19999, // window ID, optional
                                       NULL,          // control data
                                       NULL);         // presparams
    TOOLINFO    ti;
    ti.ulFlags = TTF_SUBCLASS;
    ti.hwndToolOwner = hwndOwner;
    ti.pszText = PSZ_TEXTCALLBACK;  // send TTN_NEEDTEXT
    ti.hwndTool = hwndCnr;
    // add container as tool
    WinSendMsg(G_hwndCnrTooltip,
               TTM_ADDTOOL,
               (MPARAM)0,
               &ti);
    // set timers
    WinSendMsg(G_hwndCnrTooltip,
               TTM_SETDELAYTIME,
               (MPARAM)TTDT_AUTOPOP,
               (MPARAM)(20*1000));        // 20 secs for autopop (hide)
}

/*
 *@@ GetTooltipText:
 *      implementation for tooltip's TTN_NEEDTEXT.
 *
 *@@added V0.9.2 (2000-03-10) [umoeller]
 */

VOID GetTooltipText(HWND hwndOwner,
                    HWND hwndCnr,
                    MPARAM mp2)
{
    // find record under mouse
    POINTL ptlMouse;
    WinQueryMsgPos(WinQueryAnchorBlock(hwndOwner),
                   &ptlMouse);
    PPACKAGERECORD precc
        = (PPACKAGERECORD)cnrhFindRecordFromPoint(hwndCnr,
                                                  &ptlMouse,
                                                  NULL,
                                                  CMA_ICON | CMA_TEXT,
                                                  FRFP_SCREENCOORDS);
    if (precc)
        if (precc->pJob)
        {
            FEInstallJob *pJob = precc->pJob->IsInstallJob();
            if (pJob)
            {
                PTOOLTIPTEXT pttt = (PTOOLTIPTEXT)mp2;
                pttt->ulFormat = TTFMT_PSZ;
                pttt->pszText = (PSZ)pJob->_pArcPackage->_pDecl->_strDescription.c_str();
            }
        }
}

/*
 *@@ TurnToPage:
 *      this gets called by fnwpPages when the
 *      "Next" or "Back" buttons are pressed.
 *      We update the dialogs then according
 *      to the page data which was prepared
 *      by warpin.cpp in WPIGLOBALS.
 *
 *      Throws:
 *      -- FEFatalErrorExcpt.
 *
 *@@changed 0.9.0: path entry field on cnr page now accepts 254 chars
 *@@changed V0.9.1 (2000-01-12) [umoeller]: README MLE was read/write; fixed
 *@@changed V0.9.1 (2000-02-03) [umoeller]: README MLE wasn't always scrolled to top
 *@@changed V0.9.3 (2000-04-28) [umoeller]: configure checkboxes were disabled, but checked; fixed
 *@@changed V0.9.3 (2000-05-05) [umoeller]: added FORMAT=PLAIN support
 *@@changed V0.9.3 (2000-05-06) [umoeller]: much better formatting now
 */

VOID TurnToPage(LONG lSearchPage,   // in: page index to turn to
                BOOL fStoreThis)    // in: store current index in G_WpiGlobals.lPagesList?
{
    /*
     * 1) handle "leaving page" stuff:
     *
     */

    if (G_pCurrentPageInfo)
    {
        // this still has the old page
        switch (G_pCurrentPageInfo->_PageMode)
        {
            /*
             * MODE_CONTAINER:
             *      leaving container page
             */

            case BSPageInfo::MODE_CONTAINER:
            {
                // PmPrintf("  Leaving container page");

                // if lSearchPage comes _after_ the current
                // page, we must check the validity of package
                // selections (we don't want to do this if
                // the user presses the "Back" button)

                if (lSearchPage > G_pCurrentPageInfo->_lPageIndex)
                {
                    // prompt for missing dirs
                    // and check dependencies
                    if (!wpiVerifySelections(TRUE))
                        // error found: do not leave container page
                        return;
                }

                // finally, leaving container page:
                // hide "drive info" window
                WinPostMsg(G_hwndDriveInfo, WPIM_UPDATE, (MPARAM)0, NULL);
            break; } // case MODE_CONTAINER

        } // end switch (pCurrentPageInfo->lPageMode)

        // store the index of the page that we're leaving
        // in the list in G_WpiGlobals,
        if (fStoreThis)
            G_WpiGlobals.lPagesList.push_back(G_pCurrentPageInfo->_lPageIndex);

    } // end if (pCurrentPageInfo)

    // find the page we're looking for (lSearchPage)
    G_pCurrentPageInfo = NULL;
    if (G_WpiGlobals.pPageInfoList)
    {
        list<BSPageInfo*>::iterator PageStart = G_WpiGlobals.pPageInfoList->begin(),
                                    PageEnd = G_WpiGlobals.pPageInfoList->end();
        for (; PageStart != PageEnd; PageStart++)
            if ((**PageStart)._lPageIndex == lSearchPage)
            {
                // page found: mark it
                G_pCurrentPageInfo = *PageStart;
                break;
            }

        if (G_pCurrentPageInfo == NULL)
        {
            BSString strMsg;
            strMsg._printf("Unable to find page %d in the "
                           "installation profile. The \"<PAGE ...>\" tags "
                           "always need to have an INDEX attribute, and there "
                           "needs to be at least one page.",
                           lSearchPage);
            throw (FEFatalErrorExcpt(strMsg));
        }
    }
    else
        throw (FEFatalErrorExcpt("G_WpiGlobals.pPageInfoList is NULL."));

    // set page text
    HWND hwndSubDialogTemp = NULLHANDLE;
    switch (G_pCurrentPageInfo->_PageMode)
    {
        case BSPageInfo::MODE_TEXT: hwndSubDialogTemp = G_hwndPageText; break;
        case BSPageInfo::MODE_README: hwndSubDialogTemp = G_hwndPageReadme; break;
        case BSPageInfo::MODE_CONTAINER: hwndSubDialogTemp = G_hwndPageContainer; break;
        case BSPageInfo::MODE_CONFIGURE: hwndSubDialogTemp = G_hwndPageConfigure; break;
        // case MODE_INSTALL: hwndSubDialogTemp = hwndPageInstall; break;
    }
    // each of the four subdialogs has a static text with ID_WIDI_INFOTEXT ID
    PSZ     pszNew = strdup(G_pCurrentPageInfo->_strInfoText.c_str());
    txvStripLinefeeds(&pszNew,
                      4);       // tab size
    WinSetDlgItemText(hwndSubDialogTemp, ID_WIDI_INFOTEXT,
                      pszNew);
    free(pszNew);

    // now initialize the page we're turning to

    // disable "Selections" in main menu when not on container page
    WinSendMsg(G_hmenuMain,
               MM_SETITEMATTR,
               MPFROM2SHORT(ID_WIM_SELECT, TRUE),
               MPFROM2SHORT(MIA_DISABLED,
                            (G_pCurrentPageInfo->_PageMode == BSPageInfo::MODE_CONTAINER)
                                    ? 0
                                    : MIA_DISABLED));

    switch (G_pCurrentPageInfo->_PageMode)
    {

        /*
         * MODE_TEXT:
         *      text-only page.
         */

        case BSPageInfo::MODE_TEXT:
            G_ulPageHelp = IDHI_PAGE_TEXTONLY;
        break;

        /*
         * MODE_README:
         *      set the MLE with text
         */

        case BSPageInfo::MODE_README:
        {
            HWND    hwndMLE = WinWindowFromID(hwndSubDialogTemp, ID_WIDI_READMEMLE);

            BSString str2Insert;

            if (!G_pCurrentPageInfo->_ulExtractFromPck)
                // use src directly:
                str2Insert = G_pCurrentPageInfo->_strReadmeSrc;
            else
            {
                // use _strReadmeSrc as a file name:
                BSString strTempFileName;
                APIRET arc;
                if (!(arc = wpiExtractTempFile(G_pCurrentPageInfo->_ulExtractFromPck,
                                               G_pCurrentPageInfo->_strReadmeSrc.c_str(),
                                               &strTempFileName)))
                {
                    // successfully extracted:
                    PSZ pszContent = NULL;
                    arc = doshLoadTextFile(strTempFileName.c_str(),
                                           &pszContent);
                    if (!arc)
                    {
                        str2Insert = pszContent;
                        free(pszContent);
                    }
                    else
                        str2Insert._printf("Error %d reading file \"%s\" from package %d.\n",
                                           arc,
                                           strTempFileName.c_str(),
                                           G_pCurrentPageInfo->_ulExtractFromPck);
                }
                else
                    str2Insert._printf("Error %d extracting file \"%s\" from package %d.\n",
                                       arc,
                                       strTempFileName.c_str(),
                                       G_pCurrentPageInfo->_ulExtractFromPck);
            }

            // clean up text for speed
            WinSetWindowText(hwndMLE, "");

            // get format
            XFMTPARAGRAPH xfmtp;
            WinSendMsg(hwndMLE,
                       TXM_QUERYPARFORMAT,
                       (MPARAM)0,
                       (MPARAM)(&xfmtp));

            switch (G_pCurrentPageInfo->_ReadmeFormat)
            {
                /*
                 * README_PLAIN:
                 *
                 */

                case BSPageInfo::README_PLAIN:
                {
                    xfmtp.fWordWrap = FALSE;
                    xfmtp.lLeftMargin = 0;
                    xfmtp.lRightMargin = 0;
                    xfmtp.lFirstLineMargin = 0;
                    xfmtp.lSpaceBefore = 0;
                    xfmtp.lSpaceAfter = 0;

                    PSZ     pszNew = strdup(str2Insert.c_str());
                    txvStripLinefeeds(&pszNew,
                                      4);       // tab size
                    str2Insert = pszNew;
                    free(pszNew);
                break; }

                /*
                 * README_FLOW:
                 *
                 */

                case BSPageInfo::README_FLOW:
                {
                    xfmtp.fWordWrap = TRUE;
                    xfmtp.lLeftMargin = 0;
                    xfmtp.lRightMargin = 0;
                    xfmtp.lFirstLineMargin = 0;
                    xfmtp.lSpaceBefore = 5;
                    xfmtp.lSpaceAfter = 5;

                    PSZ     pszNew = strdup(str2Insert.c_str()),
                            pTarget = pszNew;
                    const char *pSource = str2Insert.c_str();

                    // step 1:
                    // remove all line feeds
                    while (*pSource)
                    {
                        if (*pSource == '\r')
                            pSource++;
                        else
                            *pTarget++ = *pSource++;
                    }
                    *pTarget = 0;

                    str2Insert = pszNew;
                    free(pszNew);

                    // step 2:
                    // remove all simple newlines; keep double newlines
                    pszNew = strdup(str2Insert.c_str()),
                    pSource = str2Insert.c_str(),
                    pTarget = pszNew;
                    while (*pSource)
                    {
                        if (*pSource == '\n')
                        {
                            // newline:
                            if (*(pSource+1) == '\n')
                            {
                                // empty line:
                                // keep one \n
                                *pTarget++ = *pSource++;
                                // *pTarget++ = *pSource++;
                                // skip all the following
                                while ((*pSource) && (*pSource == '\n'))
                                    pSource++;
                                // skip leading spaces
                                while ((*pSource) && (*pSource == ' '))
                                    pSource++;
                            }
                            else
                            {
                                BOOL fAddSpace = FALSE;
                                // single newline:
                                // skip
                                pSource++;
                                fAddSpace = TRUE;
                                // and skip leading spaces after newline
                                while ((*pSource) && (*pSource == ' '))
                                    pSource++;

                                // check if we have "---" or "===" next
                                if (*(pSource))
                                    if (*(pSource+1))
                                        if (*(pSource+2))
                                        {
                                            if (   (     (*(pSource) == '=')
                                                      && (*(pSource+1) == '=')
                                                      && (*(pSource+2) == '=')
                                                   )
                                                || (     (*(pSource) == '-')
                                                      && (*(pSource+1) == '-')
                                                      && (*(pSource+2) == '-')
                                                   )
                                                )
                                            {
                                                // add linefeed
                                                *pTarget++ = '\r';
                                                fAddSpace = FALSE;
                                            }
                                        }
                                if (fAddSpace)
                                    *pTarget++ = ' ';
                            }
                        }
                        else if (*pSource == ' ')
                        {
                            // space: keep one
                            *pTarget++ = *pSource++;
                            // skip all the following
                            while ((*pSource) && (*pSource == ' '))
                                pSource++;
                        }
                        else
                            // any other character: copy
                            *pTarget++ = *pSource++;
                    }
                    *pTarget = 0;

                    str2Insert = pszNew;
                    free(pszNew);
                break; }

                /*
                 * README_HTML:
                 *
                 */

                case BSPageInfo::README_HTML:
                {
                    xfmtp.fWordWrap = TRUE;
                    xfmtp.lLeftMargin = 0;
                    xfmtp.lRightMargin = 0;
                    xfmtp.lFirstLineMargin = 0;
                    xfmtp.lSpaceBefore = 5;
                    xfmtp.lSpaceAfter = 5;

                    PSZ     pszNew = strdup(str2Insert.c_str());
                    txvConvertFromHTML(&pszNew,
                                       NULL,
                                       NULL,
                                       NULL);
                    str2Insert = pszNew;
                    free(pszNew);
                break; }
            } // end switch

            // set format
            WinSendMsg(hwndMLE,
                       TXM_SETPARFORMAT,
                       (MPARAM)0,
                       (MPARAM)(&xfmtp));
            WinSetWindowText(hwndMLE, str2Insert.c_str());

            G_ulPageHelp = IDHI_PAGE_TEXTONLY;
        break; }

        /*
         * MODE_CONTAINER:
         *      entering container page:
         *      create record cores etc.
         */

        case BSPageInfo::MODE_CONTAINER:
        {
            // did we create record cores before?
            // (user might have pressed the "Back" button!)
            if (!G_fContainerPageReady)
            {
                Packages2Cnr(G_hwndPackagesCnr);
                G_fContainerPageReady = TRUE;

                // path entry field length (fixed 0.9.0)
                // moved this down to guiBeginInstallMode,
                // where this dlg is loaded V0.9.12 (2001-05-17) [umoeller]

                // create tooltip for packages
                CreateCnrTooltip(hwndSubDialogTemp,
                                 G_hwndPackagesCnr);
            }
            // show and update the drive info window
            WinPostMsg(G_hwndDriveInfo, WPIM_UPDATE, (MPARAM)1, NULL);

            G_ulPageHelp = IDHI_PAGE_CONTAINER;
        break; } // case MODE_CONTAINER:

        /*
         * MODE_CONFIGURE:
         *      entering configure page
         */

        case BSPageInfo::MODE_CONFIGURE:
        {
            // enable/disable the checkboxes;
            ULONG   ulConfigureFlags = 0;
            wpiCheckJobs(NULL,
                         NULL,
                         &ulConfigureFlags);

            // ulConfigureFlags now has the configure flags according
            // to the current package selections.
            // By contrast, G_WpiGlobals.ulAllowConfig has the configure
            // flags of the sum of all packages, that is, as if all
            // packages had been selected.

            // We _enable_ the dlg items according to ulConfigureFlags,
            // we _set_ the dlg items according to ulDoConfiguration.
            winhEnableDlgItem(G_hwndPageConfigure,
                              ID_WIDI_UPDATECONFIGSYS,
                              ((ulConfigureFlags & PCK_CONFIGSYS) != 0)
                             );
            winhSetDlgItemChecked(G_hwndPageConfigure,
                                  ID_WIDI_UPDATECONFIGSYS,
                                  (     ((G_WpiGlobals.ulAllowConfig & PCK_CONFIGSYS) != 0)
                                     && ((ulConfigureFlags & PCK_CONFIGSYS) != 0)
                                  ));
            winhEnableDlgItem(G_hwndPageConfigure,
                              ID_WIDI_INSTALLWPSCLASSES,
                              ((ulConfigureFlags & PCK_WPSCLASSES) != 0));
            winhSetDlgItemChecked(G_hwndPageConfigure,
                                  ID_WIDI_INSTALLWPSCLASSES,
                                  (    ((G_WpiGlobals.ulAllowConfig & PCK_WPSCLASSES) != 0)
                                    && ((ulConfigureFlags & PCK_WPSCLASSES) != 0)
                                  ));
            winhEnableDlgItem(G_hwndPageConfigure,
                              ID_WIDI_CREATEWPSOBJECTS,
                              ((ulConfigureFlags & PCK_WPSOBJECTS) != 0));
            winhSetDlgItemChecked(G_hwndPageConfigure,
                                  ID_WIDI_CREATEWPSOBJECTS,
                                  (     ((G_WpiGlobals.ulAllowConfig & PCK_WPSOBJECTS) != 0)
                                     && ((ulConfigureFlags & PCK_WPSOBJECTS) != 0)
                                  ));

            G_ulPageHelp = IDHI_PAGE_CONFIGURE;
        break; }

    } // end switch (pCurrentPageInfo->lPageMode)

    HWND    hwndFocusButton = WinWindowFromID(G_hwndMain, ID_WIDI_CANCELBUTTON);

    // set "Back" button (all pages)
    winhShowDlgItem(G_hwndMain, ID_WIDI_BACKBUTTON,
                    !(G_WpiGlobals.lPagesList.empty()));

    // set "Next" button (all pages)
    BOOL fShow =   (G_pCurrentPageInfo->_lNextButton >= 0);
    HWND hwndNextButton = WinWindowFromID(G_hwndMain, ID_WIDI_NEXTBUTTON);
    WinShowWindow(hwndNextButton, fShow);
    if (fShow)
    {
        BSString sNextButtonTitle = G_pCurrentPageInfo->_strNextButtonTitle;

        if (!sNextButtonTitle())
            // use default string:
            sNextButtonTitle = G_NLSStrings.pszNextButton;

        sNextButtonTitle.append(" ");

        WinSetWindowText(hwndNextButton, sNextButtonTitle.c_str());
        if (fShow)
            hwndFocusButton = hwndNextButton;
    }

    WinSetFocus(HWND_DESKTOP, hwndFocusButton);

    // show/hide controls
    WinShowWindow(G_hwndPageText,
                  G_pCurrentPageInfo->_PageMode == BSPageInfo::MODE_TEXT);
    WinShowWindow(G_hwndPageReadme,
                  G_pCurrentPageInfo->_PageMode == BSPageInfo::MODE_README);
    WinShowWindow(G_hwndPageContainer,
                  G_pCurrentPageInfo->_PageMode == BSPageInfo::MODE_CONTAINER);
    WinShowWindow(G_hwndPageConfigure,
                  G_pCurrentPageInfo->_PageMode == BSPageInfo::MODE_CONFIGURE);

    WinSetActiveWindow(HWND_DESKTOP, G_hwndMain);
}

/* ******************************************************************
 *
 *  Dialog procs for "Install" mode
 *
 ********************************************************************/

/*
 *@@ ARCHIVEVIEW:
 *
 *@@added V0.9.9 (2001-03-27) [umoeller]
 */

typedef struct _ARCHIVEVIEW
{
    HWND        hwndFrame,
                hwndClient;

} ARCHIVEVIEW, *PARCHIVEVIEW;

/*
 *@@ ViewArchives:
 *      implementation for "View archives" menu command.
 *      Displays all packages and archives.
 *
 *@@added V0.9.9 (2001-03-27) [umoeller]
 *@@changed V0.9.13 (2001-07-06) [umoeller]: added more display data
 */

VOID ViewArchives(HWND hwndOwner)
{
    SWP swpFrame;
    swpFrame.x = 100;
    swpFrame.y = 100;
    swpFrame.cx = 500;
    swpFrame.cy = 500;
    swpFrame.hwndInsertBehind = HWND_TOP;
    swpFrame.fl = SWP_MOVE | SWP_SIZE;

    ARCHIVEVIEW View = {0};
    View.hwndFrame = winhCreateStdWindow(HWND_DESKTOP,           // frame parent
                                         &swpFrame,
                                         FCF_SYSMENU
                                              | FCF_SIZEBORDER
                                              | FCF_TITLEBAR
                                              | FCF_MAXBUTTON
                                              | FCF_NOBYTEALIGN,
                                         WS_ANIMATE,
                                         "WarpIN",         // title bar
                                         0,                // res IDs
                                         WC_CONTAINER,   // client class
                                         WS_VISIBLE,       // client wnd style
                                         1,                // ID
                                         0,
                                         &View.hwndClient);
    WinSetWindowPos(View.hwndFrame,
                    HWND_TOP,
                    0, 0, 0, 0,
                    SWP_SHOW | SWP_ZORDER | SWP_ACTIVATE);

    winhSetWindowFont(View.hwndClient, "9.WarpSans");

    BEGIN_CNRINFO()
    {
        cnrhSetView(CV_TREE | CV_TEXT | CA_TREELINE);
    } END_CNRINFO(View.hwndClient);

    // list of strings to be freed
    list<PSZ>       FreeList;

    // go for all archives
    list<FEArchive*>::iterator ArcStart = G_WpiGlobals.ArchivesList.begin(),
                               ArcEnd = G_WpiGlobals.ArchivesList.end();
    for (;
         ArcStart != ArcEnd;
         ArcStart++)
    {
        FEArchive *parc = *ArcStart;

        PRECORDCORE precArchive = cnrhAllocRecords(View.hwndClient,
                                                   sizeof(RECORDCORE),
                                                   1);

        string      str;
        str._printf("Archive: \"%s\"\n"
                    "WPI revision: %d",
                    parc->_strArcFilename.c_str(),
                    parc->_Arc.getArcHeader()->wi_revision_needed);
        PSZ pszThis = strdup(str.c_str());
        FreeList.push_back(pszThis);

        cnrhInsertRecords(View.hwndClient,
                          NULL,     // parent
                          precArchive,
                          TRUE,
                          pszThis,
                          CRA_RECORDREADONLY | CRA_EXPANDED,
                          1);

        // insert packages for archive
        list<FEArcPackageBase*>::iterator PckStart = parc->_PackagesList.begin(),
                                          PckEnd = parc->_PackagesList.end();
        for (;
             PckStart != PckEnd;
             PckStart++)
        {
            FEArcPackageBase *ppck = *PckStart;

            FEArcPackagePck *ppckpck = ppck->IsPackage();

            if (ppckpck)
            {
                CHAR        szIndex[200];
                sprintf(szIndex,
                        "Package %d: \n\"",
                        ppckpck->_PackHeader.number);
                str = szIndex;
                str += ppckpck->_pDecl->_strID;
                sprintf(szIndex,
                        "\"\n%d KB of uncompressed files",
                        ppckpck->_ulSizeRounded);
                str += szIndex;

                PRECORDCORE precPck = cnrhAllocRecords(View.hwndClient,
                                                       sizeof(RECORDCORE),
                                                       1);

                pszThis = strdup(str.c_str());
                FreeList.push_back(pszThis);

                cnrhInsertRecords(View.hwndClient,
                                  precArchive,     // parent
                                  precPck,
                                  TRUE,
                                  pszThis,
                                  CRA_RECORDREADONLY,
                                  1);

                // now for each package, go for config list
                list<BSConfigBase*>::iterator
                    CfgStart = ppckpck->_pDecl->_listConfigObjects.begin(),
                    CfgEnd = ppckpck->_pDecl->_listConfigObjects.end();
                for (;
                     CfgStart != CfgEnd;
                     CfgStart++)
                {
                    BSConfigBase *pcfg = *CfgStart;

                    string str = pcfg->DescribeType();
                    str += ": ";
                    str += pcfg->DescribeData();

                    PRECORDCORE precCfg = cnrhAllocRecords(View.hwndClient,
                                                           sizeof(RECORDCORE),
                                                           1);

                    pszThis = strdup(str.c_str());
                    FreeList.push_back(pszThis);

                    cnrhInsertRecords(View.hwndClient,
                                      precPck,     // parent
                                      precCfg,
                                      TRUE,
                                      pszThis,
                                      CRA_RECORDREADONLY,
                                      1);
                }
            }
        }
    }

    HAB hab = WinQueryAnchorBlock(View.hwndFrame);
    QMSG qmsg;

    while (WinGetMsg(hab, &qmsg, 0, 0, 0))
    {
        if (    (qmsg.msg == WM_CLOSE)
             && (qmsg.hwnd == View.hwndClient)
           )
            // do not dispatch this, this produces WM_QUIT
            break;

        WinDispatchMsg(hab, &qmsg);
    }

    list<PSZ>::iterator     Start = FreeList.begin(),
                            End = FreeList.end();
    for (;
         Start != End;
         Start++)
    {
        PSZ pszThis = *Start;
        free(pszThis);
    }

    WinDestroyWindow(View.hwndFrame);
}

/* ******************************************************************
 *
 *  Dialog procs for "Install" mode
 *
 ********************************************************************/

/*
 *@@ fnwpPages:
 *      shared window proc for the main window and the
 *      sub-dialogs.
 *
 *      This is responsible for almost everything that the
 *      front-end can do in "full install" mode.
 *
 *@@changed V0.9.3 (2000-05-11) [umoeller]: "Cancel" didn't close main window; fixed
 *@@changed V0.9.4 (2000-07-26) [umoeller]: fixed CID filename
 */

MRESULT EXPENTRY fnwpPages(HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    MRESULT mrc = 0;

    switch (msg)
    {
        /*
         * WM_INITDLG:
         *      initialize some more controls
         */

        case WM_INITDLG:
        {
            mrc = WinDefDlgProc(hwndDlg, msg, mp1, mp2);

            gshrSetControlsFont(hwndDlg, 0, 500);
        break; }

        /*
         * WPIM_TURNTOPAGE:
         *      this use message gets posted when
         *      we need to switch to another page.
         *      The new page is in (ULONG)mp1;
         *      if ((BOOL)mp2 == TRUE), the new page
         *      will be stored in the "visited" list.
         */

        case WPIM_TURNTOPAGE:
        {
            // set up the dialog controls for the page
            // that we want
            TurnToPage((LONG)mp1, (BOOL)mp2);
        break; }

        /*
         * WM_CONTROL:
         *
         */

        case WM_CONTROL:
        {
            USHORT  usControl = SHORT1FROMMP(mp1);
            USHORT  usNotifyCode = SHORT2FROMMP(mp1);

            switch (usControl)
            {
                /*
                 * ID_WIDI_CNR:
                 *      selection container
                 *      on "Container" page
                 */

                case ID_WIDI_CNR:
                {
                    switch (usNotifyCode)
                    {
                        /*
                         * CN_ENTER:
                         *      double-click or enter on a
                         *      container record core; update
                         *      package selections then.
                         */

                        case CN_ENTER:
                        {
                            PNOTIFYRECORDENTER pnre = (PNOTIFYRECORDENTER)mp2;
                            PPACKAGERECORD  precc = (PPACKAGERECORD)pnre->pRecord;

                            if (precc)
                            {
                                // double-click on container, not whitespace:
                                FEJobBase *pJob = precc->pJob;
                                if (pJob)
                                {
                                    pJob->ToggleSelection();
                                        // this calls guiOnJobSelectionChanged in turn

                                    // update drive info
                                    WinPostMsg(G_hwndDriveInfo, WPIM_UPDATE, (MPARAM)1, NULL);

                                    UpdatePackageDisplay();
                                }
                            }
                        break; }

                        /*
                         * CN_EMPHASIS:
                         *      package selection changed; update
                         *      dependent info in dialog
                         */

                        case CN_EMPHASIS:
                        {
                            PNOTIFYRECORDEMPHASIS pnre = (PNOTIFYRECORDEMPHASIS)mp2;
                            G_preccSelected = (PPACKAGERECORD)pnre->pRecord;
                            UpdatePackageDisplay();
                        break; }

                    }
                break; }

                /*
                 * ID_WIDI_PATHENTRY:
                 *      "Path" entry field
                 *      on "Container" page
                 */

                case ID_WIDI_PATHENTRY:
                {
                    switch (usNotifyCode)
                    {
                        /*
                         * EN_CHANGE:
                         *      PATH entry field has some new content;
                         *      we check for whether a new drive was enterd,
                         *      and if so, we need to update the recc data and,
                         *      if this is the "base" recc, examine all
                         *      other record cores too
                         */

                        case EN_CHANGE:
                        {
                            FEInstallJob *pInstallJob = G_preccSelected->pJob->IsInstallJob();
                            if (pInstallJob)
                            {
                                CHAR    szNewTargetPath[CCHMAXPATH];
                                WinQueryDlgItemText(G_hwndPageContainer, ID_WIDI_PATHENTRY,
                                                    sizeof(szNewTargetPath),
                                                    szNewTargetPath);
                                if (pInstallJob->SetTargetPath(szNewTargetPath,
                                                               G_WpiGlobals.AllJobsList))
                                    // drive letter changed:
                                    WinPostMsg(G_hwndDriveInfo, WPIM_UPDATE, (MPARAM)1, NULL);
                            }
                        break; }
                    }
                } break;

                /*
                 * ID_WIDI_UPDATECONFIGSYS etc.:
                 *      checkboxes on "Configure" page.
                 */

                case ID_WIDI_UPDATECONFIGSYS:
                case ID_WIDI_INSTALLWPSCLASSES:
                case ID_WIDI_CREATEWPSOBJECTS:
                    if (    (usNotifyCode == BN_CLICKED)
                         || (usNotifyCode == BN_DBLCLICKED)
                       )
                    {
                        G_WpiGlobals.ulAllowConfig = 0;
                        if (winhIsDlgItemChecked(G_hwndPageConfigure, ID_WIDI_UPDATECONFIGSYS))
                            G_WpiGlobals.ulAllowConfig |= PCK_CONFIGSYS;
                        else
                            G_WpiGlobals.ulAllowConfig &= ~PCK_CONFIGSYS;
                        if (winhIsDlgItemChecked(G_hwndPageConfigure, ID_WIDI_INSTALLWPSCLASSES))
                            G_WpiGlobals.ulAllowConfig |= PCK_WPSCLASSES;
                        else
                            G_WpiGlobals.ulAllowConfig &= ~PCK_WPSCLASSES;
                        if (winhIsDlgItemChecked(G_hwndPageConfigure, ID_WIDI_CREATEWPSOBJECTS))
                            G_WpiGlobals.ulAllowConfig |= PCK_WPSOBJECTS;
                        else
                            G_WpiGlobals.ulAllowConfig &= ~PCK_WPSOBJECTS;
                    }

                    guiUpdateGlobalStatus(GSTAT_COLLECTING);
                break;

                // tooltip control
                case 19999:
                    if (usNotifyCode == TTN_NEEDTEXT)
                    {
                        GetTooltipText(hwndDlg,
                                       G_hwndPackagesCnr,
                                       mp2);
                    }
                break;

            } // end switch(usControl)
        break; }

        /*
         * WM_DRAWITEM:
         *      container record core owner-draw
         */

        case WM_DRAWITEM:
        {
            // get generic DRAWITEM structure
            POWNERITEM poi = (POWNERITEM)mp2;
            // call helper func
            mrc = CnrPackageOwnerDraw(poi);
        break; }

        /*
         * WM_COMMAND:
         *
         */

        case WM_COMMAND:
        {
            USHORT  usCommand = SHORT1FROMMP(mp1);
            switch (usCommand)
            {
                /*
                 * ID_WIDI_NEXTBUTTON:
                 *      "Next" button pressed.
                 */

                case ID_WIDI_NEXTBUTTON:
                    if (G_pCurrentPageInfo->_lNextButton > 0)
                        WinPostMsg(G_hwndMain, WPIM_TURNTOPAGE,
                                   (MPARAM)(G_pCurrentPageInfo->_lNextButton),
                                   (MPARAM)TRUE);  // store in "visited" list
                    else
                    {
                        // button index == 0:
                        // this means start installation.
                        // Display summary:

                        ULONG   ulToAdd = 0, ulToRemove = 0, ulConfig = 0;
                        BOOL    fContinue = FALSE;
                        CHAR    szNo1[10], szNo2[10];
                        PSZ     apsz[2] = { szNo1, szNo2 };

                        wpiCheckJobs(&ulToAdd, &ulToRemove, &ulConfig);

                        if ((ulToAdd) && (ulToRemove))
                        {
                            // both install and de-install:
                            sprintf(szNo1, "%d", ulToAdd);
                            sprintf(szNo2, "%d", ulToRemove);
                            if (wpiMessageBox(100,
                                              175,
                                              MB_YESNO,
                                              apsz, 2)
                                    == MBID_YES)
                                fContinue = TRUE;
                        }
                        else if (ulToAdd)
                        {
                            // install only:
                            sprintf(szNo1, "%d", ulToAdd);
                            if (wpiMessageBox(100,
                                              173,
                                              MB_YESNO,
                                              apsz, 1)
                                    == MBID_YES)
                                fContinue = TRUE;
                        }
                        else if (ulToRemove)
                        {
                            // install only:
                            sprintf(szNo1, "%d", ulToRemove);
                            if (wpiMessageBox(100,
                                              174,
                                              MB_YESNO,
                                              apsz, 1)
                                    == MBID_YES)
                                fContinue = TRUE;
                        }

                        if (fContinue)
                            // WinDismissDlg(hwndDlg, DID_OK);
                            WinDismissDlg(G_hwndMain, DID_OK);
                    }
                break;

                /*
                 * ID_WIDI_BACKBUTTON:
                 *      "Back" button pressed.
                 */

                case ID_WIDI_BACKBUTTON:
                    // TurnToPage has stored the last page in G_WpiGlobals
                    if (!G_WpiGlobals.lPagesList.empty())
                    {
                        LONG lLast = G_WpiGlobals.lPagesList.back();
                        // delete last item in list
                        G_WpiGlobals.lPagesList.pop_back();
                        WinPostMsg(G_hwndMain, WPIM_TURNTOPAGE,
                                   (MPARAM)lLast,
                                   (MPARAM)FALSE);  // store in "visited" list
                    }
                break;

                /*
                 * ID_WIDI_CREATECIDCMD:
                 *      "Create CID file" button pressed.
                 */

                case ID_WIDI_CREATECIDCMD:
                {
                    FILEDLG fd;
                    memset(&fd, 0, sizeof(FILEDLG));
                    fd.cbSize = sizeof(FILEDLG);
                    fd.fl = FDS_SAVEAS_DIALOG
                              | FDS_ENABLEFILELB
                              | FDS_CENTER;

                    // file name of cid file: like archive,
                    // but replace extension with .cmd
                    strcpy(fd.szFullFile,
                           G_WpiGlobals.pFirstArchive->_strArcFilename.c_str());
                    PSZ p = strrchr(fd.szFullFile, '.');
                    if (p)
                        strcpy(p, ".cmd");

                    if (    WinFileDlg(HWND_DESKTOP,    // parent
                                       hwndDlg,
                                       &fd)
                        && (fd.lReturn == DID_OK)
                       )
                    {
                        PSZ pszTable = fd.szFullFile;
                        if (access(fd.szFullFile, 0) == 0)
                            // file exists:
                            if (wpiMessageBox(108,
                                              125,
                                              MB_ICONQUESTION | MB_YESNO | MB_MOVEABLE,
                                              &pszTable, 1)
                                    != MBID_YES)
                                break;
                        // else:
                        wpiCreateCIDFile(fd.szFullFile,
                                         G_WpiGlobals.pFirstArchive->_strArcFilename.c_str());
                    }
                break; }

                /*
                 * ID_WIMI_ARCHIVE_VIEW:
                 *      "view archive" menu item selected.
                 */

                case ID_WIMI_ARCHIVE_VIEW:
                    ViewArchives(hwndDlg);
                break;

                /*
                 * ID_WIMI_SELECT_DEFAULT etc.:
                 *      menu items from "selections" menu selected.
                 */

                case ID_WIMI_SELECT_DEFAULT:
                case ID_WIMI_SELECT_REPAIR:
                case ID_WIMI_SELECT_UPDATE:
                case ID_WIMI_SELECT_FULL:
                case ID_WIMI_SELECT_DEINSTALL:
                {
                    // items from "Selections" menu:
                    // go thru all jobs
                    list<FEJobBase*>::iterator JobStart = G_WpiGlobals.AllJobsList.begin(),
                                               JobEnd = G_WpiGlobals.AllJobsList.end();
                    for (; JobStart != JobEnd; JobStart++)
                    {
                        FEJobBase *pTemp = *JobStart;
                        FEInstallJob *pJobThis = pTemp->IsInstallJob();
                        if (pJobThis)
                        {
                            const FEArcPackageBase* pPackageThis = pJobThis->_pArcPackage;
                            if (pPackageThis->IsPackage())
                            {
                                switch (usCommand)
                                {
                                    case ID_WIMI_SELECT_DEFAULT:
                                        pJobThis->DefaultSelect();
                                    break;

                                    case ID_WIMI_SELECT_REPAIR:
                                        pJobThis->RepairSelect();
                                    break;

                                    case ID_WIMI_SELECT_UPDATE:
                                        pJobThis->UpdateSelect();
                                    break;

                                    case ID_WIMI_SELECT_FULL:
                                        pJobThis->Select(JOB_INSTALL);
                                    break;

                                    case ID_WIMI_SELECT_DEINSTALL:
                                        pJobThis->Select(JOB_DEINSTALL);
                                    break;
                                }
                            }
                        }
                    }
                break; }

                default:
                    // other command: check default menu items (gui_shared.cpp)
                    if (!gshrDefWindowProc(hwndDlg, msg, mp1, mp2,
                                           G_ulPageHelp))
                        // not processed:
                        if (wpiMessageBox(100,
                                          172,  // "Sure you wanna cancel?"
                                          MB_ICONQUESTION | MB_YESNO | MB_MOVEABLE)
                                == MBID_YES)
                            WinPostMsg(hwndDlg, WM_QUIT, 0, 0);
            }

        break; }

        default:
            if (!gshrDefWindowProc(hwndDlg, msg, mp1, mp2,
                                   G_ulPageHelp))   // help panel
                mrc = WinDefDlgProc(hwndDlg, msg, mp1, mp2);
    }

    return (mrc);
}

/* ******************************************************************
 *
 *  PM implementation part 2:
 *      Required callbacks on thread 1.
 *
 ********************************************************************/

/*
 *@@ guiInitialize:
 *      this is supposed to set up the GUI environment.
 *      This _always_ gets called at the very beginning,
 *      no matter what WarpIN will do afterwards.
 *
 *      Note that this gets called _before_ WarpIN sets
 *      the various fields in G_WpiGlobals from the environment
 *      variables, so it's safe to set fields here from
 *      the user settings, because these will be overridden
 *      if environment variables have been defined.
 *
 *      At this point, the WARPIN.INI file has been opened
 *      and its HINI has been stored in G_WpiGlobals.
 *
 *      This also checks for whether a program object has already
 *      been created for WarpIN.
 *
 *      Required callback.
 *
 *      If this returns FALSE, WarpIN is terminated.
 *
 *@@changed V0.9.0 (99-10-22) [umoeller]: added installation check
 *@@changed V0.9.2 (2000-03-10) [umoeller]: now calling txvRegisterTextView in time
 *@@changed V0.9.4 (2000-07-26) [umoeller]: now checking WPS objects only if WPS is up
 */

BOOL guiInitialize(void)
{
    BOOL brc = FALSE;

    if (!(G_hmqThread1 = WinCreateMsgQueue(G_WpiGlobals.habThread1, 0)))
        return FALSE;

    gshrLoadSettings();

    // "please wait, initializing"
    gshrCreateSplash();

    HWND hwndText = WinWindowFromID(G_hwndWarpINStartup, ID_WIDI_INFOTEXT);
    PSZ pszText = winhQueryWindowText(hwndText);
    if (pszText)
    {
        ULONG ulOfs = 0;
        strhFindReplace(&pszText,
                        &ulOfs,
                        "@@@version",
                        BLDLEVEL_VERSION);
        WinSetWindowText(hwndText, pszText);
        free(pszText);
    }
    winhCenterWindow(G_hwndWarpINStartup);
    WinShowWindow(G_hwndWarpINStartup, TRUE);

    // query all kinds of NLS data from OS2.INI
    // which we will use to display data
    G_ulDateFormat =
        PrfQueryProfileInt(HINI_USER, "PM_National", "iDate", 0);
    G_ulTimeFormat =
        PrfQueryProfileInt(HINI_USER, "PM_National", "iTime", 0);
    PrfQueryProfileString(HINI_USER, "PM_National", "sDate", "/",
        G_szDateSep, sizeof(G_szDateSep)-1);
    PrfQueryProfileString(HINI_USER, "PM_National", "sTime", ":",
        G_szTimeSep, sizeof(G_szTimeSep)-1);
    PrfQueryProfileString(HINI_USER, "PM_Default_National", "sThousand",
            ",", G_szThousand, sizeof(G_szThousand)-1);

    // load icons which we'll need; these must be
    // destroyed in guiCleanupBeforeExit
    G_hptrMain     = WinLoadPointer(HWND_DESKTOP,
                                  NULLHANDLE, ID_ICON);
    G_hptrSelect   = WinLoadPointer(HWND_DESKTOP,
                                  NULLHANDLE, IDP_SELECTED);
    G_hptrDeselect = WinLoadPointer(HWND_DESKTOP,
                                  NULLHANDLE, IDP_DESELECTED);
    G_hptrSomeselect = WinLoadPointer(HWND_DESKTOP,
                                  NULLHANDLE, IDP_SOMESELECTED);
    G_hptrDeinstall = WinLoadPointer(HWND_DESKTOP,
                                   NULLHANDLE, IDP_DEINSTALL);
    G_hptrError    = WinLoadPointer(HWND_DESKTOP,
                                  NULLHANDLE, IDP_DB_ERROR);
    G_hptrQMark    = WinLoadPointer(HWND_DESKTOP,
                                  NULLHANDLE, IDP_DB_QMARK);
    G_hptrOK       = WinLoadPointer(HWND_DESKTOP,
                                  NULLHANDLE, IDP_DB_OK);
    G_hptrNewer    = WinLoadPointer(HWND_DESKTOP,
                                  NULLHANDLE, IDP_DB_NEWER);
    G_hptrApp      = WinLoadPointer(HWND_DESKTOP,
                                  NULLHANDLE, IDP_DB_APP);

    // system icon size
    G_ulMiniIconSize = WinQuerySysValue(HWND_DESKTOP, SV_CYICON) / 2;

    // load NLS strings
    LoadString(WPSI_FILEERROR, &G_NLSStrings.pszFileError);
    LoadString(WPSI_EXISTINGFILENEWER, &G_NLSStrings.pszExistingFileNewer);
    LoadString(WPSI_EXISTINGFILEOLDER, &G_NLSStrings.pszExistingFileOlder);
    LoadString(WPSI_DELETEERROR, &G_NLSStrings.pszDeleteError);
    LoadString(WPSI_DELETEDIRERROR, &G_NLSStrings.pszDeleteDirError);
    LoadString(WPSI_PACKAGEREMOVED, &G_NLSStrings.pszPackageRemoved);
    LoadString(WPSI_CLASSDEREGISTERED, &G_NLSStrings.pszClassDeregistered);
    LoadString(WPSI_ERRORDEREGISTERINGCLASS, &G_NLSStrings.pszErrorDeregisteringClass);
    LoadString(WPSI_CLASSUNREPLACED, &G_NLSStrings.pszClassUnreplaced);
    LoadString(WPSI_ERRORUNREPLACINGCLASS, &G_NLSStrings.pszErrorUnreplacingClass);
    LoadString(WPSI_OBJECTDELETED, &G_NLSStrings.pszObjectDeleted);
    LoadString(WPSI_ERRORDELETINGOBJECT, &G_NLSStrings.pszErrorDeletingObject);
    LoadString(WPSI_CFGSYSUNDONE, &G_NLSStrings.pszCfgSysUndone);
    LoadString(WPSI_ERRORUNDOINGCFGSYS, &G_NLSStrings.pszErrorUndoingCfgSys);
    LoadString(WPSI_PROFILECLEARED, &G_NLSStrings.pszProfileCleared);
    LoadString(WPSI_ERRORCLEARINGPROFILE, &G_NLSStrings.pszErrorClearingProfile);

    LoadString(WPSI_DEEXECUTING, &G_NLSStrings.pszDeExecuting);
    LoadString(WPSI_DEEXECUTEDONE, &G_NLSStrings.pszDeExecuteDone);
    LoadString(WPSI_DEEXECUTEERROR, &G_NLSStrings.pszDeExecuteError);

    LoadString(WPSI_WARNINGEXISTINGNEWER, &G_NLSStrings.pszWarningExistingNewer);

    LoadString(WPSI_UPDATINGCONFIGSYS, &G_NLSStrings.pszUpdatingConfigSys);
    LoadString(WPSI_REGISTERINGCLASSES, &G_NLSStrings.pszRegisteringClasses);
    LoadString(WPSI_CREATINGOBJECTS, &G_NLSStrings.pszCreatingObjects);

    LoadString(WPSI_DRIVEXFREE, &G_NLSStrings.pszDriveXFree);
    LoadString(WPSI_ELAPSED, &G_NLSStrings.pszElapsed);
    LoadString(WPSI_TOGO, &G_NLSStrings.pszToGo);
    LoadString(WPSI_PACKAGEX, &G_NLSStrings.pszPackageX);
    LoadString(WPSI_FILEX, &G_NLSStrings.pszFileX);

    LoadString(WPSI_DRIVE, &G_NLSStrings.pszDrive);
    LoadString(WPSI_FREEKB, &G_NLSStrings.pszFreeKB);
    LoadString(WPSI_REQUIREDKB, &G_NLSStrings.pszRequiredKB);

    LoadString(WPSI_DB_MAINWINDOWTITLE, &G_NLSStrings.pszDBMainWindowTitle);
    LoadString(WPSI_DB_DEINSTALLINGARCHIVE, &G_NLSStrings.pszDBDeinstallingArchive);
    LoadString(WPSI_DB_DEINSTALLINGPACKAGE, &G_NLSStrings.pszDBDeinstallingPackages);
    LoadString(WPSI_DB_VERIFYINGPACKAGE, &G_NLSStrings.pszDBVerifyingPackages);

    LoadString(WPSI_FILENAME, &G_NLSStrings.pszFileName);
    LoadString(WPSI_SIZE, &G_NLSStrings.pszSize);
    LoadString(WPSI_CREATIONDATE, &G_NLSStrings.pszCreationDate);
    LoadString(WPSI_LASTWRITEDATE, &G_NLSStrings.pszLastWriteDate);

    LoadString(WPSI_APPLICATION, &G_NLSStrings.pszApplication);
    LoadString(WPSI_PACKAGENAME, &G_NLSStrings.pszPackageName);
    LoadString(WPSI_VERSION, &G_NLSStrings.pszVersion);
    LoadString(WPSI_AUTHOR, &G_NLSStrings.pszAuthor);
    LoadString(WPSI_TARGETPATH, &G_NLSStrings.pszTargetPath);
    LoadString(WPSI_FILES, &G_NLSStrings.pszFiles);
    LoadString(WPSI_INSTALLDATE, &G_NLSStrings.pszInstallDate);
    LoadString(WPSI_INSTALLTIME, &G_NLSStrings.pszInstallTime);

    LoadString(WPSI_VERIFYING, &G_NLSStrings.pszVerifying);
    LoadString(WPSI_DELETING, &G_NLSStrings.pszDeleting);
    LoadString(WPSI_DONE, &G_NLSStrings.pszDone);

    LoadString(WPSI_NEXTBUTTON, &G_NLSStrings.pszNextButton); // V0.9.2 (2000-03-11) [umoeller]

    // register comctl classes
    txvRegisterTextView(G_WpiGlobals.habThread1);
    ctlRegisterTooltip(G_WpiGlobals.habThread1);

    // load help instance
    if (    (G_hwndHelpInstance = winhCreateHelp(NULLHANDLE, // frame
                                                 NULL, // file name
                                                 NULLHANDLE, // module
                                                 (PHELPTABLE)MAKELONG(WPH_FULLINSTALL, 0xffff),
                                                 "WarpIN"))
         == NULLHANDLE)
    {
        wpiMessageBox(102,
                      195,
                      MBID_OK);
    }

    brc = CheckInstall();

    // create event semaphore for thread communication
    // (used for both the Install and Database threads)
    /* DosCreateEventSem(NULL,         // unnamed
                      &G_hevGUIDone,
                      0,            // unshared
                      FALSE);       // not posted initially
                      */

    // OK
    return (brc);
}

/*
 *@@ guiSetCodepage:
 *      this gets called from FEArchive::ParseWARPINTag
 *      while the install script is being parsed. This
 *      must return one of the following:
 *
 *      -- GUICP_SET: codepage successfully switched,
 *         no translation necessary.
 *      -- GUICP_TRANSLATED: the install script in
 *         strScript was translated to ulCodepage so
 *         that the script needs to be re-parsed.
 *      -- GUICP_FAILED: neither switching nor translating
 *         succeeded. This instructs the caller to display
 *         a warning message.
 *
 *      Required callback. This is in the GUI because we
 *      need Win* functions for codepage translations.
 *
 *@@added V0.9.2 (2000-03-11) [umoeller]
 */

ULONG guiSetCodepage(ULONG ulCodepage,
                     BSString &strScript)
{
    ULONG ulrc = GUICP_FAILED;

    if (WinSetCp(G_hmqThread1, ulCodepage))
        // success:
        ulrc = GUICP_SET;
    else
    {
        ULONG cbDest = strScript.size() * 3;        // for DBCS maybe?
        PSZ pszDest = (PSZ)malloc(cbDest);
        // not set: try translating
        if (WinCpTranslateString(G_WpiGlobals.habThread1,
                                 // source cp:
                                 WinQueryCp(G_hmqThread1),
                                 // string:
                                 (PSZ)strScript.c_str(),
                                 // target cp:
                                 ulCodepage,
                                 // target string:
                                 cbDest, pszDest))
        {
            // successfully translated:
            strScript = pszDest;
            free(pszDest);
            ulrc = GUICP_TRANSLATED;
        }
        else
            free(pszDest);
            // ulrc is still GUICP_FAILED
    }

    return (ulrc);
}

/*
 *@@ guiCleanupBeforeExit:
 *      this _always_ gets called before WarpIN exists.
 *
 *      Required callback.
 */

VOID guiCleanupBeforeExit(void)
{
    winhDestroyHelp(G_hwndHelpInstance, NULLHANDLE);

    WinDestroyPointer(G_hptrMain);
    WinDestroyPointer(G_hptrSelect);
    WinDestroyPointer(G_hptrDeselect);
    WinDestroyPointer(G_hptrSomeselect);
    WinDestroyPointer(G_hptrDeinstall);
    WinDestroyPointer(G_hptrError);
    WinDestroyPointer(G_hptrQMark);
    WinDestroyPointer(G_hptrOK);
    WinDestroyPointer(G_hptrNewer);
    WinDestroyPointer(G_hptrApp);

    // clean up on the way out
    WinDestroyMsgQueue(G_hmqThread1);
}

/*
 *@@ guiShowMessage:
 *      this should display a message box
 *      with the given strings.
 *
 *      Required callback.
 *
 *@@changed V0.9.0 (99-11-01) [umoeller]: renamed, the other GUI message box prototypes have been removed.
 *@@changed V0.9.12 (2001-05-21) [umoeller]: now doing our own messagebox
 */

ULONG guiShowMessage(const BSString &strTitle,
                     const BSString &strMessage,
                     ULONG ulFlags)
{
    HWND hwndOwner = G_hwndMain;
    if (!WinIsWindow(G_WpiGlobals.habThread1, hwndOwner))
        hwndOwner = NULLHANDLE; // HWND_DESKTOP; V0.9.9 (2001-02-01) [umoeller]
    else if (!WinIsWindowVisible(hwndOwner))
        hwndOwner = NULLHANDLE; // HWND_DESKTOP; V0.9.9 (2001-02-01) [umoeller]


    return (dlghMessageBox(hwndOwner,
                           G_hptrMain,
                           strTitle.c_str(),
                           strMessage.c_str(),
                           ulFlags,
                           "9.WarpSans",
                           &G_MsgBoxStrings));

    /* return (WinMessageBox(HWND_DESKTOP,     // parent
                          hwndOwner,        // owner
                          strMessage.c_str(),
                          strTitle.c_str(),
                          0,
                          ulFlags | MB_MOVEABLE)); */
}

/*
 *@@ guiSetupGlobalStatus:
 *      this loads, sets up and shows the "Global status"
 *      window. Items are disabled according to the
 *      current status.
 *
 *      Required callback.
 *
 *@@added V0.9.0 (99-11-01) [umoeller]
 */

VOID guiSetupGlobalStatus(BOOL fFullInstall)
{
    G_hwndGlobalStatus = gshrLoadDlg(HWND_DESKTOP, HWND_DESKTOP,
                                  WinDefDlgProc,
                                  NULLHANDLE, ID_WID_GLOBALSTATUS,
                                  NULL);
    gshrSetControlsFont(G_hwndGlobalStatus, 0, 1000);

    if (fFullInstall)
        // in full install, we never remove packages
        winhEnableDlgItem(G_hwndGlobalStatus, ID_WIDI_REMOVING, FALSE);

    G_ulCurrentBoldID = 0;
    guiUpdateGlobalStatus(GSTAT_COLLECTING);

    WinShowWindow(G_hwndGlobalStatus, TRUE);
}

/*
 *@@ guiUpdateGlobalStatus:
 *      this makes the specified dlg item in the
 *      "Global status" dialog bold and puts the
 *      small arrow next to it.
 *
 *      Required callback.
 *
 *@@added V0.9.0 (99-11-01) [umoeller]
 */

VOID guiUpdateGlobalStatus(BYTE bStatus)
{
    ULONG   ulToAdd = 0,
            ulToRemove = 0,
            ulConfig = 0;
    PSZ     pszCurrentConfig = NULL;
    BOOL    fDestroyConfigStatus = FALSE;

    wpiCheckJobs(&ulToAdd, &ulToRemove, &ulConfig);

    winhEnableDlgItem(G_hwndGlobalStatus, ID_WIDI_REMOVING, (ulToRemove != 0));
    winhEnableDlgItem(G_hwndGlobalStatus, ID_WIDI_UNPACKING, (ulToAdd != 0));
    winhEnableDlgItem(G_hwndGlobalStatus, ID_WIDI_CONFIGSYS,
                      ((ulConfig & PCK_CONFIGSYS) != 0));
    winhEnableDlgItem(G_hwndGlobalStatus, ID_WIDI_REGISTERING,
                      ((ulConfig & PCK_WPSCLASSES) != 0));
    winhEnableDlgItem(G_hwndGlobalStatus, ID_WIDI_CREATINGOBJECTS,
                      ((ulConfig & PCK_WPSOBJECTS) != 0));

    CHAR szFont[200];
    if (G_ulCurrentBoldID)
    {
        HWND hwndItem = WinWindowFromID(G_hwndGlobalStatus, G_ulCurrentBoldID);
        WinQueryPresParam(hwndItem,
                          PP_FONTNAMESIZE, 0,
                          NULL,
                          sizeof(szFont),
                          szFont,
                          QPF_NOINHERIT);
        PSZ p = strstr(szFont, " Bold");
        if (p)
        {
            *p = 0;
            WinSetPresParam(hwndItem,
                            PP_FONTNAMESIZE,
                            strlen(szFont)+1,
                            szFont);
        }
    }
    ULONG ulDlgID = 0;
    switch (bStatus)
    {
        case GSTAT_COLLECTING:
            ulDlgID = ID_WIDI_COLLECTING;
            fDestroyConfigStatus = TRUE;
        break;
        case GSTAT_REMOVING:
            ulDlgID = ID_WIDI_REMOVING;
            fDestroyConfigStatus = TRUE;
        break;
        case GSTAT_UNPACKING:
            ulDlgID = ID_WIDI_UNPACKING;
            fDestroyConfigStatus = TRUE;
        break;
        case GSTAT_CONFIGSYS:
            ulDlgID = ID_WIDI_CONFIGSYS;
            pszCurrentConfig = G_NLSStrings.pszUpdatingConfigSys;
                            // "Updating CONFIG.SYS");
        break;
        case GSTAT_WPSCLASSES:
            ulDlgID = ID_WIDI_REGISTERING;
            pszCurrentConfig = G_NLSStrings.pszRegisteringClasses;
                            // "Registering WPS classes");
        break;
        case GSTAT_WPSOBJECTS:
            ulDlgID = ID_WIDI_CREATINGOBJECTS;
            pszCurrentConfig = G_NLSStrings.pszCreatingObjects;
                            // "Creating WPS objects");
        break;
        case GSTAT_DONEWITHALL:
            fDestroyConfigStatus = TRUE;
        break;
    }

    HWND    hwndArrow = WinWindowFromID(G_hwndGlobalStatus, ID_WIDI_ARROW);
    if (bStatus != GSTAT_DONEWITHALL)
    {
        HWND hwndItem = WinWindowFromID(G_hwndGlobalStatus, ulDlgID);
        if (hwndItem)
        {
            SWP     swpArrow, swpBold;
            WinQueryWindowPos(hwndArrow,
                              &swpArrow);
            WinQueryWindowPos(hwndItem,
                              &swpBold);
            WinSetWindowPos(hwndArrow, HWND_TOP,
                            swpArrow.x, swpBold.y, swpArrow.cx, swpArrow.cy,
                            SWP_MOVE);
            WinQueryPresParam(hwndItem,
                              PP_FONTNAMESIZE, 0,
                              NULL,
                              sizeof(szFont),
                              szFont,
                              QPF_NOINHERIT);
            strcat(szFont, " Bold");
            WinSetPresParam(hwndItem,
                            PP_FONTNAMESIZE,
                            strlen(szFont)+1,
                            szFont);
            G_ulCurrentBoldID = ulDlgID;
        }
    }
    else
        WinShowWindow(hwndArrow, FALSE);

    static HWND hwndConfigStatus = NULLHANDLE;

    if (pszCurrentConfig)
    {
        // configuring: then show configuration window also
        if (hwndConfigStatus == NULLHANDLE)
        {
            // not loaded yet:
            hwndConfigStatus = gshrLoadDlg(HWND_DESKTOP,
                                          HWND_DESKTOP,
                                          WinDefDlgProc,
                                          NULLHANDLE,      // our own EXE
                                          ID_WID_WARPINSTARTUP,
                                          NULL);
            winhCenterWindow(hwndConfigStatus);
        }
        HWND hwndInfoInStatus = WinWindowFromID(hwndConfigStatus, ID_WIDI_INFOTEXT);

        WinSetWindowText(hwndInfoInStatus, pszCurrentConfig);
        WinShowWindow(hwndConfigStatus, TRUE);
    }

    if ( (fDestroyConfigStatus) && (hwndConfigStatus) )
    {
        WinDestroyWindow(hwndConfigStatus);
        hwndConfigStatus = NULLHANDLE;
    }
}

/*
 *@@ fnwpConfirmRexxDlg:
 *
 *@@added V0.9.12 (2001-05-31) [umoeller]
 */

MRESULT EXPENTRY fnwpConfirmRexxDlg(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    BOOL fCallDefault = TRUE;
    MRESULT mrc = 0;

    switch (msg)
    {
        case WM_COMMAND:
            if (SHORT1FROMMP(mp1) == ID_WIMI_VIEW_CONFIG)
            {
                XTEXTVIEWCDATA xtxvCData;
                memset(&xtxvCData, 0, sizeof(xtxvCData));
                xtxvCData.cbData = sizeof(xtxvCData);
                xtxvCData.flStyle  =   XTXF_VSCROLL | XTXF_AUTOVHIDE
                                     | XTXF_HSCROLL | XTXF_AUTOHHIDE;
                xtxvCData.ulXBorder = 5;
                xtxvCData.ulYBorder = 5;

                CONTROLDEF
                    TextView =
                            {
                                WC_XTEXTVIEW,
                                NULL,       // text, set below
                                WS_VISIBLE | WS_TABSTOP,
                                999,          // ID
                                "4.System VIO",
                                XAC_SIZEX | XAC_SIZEY,
                                { winhQueryScreenCX() * 9 / 10,
                                    winhQueryScreenCY() * 7 / 10 },                                5,
                                &xtxvCData
                            },
                    CancelButton =  {
                                     WC_BUTTON,
                                     "~Close",      // @@todo localize
                                     WS_VISIBLE | WS_TABSTOP | BS_PUSHBUTTON | BS_DEFAULT,
                                     DID_OK,          // ID
                                     CTL_COMMON_FONT,  // no font
                                     0,
                                     { 100, 30 },
                                     5
                                 };

                DLGHITEM ViewDlg[] =
                             {
                                START_TABLE,
                                    START_ROW(ROW_VALIGN_CENTER),
                                        CONTROL_DEF(&TextView),
                                    START_ROW(ROW_VALIGN_CENTER),
                                        CONTROL_DEF(&CancelButton),
                                END_TABLE
                             };

                HWND hwndDlg;
                APIRET arc = dlghCreateDlg(&hwndDlg,
                                           hwnd,
                                           FCF_TITLEBAR | FCF_SYSMENU | FCF_SIZEBORDER | FCF_NOBYTEALIGN,
                                           WinDefDlgProc,
                                           "Script",        // @@todo
                                           ViewDlg,
                                           ARRAYITEMCOUNT(ViewDlg),
                                           NULL,
                                           "9.WarpSans");

                PSZ psz = strdup((const char*)WinQueryWindowPtr(hwnd, QWL_USER));
                txvStripLinefeeds(&psz, 4);
                WinSetDlgItemText(hwndDlg, 999, psz);

                winhCenterWindow(hwndDlg);
                WinProcessDlg(hwndDlg);
                WinDestroyWindow(hwndDlg);

                free(psz);

                fCallDefault = FALSE;
            }
    }

    if (fCallDefault)
        mrc = WinDefDlgProc(hwnd, msg, mp1, mp2);

    return (mrc);
}

/*
 *@@ guiConfirmRexxAllowed:
 *      confirmation callback for the script
 *      classes which gets called if any REXX
 *      code was found in the script.
 *
 *      If this returns TRUE, parsing continues,
 *      otherwise installation is aborted. This
 *      allows paranoid users to stop install
 *      if they are worrying that evil, evil
 *      WarpIN might overwrite their master boot
 *      record or something.
 *
 *@@added V0.9.12 (2001-05-31) [umoeller]
 */

BOOL XWPENTRY guiConfirmRexxAllowed(const char *pcszScript)
{
    static CONTROLDEF
        Icon = {
                        WC_STATIC,
                        NULL,           // text, set below
                        WS_VISIBLE | SS_ICON,
                        0,          // ID
                        NULL,       // no font
                        0,
                        { SZL_AUTOSIZE, SZL_AUTOSIZE },
                        5
                    },
        InfoText =
                    {
                        WC_STATIC,
                        NULL,       // text, set below
                        WS_VISIBLE | SS_TEXT | DT_WORDBREAK | DT_LEFT | DT_TOP,
                        -1,          // ID
                        CTL_COMMON_FONT,
                        0,
                        { 500, SZL_AUTOSIZE },
                        5
                    },
        InfoText2 =
                    {
                        WC_STATIC,
                        NULL,       // text, set below
                        WS_VISIBLE | SS_TEXT | DT_WORDBREAK | DT_LEFT | DT_TOP,
                        -1,          // ID
                        CTL_COMMON_FONT,
                        0,
                        { 500, SZL_AUTOSIZE },
                        5
                    },
        OKButton =  {
                        WC_BUTTON,
                        "~OK",          // @@todo localize
                        WS_VISIBLE | WS_TABSTOP | BS_PUSHBUTTON | BS_DEFAULT,
                        DID_OK,          // ID
                        CTL_COMMON_FONT,  // no font
                        0,
                        { 100, 30 },
                        5
                    },
        CancelButton =  {
                        WC_BUTTON,
                        "~Cancel",      // @@todo localize
                        WS_VISIBLE | WS_TABSTOP | BS_PUSHBUTTON,
                        DID_CANCEL,          // ID
                        CTL_COMMON_FONT,  // no font
                        0,
                        { 100, 30 },
                        5
                    },
        ViewButton =  {
                        WC_BUTTON,
                        "~View script...",     // @@todo localize
                        WS_VISIBLE | WS_TABSTOP | BS_PUSHBUTTON,
                        ID_WIMI_VIEW_CONFIG,   // ID
                        CTL_COMMON_FONT,  // no font
                        0,
                        { 150, 30 },
                        5
                    },
        ShowAgainCheckbox = {
                        WC_BUTTON,
                        "~Show this message again", // @@todo localize
                        WS_VISIBLE | WS_TABSTOP | BS_AUTOCHECKBOX,
                        1000,          // ID
                        CTL_COMMON_FONT,  // no font
                        0,
                        { -1, -1 },
                        5
                    };

    static DLGHITEM ConfirmDlg[] =
                 {
                    START_TABLE,
                        START_ROW(ROW_VALIGN_CENTER),
                            CONTROL_DEF(&Icon),
                        START_TABLE,
                            START_ROW(ROW_VALIGN_CENTER),
                                CONTROL_DEF(&InfoText),
                            START_ROW(ROW_VALIGN_CENTER),
                                CONTROL_DEF(&ViewButton),
                            START_ROW(ROW_VALIGN_CENTER),
                                CONTROL_DEF(&InfoText2),
                            START_ROW(ROW_VALIGN_CENTER),
                                CONTROL_DEF(&OKButton),
                                CONTROL_DEF(&CancelButton),
                            START_ROW(ROW_VALIGN_CENTER),
                                CONTROL_DEF(&ShowAgainCheckbox),
                        END_TABLE,
                    END_TABLE
                 };

    if (G_WpiGlobals.fSuppressREXXConfirm)
        // user has previously selected to disable this:
        return TRUE;

    HWND hwndDlg;
    string strTitle, strMsg, strMsg2;
    wpiGetMessage(strTitle, 108);       // warning
    wpiGetMessage(strMsg, 206);         // explanation
    wpiGetMessage(strMsg2, 207);         // "ok = continue, "cancel" = paranoid
    Icon.pcszText = (const char *)G_hptrMain;
    InfoText.pcszText = strMsg.c_str();
    InfoText2.pcszText = strMsg2.c_str();
    APIRET arc = dlghCreateDlg(&hwndDlg,
                               NULLHANDLE,
                               FCF_TITLEBAR | FCF_SYSMENU | FCF_DLGBORDER | FCF_NOBYTEALIGN,
                               fnwpConfirmRexxDlg,
                               strTitle.c_str(),
                               ConfirmDlg,
                               ARRAYITEMCOUNT(ConfirmDlg),
                               NULL,
                               "9.WarpSans");

    ULONG ulrc = DID_CANCEL;

    if (!arc && hwndDlg)
    {
        winhCenterWindow(hwndDlg);
        winhSetDlgItemChecked(hwndDlg, 1000, TRUE);

        WinSetWindowPtr(hwndDlg, QWL_USER, (PVOID)pcszScript);

        ulrc = WinProcessDlg(hwndDlg);

        if (!winhIsDlgItemChecked(hwndDlg, 1000))
        {
            G_WpiGlobals.fSuppressREXXConfirm = TRUE;
            gshrSaveSettings(0);
        }

        WinDestroyWindow(hwndDlg);
    }

    /* return (wpiMessageBox(108,      // warning
                          206,      // ok = continue, cancel = stop, paranoid
                          MB_OKCANCEL)
               == MBID_OK); */

    return (ulrc == DID_OK);
}

/*
 *@@ guiBeginInstallMode:
 *      this gets called from EnterArchiveMode for preparing
 *      "install" mode, that is, when all the following are
 *      true:
 *
 *      --  an archive was passed on the command line,
 *
 *      --  the archive was successfully loaded,
 *
 *      --  the install script was successfully parsed,
 *
 *      --  the WARPIN_DISPLAYPAGES environment variable
 *          has not been set to "NO".
 *
 *      Required callback.
 *
 *      This gets called no matter if packages from the
 *      archive have already been installed.
 *
 *      This func has no arguments, because all the
 *      needed data has been stored in WPIGLOBALS.
 *
 *      It is the responsibility of this function to start
 *      displaying all the pages described in the install
 *      script. After the last page has been turned to
 *      (i.e. installation is about to begin), this function
 *      must return with TRUE. If the user chooses to cancel
 *      the install process beforehand, this must return FALSE.
 *
 *      If TRUE is returned, the front end will process the list
 *      of packages and build a job list accordingly and start
 *      installing.
 *
 *      If FALSE is returned, WarpIN is terminated.
 *
 *@@changed V0.9.0 (99-10-31) [umoeller]: renamed from guiBeginAddRemove
 *@@changed V0.9.4 (2000-07-26) [umoeller]: renamed from guiPrepareFullInstall
 *@@changed V0.9.12 (2001-05-17) [umoeller]: path entry field still had truncated paths, fixed
 */

BOOL guiBeginInstallMode(VOID)
{

    /*
     * Initialize GUI:
     *
     */

    // load the WarpIN main window.
    // All further processing is done in fnwpPages.
    G_hwndMain = gshrLoadDlg(HWND_DESKTOP, HWND_DESKTOP,
                             fnwpPages,
                             0, ID_WID_MAIN,
                             NULL);
    if (G_hwndMain == NULLHANDLE)
        guiShowMessage("WarpIN Error",
                       "The main window could not be loaded. ",
                       MB_ICONEXCLAMATION | MB_OK);

    // the main window is still hidden!

    // set main window title
    BSString    strWinTitle = "WarpIN";
    if (G_WpiGlobals.strAppTitle())
    {
        strWinTitle += ": ";
        strWinTitle += G_WpiGlobals.strAppTitle;
    }
    WinSetWindowText(G_hwndMain, strWinTitle.c_str());

    // load menu -- the ol' menu in dlg trick
    G_hmenuMain = WinLoadMenu(G_hwndMain, NULLHANDLE, ID_WIM_MAINMENU);
    // remove "View" menu V0.9.2 (2000-02-19) [umoeller]
    WinSendMsg(G_hmenuMain,
               MM_REMOVEITEM,
               MPFROM2SHORT(ID_WIM_VIEW, TRUE),
               0);
    WinSendMsg(G_hwndMain, WM_UPDATEFRAME, (MPARAM)FCF_MENU, 0);

    winhCenterWindow(G_hwndMain); // invisibly

    // now load the four subdialogs, which will be
    // children of the main window. They all use
    // fnwpPages as their window procedure also,
    // since we have all the message handling in
    // there.
    G_hwndPageText = gshrLoadDlg(G_hwndMain,     // parent
                                G_hwndMain,
                                fnwpPages,
                                NULLHANDLE,
                                ID_WID_PAGE_TEXT,
                                0);
    // replace static text with XTextView control
    HWND hwndTextView = txvReplaceWithTextView(G_hwndPageText,
                                               ID_WIDI_INFOTEXT,
                                               WS_VISIBLE | WS_TABSTOP,
                                               XTXF_VSCROLL | XTXF_AUTOVHIDE,
                                               2);
    WinSendMsg(hwndTextView, TXM_SETWORDWRAP, (MPARAM)TRUE, 0);

    // load readme page subdialog
    G_hwndPageReadme = gshrLoadDlg(G_hwndMain,     // parent
                                  G_hwndMain,
                                  fnwpPages,
                                  NULLHANDLE,
                                  ID_WID_PAGE_README,
                                  0);
    // replace static text with XTextView control
    hwndTextView = txvReplaceWithTextView(G_hwndPageReadme,
                                          ID_WIDI_INFOTEXT,
                                          WS_VISIBLE | WS_TABSTOP,
                                          XTXF_VSCROLL | XTXF_AUTOVHIDE,
                                          2);
    WinSendMsg(hwndTextView, TXM_SETWORDWRAP, (MPARAM)TRUE, 0);

    // replace readme text with XTextView control
    hwndTextView = txvReplaceWithTextView(G_hwndPageReadme,
                                          ID_WIDI_READMEMLE,
                                          WS_VISIBLE | WS_TABSTOP,
                                          XTXF_VSCROLL | XTXF_AUTOVHIDE
                                            | XTXF_HSCROLL | XTXF_AUTOHHIDE,
                                          2);

    // load container page subdialog
    G_hwndPageContainer = gshrLoadDlg(G_hwndMain,     // parent
                                     G_hwndMain,
                                     fnwpPages,
                                     NULLHANDLE,
                                     ID_WID_PAGE_CONTAINER,
                                     0);
    hwndTextView = txvReplaceWithTextView(G_hwndPageContainer,
                                          ID_WIDI_INFOTEXT,
                                          WS_VISIBLE | WS_TABSTOP,
                                          XTXF_VSCROLL | XTXF_AUTOVHIDE,
                                          2);
    WinSendMsg(hwndTextView, TXM_SETWORDWRAP, (MPARAM)TRUE, 0);
    // set entry field path limit V0.9.12 (2001-05-17) [umoeller]
    winhSetEntryFieldLimit(WinWindowFromID(G_hwndPageContainer, ID_WIDI_PATHENTRY),
                           254);

    // load configure page subdialog
    G_hwndPageConfigure = gshrLoadDlg(G_hwndMain,     // parent
                                     G_hwndMain,
                                     fnwpPages,
                                     NULLHANDLE,
                                     ID_WID_PAGE_CONFIGURE,
                                     0);
    hwndTextView = txvReplaceWithTextView(G_hwndPageConfigure,
                                          ID_WIDI_INFOTEXT,
                                          WS_VISIBLE | WS_TABSTOP,
                                          XTXF_VSCROLL | XTXF_AUTOVHIDE,
                                          2);
    WinSendMsg(hwndTextView, TXM_SETWORDWRAP, (MPARAM)TRUE, 0);

    // this loads the drive-info window, which just
    // has the cnr for displaying drives.
    G_hwndDriveInfo = gshrLoadDlg(HWND_DESKTOP,        // parent
                                 G_hwndMain,         // owner is the main window
                                 fnwpDriveInfo,
                                 0, ID_WID_DRIVEINFO,
                                 NULL);

    // add ourselves to the tasklist
    winhAddToTasklist(G_hwndMain,
                      G_hptrMain);

    // now set up the controls in the dialogs;
    // most of these are currently hidden, but
    // we can initialize them anyway

    // setup the container
    G_hwndPackagesCnr = WinWindowFromID(G_hwndPageContainer, ID_WIDI_CNR);
    BEGIN_CNRINFO()
    {
        cnrhSetView(CV_TREE | CV_ICON | CA_TREELINE | CA_OWNERDRAW);
        // CnrInfo.cxTreeIndent = 15;
        // set icon sizes to system mini-icon size
        cnrhSetBmpOrIconSize((G_ulMiniIconSize * 2 + 5), G_ulMiniIconSize);
    } END_CNRINFO(G_hwndPackagesCnr);

    // hide the startup window
    WinDestroyWindow(G_hwndWarpINStartup);
    G_hwndWarpINStartup = NULLHANDLE;

    /*
     * Start user interaction:
     *
     */

    // start with page 1
    WinPostMsg(G_hwndMain, WPIM_TURNTOPAGE, (MPARAM)1, MPNULL);
    // go!
    ULONG ulrc = WinProcessDlg(G_hwndMain);
            // fnwpPages takes over;
            // this does not return until we either
            // get to the last page and the button was
            // pressed there (then DID_OK is returned)
            // or everything has been cancelled

    WinDestroyWindow(G_hwndMain);
    G_hwndMain = NULLHANDLE;

    // return FALSE if Cancel was pressed or something went wrong
    return (ulrc == DID_OK);
}

/*
 *@@ guiOnJobSelectionChanged:
 *      this gets called from the FEJobBase::Select
 *      method for every package selection that has
 *      changed.
 *
 *      The GUI should update its display for that
 *      package then.
 *
 *      Required callback.
 *
 *@@changed V0.9.4 (2000-07-26) [umoeller]: renamed from guiJobSelectionChanged
 */

VOID guiOnJobSelectionChanged(FEJobBase *pJob)
{
    if (G_fContainerPageReady)
    {
        // get the record core which we created for
        // this package
        PPACKAGERECORD precc = (PPACKAGERECORD)(pJob->_pvGuiData);

        // repaint the record core
        WinSendMsg(G_hwndPackagesCnr,
                   CM_INVALIDATERECORD,
                   (MPARAM)&precc,
                   MPFROM2SHORT(1, CMA_ERASE));
    }
}

/*
 *@@ guiVarPrompt:
 *      this gets called by wpiResolveMacros if an
 *      environment variable needs to be resolved and
 *      a value could not be found.
 *
 *      This must return the new value for that variable
 *      in a new buffer allocated using malloc().
 *
 *      Required callback.
 *
 *      Throws:
 *      -- BSCancelExcpt.
 *
 *@@changed V0.9.4 (2000-07-26) [umoeller]: text wasn't displayed right. Fixed.
 */

PSZ guiVarPrompt(BSVarPrompt &VarPrompt)    // in: VARPROMPT definition
{
    PSZ     pszValue;
    BOOL    fContinue = FALSE;
    do
    {
        fContinue = FALSE;

        HWND hwndDlg = gshrLoadDlg(HWND_DESKTOP, HWND_DESKTOP,
                                  gshr_fnwpDialogWithHelp,
                                  NULLHANDLE, ID_WID_ENTERVARIABLE,
                                  NULL);
        WinSetWindowULong(hwndDlg, QWL_USER, IDHI_VARPROMPT_DLG);
        winhCenterWindow(hwndDlg);
        gshrSetControlsFont(hwndDlg, 0, 2000);

        PSZ pszText;

        // ### handle fail condition here

        // description given?
        if (VarPrompt._strDescription())
            // yes: use that
            pszText = strdup(VarPrompt._strDescription.c_str());
        else
            // use default text from dlg template...
            pszText = winhQueryWindowText(WinWindowFromID(hwndDlg, ID_WIDI_EV_INFOTEXT));

        HWND hwndTextView = txvReplaceWithTextView(hwndDlg,
                                                   ID_WIDI_EV_INFOTEXT,
                                                   WS_VISIBLE | WS_TABSTOP,
                                                   XTXF_VSCROLL | XTXF_AUTOVHIDE,
                                                   2);  // border
        WinSendMsg(hwndTextView,
                   TXM_SETWORDWRAP,
                   (MPARAM)(TRUE),
                   0);

        txvStripLinefeeds(&pszText, 4);

        // set description text
        WinSetWindowText(hwndTextView, pszText);
        free(pszText);

        CHAR szVarName[300];
        sprintf(szVarName, "SET %s=", VarPrompt._strVarName.c_str());
        WinSetDlgItemText(hwndDlg, ID_WIDI_EV_SETTEXT, szVarName);

        HWND hwndEntryField = WinWindowFromID(hwndDlg, ID_WIDI_EV_ENTRYFIELD);
        winhSetEntryFieldLimit(hwndEntryField, 255);

        if (WinProcessDlg(hwndDlg) == DID_OK)
        {
            pszValue = winhQueryWindowText(hwndEntryField);
        }
        else
            throw BSCancelExcpt();

        WinDestroyWindow(hwndDlg);

        if (!VarPrompt.Validate(pszValue))
        {
            if (pszValue == NULL)
                pszValue = strdup("NULL");

            if (wpiMessageBox(102,      // error
                              185,      // %1 not valid
                              MB_RETRYCANCEL,
                              &pszValue,
                              1)
                    == MBID_CANCEL)
                throw BSCancelExcpt();

            fContinue = TRUE;
        }

        if (fContinue)
            free(pszValue);
    } while (fContinue);

    return (pszValue);
}

/*
 *@@ GUIEXECUTE:
 *      temporary structure created for fntExecute
 *      parameters.
 *
 *@@added V0.9.9 (2001-03-30) [umoeller]
 */

typedef struct _GUIEXECUTE
{
    BSExecute       *pExec;     // executable instance
    BSFileLogger    *pLogFile;  // log file or NULL
} GUIEXECUTE, *PGUIEXECUTE;

/*
 *@@ fntExecute:
 *      synchronous PM thread for executing a
 *      program. This has a PM msg queue and
 *      is run using thrRunSync from guiExecute.
 *
 *@@added V0.9.5 (2000-08-26) [umoeller]
 *@@changed V0.9.9 (2001-03-30) [umoeller]: added log file
 */

void _Optlink fntExecute(PTHREADINFO ptiMyself)
{
    ULONG   ulrc = 0;
    PGUIEXECUTE pge = (PGUIEXECUTE)(ptiMyself->ulData);

    try
    {
        pge->pExec->Execute(pge->pLogFile);
    }
    catch(BSConfigExcpt& X)
    {
        // error occured:
        ulrc = X._iData;
    }

    WinPostMsg(ptiMyself->hwndNotify,
               WM_USER,
               (MPARAM)ulrc,
               0);
}

/*
 *@@ guiExecute:
 *      this gets called from wpiExecutes for each
 *      executable which is run because of an
 *      EXECUTE tag.
 *
 *      Required callback. This is a separate callback
 *      because we don't want to block thread 1 while
 *      the program is running. So we run a second
 *      thread for that and keep processing thread 1's
 *      message queue.
 *
 *@@added V0.9.5 (2000-08-26) [umoeller]
 *@@changed V0.9.9 (2001-03-30) [umoeller]: added log file
 */

BOOL guiExecute(BSExecute *pExec,       // in: executable instance
                BSFileLogger *pLogFile) // in: log file or NULL
{
    BOOL brc = FALSE;

    GUIEXECUTE ge;
    ge.pExec = pExec;
    ge.pLogFile = pLogFile;

    ULONG iError = thrRunSync(G_WpiGlobals.habThread1,
                              fntExecute,
                              "Execute",
                              (ULONG)&ge);

    if (iError)
    {
        PSZ pszSysError = doshQuerySysErrorMsg(iError);
        PSZ apsz[2] =  {
                          pExec->_pszExecutable,
                          pszSysError
                       };

        brc = FALSE;
        wpiMessageBox(102, // "WarpIN: Error"
                      177,   // "error executing program"
                      MB_ICONEXCLAMATION | MB_OK,
                      apsz, 2);
    }

    return (iError == 0);
}

/*
 *@@ guiProductInfo:
 *      this should display the WarpIN product
 *      info window.
 *
 *      Required callback.
 */

VOID guiProductInfo(HWND hwndOwner,
                    BOOL fLongVersion)
{
    CHAR    szVersion[100] = BLDLEVEL_VERSION,
            szDate[100] = __DATE__,
            szCodepage[20];
#ifdef __EMX__
    CHAR    szEMXRev[40];
    sprintf(szEMXRev, "EMX/GCC %s", _emx_vprt);
#endif
    PSZ     apszTable[4] =
                {
                    szVersion,
                    szDate,
#ifdef __IBMCPP__
                    "IBM VisualAge C++ 3.08",
#else
                    szEMXRev,
#endif
                    szCodepage
                };
    sprintf(szCodepage, "%d", WinQueryCp(G_hmqThread1));

    gshrMLEMessageBox(150,               // title
                      (fLongVersion) ? 156 : 101,
                      FALSE,             // hide cancel
                      apszTable, 4);     // string replacements
}


