/* $Id: gui_db.cpp,v 1.32 2001/05/22 17:22:09 umoeller Exp $ */

/*
 *@@sourcefile gui_db.cpp:
 *      this contains the PM-specific stuff for
 *      "database mode".
 *
 *      This has been separated from gui.cpp (99-10-24) [umoeller].
 *
 *      This has the required guiBeginDatabaseMode
 *      callback, which gets called from warpin.cpp
 *      when database mode is entered to have the
 *      database displayed.
 *
 *      Also, this has the required guiStartRemove and
 *      guiStartVerify callbacks.
 *
 *@@header "frontend\gui.h"
 */

/*
 *
 *      This file Copyright (C) 1998-2000 Ulrich Mller.
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, in version 2 as it comes in the COPYING
 *      file of this distribution.
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 */

#define OS2EMX_PLAIN_CHAR
    // this is needed for "os2emx.h"; if this is defined,
    // emx will define PSZ as _signed_ char, otherwise
    // as unsigned char

#define INCL_DOS
#define INCL_DOSERRORS
#define INCL_WIN
#define INCL_WINWORKPLACE
#define INCL_GPILOGCOLORTABLE
#define INCL_GPIPRIMITIVES
#include <os2.h>

#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <setjmp.h>             // needed for except.h
#include <assert.h>             // needed for except.h
#include <io.h>

#include "setup.h"

// include's from helpers
#include "helpers\configsys.h"
#include "helpers\datetime.h"
#include "helpers\dosh.h"
#include "helpers\cnrh.h"
#include "helpers\comctl.h"
#include "helpers\gpih.h"
#include "helpers\winh.h"
#include "helpers\stringh.h"
#include "helpers\threads.h"
#include "helpers\xstring.h"

// back-end includes
#include "wiarchive\wiarchive.h"

// base includes (99-11-07) [umoeller]
#include "base\bs_string.h"
#include "base\bs_errors.h"
#include "base\bs_logger.h"
#include "base\bs_config.h"
#include "base\bs_config_impl.h"
#include "base\bs_script.h"

// front-end includes
#include "frontend\fe_base.h"
#include "frontend\fe_database.h"
#include "frontend\fe_package.h"
#include "frontend\fe_package_db.h"

#include "frontend\warpin.h"
#include "frontend\calbacks.h"

#include "frontend\dlgids.h"
#include "frontend\gui.h"

#include "helpers\except.h"

#pragma hdrstop

/* ******************************************************************
 *
 *  Private declarations
 *
 ********************************************************************/

// window class for database main frame
#define WC_DATABASEMAINFRAME    "WPI_DatabaseMainFrameClass"

MRESULT EXPENTRY fnwpDatabaseStatus(HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2);
VOID UpdateListCnr(PDATABASERECORD precc);    // in: package record core

/* ******************************************************************
 *
 *  Global variables
 *
 ********************************************************************/

HWND                    G_hwndDBPackagesDlg = NULLHANDLE,     // left dlg: packages
                        G_hwndDBFileListDlg = NULLHANDLE,     // right dlg: files
                        G_hwndDBSplitMain = NULLHANDLE;       // split window for linking

// context menus
HWND                    G_hmenuPckWhitespace = NULLHANDLE,
                        G_hmenuPckApplication = NULLHANDLE,
                        G_hmenuPckPackage = NULLHANDLE,
                        G_hmenuFilesWhitespace = NULLHANDLE;

// currently selected database record (fnwpDBPackagesDlg)
PDATABASERECORD         G_preccDBSelected = NULL;       // (99-10-25) [umoeller]

// source record core for context menus
PDATABASERECORD         G_preccLeftSource = NULL;
PFILERECORDCORE         G_preccRightSource = NULL;

// list of all record cores for applications (tree view only)
list<PDATABASERECORD>   G_AppRecordsList;
// list of all record cores for packages (all views)
list<PDATABASERECORD>   G_PckRecordsList;

// controls list for file-list window (winhAdjustControls)
MPARAM     G_ampPackagesDlg[] =
            {
                MPFROM2SHORT(ID_WIDI_CNR, XAC_SIZEX | XAC_SIZEY),
                MPFROM2SHORT(ID_WIDI_CANCELBUTTON, XAC_MOVEX)
            };

MPARAM     G_ampFileListDlg[] =
            {
                MPFROM2SHORT(ID_WIDI_TARGETPATHTXT, XAC_MOVEY),
                MPFROM2SHORT(ID_WIDI_TARGETPATHDIR, XAC_MOVEY | XAC_SIZEX),
                MPFROM2SHORT(ID_WIDI_CNR, XAC_SIZEX | XAC_SIZEY),
                MPFROM2SHORT(ID_WIDI_CANCELBUTTON, XAC_MOVEX)
            };

XADJUSTCTRLS            G_xacPackagesDlg,
                        G_xacFileListDlg;        // for winhAdjustControls

list<BSString*>         G_ConfigStrings;        // strings 2b freed

PFNWP                   G_pfnwpListboxOrig = NULLHANDLE;

/* ******************************************************************
 *
 *  Helper funcs
 *
 ********************************************************************/

/*
 *@@ CreateDBAppRecord:
 *      creates a DATABASERECORD for the given application ID.
 *      Used with the database container subwindow.
 */

PDATABASERECORD CreateDBAppRecord(HWND hwndCnrAlloc, FEPackageID &PckID)
{
    PDATABASERECORD preccApplication
        = (PDATABASERECORD)cnrhAllocRecords(hwndCnrAlloc,
                                            sizeof(DATABASERECORD),
                                            1);
    if (preccApplication)
    {
        // set package info to NULL;
        // this signifies that this is an application
        preccApplication->pPckInfo = NULL;

        preccApplication->recc.hptrIcon
            = preccApplication->recc.hptrMiniIcon
            = G_hptrApp;

        // store other fields
        preccApplication->pszAuthor = strdup(PckID._strAuthor.c_str());
        preccApplication->pszApplication = strdup(PckID._strApplication.c_str());
    }

    return (preccApplication);
}

/*
 *@@ CreateDBPckRecord:
 *      creates a DATABASERECORD for the given package info.
 *      Used with the database container subwindow.
 */

PDATABASERECORD CreateDBPckRecord(HWND hwndCnrAlloc, FEDBPackage* ppiPackage)
{
    PDATABASERECORD preccPck = NULL;

    // create package ID on the stack
    FEPackageID PckID(ppiPackage, __FILE__, __LINE__,
                      FALSE);   // don't allow short format

    preccPck = (PDATABASERECORD)cnrhAllocRecords(hwndCnrAlloc,
                                                 sizeof(DATABASERECORD),
                                                 1);
    if (preccPck)
    {
        // store package info in recc
        preccPck->pPckInfo = ppiPackage;
        // and recc in package info
        ppiPackage->_pvGuiData = (PRECORDCORE)preccPck;

        preccPck->recc.hptrIcon
            = preccPck->recc.hptrMiniIcon
            = G_hptrQMark;

        // store other fields
        preccPck->pszAuthor = strdup(PckID._strAuthor.c_str());
        preccPck->pszApplication = strdup(PckID._strApplication.c_str());
        preccPck->pszPackageName = strdup(PckID._strPackageName.c_str());
        preccPck->ulVersionMajor = PckID._ulVersionMajor;
        preccPck->ulVersionMinor = PckID._ulVersionMinor;
        preccPck->ulVersionRevision = PckID._ulVersionRevision;
        sprintf(preccPck->szVersion, "%d.%d.%d",
                PckID._ulVersionMajor,
                PckID._ulVersionMinor,
                PckID._ulVersionRevision);
        preccPck->pszVersion = preccPck->szVersion;
        preccPck->ulFiles = preccPck->pPckInfo->_PackHeader.files;
        preccPck->ulTotalSize = preccPck->pPckInfo->_PackHeader.origsize;
        preccPck->pszTargetPath = (PSZ)preccPck->pPckInfo->QueryTargetPath();

        // install date/time
        cnrhDateTimeDos2Win(&ppiPackage->_InstallDateTime,
                            &preccPck->cdateInstall,
                            &preccPck->ctimeInstall);
    }

    return (preccPck);
}

/*
 *@@ FreeStringsList:
 *      frees all strings on the specified list.
 *      This is used with the strings created by
 *      InsertConfigIntoCnr which are needed while
 *      the container displays records.
 *
 *@@added V0.9.2 (2000-02-19) [umoeller]
 */

VOID FreeStringsList(list<BSString*> &Strings2FreeList)
{
    list<BSString*>::iterator StrStart = Strings2FreeList.begin(),
                              StrEnd = Strings2FreeList.end();
    for (; StrStart != StrEnd; StrStart++)
    {
        BSString *pStrThis = *StrStart;
        if (pStrThis)
            delete pStrThis;
    }

    Strings2FreeList.clear();
}

/*
 *@@ CheckLeftViewMenuItems:
 *      checks the view items for the left view
 *      (tree or details) in the given menu.
 *
 *      This can either be the "View" pulldown or
 *      a context menu.
 *
 *@@added V0.9.5 (2000-08-10) [umoeller]
 */

VOID CheckLeftViewMenuItems(HWND hmenu)
{
    WinCheckMenuItem(hmenu,
                     ID_WIMI_DATABASE_TREEVIEW,
                     G_GuiSettings.ulLeftView == ID_WIMI_DATABASE_TREEVIEW);
    WinCheckMenuItem(hmenu,
                     ID_WIMI_DATABASE_DETAILSVIEW,
                     G_GuiSettings.ulLeftView != ID_WIMI_DATABASE_TREEVIEW);
}

/*
 *@@ CheckRightViewMenuItems:
 *      checks the view items for the right view
 *      (files or config) in the given menu.
 *
 *      This can either be the "View" pulldown or
 *      a context menu.
 *
 *@@added V0.9.5 (2000-08-10) [umoeller]
 */

VOID CheckRightViewMenuItems(HWND hmenu)
{
    WinCheckMenuItem(hmenu,
                     ID_WIMI_VIEW_FILES,
                     G_GuiSettings.ulRightView == ID_WIMI_VIEW_FILES);
    WinCheckMenuItem(hmenu,
                     ID_WIMI_VIEW_CONFIG,
                     G_GuiSettings.ulRightView != ID_WIMI_VIEW_FILES);
}

/*
 *@@ ProcessWMChar:
 *      switches the focus between the two DB dialogs.
 *
 *@@added V0.9.2 (2000-03-10) [umoeller]
 */

VOID ProcessWMChar(MPARAM mp1, MPARAM mp2)
{
    USHORT usFlags    = SHORT1FROMMP(mp1);
    // USHORT usch       = SHORT1FROMMP(mp2);
    USHORT usvk       = SHORT2FROMMP(mp2);

    if (    (usFlags & KC_VIRTUALKEY)
         && (usvk == VK_TAB)
       )
    {
        if ((usFlags & KC_KEYUP) == 0)
        {
            HWND hwndFocus = WinQueryFocus(HWND_DESKTOP);
            if (WinIsChild(hwndFocus, G_hwndDBPackagesDlg))
                WinSetFocus(HWND_DESKTOP, WinWindowFromID(G_hwndDBFileListDlg, ID_WIDI_CNR));
            else
                WinSetFocus(HWND_DESKTOP, WinWindowFromID(G_hwndDBPackagesDlg, ID_WIDI_CNR));
        }
    }
}

/*
 *@@ InsertFilesIntoCnr:
 *      switches the right container to "files" mode
 *      and adds all the files for the specified package.
 *      This has been extracted from UpdateListCnr.
 *
 *@@added V0.9.2 (2000-02-19) [umoeller]
 */

VOID InsertFilesIntoCnr(PDATABASERECORD precc,  // in: database record selected on the left
                        HWND hwndFilesCnr)      // in: right cnr window
{
    // set up data for Details view columns
    XFIELDINFO      xfi[4];

    xfi[0].ulFieldOffset = FIELDOFFSET(RECORDCORE, pszIcon);
    xfi[0].pszColumnTitle = G_NLSStrings.pszFileName; // "File name";
    xfi[0].ulDataType = CFA_STRING;
    xfi[0].ulOrientation = CFA_LEFT;

    xfi[1].ulFieldOffset = FIELDOFFSET(FILERECORDCORE, ulSize);
    xfi[1].pszColumnTitle = G_NLSStrings.pszSize; // "Size";
    xfi[1].ulDataType = CFA_ULONG;
    xfi[1].ulOrientation = CFA_RIGHT;

    xfi[2].ulFieldOffset = FIELDOFFSET(FILERECORDCORE, cdateCreation);
    xfi[2].pszColumnTitle = G_NLSStrings.pszCreationDate; // "Creation date";
    xfi[2].ulDataType = CFA_DATE;
    xfi[2].ulOrientation = CFA_RIGHT;

    xfi[3].ulFieldOffset = FIELDOFFSET(FILERECORDCORE, cdateLastWrite);
    xfi[3].pszColumnTitle = G_NLSStrings.pszLastWriteDate; // "Last write date";
    xfi[3].ulDataType = CFA_DATE;
    xfi[3].ulOrientation = CFA_RIGHT;

    cnrhClearFieldInfos(hwndFilesCnr, FALSE);

    PFIELDINFO pFieldInfoLast =
        cnrhSetFieldInfos(hwndFilesCnr,
                          &xfi[0],
                          (sizeof(xfi) / sizeof(XFIELDINFO)),   // array item count
                          TRUE,          // draw lines
                          0);            // column index to return
    BEGIN_CNRINFO()
    {
        // set split bar
        cnrhSetSplitBarAfter(pFieldInfoLast);
        cnrhSetSplitBarPos(100);
        // switch view
        cnrhSetView(CV_DETAIL | CA_DETAILSVIEWTITLES);
    } END_CNRINFO(hwndFilesCnr);

    // now fill the container
    FEDBPackage*    pPckInfo = precc->pPckInfo;
    ULONG           ulHeaderCount = 0;

    list<WIFileHeader*>* pHeaderList = pPckInfo->GetFileHeaders(
                                                         &ulHeaderCount);
    if (pHeaderList)
    {
        // allocate memory for file record cores;
        // the number of items has been returned by datGetFileHeaders
        PFILERECORDCORE preccFileFirst = (PFILERECORDCORE)cnrhAllocRecords(
                                                        hwndFilesCnr,
                                                        sizeof(FILERECORDCORE),
                                                        ulHeaderCount);

        if (preccFileFirst)
        {
            // preccFile will be used for filling the current recc;
            // the record cores are linked using the preccNextRecord fields
            PFILERECORDCORE preccFile = preccFileFirst;

            list<WIFileHeader*>::iterator HeaderStart = pHeaderList->begin(),
                                          HeaderEnd = pHeaderList->end();

            for (; (HeaderStart != HeaderEnd); HeaderStart++)
            {
                WIFileHeader *pHeaderThis = *HeaderStart;

                strcpy(preccFile->szFilename, pHeaderThis->name);
                preccFile->recc.pszIcon = preccFile->szFilename;
                preccFile->ulSize = pHeaderThis->origsize;
                // convert time_t to human-readable format
                struct tm* pTimeBuf = localtime(&(**HeaderStart).lastwrite);
                preccFile->cdateLastWrite.day    = pTimeBuf->tm_mday;
                preccFile->cdateLastWrite.month  = pTimeBuf->tm_mon + 1;
                preccFile->cdateLastWrite.year   = pTimeBuf->tm_year + 1900;
                pTimeBuf = localtime(&pHeaderThis->creation);
                preccFile->cdateCreation.day   = pTimeBuf->tm_mday;
                preccFile->cdateCreation.month = pTimeBuf->tm_mon + 1;
                preccFile->cdateCreation.year  = pTimeBuf->tm_year + 1900;

                delete pHeaderThis;

                // next record
                preccFile = (PFILERECORDCORE)(preccFile->recc.preccNextRecord);
            }

            // insert the record core linked list
            RECORDINSERT ri;
            memset(&ri, 0, sizeof(ri));
            ri.cb = sizeof(ri);
            ri.pRecordOrder = (PRECORDCORE)CMA_END;
            ri.pRecordParent = 0;
            ri.fInvalidateRecord = TRUE;
            ri.zOrder = CMA_TOP;
            ri.cRecordsInsert = ulHeaderCount;

            WinSendMsg(hwndFilesCnr,
                       CM_INSERTRECORD,
                       (MPARAM)preccFileFirst,
                       (MPARAM)&ri);
        }

        delete pHeaderList;
    }
}

/*
 *@@ SetCnrConfigFieldInfos:
 *
 *@@added V0.9.2 (2000-02-19) [umoeller]
 */

VOID SetCnrConfigFieldInfos(HWND hwndConfigCnr)
{
    // set up config cnr details view
    XFIELDINFO      xfi[2];

    xfi[0].ulFieldOffset = FIELDOFFSET(DBCONFIGRECORD, pszConfigType);
    xfi[0].pszColumnTitle = "Type";
    xfi[0].ulDataType = CFA_STRING;
    xfi[0].ulOrientation = CFA_LEFT;

    xfi[1].ulFieldOffset = FIELDOFFSET(DBCONFIGRECORD, pszConfigData);
    xfi[1].pszColumnTitle = "Configuration data";
    xfi[1].ulDataType = CFA_STRING;
    xfi[1].ulOrientation = CFA_LEFT;

    cnrhClearFieldInfos(hwndConfigCnr, FALSE);

    PFIELDINFO pFieldInfoLast =
        cnrhSetFieldInfos(hwndConfigCnr,
                          &xfi[0],
                          (sizeof(xfi) / sizeof(XFIELDINFO)),   // array item count
                          TRUE,          // draw lines
                          0);            // column index to return
    BEGIN_CNRINFO()
    {
        // set split bar
        cnrhSetSplitBarAfter(pFieldInfoLast);
        cnrhSetSplitBarPos(100);
        // switch view
        cnrhSetView(CV_DETAIL | CA_DETAILSVIEWTITLES);
    } END_CNRINFO(hwndConfigCnr);
}

/*
 *@@ InsertConfigIntoCnr:
 *      this lists the configuration of the specified
 *      FEDBPackage by inserting DBCONFIGRECORD records
 *      into the specified container.
 *
 *      Returns the no. of records inserted or 0 if none.
 *
 *      Gets called both from UpdateListCnr when the right
 *      container is in "config" mode and from ConfirmDeinstall
 *      to list the package configuration to be de-installed.
 *
 *      This function creates a lot of strings which are used
 *      by the container record cores. All those strings are
 *      appended to the given Strings2FreeList. You should
 *      invoke FreeStringsList on that list later when the
 *      DBCONFIGRECORD's are no longer used (and only then,
 *      or otherwise the container control will crash).
 *
 *@@added V0.9.1 (2000-02-12) [umoeller]
 *@@changed V0.9.2 (2000-02-19) [umoeller]: sped up display
 *@@changed V0.9.2 (2000-02-19) [umoeller]: much shorter now since we're using virtual BSConfigBase methods
 */

ULONG InsertConfigIntoCnr(HWND hwndConfigCnr,       // in: cnr to receive DBCONFIGRECORD's
                          FEDBPackage *pDBPackage,  // in: package to add info for
                          list<BSString*> &Strings2FreeList) // in: list receiving strings to be freed later
{
    ULONG   ulrc = 0;

    list<BSConfigBase*>::iterator CfgStart = pDBPackage->_listUndoConfig.begin(),
                                  CfgEnd   = pDBPackage->_listUndoConfig.end();
    for (; CfgStart != CfgEnd; CfgStart++)
    {
        BSConfigBase *pManipThis = *CfgStart;

        PDBCONFIGRECORD pcrec
            = (PDBCONFIGRECORD)cnrhAllocRecords(hwndConfigCnr,
                                                sizeof(DBCONFIGRECORD),
                                                1);
        if (pcrec)
        {
            pcrec->pszConfigType = pManipThis->DescribeType();

            BSString *pstrConfig = new BSString();
            pstrConfig->append(pManipThis->DescribeData());
            Strings2FreeList.push_back(pstrConfig);
            pcrec->pszConfigData = (PSZ)pstrConfig->c_str();
            pcrec->pConfig = pManipThis;
            cnrhInsertRecords(hwndConfigCnr,
                              (PRECORDCORE)NULL,       // root level
                              (PRECORDCORE)pcrec,
                              FALSE,
                              NULL,
                              CRA_RECORDREADONLY,
                              1);
        }
        ulrc++;
    }

    // invalidate container
    cnrhInvalidateAll(hwndConfigCnr);

    return (ulrc);
}

/*
 *@@ fnwpListboxNoselect:
 *
 *@@added V0.9.5 (2000-08-26) [umoeller]
 */

MRESULT EXPENTRY fnwpListboxNoselect(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    MRESULT mrc = 0;

    switch (msg)
    {
        case WM_BUTTON1DOWN:
            WinSetFocus(HWND_DESKTOP, hwnd);
            mrc = (MPARAM)TRUE;
        break;

        case WM_BUTTON1UP:
        case WM_BUTTON1CLICK:
            mrc = (MPARAM)TRUE;
        break;

        default:
            mrc = G_pfnwpListboxOrig(hwnd, msg, mp1, mp2);
    }

    return (mrc);
}

/*
 *@@ fnwpConfirmDeinstall:
 *      dialog proc for "Confirm deinstall" dlg (see
 *      ConfirmDeinstall).
 *
 *@@added V0.9.4 (2000-07-01) [umoeller]
 */

MRESULT EXPENTRY fnwpConfirmDeinstall(HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    MRESULT mrc = 0;

    switch (msg)
    {
        case WM_CONTROL:
        {
            USHORT usID = SHORT1FROMMP(mp1);
            USHORT usNotifyCode = SHORT2FROMMP(mp1);

            switch(usID)
            {
                case ID_WIDI_CFD_UNDOCONFIG:
                    if (    (usNotifyCode == BN_CLICKED)
                         || (usNotifyCode == BN_DBLCLICKED) // added V0.9.0
                       )
                    {
                        HWND hwndCnr = WinWindowFromID(hwndDlg, ID_WIDI_CFD_CONFIGCNR);
                        if (winhIsDlgItemChecked(hwndDlg, usID))
                        {
                            cnrhSelectAll(hwndCnr, TRUE);
                        }
                        else
                            cnrhSelectAll(hwndCnr, FALSE);
                    }
                break;

                case ID_WIDI_CFD_CONFIGCNR:
                {
                    HWND hwndCnr = WinWindowFromID(hwndDlg, ID_WIDI_CFD_CONFIGCNR);
                    if (usNotifyCode == CN_EMPHASIS)
                    {
                        PRECORDCORE pSel
                            = (PRECORDCORE)WinSendMsg(hwndCnr,
                                                      CM_QUERYRECORDEMPHASIS,
                                                      (MPARAM)CMA_FIRST,
                                                      (MPARAM)CRA_SELECTED);
                        winhSetDlgItemChecked(hwndDlg,
                                              ID_WIDI_CFD_UNDOCONFIG,
                                              ((pSel) && ((LONG)pSel != -1))
                                              );
                    }
                break; }
            }
        break; }

        case WM_HELP:
            gshrDisplayHelp(IDHI_CONFIRM_DEINSTALL);
        break;

        default:
            mrc = WinDefDlgProc(hwndDlg, msg, mp1, mp2);
    }

    return (mrc);
}

/*
 *@@ DEPENDENCYRECORD:
 *      extended record structure for dependency
 *      violation dialog (ConfirmDeinstall).
 *
 *@@added V0.9.12 (2001-05-17) [umoeller]
 */

typedef struct _DEPENDENCYRECORD
{
    RECORDCORE      recc;

            // "package"
    PSZ     pszDepApplication,
            pszDepName,
            pszDepVersion,      // points to szDepVersion
            pszDepAuthor,

            // "requires"
            pszReqApplication,
            pszReqName,
            pszReqVersion,      // points to szReqVersion
            pszReqAuthor;

    // string buffers
    CHAR    szDepVersion[100],
            szReqVersion[100];

} DEPENDENCYRECORD, *PDEPENDENCYRECORD;

/*
 *@@ ConfirmDeinstall:
 *      this displays the "Confirm Deinstall" dialog.
 *      pPackagesList has the packages to deinstall,
 *      depending on which the dialog will be filled
 *      with data and such.
 *
 *      This gets called in two situations:
 *      1)  from main() after the user has selected
 *          "Deinstall" in the "Archive installed" dialog;
 *      2)  from fnwpDatabaseMain also before deinstalling
 *          any packages.
 *
 *      This returns -1 if the user pressed Cancel.
 *      Otherwise a value >= 0 is returned, that is 0 ORed
 *      with the DBT_* flags in fe_database.h:
 *      -- DBT_DELETEFILES     0x01
 *      -- DBT_UNDOCONFIG      0x02
 *
 *@@changed V0.9.0 (99-11-02) [umoeller]: renamed function
 *@@changed V0.9.0 (99-11-02) [umoeller]: no longer required callback, only used in this file
 *@@changed V0.9.1 (2000-01-05) [umoeller]: "Files" checkbox wasn't working; fixed
 *@@changed V0.9.1 (2000-01-05) [umoeller]: reworked configuration flags
 *@@changed V0.9.1 (2000-01-05) [umoeller]: reworked configuration display; now using container
 *@@changed V0.9.2 (2000-03-10) [umoeller]: "undo config" wasn't checked right for multiple packages; fixed
 *@@changed V0.9.4 (2000-07-01) [umoeller]: added individual undo-config selection
 *@@changed V0.9.4 (2000-07-01) [umoeller]: added dependency checks
 *@@changed V0.9.12 (2001-05-17) [umoeller]: now displaying detailed dependencies, allowing ignore
 */

LONG ConfirmDeinstall(HWND hwndOwner,                   // in: dialog owner
                      list<FEDBPackage*>* pPackagesList) // in: package list to deinstall.
{
    LONG   lrc = -1;

    // list of strings to be freed
    list<BSString*> Strings2FreeList;

    // first, load dependency violations dialog;
    // while we're checking the dependencies, we add
    // records to the violations container...

    // note, the dialog is only shown if any violations
    // were actually found!

    HWND hwndConfirm = WinLoadDlg(HWND_DESKTOP,
                                  G_hwndMain,
                                  WinDefDlgProc,
                                  NULLHANDLE,
                                  ID_WID_DEPENDENCIES,
                                  NULL);
    HWND hwndCnr = WinWindowFromID(hwndConfirm, ID_WIDI_CNR);

    winhSetControlsFont(hwndConfirm,
                        0,
                        1000,
                        "9.WarpSans");

    // set up cnr details view
    XFIELDINFO      xfi[10];
    memset(&xfi, 0, sizeof(xfi));

    int i = 0;

    xfi[i].ulFieldOffset = FIELDOFFSET(DEPENDENCYRECORD, pszDepApplication);
    xfi[i].pszColumnTitle = G_NLSStrings.pszApplication, // application
    xfi[i].ulDataType = CFA_STRING;
    xfi[i++].ulOrientation = CFA_LEFT;

    xfi[i].ulFieldOffset = FIELDOFFSET(DEPENDENCYRECORD, pszDepName);
    xfi[i].pszColumnTitle = G_NLSStrings.pszPackageName; // "Package name";
    xfi[i].ulDataType = CFA_STRING;
    xfi[i++].ulOrientation = CFA_LEFT;

    xfi[i].ulFieldOffset = FIELDOFFSET(DEPENDENCYRECORD, pszDepVersion);
    xfi[i].pszColumnTitle = G_NLSStrings.pszVersion; // "Version";
    xfi[i].ulDataType = CFA_STRING;
    xfi[i++].ulOrientation = CFA_RIGHT;

    xfi[i].ulFieldOffset = FIELDOFFSET(DEPENDENCYRECORD, pszDepAuthor);
    xfi[i].pszColumnTitle = G_NLSStrings.pszAuthor; // "Author";
    xfi[i].ulDataType = CFA_STRING;
    xfi[i++].ulOrientation = CFA_LEFT;

    xfi[i].ulFieldOffset = FIELDOFFSET(DEPENDENCYRECORD, pszReqApplication);
    xfi[i].pszColumnTitle = G_NLSStrings.pszApplication, // application
    xfi[i].ulDataType = CFA_STRING;
    xfi[i++].ulOrientation = CFA_LEFT;

    xfi[i].ulFieldOffset = FIELDOFFSET(DEPENDENCYRECORD, pszReqName);
    xfi[i].pszColumnTitle = G_NLSStrings.pszPackageName; // "Package name";
    xfi[i].ulDataType = CFA_STRING;
    xfi[i++].ulOrientation = CFA_LEFT;

    xfi[i].ulFieldOffset = FIELDOFFSET(DEPENDENCYRECORD, pszReqVersion);
    xfi[i].pszColumnTitle = G_NLSStrings.pszVersion; // "Version";
    xfi[i].ulDataType = CFA_STRING;
    xfi[i++].ulOrientation = CFA_RIGHT;

    xfi[i].ulFieldOffset = FIELDOFFSET(DEPENDENCYRECORD, pszReqAuthor);
    xfi[i].pszColumnTitle = G_NLSStrings.pszAuthor; // "Author";
    xfi[i].ulDataType = CFA_STRING;
    xfi[i++].ulOrientation = CFA_LEFT;

    PFIELDINFO pFieldInfoLast =
             cnrhSetFieldInfos(hwndCnr,
                               &xfi[0],
                               i,        // array item count
                               TRUE,     // draw lines
                               3);       // PFIELDINFO index to return

    BEGIN_CNRINFO()
    {
        SWP swpCnr;
        WinQueryWindowPos(hwndCnr, &swpCnr);
        // set split bar
        cnrhSetSplitBarAfter(pFieldInfoLast);
        cnrhSetSplitBarPos(swpCnr.cx / 2);
        // switch view
        cnrhSetView(CV_DETAIL | CA_DETAILSVIEWTITLES);
    } END_CNRINFO(hwndCnr);

    /*
     * check dependencies
     *
     */

    ULONG cRequiring = 0;
    list<FEDBPackage*>::iterator PckStart = pPackagesList->begin(),
                                 PckEnd = pPackagesList->end();

    string strDependencies;

    for (; PckStart != PckEnd; PckStart++)
    {
        FEDBPackage *pPackageThis = *PckStart;
        FEPackageID PckIDThis(pPackageThis, __FILE__, __LINE__,
                              FALSE);    // don't allow short format

        list<FEDBPackage*> RequiringList;

        ULONG cThis = G_pDatabase->IsPackageRequired(PckIDThis,
                                                     RequiringList,
                                                     // packages to be ignored:
                                                     pPackagesList);
        if (cThis)
        {
            // dependency violations:

            cRequiring += cThis;

            // create temporary strings
            BSString *pstrDepApplication = new BSString(PckIDThis._strApplication);
            Strings2FreeList.push_back(pstrDepApplication);
            BSString *pstrDepName = new BSString(PckIDThis._strPackageName);
            Strings2FreeList.push_back(pstrDepName);
            BSString *pstrDepAuthor = new BSString(PckIDThis._strAuthor);
            Strings2FreeList.push_back(pstrDepAuthor);

            // allocate records for this package

            PDEPENDENCYRECORD precFirst
                = (PDEPENDENCYRECORD)cnrhAllocRecords(hwndCnr,
                                                      sizeof(DEPENDENCYRECORD),
                                                      cThis);
            if (precFirst)
            {
                PDEPENDENCYRECORD precThis = precFirst;
                ULONG ul;

                list<FEDBPackage*>::iterator ReqStart = RequiringList.begin(),
                                             ReqEnd = RequiringList.end();
                for (;
                     ReqStart != ReqEnd;
                     ReqStart++)
                {
                    FEDBPackage *pRequires = *ReqStart;

                    precThis->pszDepApplication = (PSZ)pstrDepApplication->c_str();
                    precThis->pszDepName = (PSZ)pstrDepName->c_str();
                    sprintf(precThis->szDepVersion,
                            "%u.%u.%u",
                            PckIDThis._ulVersionMajor,
                            PckIDThis._ulVersionMinor,
                            PckIDThis._ulVersionRevision);
                    precThis->pszDepVersion = precThis->szDepVersion;
                    precThis->pszDepAuthor = (PSZ)pstrDepAuthor->c_str();

                    FEPackageID ReqID(pRequires, __FILE__, __LINE__,
                                      FALSE);    // don't allow short format

                    // create temporary strings
                    BSString *pstrReqApplication = new BSString(ReqID._strApplication);
                    Strings2FreeList.push_back(pstrReqApplication);
                    BSString *pstrReqName = new BSString(ReqID._strPackageName);
                    Strings2FreeList.push_back(pstrReqName);
                    BSString *pstrReqAuthor = new BSString(ReqID._strAuthor);
                    Strings2FreeList.push_back(pstrReqAuthor);

                    precThis->pszReqApplication = (PSZ)pstrReqApplication->c_str();
                    precThis->pszReqName = (PSZ)pstrReqName->c_str();
                    sprintf(precThis->szReqVersion,
                            "%u.%u.%u",
                            ReqID._ulVersionMajor,
                            ReqID._ulVersionMinor,
                            ReqID._ulVersionRevision);
                    precThis->pszReqVersion = precThis->szReqVersion;
                    precThis->pszReqAuthor = (PSZ)pstrReqAuthor->c_str();

                    // next record in allocated list
                    precThis = (PDEPENDENCYRECORD)precFirst->recc.preccNextRecord;
                }

                // now insert the records
                cnrhInsertRecords(hwndCnr,
                                  NULL,
                                  (PRECORDCORE)precFirst,
                                  TRUE,
                                  NULL,
                                  CRA_RECORDREADONLY,
                                  cThis);
            }
        }
    }

    if (cRequiring)
    {
        // any dependencies found:
        // ask user if these violations should be ignored


        if (WinProcessDlg(hwndConfirm) == DID_OK)
            // deinstall anyway:
            cRequiring = 0;
    }

    WinDestroyWindow(hwndConfirm);

    if (!cRequiring)
    {
        // no dependencies, or deinstall anyway:

        if (pPackagesList)
        {
            HWND hwndDlg = gshrLoadDlg(HWND_DESKTOP,
                                      hwndOwner,
                                      fnwpConfirmDeinstall,
                                      NULLHANDLE, ID_WID_CONFIRMDEINSTALL,
                                      NULL);
            winhCenterWindow(hwndDlg);
            gshrSetControlsFont(hwndDlg, 0, 2000);

            BOOL        fHasFiles = FALSE;
            ULONG       ulConfigItems = 0;
            HWND        hwndPckLbox = WinWindowFromID(hwndDlg, ID_WIDI_CFD_PACKAGESLIST),
                        hwndFilesLbox = WinWindowFromID(hwndDlg, ID_WIDI_CFD_FILESLIST),
                        hwndConfigCnr = WinWindowFromID(hwndDlg, ID_WIDI_CFD_CONFIGCNR);

            // subclass listboxes
            G_pfnwpListboxOrig = WinSubclassWindow(hwndPckLbox, fnwpListboxNoselect);
            G_pfnwpListboxOrig = WinSubclassWindow(hwndFilesLbox, fnwpListboxNoselect);

            SetCnrConfigFieldInfos(hwndConfigCnr);

            // go thru all packages to collect files and config data
            list<FEDBPackage*>::iterator PckStart = pPackagesList->begin(),
                                         PckEnd = pPackagesList->end();
            for (; PckStart != PckEnd; PckStart++)
            {
                FEDBPackage* ppi = *PckStart;
                WinInsertLboxItem(hwndPckLbox, LIT_END, ppi->QueryPackageID());

                // check files
                if (ppi->_pszFilesList)
                {
                    ULONG ulHeaderCount = 0;
                    list<WIFileHeader*>* pFileHeadersList = ppi->GetFileHeaders(
                                                                          &ulHeaderCount);
                    if (pFileHeadersList)
                    {
                        list<WIFileHeader*>::iterator HeaderFirst = pFileHeadersList->begin(),
                                                      HeaderLast = pFileHeadersList->end();
                        for (; HeaderFirst != HeaderLast; HeaderFirst++)
                        {
                            BSString strFullFile(ppi->QueryTargetPath());
                            if (strFullFile[strFullFile.length() - 1] != '\\')
                                strFullFile += "\\";
                            strFullFile += (**HeaderFirst).name;
                            WinInsertLboxItem(hwndFilesLbox, LIT_END, strFullFile.c_str());
                            delete *HeaderFirst;
                        }
                        delete pFileHeadersList;
                        fHasFiles = TRUE;
                    }
                }

                ulConfigItems += InsertConfigIntoCnr(hwndConfigCnr,
                                                     ppi,
                                                     Strings2FreeList);
                cnrhSelectAll(hwndConfigCnr, TRUE);
            }

            winhEnableDlgItem(hwndDlg, ID_WIDI_CFD_DELETEFILES, fHasFiles);
            winhSetDlgItemChecked(hwndDlg, ID_WIDI_CFD_DELETEFILES, fHasFiles);

            winhEnableDlgItem(hwndDlg, ID_WIDI_CFD_UNDOCONFIG, (ulConfigItems != 0));
            winhSetDlgItemChecked(hwndDlg, ID_WIDI_CFD_UNDOCONFIG, (ulConfigItems != 0));

            // go!!!
            if (WinProcessDlg(hwndDlg) == DID_OK)
            {
                lrc = 0;
                if (winhIsDlgItemChecked(hwndDlg, ID_WIDI_CFD_DELETEFILES))
                    lrc |= DBT_DELETEFILES;
                if (winhIsDlgItemChecked(hwndDlg, ID_WIDI_CFD_UNDOCONFIG))
                {
                    // undo config (any items selected in container):
                    lrc |= DBT_UNDOCONFIG;

                    // go thru all container records and select config items
                    // accordingly

                    PDBCONFIGRECORD pcrec = (PDBCONFIGRECORD)0;
                    USHORT fsCmd = CMA_FIRST;
                    do
                    {
                        pcrec = (PDBCONFIGRECORD)WinSendMsg(hwndConfigCnr,
                                                            CM_QUERYRECORD,
                                                            (MPARAM)pcrec,
                                                                    // null on first loop
                                                            MPFROM2SHORT(fsCmd,
                                                                         CMA_ITEMORDER));
                        fsCmd = CMA_NEXT;
                        if ((pcrec) && ((LONG)pcrec != -1))
                        {
                            // first or next record found:
                            // mark config item as selected or not selected
                            // according to recc selection
                            pcrec->pConfig->Select((pcrec->recc.flRecordAttr & CRA_SELECTED)
                                                       != 0);
                        }
                        else
                            break;
                    } while (TRUE);
                }
            }
            WinDestroyWindow(hwndDlg);
        }
    }

    FreeStringsList(Strings2FreeList);

    return (lrc);
}

/* ******************************************************************
 *
 *   "Packages" subwindow
 *
 ********************************************************************/

/*
 *@@ FreeDatabaseRecord:
 *      this removes preccRemove from the given container
 *      and frees all associated data. This is used during
 *      de-installation to really remove a record.
 */

VOID FreeDatabaseRecord(HWND hwndCnr,
                        PDATABASERECORD preccRemove)
{
    if (preccRemove->pPckInfo)
        // is package:
        G_PckRecordsList.remove(preccRemove);
    else
        // is application:
        G_AppRecordsList.remove(preccRemove);

    if (preccRemove->pszAuthor)
        free(preccRemove->pszAuthor);
    if (preccRemove->pszApplication)
        free(preccRemove->pszApplication);
    if (preccRemove->pszPackageName)
        free(preccRemove->pszPackageName);

    WinSendMsg(hwndCnr,
               CM_REMOVERECORD,
               (MPARAM)&preccRemove,
               MPFROM2SHORT(1,
                            CMA_FREE | CMA_INVALIDATE));

}

/*
 *@@ ClearDatabaseContainer:
 *      this removes all record cores from the given
 *      container. It is assumed that the container
 *      uses DATABASERECORDs for its record cores.
 *      The records are removed only, not freed.
 *
 *      This is used when the display changes, but the
 *      record cores will still be used.
 *
 *@@changed V0.9.0 (99-11-04) [umoeller]: fixed the age-old display bug when switching between views
 *@@changed V0.9.2 (2000-02-19) [umoeller]: sped up display
 */

VOID ClearDatabaseContainer(HWND hwndCnr,
                            BOOL fFreeAppsToo)
{
    list<PDATABASERECORD>::iterator RecStart = G_PckRecordsList.begin(),
                                    RecEnd = G_PckRecordsList.end();
    for (; RecStart != RecEnd; RecStart++)
    {
        PDATABASERECORD preccRemove = *RecStart;
        WinSendMsg(hwndCnr,
                   CM_REMOVERECORD,
                   (MPARAM)&preccRemove,
                   MPFROM2SHORT(1,
                                0)); // CMA_INVALIDATE));  // no CMA_FREE, we still need the record
    }

    if (fFreeAppsToo)
    {
        RecStart = G_AppRecordsList.begin();
        RecEnd = G_AppRecordsList.end();
        for (; RecStart != RecEnd; RecStart++)
        {
            PDATABASERECORD preccRemove = *RecStart;
            WinSendMsg(hwndCnr,
                       CM_REMOVERECORD,
                       (MPARAM)&preccRemove,
                       MPFROM2SHORT(1,
                                    0)); // CMA_INVALIDATE));  // no CMA_FREE, we still need the record
        }
    }

    cnrhInvalidateAll(hwndCnr);
}

#define WM_FILLCNR      WM_USER

/*
 *@@ FillDatabaseContainer:
 *      this fills the container with all applications/packages.
 *
 *      Depending on G_GuiSettings.ulLeftView, this is either
 *      a Tree view or a Details view.
 *
 *      This gets called both after the main window has been
 *      created and if the database view changed.
 *
 *@@changed V0.9.2 (2000-02-19) [umoeller]: sped up display
 *@@changed V0.9.9 (2001-02-28) [umoeller]: apps tree was missing items if same app had differing authors
 */

VOID FillDatabaseContainer(HWND hwndCnr,        // in: cnr to fill
                           ULONG ulNewView)
{
    ClearDatabaseContainer(hwndCnr,
                           // free apps too?
                           G_GuiSettings.ulLeftView == ID_WIMI_DATABASE_TREEVIEW);

    G_GuiSettings.ulLeftView = ulNewView;

    /*
     * ID_WIMI_DATABASE_TREEVIEW:
     *
     */

    if (G_GuiSettings.ulLeftView == ID_WIMI_DATABASE_TREEVIEW)
    {
        // CnrInfo.cxTreeIndent = 15;

        // switch cnr to Details view
        BEGIN_CNRINFO()
        {
            // set bitmap size
            cnrhSetBmpOrIconSize(16, 16);
            // switch view
            cnrhSetView(CV_TREE | CV_ICON | CA_TREELINE | CV_MINI);
                                // | CA_DRAWBITMAP);
        } END_CNRINFO(hwndCnr);

        // insert all application record cores to root level
        list<PDATABASERECORD>::iterator AppsStart, AppsEnd;
        AppsStart = G_AppRecordsList.begin();
        AppsEnd = G_AppRecordsList.end();

        for (; (AppsStart != AppsEnd); AppsStart++)
        {
            PDATABASERECORD precc = (*AppsStart);
            cnrhInsertRecords(hwndCnr,
                              (PRECORDCORE)NULL,       // root level
                              (PRECORDCORE)precc,
                              FALSE,
                              precc->pszApplication,
                              CRA_RECORDREADONLY | CRA_COLLAPSED,
                              1);
        }

        // insert all packages as children of the applications
        AppsStart = G_AppRecordsList.begin();
        AppsEnd = G_AppRecordsList.end();

        for (; (AppsStart != AppsEnd); AppsStart++)
        {
            PDATABASERECORD preccApplication = (*AppsStart);

            // AppsStart now has the current application recc;
            // go thru the package recc list and add all package
            // records which have this application

            list<PDATABASERECORD>::iterator ReccStart = G_PckRecordsList.begin(),
                                            ReccEnd = G_PckRecordsList.end();

            for (; (ReccStart != ReccEnd); ReccStart++)
            {

                PDATABASERECORD preccPck = (*ReccStart);

                if (    /* (strcmp(preccPck->pszAuthor, preccApplication->pszAuthor) == 0)
                     && */  // fixed V0.9.9 (2001-02-28) [umoeller]
                     (strcmp(preccPck->pszApplication, preccApplication->pszApplication) == 0)
                   )
                {
                    cnrhInsertRecords(hwndCnr,
                                      (PRECORDCORE)preccApplication,   // parent
                                      (PRECORDCORE)preccPck,
                                      FALSE,
                                      preccPck->pszPackageName,
                                      CRA_RECORDREADONLY | CRA_COLLAPSED,
                                      1);
                }
            }
        }
    }

    /*
     * ID_WIMI_DATABASE_DETAILSVIEW:
     *
     */

    else if (G_GuiSettings.ulLeftView == ID_WIMI_DATABASE_DETAILSVIEW)
    {
        // switch cnr to Details view
        BEGIN_CNRINFO()
        {
            // set bitmap size
            cnrhSetBmpOrIconSize(16, 16);
            // switch view
            cnrhSetView(CV_DETAIL | CA_DETAILSVIEWTITLES);
        } END_CNRINFO(hwndCnr);

        // the container already has the details fields we need,
        // so just insert the records now
        list<PDATABASERECORD>::iterator ReccStart = G_PckRecordsList.begin(),
                                        ReccEnd = G_PckRecordsList.end();

        for (; (ReccStart != ReccEnd); ReccStart++)
        {

            PDATABASERECORD preccPck = (*ReccStart);
            cnrhInsertRecords(hwndCnr,
                              (PRECORDCORE)NULL,       // no parent for Details view
                              (PRECORDCORE)preccPck,
                              FALSE,
                              preccPck->pszPackageName,
                              CRA_RECORDREADONLY | CRA_COLLAPSED,
                              1);
        }
    }

    cnrhInvalidateAll(hwndCnr);
}

/*
 *@@ DBPackagesEmphasis:
 *      implementation for WM_CONTROL and CN_EMPHASIS
 *      in fnwpDBPackagesDlg.
 *
 *@@added V0.9.12 (2001-05-17) [umoeller]
 */

VOID DBPackagesEmphasis(HWND hwndDlg,
                        PNOTIFYRECORDEMPHASIS pnre, // in: mp2 of WM_CONTROL
                        BOOL fClearingContainers)
{
    PDATABASERECORD precc;

    if (    (pnre)
         && (pnre->fEmphasisMask & CRA_SELECTED)
         && (precc = (PDATABASERECORD)pnre->pRecord)
       )
    {
        CHAR    szVersion[40] = "";
        BSString strInstallDateTime;
        const char *pcszTargetDir = 0;
        CHAR    szFiles[200] = "";

        // set the following for both apps and pcks
        WinSetDlgItemText(hwndDlg, ID_WIDI_DB_AUTHOR,
                          precc->pszAuthor);

        // set the following for pcks only
        if (precc->pPckInfo)
        {
            sprintf(szVersion, "%d.%d.%d",
                    precc->ulVersionMajor,
                    precc->ulVersionMinor,
                    precc->ulVersionRevision);
            pcszTargetDir = precc->pPckInfo->QueryTargetPath();
            sprintf(szFiles, "%d",
                    precc->pPckInfo->_PackHeader.files);

            CHAR szDate[50], szTime[50];
            strhDateTime(szDate, szTime,
                         &precc->pPckInfo->_InstallDateTime,
                         G_ulDateFormat,
                         G_szDateSep[0],
                         G_ulTimeFormat,
                         G_szTimeSep[0]);
            strInstallDateTime = szDate;
            strInstallDateTime += " ";
            strInstallDateTime += szTime;
        }

        WinSetDlgItemText(hwndDlg, ID_WIDI_DB_VERSION,
                          szVersion);
        WinSetDlgItemText(hwndDlg, ID_WIDI_DB_INSTALLDATE,
                          strInstallDateTime.c_str());
        WinSetDlgItemText(hwndDlg, ID_WIDI_DB_TARGETDIR,
                          pcszTargetDir);
        WinSetDlgItemText(hwndDlg, ID_WIDI_DB_FILES,
                          szFiles);

        if (!fClearingContainers) // V0.9.9 (2001-02-28) [umoeller]
        {
            HWND hwndCnr = WinWindowFromID(hwndDlg, ID_WIDI_CNR);

            // count the no. of selected records in
            // the left cnr
            PDATABASERECORD precSel;

            if (    (precSel = (PDATABASERECORD)WinSendMsg(hwndCnr,
                                                           CM_QUERYRECORDEMPHASIS,
                                                           (MPARAM)CMA_FIRST,
                                                           (MPARAM)CRA_SELECTED))
                 && ((ULONG)precSel != -1)
                    // get next record to check if we have more
                    // than one selected
                 && (NULL == WinSendMsg(hwndCnr,
                                        CM_QUERYRECORDEMPHASIS,
                                        (MPARAM)precSel,
                                        (MPARAM)CRA_SELECTED))
               )
            {
                // ok, exactly one record selected:
                if (G_preccDBSelected != precSel)
                {
                    // store record
                    G_preccDBSelected = precSel;
                    UpdateListCnr(precSel);
                }
            }
            else
            {
                // zero or multiple:
                // clear right container
                G_preccDBSelected = NULL;
                UpdateListCnr(NULL);
            }
        }
    }
}

/*
 *@@ DBVerifyOrDeinstall:
 *      implementation for WM_COMMAND in fnwpDBPackagesDlg
 *      with the verify or deinstall apps or pcks commands.
 *
 *@@added V0.9.12 (2001-05-17) [umoeller]
 */

VOID DBVerifyOrDeinstall(ULONG ulCmd)
{
    // list of packages to remove or verify
    list<FEDBPackage*> PckInfoList;

    // build list
    switch (ulCmd)
    {
        // work on application:
        case ID_WIMI_DATABASE_VERIFYAPP:
        case ID_WIMI_DATABASE_DEINSTALLAPP:
        {
            // find package infos which match author
            // and application by building a comparison
            // string

            BSString str = G_preccLeftSource->pszApplication;
                    // we are now only comparing the app name...
                    // using the author only didn't work

            list<FEDBPackage*>::iterator PckStart = G_pDatabase->DBPackagesList.begin(),
                                         PckEnd = G_pDatabase->DBPackagesList.end();
            for (; PckStart != PckEnd; PckStart++)
            {
                FEDBPackage* ppiThis = *PckStart;
                FEPackageID PckID(ppiThis->QueryPackageID(),
                                  __FILE__,
                                  __LINE__,
                                  TRUE);        // allow short format
                if (PckID._strApplication == str)
                    // matches: store this package in the list
                    PckInfoList.push_back(ppiThis);
            }
        break; }

        // work on packages:
        case ID_WIMI_DATABASE_DEINSTALLPCK:
        case ID_WIMI_DATABASE_VERIFYPCK:
        {
            // V0.9.12 (2001-05-17) [umoeller]: go thru all
            // selected records
            HWND hwndCnr = WinWindowFromID(G_hwndDBPackagesDlg, ID_WIDI_CNR);
            ULONG ulSelection = 0;
            PDATABASERECORD precSel
                = (PDATABASERECORD)cnrhQuerySourceRecord(hwndCnr,
                                                         (PRECORDCORE)G_preccLeftSource,
                                                         &ulSelection);
            if (precSel)
            {
                PckInfoList.push_back(precSel->pPckInfo);

                if (ulSelection == SEL_MULTISEL)
                {
                    while (precSel = (PDATABASERECORD)cnrhQueryNextSelectedRecord(hwndCnr,
                                                                     (PRECORDCORE)precSel))
                        PckInfoList.push_back(precSel->pPckInfo);
                }
            }
        }
    } // end switch (ulCmd)

    // confirmations/setup

    switch (ulCmd)
    {
        // work on application:
        case ID_WIMI_DATABASE_VERIFYAPP:
        case ID_WIMI_DATABASE_VERIFYPCK:
            guiStartVerify(&PckInfoList,
                           FALSE,      // non-modal
                           NULL);      // do not log verify
        break;

        case ID_WIMI_DATABASE_DEINSTALLAPP:
        case ID_WIMI_DATABASE_DEINSTALLPCK:
        {
            LONG lrc = ConfirmDeinstall(G_hwndMain, &PckInfoList);
            if (lrc >= 0)
            {
                BSFileLogger *pLogFile = NULL;

                if (G_WpiGlobals.szLogFilename[0])
                    // logging enabled:
                    pLogFile = new BSFileLogger(G_WpiGlobals.szLogFilename);

                guiStartRemove(&PckInfoList,
                               lrc,
                               FALSE,      // non-modal
                               pLogFile);
            }
        break; }
    }

    // since we have a block here, the
    // list will be destroyed automatically now
}

/*
 *@@ fnwpDBPackagesDlg:
 *      window procedure for the left "Packages" subdialog.
 *
 *      This one processes all the commands from the
 *      context menus (deinstall, verify etc.).
 *
 *@@changed V0.9.0 (99-10-24) [umoeller]: reworked completely for split view
 *@@changed V0.9.0 (99-11-02) [umoeller]: reworked database thread callbacks
 *@@changed V0.9.2 (2000-02-19) [umoeller]: added "View" menu support
 *@@changed V0.9.2 (2000-03-10) [umoeller]: added keyboard focus support
 *@@changed V0.9.4 (2000-06-30) [umoeller]: right split view was empty initially
 *@@changed V0.9.5 (2000-08-10) [umoeller]: F1 displays help now
 *@@changed V0.9.9 (2001-02-28) [umoeller]: now suppressing flicker on cnr view change
 *@@changed V0.9.12 (2001-05-17) [umoeller]: extracted DBPackagesEmphasis and DBVerifyOrDeinstall for support of multiple selections
 */

MRESULT EXPENTRY fnwpDBPackagesDlg(HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    MRESULT mrc = 0;

    static fClearingContainers = FALSE;
                // while this is TRUE, we do not update the files container
                // V0.9.9 (2001-02-28) [umoeller]

    switch (msg)
    {
        /*
         * WM_INITDLG:
         *
         */

        case WM_INITDLG:
        {
            mrc = WinDefDlgProc(hwndDlg, msg, mp1, mp2);

            // set font for all the dialog controls
            gshrSetControlsFont(hwndDlg, -2, 2000);

            // prepare winhAdjustControls
            memset(&G_xacPackagesDlg, 0, sizeof(XADJUSTCTRLS));
            winhAdjustControls(hwndDlg,             // dialog
                               G_ampPackagesDlg,    // MPARAMs array
                               sizeof(G_ampPackagesDlg) / sizeof(MPARAM), // items count
                               NULL,                // init
                               &G_xacPackagesDlg);  // storage area

            // set up the container for Details view
            HWND hwndDatabaseCnr = WinWindowFromID(hwndDlg, ID_WIDI_CNR);

            // 1) set up data for Details view columns
            XFIELDINFO      xfi[10];
            memset(&xfi, 0, sizeof(xfi));

            int i = 0;

            xfi[i].ulFieldOffset = FIELDOFFSET(RECORDCORE, hptrIcon);
            xfi[i].pszColumnTitle = "";
            xfi[i].ulDataType = CFA_BITMAPORICON;       // it's an icon, really
            xfi[i++].ulOrientation = CFA_LEFT;

            xfi[i].ulFieldOffset = FIELDOFFSET(DATABASERECORD, pszApplication);
            xfi[i].pszColumnTitle = G_NLSStrings.pszApplication, // application
            xfi[i].ulDataType = CFA_STRING;
            xfi[i++].ulOrientation = CFA_LEFT;

            xfi[i].ulFieldOffset = FIELDOFFSET(RECORDCORE, pszIcon);
            xfi[i].pszColumnTitle = G_NLSStrings.pszPackageName; // "Package name";
            xfi[i].ulDataType = CFA_STRING;
            xfi[i++].ulOrientation = CFA_LEFT;

            xfi[i].ulFieldOffset = FIELDOFFSET(DATABASERECORD, pszVersion);
            xfi[i].pszColumnTitle = G_NLSStrings.pszVersion; // "Version";
            xfi[i].ulDataType = CFA_STRING;
            xfi[i++].ulOrientation = CFA_RIGHT;

            xfi[i].ulFieldOffset = FIELDOFFSET(DATABASERECORD, pszAuthor);
            xfi[i].pszColumnTitle = G_NLSStrings.pszAuthor; // "Author";
            xfi[i].ulDataType = CFA_STRING;
            xfi[i++].ulOrientation = CFA_LEFT;

            xfi[i].ulFieldOffset = FIELDOFFSET(DATABASERECORD, pszTargetPath);
            xfi[i].pszColumnTitle = G_NLSStrings.pszTargetPath; // "Target directory";
            xfi[i].ulDataType = CFA_STRING;
            xfi[i++].ulOrientation = CFA_LEFT;

            xfi[i].ulFieldOffset = FIELDOFFSET(DATABASERECORD, ulFiles);
            xfi[i].pszColumnTitle = G_NLSStrings.pszFiles; // "Files";
            xfi[i].ulDataType = CFA_ULONG;
            xfi[i++].ulOrientation = CFA_RIGHT;

            xfi[i].ulFieldOffset = FIELDOFFSET(DATABASERECORD, ulTotalSize);
            xfi[i].pszColumnTitle = G_NLSStrings.pszSize; // "Size";
            xfi[i].ulDataType = CFA_ULONG;
            xfi[i++].ulOrientation = CFA_RIGHT;

            xfi[i].ulFieldOffset = FIELDOFFSET(DATABASERECORD, cdateInstall);
            xfi[i].pszColumnTitle = G_NLSStrings.pszInstallDate; // "Install date";
            xfi[i].ulDataType = CFA_DATE;
            xfi[i++].ulOrientation = CFA_RIGHT;

            xfi[i].ulFieldOffset = FIELDOFFSET(DATABASERECORD, ctimeInstall);
            xfi[i].pszColumnTitle = G_NLSStrings.pszInstallTime; // "Install time";
            xfi[i].ulDataType = CFA_TIME;
            xfi[i++].ulOrientation = CFA_RIGHT;

            PFIELDINFO pFieldInfoLast =
                     cnrhSetFieldInfos(hwndDatabaseCnr,
                                       &xfi[0],
                                       i,        // array item count
                                       TRUE,     // draw lines
                                       3);       // PFIELDINFO index to return

            // set split bar
            BEGIN_CNRINFO()
            {
                cnrhSetSplitBarAfter(pFieldInfoLast);
                cnrhSetSplitBarPos(200);
            } END_CNRINFO(hwndDatabaseCnr);

            WinPostMsg(hwndDlg,
                       WM_FILLCNR,
                       0, 0);
        break; }

        /*
         * WM_FILLCNR:
         *      posted by WM_INITDLG to fill the left container.
         *      This is delayed because at the point WM_INITDLG
         *      comes in, we have no message loop yet and updating
         *      the right split view doesn't work.
         *      V0.9.4 (2000-06-30) [umoeller]
         */

        case WM_FILLCNR:
        {
            HWND hwndDatabaseCnr = WinWindowFromID(hwndDlg, ID_WIDI_CNR);

            // create record cores for all applications
            list<FEDBPackage*>::iterator PckStart = G_pDatabase->DBPackagesList.begin(),
                                         PckEnd = G_pDatabase->DBPackagesList.end();

            for (; (PckStart != PckEnd); PckStart++)
            {
                FEDBPackage *pDBPackage = *PckStart;
                // create package ID on the stack
                FEPackageID PckID(pDBPackage, __FILE__, __LINE__,
                                  FALSE);   // don't allow short format

                // check if this application exists in the list already
                BOOL fFound = FALSE;
                list<PDATABASERECORD>::iterator AppsStart = G_AppRecordsList.begin(),
                                                AppsEnd = G_AppRecordsList.end();
                for (; (AppsStart != AppsEnd); AppsStart++)
                {
                    PDATABASERECORD pAppThis = *AppsStart;
                    if (PckID._strApplication == pAppThis->pszApplication)
                    {
                        fFound = TRUE;
                        break;
                    }
                }

                if (!fFound)
                {
                    // not found:
                    // create application record core
                    PDATABASERECORD preccApplication = CreateDBAppRecord(hwndDatabaseCnr,
                                                                         PckID);

                    // store in global list
                    G_AppRecordsList.push_back(preccApplication);

                } // end if (!fFound)
            } // for (; (PckStart != PckEnd); PckStart++)

            // 3) create record cores for all packages
            PckStart = G_pDatabase->DBPackagesList.begin();
            PckEnd = G_pDatabase->DBPackagesList.end();
            for (; (PckStart != PckEnd); PckStart++)
            {
                PDATABASERECORD preccPck = CreateDBPckRecord(hwndDatabaseCnr, *PckStart);
                // store in global list
                G_PckRecordsList.push_back(preccPck);
            } // for (; (PckStart != PckEnd); PckStart++)

            FillDatabaseContainer(hwndDatabaseCnr,
                                  G_GuiSettings.ulLeftView);
        break; }

        /*
         * WM_CONTROL:
         *
         */

        case WM_CONTROL:
        {
            USHORT   usID = SHORT1FROMMP(mp1);
            USHORT   usNotifyCode = SHORT2FROMMP(mp1);
            switch (usID)
            {
                case ID_WIDI_CNR:
                    switch (usNotifyCode)
                    {
                        /*
                         * CN_EMPHASIS:
                         *      selection changed: update display
                         */

                        case CN_EMPHASIS:
                            DBPackagesEmphasis(hwndDlg,
                                               (PNOTIFYRECORDEMPHASIS)mp2,
                                               fClearingContainers);
                        break;

                        /*
                         * CN_CONTEXTMENU:
                         *      context menu requested for item;
                         *      mp2 has record core or NULL for
                         *      whitespace
                         */

                        case CN_CONTEXTMENU:
                        {
                            HWND    hPopupMenu = NULLHANDLE;

                            G_preccLeftSource = (PDATABASERECORD)mp2;
                            if (G_preccLeftSource)
                            {
                                // on context record:
                                if (G_preccLeftSource->pPckInfo)
                                    // on package:
                                    hPopupMenu = G_hmenuPckPackage;
                                else
                                    // on application:
                                    hPopupMenu = G_hmenuPckApplication;
                            }
                            else
                            {
                                // cnr whitespace:
                                hPopupMenu = G_hmenuPckWhitespace;
                                CheckLeftViewMenuItems(hPopupMenu);
                            }

                            cnrhShowContextMenu(WinWindowFromID(hwndDlg, usID),
                                                (PRECORDCORE)G_preccLeftSource,
                                                hPopupMenu,
                                                hwndDlg);
                        break; }

                        case CN_HELP:
                            gshrDisplayHelp(IDHI_DATABASE);
                        break;

                    } // end switch (usNotifyCode)

                break;      // case ID_WIDI_CNR
            }
        break; }

        /*
         * WM_MENUEND:
         *      if the context menu is dismissed, we'll need
         *      to remove the cnr source emphasis which was
         *      set above when showing the context menu.
         */

        case WM_MENUEND:
            cnrhSetSourceEmphasis(WinWindowFromID(hwndDlg, ID_WIDI_CNR),
                                  G_preccLeftSource,
                                  FALSE);
        break;

        /*
         * WM_CHAR:
         *      switch focus on "Tab" key.
         */

        case WM_CHAR:
        {
            ProcessWMChar(mp1, mp2);
            mrc = (MPARAM)TRUE;
        break; }

        /*
         * WM_COMMAND:
         *
         */

        case WM_COMMAND:
        {
            ULONG       ulCmd = (ULONG)mp1;

            switch (ulCmd)
            {
                /*
                 * ID_WIMI_DATABASE_VERIFYAPP etc.:
                 *      open "Status" window, which
                 *      starts the Database thread
                 */

                case ID_WIMI_DATABASE_VERIFYAPP:
                case ID_WIMI_DATABASE_VERIFYPCK:
                case ID_WIMI_DATABASE_DEINSTALLAPP:
                case ID_WIMI_DATABASE_DEINSTALLPCK:
                    DBVerifyOrDeinstall(ulCmd);
                break;

                case ID_WIMI_DATABASE_REMOVEPCK:
                    if (wpiMessageBox(108,       // "Warning"
                                      122,
                                      MB_ICONQUESTION | MB_YESNO | MB_MOVEABLE,
                                      &(G_preccLeftSource->recc.pszIcon), 1)
                                == MBID_YES)
                        G_pDatabase->RemovePackage(G_preccLeftSource->pPckInfo);
                            // this will call guiDatabaseCallback
                break;

                /*
                 * ID_WIMI_DATABASE_TREEVIEW:
                 * ID_WIMI_DATABASE_DETAILSVIEW:
                 *      switch container view
                 */

                case ID_WIMI_DATABASE_TREEVIEW:
                case ID_WIMI_DATABASE_DETAILSVIEW:
                    if (G_GuiSettings.ulLeftView != ulCmd)
                    {
                        // view changed:

                        // set the static flag in this proc to prevent flicker
                        // in the right half V0.9.9 (2001-02-28) [umoeller]
                        fClearingContainers = TRUE;

                        FillDatabaseContainer(WinWindowFromID(hwndDlg, ID_WIDI_CNR),
                                              ulCmd);
                        gshrSaveSettings(0);
                        CheckLeftViewMenuItems(G_hmenuMain);

                        fClearingContainers = FALSE;

                        // now refresh the right
                        UpdateListCnr(G_preccDBSelected);
                    }
                break;

                case ID_WIDI_CANCELBUTTON:
                    WinDismissDlg(hwndDlg, ulCmd);
                break;
            }
        break; }

        /*
         * WPIM_PACKAGEREMOVED:
         *      this gets posted by guiDatabaseCallback.
         *      mp1 has the FEPackage* which has been
         *      removed by the Database thread.
         */

        case WPIM_PACKAGEREMOVED:
        {
            FEDBPackage* pPckInfoRemove = (FEDBPackage*)mp1;
            if (pPckInfoRemove)
            {
                // get the record core from the package info
                PDATABASERECORD preccRemove = (PDATABASERECORD)pPckInfoRemove->_pvGuiData;
                if (preccRemove)
                {
                    HWND hwndDatabaseCnr = WinWindowFromID(hwndDlg, ID_WIDI_CNR);
                    PDATABASERECORD preccApplication =
                                (PDATABASERECORD)cnrhQueryParentRecord(hwndDatabaseCnr,
                                                                       preccRemove);

                    FreeDatabaseRecord(hwndDatabaseCnr, preccRemove);

                    // now check if the application record has
                    // any more children (i.e. any more packages)
                    if (preccApplication)
                        if (cnrhQueryFirstChildRecord(hwndDatabaseCnr, preccApplication)
                                        == NULL)
                            FreeDatabaseRecord(hwndDatabaseCnr, preccApplication);
                }
            }
        break; }

        /*
         * WM_WINDOWPOSCHANGED:
         *
         */

        case WM_WINDOWPOSCHANGED:
        {
            // this msg is passed two SWP structs:
            // one for the old, one for the new data
            // (from PM docs)
            PSWP pswpNew = (PSWP)(mp1);

            // resizing?
            if (pswpNew->fl & SWP_SIZE)
            {
                winhAdjustControls(hwndDlg,             // dialog
                                   G_ampPackagesDlg,    // MPARAMs array
                                   sizeof(G_ampPackagesDlg) / sizeof(MPARAM), // items count
                                   pswpNew,             // mp1
                                   &G_xacPackagesDlg);  // storage area
            }
            mrc = WinDefDlgProc(hwndDlg, msg, mp1, mp2);
        break; }

        /*
         * WM_SYSCOMMAND:
         *      pass on to frame
         */

        case WM_SYSCOMMAND:
        case WM_HELP:
            WinPostMsg(WinQueryWindow(hwndDlg, QW_OWNER),
                       msg, mp1, mp2);
        break;

        case WM_DESTROY:
            winhAdjustControls(hwndDlg,             // dialog
                               NULL,
                               NULL,
                               NULL,
                               &G_xacPackagesDlg);  // storage area
            mrc = WinDefDlgProc(hwndDlg, msg, mp1, mp2);
        break;

        default:
            mrc = WinDefDlgProc(hwndDlg, msg, mp1, mp2);
    }
    return (mrc);
}

/* ******************************************************************
 *
 *   "File list" subwindow
 *
 ********************************************************************/

/*
 *@@ fnwpDBFileListDlg:
 *      window procedure for the "File list" subdialog
 *      in the database view (on the right).
 *
 *@@changed V0.9.0 (99-10-24) [umoeller]: reworked completely for split view
 *@@changed V0.9.2 (2000-02-19) [umoeller]: added right view support
 *@@changed V0.9.2 (2000-03-10) [umoeller]: added keyboard focus support
 *@@changed V0.9.5 (2000-08-10) [umoeller]: added context menu for right view
 *@@changed V0.9.5 (2000-08-10) [umoeller]: F1 displays help now
 */

MRESULT EXPENTRY fnwpDBFileListDlg(HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    MRESULT mrc = 0;

    switch (msg)
    {
        /*
         * WM_INITDLG:
         *
         */

        case WM_INITDLG:
        {
            mrc = WinDefDlgProc(hwndDlg, msg, mp1, mp2);

            // make the thing properly sizeable (99-10-24) [umoeller]
            memset(&G_xacFileListDlg, 0, sizeof(XADJUSTCTRLS));
            winhAdjustControls(hwndDlg,
                               G_ampFileListDlg,         // global struct
                               sizeof(G_ampFileListDlg) / sizeof(MPARAM),
                               NULL,                // init
                               &G_xacFileListDlg);

            // set font
            gshrSetControlsFont(hwndDlg, -2, 2000);
        break; }

        case WM_WINDOWPOSCHANGED:   // added (99-10-24) [umoeller]
        {
            // this msg is passed two SWP structs:
            // one for the old, one for the new data
            // (from PM docs)
            PSWP pswpNew = (PSWP)mp1;

            // resizing?
            if (pswpNew->fl & SWP_SIZE)
            {
                winhAdjustControls(hwndDlg,             // dialog
                                   G_ampFileListDlg,     // MPARAMs array
                                   sizeof(G_ampFileListDlg) / sizeof(MPARAM), // items count
                                   pswpNew,             // mp1
                                   &G_xacFileListDlg);               // storage area
            }
            mrc = WinDefDlgProc(hwndDlg, msg, mp1, mp2);
        break; }

        /*
         * WM_CHAR:
         *      switch focus on "Tab" key.
         */

        case WM_CHAR:
        {
            ProcessWMChar(mp1, mp2);
            mrc = (MPARAM)TRUE;
        break; }

        /*
         * WM_SYSCOMMAND:
         *      pass on to frame
         */

        case WM_SYSCOMMAND:
        case WM_HELP:
            WinPostMsg(WinQueryWindow(hwndDlg, QW_OWNER),
                       msg, mp1, mp2);
        break;

        /*
         * WM_CONTROL:
         *
         */

        case WM_CONTROL:
        {
            USHORT   usID = SHORT1FROMMP(mp1);
            USHORT   usNotifyCode = SHORT2FROMMP(mp1);
            switch (usID)
            {
                case ID_WIDI_CNR:
                    switch (usNotifyCode)
                    {
                        /*
                         * CN_CONTEXTMENU:
                         *      context menu requested for item;
                         *      mp2 has record core or NULL for
                         *      whitespace
                         */

                        case CN_CONTEXTMENU:
                        {
                            HWND    hPopupMenu = NULLHANDLE;
                            G_preccRightSource = (PFILERECORDCORE)mp2;
                            if (G_preccRightSource)
                            {
                                /* // on context record:
                                if (G_preccDBSource->pPckInfo)
                                    // on package:
                                    hPopupMenu = G_hmenuPckPackage;
                                else
                                    // on application:
                                    hPopupMenu = G_hmenuPckApplication;
                                */
                            }
                            else
                            {
                                // cnr whitespace:
                                hPopupMenu = G_hmenuFilesWhitespace;
                                CheckRightViewMenuItems(hPopupMenu);
                            }

                            if (hPopupMenu)
                                cnrhShowContextMenu(WinWindowFromID(hwndDlg, usID),
                                                    (PRECORDCORE)G_preccRightSource,
                                                    hPopupMenu,
                                                    hwndDlg);
                        break; }

                        case CN_HELP:
                            gshrDisplayHelp(IDHI_DATABASE);
                        break;

                    } // end switch (usNotifyCode)

                break;      // case ID_WIDI_CNR
            }
        break; }

        /*
         * WM_MENUEND:
         *      if the context menu is dismissed, we'll need
         *      to remove the cnr source emphasis which was
         *      set above when showing the context menu.
         */

        case WM_MENUEND:
            cnrhSetSourceEmphasis(WinWindowFromID(hwndDlg, ID_WIDI_CNR),
                                  G_preccRightSource,
                                  FALSE);
        break;

        case WM_COMMAND:
        {
            USHORT  usCmd = SHORT1FROMMP(mp1);
            switch (usCmd)
            {
                case ID_WIMI_VIEW_FILES:
                case ID_WIMI_VIEW_CONFIG:
                    if (G_GuiSettings.ulRightView != usCmd)
                    {
                        G_GuiSettings.ulRightView = usCmd;
                        // view changed:
                        /* FillDatabaseContainer(WinWindowFromID(hwndDlg, ID_WIDI_CNR),
                                              usCmd); */
                        UpdateListCnr(G_preccDBSelected);
                        gshrSaveSettings(0);
                        CheckRightViewMenuItems(G_hmenuMain);
                    }
            }
        break; }

        case WM_DESTROY:
        {
            // cleanup adjust-controls data
            winhAdjustControls(hwndDlg,
                               NULL,
                               0,
                               NULL,
                               &G_xacFileListDlg);

            mrc = WinDefDlgProc(hwndDlg, msg, mp1, mp2);
        break; }

        default:
            mrc = WinDefDlgProc(hwndDlg, msg, mp1, mp2);
    }

    return (mrc);
}

/*
 *@@ UpdateListCnr:
 *      this gets called from fnwpDBPackagesDlg when
 *      a new application/package has been selected
 *      in the packages list subdialog. This then
 *      updates the file list.
 *
 *      Note that precc must be a package record core
 *      from the packages  container (not an application
 *      record core, because applications have no
 *      file lists).
 *
 *@@changed V0.9.0 (99-10-24) [umoeller]: reworked completely for split view
 *@@changed V0.9.0 (99-10-24) [umoeller]: changed name from OpenFileList
 *@@changed V0.9.0 (99-11-04) [umoeller]: fixed baaaad memory leak
 *@@changed V0.9.1 (2000-01-06) [umoeller]: now displaying target path on top of file list
 *@@changed V0.9.2 (2000-02-19) [umoeller]: reworked, can now display config too
 *@@changed V0.9.12 (2001-05-17) [umoeller]: added NULL check
 */

VOID UpdateListCnr(PDATABASERECORD precc)    // in: package record core in left view
{
    HWND    hwndFilesCnr = WinWindowFromID(G_hwndDBFileListDlg, ID_WIDI_CNR);

    // clear container
    cnrhRemoveAll(hwndFilesCnr);

    FreeStringsList(G_ConfigStrings);

    PCSZ pcszTargetPath = "";

    if (precc)      // V0.9.12 (2001-05-17) [umoeller]
    {
        // is precc a package record core?
        if (precc->pPckInfo)
        {
            // set up info text (99-10-24) [umoeller]
            pcszTargetPath = precc->pPckInfo->QueryTargetPath();

            if (G_GuiSettings.ulRightView == ID_WIMI_VIEW_FILES)
                InsertFilesIntoCnr(precc,
                                   hwndFilesCnr);
            else
            {
                SetCnrConfigFieldInfos(hwndFilesCnr);
                InsertConfigIntoCnr(hwndFilesCnr,
                                    precc->pPckInfo,
                                    G_ConfigStrings);
            }
        } // end if (precc->pPckInfo)
    }

    WinSetWindowText(WinWindowFromID(G_hwndDBFileListDlg,
                                     ID_WIDI_TARGETPATHDIR),
                     pcszTargetPath);

}

/* ******************************************************************
 *
 *   main window
 *
 ********************************************************************/

/*
 *@@ fnwpDBMainClient:
 *      window proc for the client of the database window.
 *      This initializes all the subwindows in turn and
 *      manages the split windows upon resizes etc.
 *
 *@@added V0.9.0 (99-10-24) [umoeller]
 *@@changed V0.9.2 (2000-02-19) [umoeller]: added "view" menu support
 *@@changed V0.9.2 (2000-02-19) [umoeller]: "Exit" didn't work; fixed
 */

MRESULT EXPENTRY fnwpDBMainClient(HWND hwndClient, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    MRESULT mrc = 0;

    switch (msg)
    {
        case WM_CREATE:
        {
            /*
             * left window: packages dialog
             *
             */

            G_hwndDBPackagesDlg = gshrLoadDlg(hwndClient, hwndClient, // parent and owner
                                             fnwpDBPackagesDlg,
                                             NULLHANDLE,      // our own EXE
                                             ID_WID_DATABASE,
                                             NULL);
            WinSetWindowBits(G_hwndDBPackagesDlg,
                             QWL_STYLE,
                             WS_CLIPCHILDREN,         // set bit
                             WS_CLIPCHILDREN);

            /*
             * right window: files list
             *
             */

            G_hwndDBFileListDlg = gshrLoadDlg(hwndClient, hwndClient,
                                             fnwpDBFileListDlg,
                                             NULLHANDLE,      // our own EXE
                                             ID_WID_FILELIST,
                                             NULL);          // for WM_INITDLG
            WinSetWindowBits(G_hwndDBFileListDlg,
                             QWL_STYLE,
                             WS_CLIPCHILDREN,         // set bit
                             WS_CLIPCHILDREN);

            SPLITBARCDATA   sbcd;
            memset(&sbcd, 0, sizeof(SPLITBARCDATA));
            sbcd.ulCreateFlags = SBCF_VERTICAL          // vertical split bar
                                    | SBCF_PERCENTAGE   // lPos has split bar pos
                                                        // in percent of the client
                                    | SBCF_3DSUNK       // draw 3D "sunk" frame
                                    | SBCF_MOVEABLE;    // moveable split bar
            sbcd.lPos = 50;     // percent
            sbcd.hwndParentAndOwner = hwndClient;
            sbcd.ulLeftOrBottomLimit = 100;
            sbcd.ulRightOrTopLimit = 100;
            sbcd.ulSplitWindowID = 1000;
            G_hwndDBSplitMain = ctlCreateSplitWindow(G_WpiGlobals.habThread1,
                                                     &sbcd);

            // now link the split windows
            WinSendMsg(G_hwndDBSplitMain,
                       SPLM_SETLINKS,
                       // left window
                       (MPARAM)G_hwndDBPackagesDlg,
                       // right window:
                       (MPARAM)G_hwndDBFileListDlg);
            ctlUpdateSplitWindow(G_hwndDBSplitMain);

            WinShowWindow(G_hwndDBPackagesDlg, TRUE);
            WinShowWindow(G_hwndDBFileListDlg, TRUE);
        break; }

        /*
         * WM_WINDOWPOSCHANGED:
         *
         */

        case WM_WINDOWPOSCHANGED:
        {
            // this msg is passed two SWP structs:
            // one for the old, one for the new data
            // (from PM docs)
            PSWP pswpNew = (PSWP)(mp1);

            // resizing?
            if (pswpNew->fl & SWP_SIZE)
            {
                WinSetWindowPos(G_hwndDBSplitMain, HWND_TOP,
                                0, 0,
                                pswpNew->cx, pswpNew->cy,
                                SWP_SIZE);
            }

            // return default NULL
        break; }

        case WM_PAINT:
        {
            HPS     hps = WinBeginPaint(hwndClient, (HPS)0, NULL);
            // do nothing
            WinEndPaint(hps);

            // return default NULL
        break; }

        /*
         * WM_MINMAXFRAME:
         *      when minimizing, we hide the "split window",
         *      because otherwise the child dialogs will
         *      display garbage
         */

        case WM_MINMAXFRAME:
        {
            PSWP pswp = (PSWP)mp1;
            if (pswp->fl & SWP_MINIMIZE)
                WinShowWindow(G_hwndDBSplitMain, FALSE);
            else if (pswp->fl & SWP_RESTORE)
                WinShowWindow(G_hwndDBSplitMain, TRUE);
        break; }

        /*
         * WM_CHAR:
         *      switch focus on "Tab" key.
         */

        case WM_CHAR:
        {
            ProcessWMChar(mp1, mp2);
            mrc = (MPARAM)TRUE;
        break; }

        /*
         * WM_COMMAND:
         *
         */

        case WM_COMMAND:
        {
            USHORT  usCmd = SHORT1FROMMP(mp1);

            switch (usCmd)
            {
                case ID_WIMI_DATABASE_TREEVIEW:
                case ID_WIMI_DATABASE_DETAILSVIEW:
                    WinPostMsg(G_hwndDBPackagesDlg, WM_COMMAND, mp1, mp2);
                    break;

                case ID_WIMI_VIEW_FILES:
                case ID_WIMI_VIEW_CONFIG:
                    WinPostMsg(G_hwndDBFileListDlg, WM_COMMAND, mp1, mp2);

                default:
                    gshrDefWindowProc(hwndClient, msg, mp1, mp2,
                                      IDHI_DATABASE);
            }

        break; }

        /*
         * WM_CLOSE:
         *      window list is being closed:
         *      store the window position
         */

        case WM_CLOSE:
        {
            HWND hwndFrame = WinQueryWindow(hwndClient, QW_PARENT);
            // get the object pointer and the use list item from the window

            // save window position
            winhSaveWindowPos(hwndFrame,
                              G_WpiGlobals.hiniWarpIN,
                              "GUI", "DBWndPos");

            mrc = WinDefWindowProc(hwndClient, msg, mp1, mp2);
                    // posts WM_QUIT
        break; }

        default:
            if (!gshrDefWindowProc(hwndClient, msg, mp1, mp2,
                                   IDHI_DATABASE))
                mrc = WinDefWindowProc(hwndClient, msg, mp1, mp2);
    }

    return (mrc);
}

/*
 *@@ guiBeginDatabaseMode:
 *      this gets called from warpin.cpp when WarpIN is
 *      started without any archive as a parameter,
 *      and the database is not empty.
 *
 *      The global variable pDatabase points to the
 *      instance of FEDatabase which has been created
 *      upon startup.
 *
 *      This routine is responsible for displaying
 *      the installed packages and calling the database
 *      methods for package removal and verification,
 *      if the user desires this.
 *
 *      For the PM version of this function, this
 *      is done from the Database Status dialog window
 *      (fnwpDatabaseStatus), which is created from
 *      this function.
 *
 *      Required callback. After this returns, WarpIN
 *      terminates.
 *
 *@@changed V0.9.0 (99-10-24) [umoeller]: reworked completely for split view
 *@@changed V0.9.2 (2000-02-19) [umoeller]: added "view" menu support
 *@@changed V0.9.2 (2000-03-10) [umoeller]: added keyboard focus support
 */

BOOL guiBeginDatabaseMode(void)
{
    // register the frame class, adding a user word to the window data
    WinRegisterClass(G_WpiGlobals.habThread1,
                     WC_DATABASEMAINFRAME,
                     fnwpDBMainClient,
                     CS_SIZEREDRAW | CS_SYNCPAINT,
                     sizeof(PVOID)); // additional bytes to reserve

    // create the frame window
    ULONG flCreate = FCF_SYSMENU
                        | FCF_SIZEBORDER
                        | FCF_TITLEBAR
                        | FCF_MINMAX
                        | FCF_NOBYTEALIGN;

    SWP swpFrame;
    swpFrame.x = 100;
    swpFrame.y = 100;
    swpFrame.cx = 500;
    swpFrame.cy = 500;
    swpFrame.hwndInsertBehind = HWND_TOP;
    swpFrame.fl = SWP_MOVE | SWP_SIZE;

    HWND        hwndClient = NULLHANDLE;
    G_hwndMain = winhCreateStdWindow(HWND_DESKTOP,           // frame parent
                                     &swpFrame,
                                     flCreate,
                                     WS_ANIMATE,
                                     G_NLSStrings.pszDBMainWindowTitle, // title bar
                                     0,                      // res IDs
                                     WC_DATABASEMAINFRAME,     // client class
                                     0L,                     // client wnd style
                                     1,          // ID
                                     0,
                                     &hwndClient);
            // this cascades into all the window procs to initialize
            // the subwindows

    if (G_hwndMain)
    {
        // add to tasklist
        winhAddToTasklist(G_hwndMain,
                          G_hptrMain);

        // load main menu
        G_hmenuMain = WinLoadMenu(G_hwndMain, NULLHANDLE, ID_WIM_MAINMENU);
        // remove "Archive" menu V0.9.9 (2001-03-27) [umoeller]
        WinSendMsg(G_hmenuMain,
                   MM_REMOVEITEM,
                   MPFROM2SHORT(ID_WIM_ARCHIVE, TRUE),
                   0);
        // remove "Selections" menu (99-11-01) [umoeller]
        WinSendMsg(G_hmenuMain,
                   MM_REMOVEITEM,
                   MPFROM2SHORT(ID_WIM_SELECT, TRUE),
                   0);
        WinSendMsg(G_hwndMain, WM_UPDATEFRAME, (MPARAM)FCF_MENU, 0);
        CheckLeftViewMenuItems(G_hmenuMain);
        CheckRightViewMenuItems(G_hmenuMain);

        // load context menus
        G_hmenuPckWhitespace = WinLoadMenu(HWND_OBJECT,
                                           NULL,     // our EXE
                                           ID_WIM_DATABASE_LEFT_WHTSPACE);
        G_hmenuPckApplication = WinLoadMenu(HWND_OBJECT,
                                            NULL,     // our EXE
                                            ID_WIM_DATABASE_APP);
        G_hmenuPckPackage = WinLoadMenu(HWND_OBJECT,
                                        NULL,     // our EXE
                                        ID_WIM_DATABASE_PACKAGE);
        G_hmenuFilesWhitespace = WinLoadMenu(HWND_OBJECT,
                                             NULL,     // our EXE
                                             ID_WIM_DATABASE_RIGHT_WHTSPACE);

        // now position the frame and the client:
        // 1) frame
        if (!winhRestoreWindowPos(G_hwndMain,
                                  G_WpiGlobals.hiniWarpIN,
                                  "GUI", "DBWndPos",
                                  SWP_MOVE | SWP_SIZE))
            // INI data not found:
            WinSetWindowPos(G_hwndMain,
                            HWND_TOP,
                            100, 100,
                            500, 500,
                            SWP_MOVE | SWP_SIZE);

        WinShowWindow(G_hwndMain, TRUE);

        WinSetFocus(HWND_DESKTOP,
                    WinWindowFromID(G_hwndDBPackagesDlg,
                                    ID_WIDI_CNR));

        // kill the startup window
        gshrDestroySplash();
        // WinDestroyWindow(G_hwndWarpINStartup);

        QMSG qmsg;
        while (WinGetMsg(G_WpiGlobals.habThread1,
                         &qmsg, 0, 0, 0))
            WinDispatchMsg(G_WpiGlobals.habThread1, &qmsg);

        // done:
        WinDestroyWindow(G_hwndMain);

        return (TRUE);
    }

    return (FALSE);
}

/*
 *@@ guiStartVerify:
 *      this is supposed to display a dialog for the
 *      database progress and call pDatabase->VerifyPackages.
 *
 *      warpin.cpp calls this callback in "add/remove"
 *      mode, but this also gets called from the GUI
 *      in "Database" mode.
 *
 *      This creates the database status window. See
 *      fnwpDatabaseStatus for additional remarks.
 *
 *      Required callback.
 *
 *@@added V0.9.0 (99-11-02) [umoeller]
 *@@changed V0.9.9 (2001-03-30) [umoeller]: added log file
 */

BOOL guiStartVerify(list<FEDBPackage*>* pPackagesList,
                    BOOL fModal,
                            // if TRUE, this function does not return
                            // until we're done (modal dialog)
                    BSFileLogger *pLogFile)     // in: log file or NULL
{
    // load the database thread status window
    // (fnwpDatabaseStatus)
    DATABASESTATUSINFO dbsi(DBT_VERIFY,
                            0,          // flags, this is for remove only
                            // log file: pass this only in database (non-modal)
                            // mode, the status window only needs this for
                            // closing the log file when db thread is done
                            (fModal)
                                ? NULL
                                : pLogFile,
                            G_NLSStrings.pszDBVerifyingPackages);

    // load the status window
    HWND hwndDlg = gshrLoadDlg(HWND_DESKTOP,   // parent
                               G_hwndMain,        // owner
                               fnwpDatabaseStatus,
                               NULLHANDLE,      // our own EXE
                               ID_WID_DATABASETHREAD,
                               // pass the command to the dlg
                               (PVOID)&dbsi);
    winhCenterWindow(hwndDlg);
    WinShowWindow(hwndDlg, TRUE);

    // start database thread
    G_pDatabase->VerifyPackages(pPackagesList,
                                hwndDlg,
                                pLogFile);

    if (fModal)
    {
        WinProcessDlg(hwndDlg);
        WinDestroyWindow(hwndDlg);
    }
    else
        G_preccLeftSource->hwndStatus = hwndDlg;

    return (TRUE);
}

/*
 *@@ guiStartRemove:
 *      this is supposed to display a dialog for the
 *      database progress and call pDatabase->DeinstallPackages.
 *
 *      warpin.cpp calls this callback two situations:
 *
 *      -- in "add/remove" mode, when packages need to be removed;
 *         in that case, this dialog runs modally;
 *
 *      -- from the GUI in "Database" mode; in that case, this
 *         dialog runs modelessly.
 *
 *      This creates the database status window. See
 *      fnwpDatabaseStatus for additional remarks.
 *
 *      Required callback.
 *
 *@@added V0.9.0 (99-11-02) [umoeller]
 *@@changed V0.9.4 (2000-07-01) [umoeller]: added process kills
 */

BOOL guiStartRemove(list<FEDBPackage*>* pPackagesList,
                            // list of packages to be removed
                    ULONG ulConfigFlags,
                            // any combination of DBT_DELETEFILES, DBT_UNDOCONFIG
                    BOOL fModal,
                            // if TRUE, this function does not return
                            // until we're done (modal dialog)
                    BSFileLogger *pLogFile)     // in: log file or NULL
{
    BOOL brc = FALSE;

    if (ulConfigFlags >= 0)
    {
        // kill processes
        list<FEDBPackage*>::iterator PckStart = pPackagesList->begin(),
                                     PckEnd = pPackagesList->end();
        for (; PckStart != PckEnd; PckStart++)
        {
            FEDBPackage *pPackageThis = *PckStart;

            // go thru the list of all config objects; this
            // list holds all objects as base objects, so
            // we need to type check
            list<BSConfigBase*>::iterator
                    CfgStart = pPackageThis->_listUndoConfig.begin(),
                    CfgEnd = pPackageThis->_listUndoConfig.end();
            for (; CfgStart != CfgEnd; CfgStart++)
            {
                BSConfigBase* pCfgThis = *CfgStart;
                if (pCfgThis)   // V0.9.7 (2000-12-16) [umoeller]
                {
                    // check if this is our class
                    BSKillProcess *pKill = pCfgThis->IsKillProcess();
                    if (pKill)
                    {
                        BOOL fReloop = FALSE;

                        do
                        {
                            // OK, we got a KILLPROCESS attribute:
                            fReloop = wpiKillOneProcess(pKill,
                                                        pLogFile);
                        } while (fReloop);
                    }
                    // else
                        // not the class we need: take next then
                }
            }
        }

        // load the database thread status window
        // (fnwpDatabaseStatus)
        DATABASESTATUSINFO dbsi(DBT_DEINSTALL,
                                ulConfigFlags,
                                // log file: pass this only in database (non-modal)
                                // mode, the status window only needs this for
                                // closing the log file when db thread is done
                                (fModal)
                                    ? NULL
                                    : pLogFile,
                                G_NLSStrings.pszDBDeinstallingPackages);

        // load the status window
        HWND hwndDlg = gshrLoadDlg(HWND_DESKTOP,   // parent
                                   G_hwndMain,        // owner
                                   fnwpDatabaseStatus,
                                   NULLHANDLE,      // our own EXE
                                   ID_WID_DATABASETHREAD,
                                   // pass the command to the dlg
                                   (PVOID)&dbsi);
        winhCenterWindow(hwndDlg);
        WinShowWindow(hwndDlg, TRUE);

        // start database thread
        G_pDatabase->DeinstallPackages(pPackagesList,
                                       ulConfigFlags,
                                       hwndDlg, // passed to callback
                                       pLogFile);

        if (fModal)
        {
            WinProcessDlg(hwndDlg);
            WinDestroyWindow(hwndDlg);
        }
        else
            G_preccLeftSource->hwndStatus = hwndDlg;

        brc = TRUE;
    }

    return (brc);
}


