/* $Id: gui_installing.cpp,v 1.27 2001/06/04 07:26:58 umoeller Exp $ */

/*
 *@@sourcefile gui_installing.cpp:
 *      this contains the PM-specific stuff for while
 *      the front end is installing packages. See
 *      guiStartInstall, which is the main entry point
 *      into all this.
 *
 *      This has been separated from gui.cpp (99-10-26) [umoeller].
 *
 *@@header "frontend\gui.h"
 */

/*
 *      This file Copyright (C) 1998-2000 Ulrich Mller.
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, in version 2 as it comes in the COPYING
 *      file of this distribution.
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 */

#define OS2EMX_PLAIN_CHAR
    // this is needed for "os2emx.h"; if this is defined,
    // emx will define PSZ as _signed_ char, otherwise
    // as unsigned char

#define INCL_DOS
#define INCL_DOSERRORS
#define INCL_WIN
#define INCL_WINWORKPLACE
#define INCL_GPILOGCOLORTABLE
#define INCL_GPIPRIMITIVES
#include <os2.h>

#include <stdlib.h>
// #include <stdio.h>           // included with wiarchive.h
#ifdef __IBMCPP__
    #include <direct.h>
#endif
#include <string.h>
#include <ctype.h>
#include <setjmp.h>             // needed for except.h
#include <assert.h>             // needed for except.h
#include <io.h>

#include "setup.h"

// include's from helpers
#include "helpers\datetime.h"
#include "helpers\dosh.h"
#include "helpers\cnrh.h"
#include "helpers\comctl.h"
#include "helpers\gpih.h"
#include "helpers\winh.h"
#include "helpers\stringh.h"
#include "helpers\threads.h"
#include "helpers\xstring.h"

// back-end includes
#include "wiarchive\wiarchive.h"

#include "libbz2\bzlib.h"               // for the error codes V0.9.12 (2001-04-29) [umoeller]

// base includes (99-11-07) [umoeller]
#include "base\bs_string.h"
#include "base\bs_errors.h"
// #include "base\bs_config.h"
// #include "base\bs_script.h"

// front-end includes
#include "frontend\fe_base.h"
#include "frontend\fe_job.h"
// #include "frontend\fe_package.h"
// #include "frontend\fe_package_arc.h"

#include "frontend\warpin.h"
#include "frontend\calbacks.h"

#include "frontend\dlgids.h"
#include "frontend\gui.h"

#include "helpers\except.h"

#pragma hdrstop

/* ******************************************************************
 *
 *  Global variables
 *
 ********************************************************************/

// LONG                G_lErrorRC = -1;             // changed (99-11-03) [umoeller]
            // removed V0.9.12 (2001-05-22) [umoeller]

ULONG               G_ulInstallTimeBegin = 0,
                    G_ulSecsElapsed = 0,
                    G_ulPercent = 0;

/* ******************************************************************
 *
 *    GUI functions on thread 1
 *
 ********************************************************************/

/*
 *@@ ConfirmCancelInstall:
 *      helper func which is used every time
 *      the user tries to cancel installation.
 *      This wants this decision confirmed
 *      and does all the cleanup, if neccessary.
 */

BOOL ConfirmCancelInstall(HWND hwndOwner)
{
    ULONG ulmbrc = MBID_NO;
    BOOL  brc = FALSE;

    if (!G_WpiGlobals.fInstallComplete)
    {
        TID     tidInstall;

        // suspend Install thread if it's running;
        // we don't want the unpacking of files to
        // continue while the dialog is being
        // displayed
        if ((tidInstall = thrQueryID(&G_WpiGlobals.tiInstallThread)))
            DosSuspendThread(tidInstall);

        ulmbrc = wpiMessageBox(108, // "Warning"
                               109, // "Damage, Danger, Really cancel?"
                               MB_ICONQUESTION | MB_YESNO | MB_MOVEABLE);

        if (ulmbrc == MBID_YES)
        {
            // terminate install thread
            G_WpiGlobals.tiInstallThread.fExit = TRUE;

            WinPostMsg(hwndOwner, WM_CLOSE, 0, 0);
            brc = TRUE;
        }

        if (tidInstall)
            DosResumeThread(tidInstall);
                    // if the exit flag has been set above,
                    // this will quickly exit
    }
    else
    {
        WinPostMsg(hwndOwner, WM_CLOSE, 0, 0);
        brc = TRUE;
    }

    return (brc);
}

/*
 *@@ fnwpFileExistsOrLocked:
 *      window proc for "file exists" confirmation
 *      dialog. This is only for an additional
 *      confirmation when "Cancel" or "Confirmations"
 *      are pressed.
 *
 *@@changed V0.9.0 (99-10-26) [umoeller]: renamed from fnwpFileExists
 *@@changed V0.9.0 (99-10-26) [umoeller]: added support for file-locked dlg
 */

MRESULT EXPENTRY fnwpFileExistsOrLocked(HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    MRESULT mrc = 0;

    switch (msg)
    {
        case WM_COMMAND:
        {
            switch (SHORT1FROMMP(mp1))
            {
                case ID_WIMI_PREFERENCES:
                    gshrPreferencesDlg(hwndDlg);     // only in file exists
                break;

                case DID_CANCEL:
                    if (ConfirmCancelInstall(hwndDlg))
                        WinDismissDlg(hwndDlg, DID_CANCEL);
                    else
                        // continue
                        break;

                default:
                    mrc = WinDefDlgProc(hwndDlg, msg, mp1, mp2);
            }
        break; }

        default:
            mrc = WinDefDlgProc(hwndDlg, msg, mp1, mp2);
    }

    return (mrc);
}

/*
 *@@ ShowFileLockedDlg:
 *      this gets called from fnwpInstallingStatus if a
 *      file to be overwritten is locked by another process
 *      (in a response to a guiFileLockedError call).
 *
 *      This must return one of the following:
 *      --  CBRC_SKIP:    skip locked file
 *      --  CBRC_RETRY:   retry same file
 *      --  CBRC_UNLOCK:  DosReplaceModule
 *      --  CBRC_DEFER:   defer thru CONFIG.SYS
 *      --  CBRC_ABORT:   abort install
 *
 *@@added V0.9.0 (99-10-26) [umoeller]
 */

int ShowFileLockedDlg(HWND hwndOwner,
                      PGUIFILEERROR pfi)
{
    int irc = CBRC_SKIP;

    // last_sel is the global (static) flag
    // in which we remember the last user selection
    static int      last_sel = ID_WIDI_EXISTS_SKIP;

    HWND hwndConfirm = gshrLoadConfirmDlg(hwndOwner,
                                          ID_WID_FILELOCKED,
                                          fnwpFileExistsOrLocked,
                                          pfi);

    // set file name (replace %1 in static text)
    HWND        hwndItem = WinWindowFromID(hwndConfirm, ID_WIDI_EXISTS_TXT1);
    PSZ         pszMessage = winhQueryWindowText(hwndItem);
    ULONG       ulOfs = 0;
    strhFindReplace(&pszMessage, &ulOfs, "%1", pfi->szFilename);
    WinSetWindowText(hwndItem, pszMessage);
    free(pszMessage);

    // disable "defer"; not implemented yet
    WinEnableControl(hwndConfirm, ID_WIDI_LOCKED_DEFER, FALSE);

    // check last selection item
    winhSetDlgItemChecked(hwndConfirm, last_sel, 1);
    WinSetFocus(HWND_DESKTOP, WinWindowFromID(hwndConfirm, last_sel));

    // run confirmation dialog
    ULONG ulrc = WinProcessDlg(hwndConfirm);

    if (ulrc == DID_OK)
    {
        // and now, do what?
        if (winhIsDlgItemChecked(hwndConfirm, ID_WIDI_EXISTS_SKIP))
        {
            last_sel = ID_WIDI_EXISTS_SKIP;
            irc = CBRC_SKIP;
        }
        else if (winhIsDlgItemChecked(hwndConfirm, ID_WIDI_LOCKED_RETRY))
        {
            last_sel = ID_WIDI_LOCKED_RETRY;
            irc = CBRC_RETRY;
        }
        else if (winhIsDlgItemChecked(hwndConfirm, ID_WIDI_LOCKED_UNLOCK))
        {
            last_sel = ID_WIDI_LOCKED_UNLOCK;
            irc = CBRC_UNLOCK;
        }
        else if (winhIsDlgItemChecked(hwndConfirm, ID_WIDI_LOCKED_DEFER))
        {
            last_sel = ID_WIDI_LOCKED_DEFER;
            irc = CBRC_DEFER;
        }
    }
    else
        // "Cancel":
        irc = CBRC_ABORT;

    WinDestroyWindow(hwndConfirm);

    return (irc);
}

/*
 *@@ ShowFileExistsDlg:
 *      this gets called from fnwpInstallingStatus if a
 *      file already exists on disk and we need to prompt
 *      the user.
 *
 *      This must return one of the following:
 *      --  CBRC_PROCEED    replace this file
 *      --  CBRC_SKIP       skip this file
 *      --  CBRC_ABORT      abort installation
 *
 *@@changed V0.9.0 (99-10-26) [umoeller]: renamed from FileExistsDlg
 *@@changed V0.9.0 (99-10-26) [umoeller]: fixed "Cancel" not working
 *@@changed V0.9.0 (99-10-26) [umoeller]: exported stuff to LoadConfirmDlg
 */

int ShowFileExistsDlg(HWND hwndOwner,
                      PGUIFILEERROR pfi)
{
    int irc = CBRC_SKIP;

    // last_sel is the global (static) flag
    // in which we remember the last user selection
    static int      last_sel = ID_WIDI_EXISTS_SKIP;

    HWND hwndConfirm = gshrLoadConfirmDlg(hwndOwner,
                                          ID_WID_FILEEXISTS,
                                          fnwpFileExistsOrLocked,
                                          pfi);

    // set file name (replace %1 and %2 in static text)
    HWND        hwndItem = WinWindowFromID(hwndConfirm, ID_WIDI_EXISTS_TXT1);
    PSZ         pszMessage = winhQueryWindowText(hwndItem);
    PSZ         pLastBackslash = strrchr(pfi->szFilename, '\\');
    ULONG       ulOfs = 0;
    strhFindReplace(&pszMessage, &ulOfs, "%1", pLastBackslash+1);
    *pLastBackslash = 0;
    ulOfs = 0;
    strhFindReplace(&pszMessage, &ulOfs, "%2", pfi->szFilename);
    WinSetWindowText(hwndItem, pszMessage);
    free(pszMessage);

    // check last selection item
    winhSetDlgItemChecked(hwndConfirm, last_sel, 1);
    WinSetFocus(HWND_DESKTOP, WinWindowFromID(hwndConfirm, last_sel));

    // run confirmation dialog
    ULONG ulrc = WinProcessDlg(hwndConfirm);

    if (ulrc == DID_OK)
    {
        // and now, do what?
        if (winhIsDlgItemChecked(hwndConfirm, ID_WIDI_EXISTS_SKIP))
        {
            last_sel = ID_WIDI_EXISTS_SKIP;
            irc = CBRC_SKIP;
        } else if (winhIsDlgItemChecked(hwndConfirm, ID_WIDI_EXISTS_RENAME))
        {
            last_sel = ID_WIDI_EXISTS_RENAME;
            irc = CBRC_PROCEED;
        }
    }
    else
        // "Cancel":
        irc = CBRC_ABORT;

    WinDestroyWindow(hwndConfirm);

    // last_sel always has the action to do now
    return (irc);
}

/*
 *@@ fnwpInstallingStatus:
 *      window proc for the "Install" window after we're
 *      done with all the pages and the Install thread is
 *      running.
 *
 *      This runs on thread 1.
 *
 *      The various gui* callbacks on the Install thread
 *      keep posting messages to us.
 *
 *      This gets started via WinProcessDlg from guiStartInstall.
 *      If this dialog returns TRUE via WinDismissDlg, this
 *      means installation went OK.
 *
 *@@changed V0.9.0 (99-10-26) [umoeller]: renamed from fnwpInstall
 *@@changed V0.9.0 (99-10-26) [umoeller]: error string handling reworked completely
 *@@changed V0.9.0 (99-10-26) [umoeller]: added WPIM_FILELOCKEDERROR
 */

MRESULT EXPENTRY fnwpInstallingStatus(HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2)
{
    MRESULT mrc = 0;

    static HWND         hwndProgressBar = NULLHANDLE;

    switch (msg)
    {
        /*
         * WM_INITDLG:
         *      initialize some more controls
         */

        case WM_INITDLG:
        {
            mrc = WinDefDlgProc(hwndDlg, msg, mp1, mp2);

            gshrSetControlsFont(hwndDlg, 0, 500);

            // create a progress bar
            hwndProgressBar = WinWindowFromID(hwndDlg, ID_WIDI_PROGRESSBAR);
            ctlProgressBarFromStatic(hwndProgressBar,
                        PBA_ALIGNCENTER | PBA_BUTTONSTYLE);  // (*UM#4)

            // start the timer
            G_ulInstallTimeBegin = dtGetULongTime();
            WinStartTimer(G_WpiGlobals.habThread1, hwndDlg, 1, 1000);
        break; }

        /*
         *@@ WPIM_UPDATEPACKAGE:
         *      user msg posted from the Install thread
         *      when we advance to the next package.
         *      Parameters:
         *          PSZ mp1  current FEInstallJob*
         *          PSZ mp2  string with "pck 1 out of 9"
         *
         *      Strings must be free()'d here.
         */

        case WPIM_UPDATEPACKAGE:
        {
            if (mp1 != NULL)
            {
                FEInstallJob *pJob = (FEInstallJob*)mp1;
                CHAR szSizeRounded[20];
                strhThousandsULong(szSizeRounded, pJob->QueryPckSizeRounded(),
                                   G_szThousand[0]);
                CHAR    szPck[2000];
                sprintf(szPck, "%s (%s KB)",
                        pJob->QueryPckTitle(),
                        szSizeRounded);
                WinSetDlgItemText(hwndDlg, ID_WIDI_CURPCKTITLE,
                                  szPck);
                WinSetDlgItemText(hwndDlg, ID_WIDI_TARGETPATHDIR,
                                  pJob->QueryTargetPath());
            }
            if (mp2 != NULL)
            {
                WinSetDlgItemText(hwndDlg, ID_WIDI_CURPCKCOUNT, (PSZ)mp2);
                free(mp2);
            }
        break; }

        /*
         *@@ WPIM_UPDATEFILE:
         *      user msg posted from the Install thread
         *      when we advance to the next file.
         *      Parameters:
         *          PSZ mp1  current file name
         *          PSZ mp2  string with "file 1 out of 9"
         *
         *      Both strings must be free()'d here.
         */

        case WPIM_UPDATEFILE:
        {
            if (mp1 != NULL)
            {
                WinSetDlgItemText(hwndDlg, ID_WIDI_CURFILENAME, (PSZ)mp1);
                free(mp1);
            }
            if (mp2 != NULL)
            {
                WinSetDlgItemText(hwndDlg, ID_WIDI_CURFILECOUNT, (PSZ)mp2);
                free(mp2);
            }
            ULONG ulDrive, ulMap;
            APIRET arc = DosQueryCurrentDisk(&ulDrive, &ulMap);
            CHAR szMsg[100] = "?";
            if (arc == NO_ERROR)
            {
                double d = 0;
                arc = doshQueryDiskFree(ulDrive, &d);
                if (arc == NO_ERROR)
                {
                    CHAR sz2[20];
                    sprintf(szMsg, G_NLSStrings.pszDriveXFree, // "Drive %c: %s KB free",
                                UCHAR(ulDrive & 0xFF) + 'A' - 1,
                                strhThousandsDouble(sz2,
                                                    d / 1024,
                                                    G_szThousand[0]));
                }
            }
            WinSetDlgItemText(hwndDlg, ID_WIDI_TARGETPATHFREE, szMsg);
        break; }

        /*
         *@@ WPIM_UPDATEBYTES:
         *      user msg posted from the Install thread
         *      when the "bytes" display needs updating.
         *      Parameters:
         *          SHORT mp1  current percentage
         */

        case WPIM_UPDATEBYTES:
        {
            G_ulPercent = (ULONG)mp1;
            WinPostMsg(hwndProgressBar, WM_UPDATEPROGRESSBAR,
                       (MPARAM)G_ulPercent,      // current value
                       (MPARAM)100);           // maximum value
        break; }

        /*
         * WM_TIMER:
         *      the timer is only running on the
         *      "Install" page to update the
         *      elapsed time once a second.
         *      We also attempt to have a meaningful
         *      "time to go" display, but this isn't
         *      always working right.
         */

        case WM_TIMER:
        {
            ULONG ulTime2 = dtGetULongTime();

            G_ulSecsElapsed = (ulTime2 - G_ulInstallTimeBegin) / 1000;

            ULONG   ulHours = G_ulSecsElapsed / 60 /60;
            ULONG   ulMinutes = ((G_ulSecsElapsed / 60) - (ulHours * 60));
            ULONG   ulSecs = G_ulSecsElapsed
                                - (ulHours * 60 * 60)
                                - (ulMinutes * 60);
            CHAR    szTime[30];
            sprintf(szTime, "%02d%c%02d%c%02d %s",
                    ulHours,    G_szTimeSep[0],
                    ulMinutes,  G_szTimeSep[0],
                    ulSecs,
                    G_NLSStrings.pszElapsed);

            WinSetDlgItemText(hwndDlg, ID_WIDI_TIMEELAPSED, szTime);

            // calculate time to go
            if ((G_ulSecsElapsed) && (G_ulPercent > 10))
            {
                ULONG   ulSecsToGo = (G_ulSecsElapsed * (100 - G_ulPercent) ) / G_ulPercent;

                ULONG   ulHours = ulSecsToGo / 60 /60;
                ULONG   ulMinutes = ((ulSecsToGo / 60) - (ulHours * 60));
                ULONG   ulSecs = ulSecsToGo
                                    - (ulHours * 60 * 60)
                                    - (ulMinutes * 60);
                CHAR    szTime[30];
                sprintf(szTime, "%02d%c%02d%c%02d %s",
                        ulHours,    G_szTimeSep[0],
                        ulMinutes,  G_szTimeSep[0],
                        ulSecs,
                        G_NLSStrings.pszToGo);

                WinSetDlgItemText(hwndDlg, ID_WIDI_TIMELEFT, szTime);
            }
        break; }

        /*
         *@@ WPIM_FILEEXISTS:
         *      this is posted by the guiFileExists callback
         *      on the Install thread if a file exists already.
         *
         *      Parameters:
         *      -- PFILEEXISTS mp1: more info.
         *
         *      Note that guiFileExists waits until the global
         *      iErrorRC variable is set to anything but -1.
         *      The Install thread then resumes operation
         *      depending on that value.
         *
         *      This must set iErrorRC to one of the following:
         *
         *      --  CBRC_PROCEED:   replace this file
         *
         *      --  CBRC_SKIP:      skip this file
         *
         *      --  CBRC_ABORT:     stop processing.
         *
         *      Those are the return values of ShowFileExistsDlg.
         */

        case WPIM_FILEEXISTS:
            mrc = (MPARAM)ShowFileExistsDlg(hwndDlg,
                                            (PGUIFILEERROR)mp1);
        break;

        /*
         *@@ WPIM_INSTALLERROR:
         *      posted from the Install thread also.
         *      Parameters:
         *      --  PSZ mp1:    the error message (static buffer (99-10-26) [umoeller])
         *      --  ULONG mp2:  CBREC_* flags (wiarchive.h), allowed return codes.
         *
         *      Note that guiFileExists waits until the global
         *      iErrorRC variable is set to anything but -1.
         *      The Install thread then resumes operation
         *      depending on that value.
         *
         *      Depending on mp2, the following values may be returned:
         *      --  CBRC_RETRY:     have the install thread try again.
         *      --  CBRC_IGNORE:    ignore the error, go on (skip the file).
         *      --  CBRC_ABORT:     have guiFileExists terminate the
         *                          install thread and quit.
         */

        case WPIM_INSTALLERROR:
        {
            ULONG ulErrRsp;

            switch ((ULONG)mp2)
            {
                case CBREC_RETRYCANCEL:         ulErrRsp = MB_RETRYCANCEL; break;
                case CBREC_ABORTRETRYIGNORE:    ulErrRsp = MB_ABORTRETRYIGNORE; break;
                default:                        ulErrRsp = MB_CANCEL;
            }

            ULONG ulmbrc = WinMessageBox(HWND_DESKTOP,      // parent
                                         hwndDlg,       // owner
                                         (PCSZ)mp1,
                                         "WarpIN: Install Error",   // xxx
                                         0,
                                         MB_ICONEXCLAMATION | ulErrRsp | MB_MOVEABLE);

            switch (ulmbrc)
            {
                case MBID_RETRY:  mrc = (MPARAM)CBRC_RETRY; break;
                case MBID_IGNORE: mrc = (MPARAM)CBRC_IGNORE; break;
                // case MBID_CANCEL: mrc = (MPARAM)CBRC_ABORT; break;
                // case MBID_ABORT:  mrc = (MPARAM)CBRC_ABORT; break;
                default:          mrc = (MPARAM)CBRC_ABORT; break;
            }
        break; }

        /*
         *@@ WPIM_FILELOCKEDERROR:
         *      posted from guiFileLockedError when a file is
         *      locked and cannot be overwritten.
         *
         *      Parameters:
         *      -- PFILEEXISTS mp1: more info.
         *
         *      This must set iErrorRC to one of these:
         *      --  CBRC_SKIP:    ignore the error, go on (skip the file).
         *      --  CBRC_RETRY:   have the install thread retry.
         *      --  CBRC_UNLOCK:  have the install thread unlock the file (DosReplaceModule).
         *      --  CBRC_DEFER:   have the install thread defer processing thru CONFIG.SYS.
         *      --  CBRC_ABORT:   have guiFileExists terminate the
         *                          install thread and quit.
         *
         *      This was added (99-10-26) [umoeller].
         */

        case WPIM_FILELOCKEDERROR:
        {
            PGUIFILEERROR pfi = (PGUIFILEERROR)mp1;

            mrc = (MPARAM)ShowFileLockedDlg(hwndDlg,
                                            pfi);
        break; }

        /*
         *@@ WPIM_DONEWITHINSTALL:
         *      posted (!) from the Install thread
         *      when all files have been unpacked.
         *
         *      The Install thread terminates itself
         *      automatically. We need to dismiss
         *      the dialog and continue working
         *      on thread 1.
         *
         *      mp1 and mp2 are both NULL.
         *
         *      NOTE: even though we converted all
         *      WinPostMsg to WinSendMsg, this msg
         *      is still _posted_ (V0.9.12 (2001-05-22) [umoeller])
         */

        case WPIM_DONEWITHINSTALL:
        {
            // stop timer on container page
            WinStopTimer(G_WpiGlobals.habThread1, hwndDlg, 1);

            // get outta here
            WinDismissDlg(hwndDlg, DID_OK);
        break; }

        case WM_COMMAND:
            ConfirmCancelInstall(hwndDlg);
        break;

        default:
            mrc = WinDefDlgProc(hwndDlg, msg, mp1, mp2);

    } // end switch

    return (mrc);
}

/*
 *@@ guiStartInstall:
 *      this gets called from warpin.cpp to actually
 *      start the install process. This gets called
 *      in both "full install" and "add/remove" modes
 *      after all information has been collected
 *      from the user.
 *
 *      Required callback.
 *
 *      It is the responsibility of this function to
 *      show up the "Installing..." dialog
 *      (progress bar, current packages, etc.)
 *      and call wpiStartInstallThread, which will
 *      process the FEInstallJob list.
 *
 *      If this returns TRUE, this means the packages
 *      are to be stored in the database afterwards
 *      (i.e. installation went OK).
 *
 *      If this returns FALSE, packages are not
 *      stored, because installation was cancelled or
 *      errors occured.
 */

BOOL guiStartInstall(BSFileLogger *pLogFile)
{
    G_hwndMain = gshrLoadDlg(HWND_DESKTOP,     // parent
                            HWND_DESKTOP,
                            fnwpInstallingStatus,
                            NULLHANDLE,
                            ID_WID_PAGE_INSTALL,
                            0);
                // this calls wpiStartInstallThread in turn

    // add ourselves to the tasklist
    winhAddToTasklist(G_hwndMain,
                      G_hptrMain);

    winhCenterWindow(G_hwndMain);
    WinShowWindow(G_hwndMain, TRUE);

    // start Install thread
    wpiStartInstallThread(pLogFile);
    // go!!
    ULONG ulrc = WinProcessDlg(G_hwndMain);

    WinDestroyWindow(G_hwndMain);
    G_hwndMain = NULLHANDLE;
    return (ulrc == DID_OK);
}

/* ******************************************************************
 *
 *    GUI functions on Install thread
 *
 ********************************************************************/

/*  General note for the following functions:
 *      Most of these are callbacks from the
 *      main Install thread on which the "Unpack"
 *      methods of the WIArchive class are
 *      running. In other words, it's the Install
 *      thread which creates all the files.
 *      Since all the following functions therefore
 *      run on the Install thread, and since the
 *      Install thread does _not_ have a message
 *      queue, we better not manipulate windows
 *      here. Instead, we post msgs to the main
 *      window which does the updating on thread 1.
 *
 *      This runs on the Install thread.
 */

/*
 *@@ guiUpdatePackage:
 *      GUI callback from warpin.cpp
 *      to update the "Package" display.
 *
 *      Required callback.
 *
 *      This runs on the Install thread.
 */

void guiUpdatePackage(FEInstallJob *pJob, ULONG ulPckThis, ULONG ulPcksTotal)
{
    CHAR szMsg[100];
    sprintf(szMsg, G_NLSStrings.pszPackageX,  // "Package %d out of %d"
                   ulPckThis, ulPcksTotal);
    WinPostMsg(G_hwndMain, WPIM_UPDATEPACKAGE,
               pJob,
               strdup(szMsg));
        // the window proc will release the string memory
}

/*
 *@@ guiUpdateFile:
 *      GUI callback from warpin.cpp
 *      to update the "File" display.
 *
 *      Required callback.
 *
 *      This runs on the Install thread.
 */

void guiUpdateFile(WIFileHeader *pwifh, ULONG ulFileThis, ULONG ulFilesTotal)
{
    CHAR szSizeRounded[20];
    strhThousandsULong(szSizeRounded, ((pwifh->origsize + 512) / 1024), G_szThousand[0]);

    CHAR    szFile[CCHMAXPATH + 100];
    CHAR    szMsg[100];
    sprintf(szFile, "%s (%s KB)", pwifh->name, szSizeRounded);
    sprintf(szMsg, G_NLSStrings.pszFileX, // "File %d out of %d",
                   ulFileThis, ulFilesTotal);
    WinPostMsg(G_hwndMain, WPIM_UPDATEFILE,
               strdup(szFile), strdup(szMsg));
        // the window proc will release the string memory
}

/*
 *@@ guiUpdateBytes:
 *      GUI callback from warpin.cpp
 *      to update the progress display.
 *
 *      Required callback.
 *
 *      This runs on the Install thread.
 */

void guiUpdateBytes(double dBytesDone, double dBytesTotal)
{
    SHORT sPercent = (SHORT)((dBytesDone / dBytesTotal) * 100);
    WinPostMsg(G_hwndMain, WPIM_UPDATEBYTES,
               (MPARAM)sPercent, 0);
}

/*
 *@@ guiFileExists:
 *      callback for the Install thread to confirm
 *      whether a file should be replaced.
 *
 *      This only gets called if WICallback (warpin.cpp)
 *      has determined that the user should be prompted
 *      for what to do, depending on ulIfSameDate,
 *      ulIfExistingNewer, and ulIfExistingOlder in WpiGlobals.
 *
 *      This must return one of the following:
 *
 *      --  CBRC_PROCEED:   replace this file
 *
 *      --  CBRC_SKIP:      skip this file
 *
 *      If the user presses "Cancel", it's our own
 *      responsibility to terminate everthing.
 *      We must _not_ return from this function then.
 *
 *      This callback should also offer the user an
 *      opportunity to reset the three flags in WpiGlobals
 *      so that prompting can be disabled for subsequent
 *      files.
 *
 *      Required callback.
 *
 *      This runs on the Install thread.
 *
 *@@changed V0.9.12 (2001-05-22) [umoeller]: converted all WinPostMsg to WinSendMsg
 */

int guiFileExists(PGUIFILEERROR pfi)
{
    // we cannot do a WinSendMsg, because this would
    // require the Install thread to have a PM message
    // queue.
    // WinPostMsg works though, so we rely on a global
    // variable to get the return code. This is safe
    // since this variable is only set by thread 1
    // after it receives our message.

    // changed V0.9.12 (2001-05-22) [umoeller]: this
    // thread now has a PM message queue, so use
    // WinSendMsg to fix serialization problems on SMP.
    int irc = (int)WinSendMsg(G_hwndMain,
                              WPIM_FILEEXISTS,
                              (MPARAM)pfi,     // pass structure to main window
                              (MPARAM)0);

    // wait until the user has selected something
    // in the "File exists" dialog on thread 1;
    // WPIM_FILEEXISTS in fnwpPages (above) sets
    // iErrorRC to either CBRC_CANCEL, CBRC_SKIP
    // or CBRC_PROCEED
    // DosWaitEventSem(G_hevGUIDone, SEM_INDEFINITE_WAIT);

    if (irc == CBRC_ABORT)
        // "Cancel" pressed: we need to terminate
        // our thread then, or OS/2 will hang
        _endthread();       // @@todo this doesn't work

    return (irc);
}

/*
 *@@ guiFileError:
 *      this is called from WICallback (warpin.cpp)
 *      whenever an error occurs on the Install thread.
 *
 *      This can be various errors, depending on
 *      the iErrorClass and iErrorCode values.
 *
 *      This must return one of the following:
 *          CBRC_RETRY     retry the failing operation
 *
 *      If the user presses "Cancel", it's our own
 *      responsibility to terminate the Install thread.
 *      We must _not_ return from this function then.
 *
 *      Required callback.
 *
 *      This runs on the Install thread.
 *
 *@@changed V0.9.0 (99-10-26) [umoeller]: function prototype changed
 *@@changed V0.9.0 (99-10-26) [umoeller]: string and code handling reworked completely
 *@@changed V0.9.1 (2000-02-06) [umoeller]: renamed from guiInstallError
 *@@changed V0.9.12 (2001-05-22) [umoeller]: converted all WinPostMsg to WinSendMsg
 */

int guiFileError(// FEArcPackagePck *pCurrentArcPck, // in: current package, ptr can be NULL!
                 FEInstallJob *pJob,    // in: current install job, ptr can be NULL!
                 PVOID pData,        // in: error data; meaning depends on iErrorClass
                 int iErrorClass,    // in: ECLS_* flag
                 int iErrorCode,     // in: error code; meaning depends on iErrorClass
                 int iErrorRsp)      // in: possible response (ERSP_* flags)
{
    // we cannot do a WinSendMsg, because this would
    // require the Install thread to have a PM message
    // queue.
    // WinPostMsg works though, so we rely on a global
    // variable to get the return code. This is safe
    // since this variable is only set by thread 1
    // after it receives our message.
    // changed V0.9.12 (2001-05-22) [umoeller]: this
    // thread now has a PM message queue, so use
    // WinSendMsg to fix serialization problems on SMP.

    BSString    strErrorMsg;

    switch (iErrorClass)
    {
        /*
         * ECLS_DOS_CREATEDIR:
         *      error in wpiInstallThread while creating
         *      target directory.
         *
         *      pData is a PSZ to the directory name.
         */

        case ECLS_DOS_CREATEDIR:
        {
            PSZ     apsz = (PSZ)pData;
            wpiGetMessage(strErrorMsg,
                          134,   // The directory \"%s\" could not be created.
                          &apsz, 1);
        break; }

        /*
         * ECLS_DOS_WRITE:
         *      DOS error. In this case iErrorCode has
         *      the OS/2 DOS error code, and pData points
         *      to the WIFileHeader in question.
         */

        case ECLS_DOS_WRITE:
        {
            // WIFileHeader *pwifh = (WIFileHeader*)pData;
            PSZ pszSysError = doshQuerySysErrorMsg(iErrorCode);
            if (pszSysError)
            {
                PSZ pTable[1] = {pszSysError};

                wpiGetMessage(strErrorMsg,
                              118,       // "OS/2 reported..."
                              pTable, 1);

                free(pszSysError);
            }
        break; }

        /*
         * ECLS_EXCEPTION:
         *      exception occured. pData is a PSZ
         *      to an error message.
         */

        case ECLS_EXCEPTION:
            strErrorMsg = (PSZ)pData;
        break;

        /*
         * ECLS_DECOMP:
         *      compression library error.
         */

        case ECLS_DECOMP:
        {
            // pszMessage2 = (PSZ)malloc(400);

            PSZ pszExpl = "Unknown error";
            switch (iErrorCode)
            {
                case BZ_SEQUENCE_ERROR:
                        pszExpl = "Invalid sequence"; break;
                case BZ_PARAM_ERROR:
                        pszExpl = "Invalid parameter"; break;
                case BZ_MEM_ERROR:
                        pszExpl = "Not enough memory"; break;
                case BZ_DATA_ERROR:         // -4
                        pszExpl = "Invalid data (CRC mismatch)"; break;
                case BZ_DATA_ERROR_MAGIC:
                        pszExpl = "Invalid data (magic bytes didn't match)"; break;
                case BZ_IO_ERROR:
                        pszExpl = "I/O error"; break;
                case BZ_UNEXPECTED_EOF:
                        pszExpl = "Unexpected end of file"; break;
                case BZ_OUTBUFF_FULL:
                        pszExpl = "Not enough room in output buffer"; break;
                case BZ_CONFIG_ERROR:
                        pszExpl = "Configuration error"; break;
                // the following are added by WIArchive
                case ZLIBERR_NO_WIFH_MAGIC:
                        pszExpl = "Invalid file header format in archive; probably outdated archive format"; break;
            }

            CHAR szCode[20];
            PSZ pTable[3] = { pszExpl, szCode };
            sprintf(szCode, "%d", iErrorCode);
            wpiGetMessage(strErrorMsg,
                          119,       // "Decompression engine reported..."
                          pTable, 2);
        break; }

        default:
            strErrorMsg = "Unknown error (guiFileError)";
    }

    // add button explanations from message file
    ULONG ulMsg = 0;
    switch(iErrorRsp)
    {
        case CBREC_RETRYCANCEL:
            ulMsg = 111;
        break;

        case CBREC_ABORTRETRYIGNORE:
            ulMsg = 112;
        break;

        default:
            ulMsg = 113;
    }

    // append msg after existing one
    BSString    strMsg2;
    wpiGetMessage(strMsg2,
                  ulMsg);
    strErrorMsg += strMsg2;

    // post msg to fnwpInstallingStatus to display msg
    int irc = (int)WinSendMsg(G_hwndMain,
                              WPIM_INSTALLERROR,
                              (MPARAM)strErrorMsg.c_str(),
                              (MPARAM)iErrorRsp);

    if (irc == CBRC_ABORT)
    {
        // "Cancel" pressed: we need to terminate
        // our thread then, or OS/2 will hang
        WinPostMsg(G_hwndMain, WM_CLOSE, 0, 0);
        _endthread();       // @@todo this doesn't work
    }

    return (irc);
}

/*
 *@@ guiFileLockedError:
 *      this is a special case of guiFileError
 *      when warpin.cpp has detected that a file
 *      should be overwritten which is currently
 *      in use by another process.
 *
 *      This must return one of these:
 *      --  CBRC_SKIP:    ignore the error, go on (skip the file).
 *      --  CBRC_RETRY:   have the install thread retry.
 *      --  CBRC_UNLOCK:  have the install thread unlock the file (DosReplaceModule).
 *      --  CBRC_DEFER:   have the install thread defer processing thru CONFIG.SYS.
 *      --  CBRC_ABORT:   have guiFileExists terminate the
 *                          install thread and quit.
 *
 *      Required callback.
 *
 *      This runs on the Install thread.
 *
 *@@added V0.9.0 (99-10-26) [umoeller]
 *@@changed V0.9.12 (2001-05-22) [umoeller]: converted all WinPostMsg to WinSendMsg
 *@@changed V0.9.12 (2001-05-31) [umoeller]: _really_ converted WinPostMsg to WinSendMsg, previous broke "locked" support
 */

int guiFileLockedError(PGUIFILEERROR pfi,
                       int iErrorCode)      // in: usually 32 == ERROR_SHARING_VIOLATION
{
    // V0.9.12 (2001-06-03) [umoeller]: this was still WinPostMsg
    int irc = (int)WinSendMsg(G_hwndMain,
                              WPIM_FILELOCKEDERROR,
                              (MPARAM)pfi,
                              (MPARAM)0);

    if (irc == CBRC_ABORT)
    {
        // "Cancel" pressed: we need to terminate
        // our thread then, or OS/2 will hang
        WinPostMsg(G_hwndMain, WM_CLOSE, 0, 0);
        _endthread();       // @@todo this doesn't work
    }

    return (irc);
}

/*
 *@@ guiDoneWithInstall:
 *      callback when all files have been
 *      unpacked successfully.
 *      After this, the Install thread will
 *      terminate itself (warpin.cpp).
 *
 *      Required callback.
 *
 *      This runs on the Install thread.
 */

void guiDoneWithInstall(void)
{
    WinPostMsg(G_hwndMain,
               WPIM_DONEWITHINSTALL,
               NULL,
               NULL);
        // this will call Configure() below
}


