
/*
 *@@sourcefile gui_shared.cpp:
 *      code shared by the various gui* files.
 *      These things are prototyped in gui.h.
 *
 *      All functions in here have the gshr* prefix.
 *
 *@@added V0.9.0 (99-10-24) [umoeller]
 *@@header "frontend\gui.h"
 */

/*
 *
 *      This file Copyright (C) 1998-2000 Ulrich Mller.
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, in version 2 as it comes in the COPYING
 *      file of this distribution.
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 */

#define OS2EMX_PLAIN_CHAR
    // this is needed for "os2emx.h"; if this is defined,
    // emx will define PSZ as _signed_ char, otherwise
    // as unsigned char

#define INCL_DOS
#define INCL_WIN
#include <os2.h>

#include <stdlib.h>
#include <stdio.h>

#include "setup.h"
#include "bldlevel.h"

// include's from helpers
#include "helpers\dosh.h"
#include "helpers\dialog.h"
#include "helpers\prfh.h"
#include "helpers\winh.h"
#include "helpers\threads.h"
#include "helpers\stringh.h"
#include "helpers\textview.h"
#include "helpers\textv_html.h"
#include "helpers\xstring.h"

// back-end includes
#include "wiarchive\wiarchive.h"

// base includes (99-11-07) [umoeller]
#include "base\bs_string.h"
#include "base\bs_errors.h"

// front-end includes
#include "frontend\fe_base.h"
#include "frontend\warpin.h"
#include "frontend\calbacks.h"

#include "frontend\dlgids.h"
#include "frontend\gui.h"

#pragma hdrstop

/* ******************************************************************
 *
 *  Global variables
 *
 ********************************************************************/

GUISETTINGS         G_GuiSettings = {0};  // implementation

HWND                G_hwndSplashDlg = NULLHANDLE;

static CONTROLDEF
    G_Bitmap =  {
                    WC_STATIC,
                    NULL,
                    WS_VISIBLE | SS_BITMAP,
                    0,          // ID
                    NULL,       // no font
                    0,
                    { SZL_AUTOSIZE, SZL_AUTOSIZE },
                    5
                },
    G_InfoText =
                {
                    WC_STATIC,
                    "WarpIN V" BLDLEVEL_VERSION " (built " __DATE__ ")\n"
                        "(C) 1998-2001 Ulrich Mller, Jens Bckman,\n"
                        "  Cornelis Bockemhl and others\n",
                    WS_VISIBLE | SS_TEXT | DT_WORDBREAK | DT_CENTER | DT_TOP,
                    1,          // ID
                    CTL_COMMON_FONT,
                    0,
                    { SZL_AUTOSIZE, SZL_AUTOSIZE },
                    0
                };

static DLGHITEM G_SplashDialogTemplate[] =
             {
                START_TABLE,
                    START_ROW(ROW_VALIGN_CENTER),
                        // column 1: bitmap
                        CONTROL_DEF(&G_Bitmap),

                        // column 2: info
                        CONTROL_DEF(&G_InfoText),
                END_TABLE
             };

/* ******************************************************************
 *
 *  Shared functions
 *
 ********************************************************************/

/*
 *@@ gshrCreateSplash:
 *
 *@@added V0.9.9 (2001-04-01) [umoeller]
 */

VOID gshrCreateSplash(VOID)
{
    if (!G_hwndSplashDlg)
    {
        APIRET arc;
        HPS hps = WinGetScreenPS(HWND_DESKTOP);
        HBITMAP hbm = GpiLoadBitmap(hps,
                                    NULLHANDLE,
                                    IDB_WARPINLOGO,
                                    0, 0);
        if (hbm)
            // bad hack
            G_Bitmap.pcszText = (const char *)hbm;

        if (!(arc = dlghCreateDlg(&G_hwndSplashDlg,
                                  NULLHANDLE,
                                  FCF_DLGBORDER | FCF_NOBYTEALIGN,
                                  WinDefDlgProc,
                                  NULL,
                                  G_SplashDialogTemplate,
                                  sizeof(G_SplashDialogTemplate) / sizeof(G_SplashDialogTemplate[0]),
                                  NULL,
                                  "9.WarpSans Bold")))
        {
            LONG lColor = 0xFFFFFF;     // white
            WinSetPresParam(G_hwndSplashDlg,
                            PP_BACKGROUNDCOLOR,
                            sizeof(lColor),
                            &lColor);
            winhCenterWindow(G_hwndSplashDlg);
            WinShowWindow(G_hwndSplashDlg, TRUE);
        }
        else
        {
            CHAR szErr[100];
            sprintf(szErr, "Error %d", arc);
            winhDebugBox(NULLHANDLE, "Error", szErr);
        }
    }
}

/*
 *@@ gshrDestroySplash:
 *
 *@@added V0.9.9 (2001-04-01) [umoeller]
 */

VOID gshrDestroySplash(VOID)
{
    WinDestroyWindow(G_hwndSplashDlg);
    G_hwndSplashDlg = NULLHANDLE;
}

/*
 *@@ gshrLoadDlg:
 *      wrapper for WinLoadDlg which throws
 *      FEFatalErrorExcpt on errors. I've had
 *      this fail recently, so this was added
 *      for future problems.
 *
 *@@added V0.9.3 (2000-04-28) [umoeller]
 */

HWND gshrLoadDlg(HWND hwndParent,
                 HWND hwndOwner,
                 PFNWP pfnDlgProc,
                 HMODULE hmod,
                 ULONG idDlg,
                 PVOID pCreateParams)
{
    HWND hwnd = WinLoadDlg(hwndParent,
                           hwndOwner,
                           pfnDlgProc,
                           hmod,
                           idDlg,
                           pCreateParams);
    if (!hwnd)
        throw (FEFatalErrorExcpt("WinLoadDlg failed."));

    return (hwnd);
}

/*
 *@@ gshrLoadSettings:
 *      this loads the user settings from WARPIN.INI.
 *      This gets called from guiInitialize.
 */

VOID gshrLoadSettings(VOID)
{
    G_GuiSettings.ulLeftView = ID_WIMI_DATABASE_DETAILSVIEW;
    G_GuiSettings.ulRightView = ID_WIMI_VIEW_FILES;

    ULONG cbData = sizeof(G_GuiSettings);
    PrfQueryProfileData(G_WpiGlobals.hiniWarpIN,
                        "GUI", "Database",
                        &G_GuiSettings, &cbData);
    cbData = sizeof(G_WpiGlobals.ulIfSameDate);
    PrfQueryProfileData(G_WpiGlobals.hiniWarpIN,
                        "Settings", "ulIfSameDate",
                        &G_WpiGlobals.ulIfSameDate, &cbData);
    cbData = sizeof(G_WpiGlobals.ulIfExistingOlder);
    PrfQueryProfileData(G_WpiGlobals.hiniWarpIN,
                        "Settings", "ulIfExistingOlder",
                        &G_WpiGlobals.ulIfExistingOlder, &cbData);
    cbData = sizeof(G_WpiGlobals.ulIfExistingNewer);
    PrfQueryProfileData(G_WpiGlobals.hiniWarpIN,
                        "Settings", "ulIfExistingNewer",
                        &G_WpiGlobals.ulIfExistingNewer, &cbData);

    cbData = sizeof(G_WpiGlobals.fSuppressREXXConfirm);
    PrfQueryProfileData(G_WpiGlobals.hiniWarpIN,
                        "Settings", "SuppressRexxConfirm",
                        &G_WpiGlobals.fSuppressREXXConfirm, &cbData);

    CHAR    szTemp[300];
    const char *pcszCommand = "WARPIN_DEFAULTAPPSPATH";
    if (getenv(pcszCommand) == NULL)
    {
        // environment variable not set:
        cbData = sizeof(szTemp);
        if (PrfQueryProfileData(G_WpiGlobals.hiniWarpIN,
                                "Settings", "DefaultAppsPath",
                                szTemp, &cbData))
        {
            // found:
            sprintf(G_WpiGlobals.szDefaultAppsPath,
                    "%s=%s",
                    pcszCommand,
                    szTemp);
            putenv(G_WpiGlobals.szDefaultAppsPath);
        }
    }

    pcszCommand = "WARPIN_DEFAULTTOOLSPATH";
    if (getenv(pcszCommand) == NULL)
    {
        // environment variable not set:
        cbData = sizeof(szTemp);
        if (PrfQueryProfileData(G_WpiGlobals.hiniWarpIN,
                                "Settings", "DefaultToolsPath",
                                szTemp, &cbData))
        {
            // found:
            sprintf(G_WpiGlobals.szDefaultToolsPath,
                    "%s=%s",
                    pcszCommand,
                    szTemp);
            putenv(G_WpiGlobals.szDefaultToolsPath);
        }
    }

    PSZ pszLogFilename = prfhQueryProfileData(G_WpiGlobals.hiniWarpIN,
                                              "Settings", "LogFile",
                                              NULL);
    if (pszLogFilename)
    {
        strcpy(G_WpiGlobals.szLogFilename, pszLogFilename);
        free(pszLogFilename);
    }
}

/*
 *@@ gshrSaveSettings:
 *      this saves the user settings back into WARPIN.INI.
 *
 *      If fl has the SETFL_SAVELOGFILEPATH bit set, the
 *      log file name is also saved.
 *
 *@@changed V0.9.9 (2001-03-30) [umoeller]: added fl
 */

VOID gshrSaveSettings(ULONG fl)
{
    PrfWriteProfileData(G_WpiGlobals.hiniWarpIN, "GUI", "Database",
                        &G_GuiSettings, sizeof(G_GuiSettings));
    PrfWriteProfileData(G_WpiGlobals.hiniWarpIN, "Settings", "ulIfSameDate",
                        &G_WpiGlobals.ulIfSameDate, sizeof(G_WpiGlobals.ulIfSameDate));
    PrfWriteProfileData(G_WpiGlobals.hiniWarpIN, "Settings", "ulIfExistingOlder",
                        &G_WpiGlobals.ulIfExistingOlder, sizeof(G_WpiGlobals.ulIfExistingOlder));
    PrfWriteProfileData(G_WpiGlobals.hiniWarpIN, "Settings", "ulIfExistingNewer",
                        &G_WpiGlobals.ulIfExistingNewer, sizeof(G_WpiGlobals.ulIfExistingNewer));

    PrfWriteProfileData(G_WpiGlobals.hiniWarpIN,
                        "Settings", "SuppressRexxConfirm",
                        &G_WpiGlobals.fSuppressREXXConfirm, sizeof(G_WpiGlobals.fSuppressREXXConfirm));


    PSZ pszCommand = "WARPIN_DEFAULTAPPSPATH";
    PSZ pszValue;
    if ((pszValue = getenv(pszCommand)))
    {
        // environment variable set:
        PrfWriteProfileString(G_WpiGlobals.hiniWarpIN,
                              "Settings", "DefaultAppsPath",
                              pszValue);
    }

    pszCommand = "WARPIN_DEFAULTTOOLSPATH";
    if ((pszValue = getenv(pszCommand)))
    {
        // environment variable set:
        PrfWriteProfileString(G_WpiGlobals.hiniWarpIN,
                              "Settings", "DefaultToolsPath",
                              pszValue);
    }

    if (fl & SETFL_SAVELOGFILEPATH)
        PrfWriteProfileString(G_WpiGlobals.hiniWarpIN,
                              "Settings", "LogFile",
                              G_WpiGlobals.szLogFilename);
}

/*
 *@@ gshrSetControlsFont:
 *      calls winhSetControlsFont with either 8.Helv or
 *      9.WarpSans.
 */

VOID gshrSetControlsFont(HWND hwnd, SHORT sIDMin, SHORT sIDMax)
{
    winhSetControlsFont(hwnd, sIDMin, sIDMax,
                        (doshIsWarp4())
                            ? "9.WarpSans"
                            : "8.Helv");
}

/*
 *@@ gshrLoadMLE:
 *      loads the MLE dialog template and creates a textview
 *      control in it.
 *
 *@@added V0.9.7 (2001-01-06) [umoeller]
 */

HWND gshrLoadMLE(ULONG ulDlgTemplateID,
                 BSString &strText,
                 BOOL fHTML)            // convert strText from HTML?
{
    HWND hwndOwner = G_hwndMain;
    if (!WinIsWindow(G_WpiGlobals.habThread1, hwndOwner))
        hwndOwner = HWND_DESKTOP;

    HWND hwnd = gshrLoadDlg(HWND_DESKTOP,
                            hwndOwner,
                            WinDefDlgProc,
                            NULLHANDLE,
                            ulDlgTemplateID,
                            NULL);
    HWND hwndTextView = txvReplaceWithTextView(hwnd,
                                               ID_WIDI_INFOTEXT,
                                               WS_VISIBLE | WS_TABSTOP,
                                               XTXF_VSCROLL | XTXF_AUTOVHIDE,
                                               2);
    PSZ psz = strdup(strText.c_str());
    if (fHTML)
    {
        // set format
        XFMTPARAGRAPH xfmtp = {0};
        xfmtp.fWordWrap = TRUE;
        xfmtp.lSpaceBefore = 5;
        xfmtp.lSpaceAfter = 5;
        WinSendMsg(hwndTextView,
                   TXM_SETPARFORMAT,
                   (MPARAM)0,
                   (MPARAM)(&xfmtp));
        txvConvertFromHTML(&psz, NULL, NULL, NULL);
    }
    else
    {
        WinSendMsg(hwndTextView,
                   TXM_SETWORDWRAP,
                   (MPARAM)(TRUE),
                   0);
        txvStripLinefeeds(&psz, 4);
    }
    WinSetWindowText(hwndTextView, psz);
    free(psz);

    return (hwnd);
}


/*
 *@@ gshrMLEMessageBox:
 *      this displays a dialog with an MLE and
 *      fills the MLE with a text from the TMF
 *      message file.
 *
 *      Used in various places, most notably the
 *      "Product info" and "Install WarpIN" dialogs.
 *
 *      Note: the internal buffer is limited to
 *      4000 characters.
 *
 *      Returns DID_OK or DID_CANCEL, depending on
 *      the user selection.
 */

ULONG gshrMLEMessageBox(ULONG ulTitleID,     // msg id from TMF file for title
                        ULONG ulMsgID,       // msg id from TMF file to display in MLE
                        BOOL fShowCancel,    // TRUE: OK and Cancel buttons, FALSE: OK only
                        PSZ* ppTable,        // for string replacements (tmfGetMessage; defaults to NULL)
                        ULONG cbTable)       // defaults to 0
{
    ULONG   ulrc = 0;

    // create install text
    BSString    strInstallInfo;
    wpiGetMessage(strInstallInfo,
                  ulMsgID,
                  ppTable,
                  cbTable);

    HWND hwndProdInfo = gshrLoadMLE(ID_WID_PRODUCTINFO,
                                    strInstallInfo,
                                    FALSE);     // no HTML

    // set window title
    BSString    strTitle;
    wpiGetMessage(strTitle,
                  ulTitleID);
    WinSetWindowText(hwndProdInfo, strTitle.c_str());

    // hide "Cancel" button
    if (!fShowCancel)
        WinShowWindow(WinWindowFromID(hwndProdInfo, DID_CANCEL), FALSE);

    winhCenterWindow(hwndProdInfo);

    // go!!
    ulrc = WinProcessDlg(hwndProdInfo);
    WinDestroyWindow(hwndProdInfo);

    return (ulrc);
}

/*
 *@@ fnwpDialogWithHelp:
 *      same as WinDefDlgProc, but with help button support.
 *      The help panel to be displayed must be in QWL_USER.
 *
 *@@added V0.9.4 (2000-07-26) [umoeller]
 */

MRESULT EXPENTRY gshr_fnwpDialogWithHelp(HWND hwndDlg,
                                         ULONG msg,
                                         MPARAM mp1,
                                         MPARAM mp2)
{
    MRESULT mrc = 0;

    switch (msg)
    {
        case WM_HELP:
            gshrDisplayHelp(WinQueryWindowULong(hwndDlg, QWL_USER));
        break;

        default:
            mrc = WinDefDlgProc(hwndDlg, msg, mp1, mp2);
    }

    return (mrc);
}

/*
 *@@ gshrPreferencesDlg:
 *      this shows the "Confirmations" dialog and
 *      updates the flags in G_WpiGlobals, if necessary.
 *
 *@@changed V0.9.12 (2001-05-31) [umoeller]: added REXX confirm
 */

VOID gshrPreferencesDlg(HWND hwndOwner)
{
    HWND hwndDlg = gshrLoadDlg(HWND_DESKTOP,
                               hwndOwner,
                               gshr_fnwpDialogWithHelp,
                               NULLHANDLE,
                               ID_WID_CONFIRMATIONS,
                               NULL);
    // set help panel for fnwpDialogWithHelp
    WinSetWindowULong(hwndDlg, QWL_USER, IDHI_CONFIRMATIONS_DLG);
    winhCenterWindow(hwndDlg);
    gshrSetControlsFont(hwndDlg, 0, 2000);

    // set controls
    ULONG idSet = 0;
    switch (G_WpiGlobals.ulIfExistingOlder)
    {
        case 0:  idSet = ID_WID_EXISTOLDER_PROMPT; break;
        case 1:  idSet = ID_WID_EXISTOLDER_SKIP; break;
        case 2:  idSet = ID_WID_EXISTOLDER_OVERWRITE; break;
    }
    winhSetDlgItemChecked(hwndDlg, idSet, TRUE);
    switch (G_WpiGlobals.ulIfSameDate)
    {
        case 0:  idSet = ID_WID_EXISTSAME_PROMPT; break;
        case 1:  idSet = ID_WID_EXISTSAME_SKIP; break;
        case 2:  idSet = ID_WID_EXISTSAME_OVERWRITE; break;
    }
    winhSetDlgItemChecked(hwndDlg, idSet, TRUE);
    switch (G_WpiGlobals.ulIfExistingNewer)
    {
        case 0:  idSet = ID_WID_EXISTNEWER_PROMPT; break;
        case 1:  idSet = ID_WID_EXISTNEWER_SKIP; break;
        case 2:  idSet = ID_WID_EXISTNEWER_OVERWRITE; break;
    }
    winhSetDlgItemChecked(hwndDlg, idSet, TRUE);

    // V0.9.12 (2001-05-31) [umoeller]
    winhSetDlgItemChecked(hwndDlg,
                          ID_WID_SHOWREXXCONFIRM,
                          (!G_WpiGlobals.fSuppressREXXConfirm));

    // log file name
    HWND hwndEF = WinWindowFromID(hwndDlg, ID_WID_LOGFILE_ENTRY);
    winhSetEntryFieldLimit(hwndEF, CCHMAXPATH);
    WinSetWindowText(hwndEF,
                     G_WpiGlobals.szLogFilename);

    if (WinProcessDlg(hwndDlg) == DID_OK)
    {
        // read controls
        if (winhIsDlgItemChecked(hwndDlg, ID_WID_EXISTOLDER_PROMPT))
            G_WpiGlobals.ulIfExistingOlder = 0;
        if (winhIsDlgItemChecked(hwndDlg, ID_WID_EXISTOLDER_SKIP))
            G_WpiGlobals.ulIfExistingOlder = 1;
        if (winhIsDlgItemChecked(hwndDlg, ID_WID_EXISTOLDER_OVERWRITE))
            G_WpiGlobals.ulIfExistingOlder = 2;
        if (winhIsDlgItemChecked(hwndDlg, ID_WID_EXISTSAME_PROMPT))
            G_WpiGlobals.ulIfSameDate = 0;
        if (winhIsDlgItemChecked(hwndDlg, ID_WID_EXISTSAME_SKIP))
            G_WpiGlobals.ulIfSameDate = 1;
        if (winhIsDlgItemChecked(hwndDlg, ID_WID_EXISTSAME_OVERWRITE))
            G_WpiGlobals.ulIfSameDate = 2;
        if (winhIsDlgItemChecked(hwndDlg, ID_WID_EXISTNEWER_PROMPT))
            G_WpiGlobals.ulIfExistingNewer = 0;
        if (winhIsDlgItemChecked(hwndDlg, ID_WID_EXISTNEWER_SKIP))
            G_WpiGlobals.ulIfExistingNewer = 1;
        if (winhIsDlgItemChecked(hwndDlg, ID_WID_EXISTNEWER_OVERWRITE))
            G_WpiGlobals.ulIfExistingNewer = 2;

        // V0.9.12 (2001-05-31) [umoeller]
        G_WpiGlobals.fSuppressREXXConfirm
                = !winhIsDlgItemChecked(hwndDlg,
                                        ID_WID_SHOWREXXCONFIRM);

        ULONG fl = 0;

        PSZ pszLogFilename = winhQueryWindowText(hwndEF);
        if (pszLogFilename)
        {
            if (strcmp(pszLogFilename, G_WpiGlobals.szLogFilename))
            {
                // changed:
                if (0 == doshIsValidFileName(pszLogFilename,
                                             TRUE))
                {
                    // valid file name:
                    strcpy(G_WpiGlobals.szLogFilename, pszLogFilename);
                    fl |= SETFL_SAVELOGFILEPATH;
                }
                else
                    wpiMessageBox(102,      // error
                                  204,      // invalid log file name
                                  MB_OK);
            }

            free(pszLogFilename);
        }
        else
        {
            // no log file specified:
            G_WpiGlobals.szLogFilename[0] = '\0';
            fl |= SETFL_SAVELOGFILEPATH;
        }

        gshrSaveSettings(fl);
    }
    WinDestroyWindow(hwndDlg);
}

/*
 *@@ gshrDefaultPathsDlg:
 *      this shows the "default directories" dialog
 *      and updates G_WpiGlobals.
 *
 *@@added V0.9.4 (2000-07-26) [umoeller]
 */

VOID gshrDefaultPathsDlg(HWND hwndOwner)
{
    HWND hwndDlg = gshrLoadDlg(HWND_DESKTOP,
                               hwndOwner,
                               gshr_fnwpDialogWithHelp,
                               NULLHANDLE,
                               ID_WID_DEFAULTPATHS,
                               NULL);
    // set help panel for fnwpDialogWithHelp
    WinSetWindowULong(hwndDlg, QWL_USER, IDHI_DEFAULTPATHS_DLG);
    winhCenterWindow(hwndDlg);
    gshrSetControlsFont(hwndDlg, 0, 2000);

    HWND    hwndApps = WinWindowFromID(hwndDlg, ID_WIDI_DEFAULTAPPSPATH),
            hwndTools = WinWindowFromID(hwndDlg, ID_WIDI_DEFAULTTOOLSPATH);

    PSZ p = getenv("WARPIN_DEFAULTAPPSPATH");
    WinSetWindowText(hwndApps, p);
    p = getenv("WARPIN_DEFAULTTOOLSPATH");
    WinSetWindowText(hwndTools, p);

    if (WinProcessDlg(hwndDlg) == DID_OK)
    {
        PSZ psz = winhQueryWindowText(hwndApps);
        if (psz)
        {
            sprintf(G_WpiGlobals.szDefaultAppsPath,
                    "WARPIN_DEFAULTAPPSPATH=%s", psz);
            putenv(G_WpiGlobals.szDefaultAppsPath);
            free(psz);
        }

        psz = winhQueryWindowText(hwndTools);
        if (psz)
        {
            sprintf(G_WpiGlobals.szDefaultToolsPath,
                    "WARPIN_DEFAULTTOOLSPATH=%s", psz);
            putenv(G_WpiGlobals.szDefaultToolsPath);
            free(psz);
        }

        gshrSaveSettings(0);
    }

    WinDestroyWindow(hwndDlg);
}

/*
 *@@ gshrLoadConfirmDlg:
 *      loads the specified confirmation window
 *      and sets the dialog fields according to
 *      the file data.
 *
 *      This is shared by ShowFileExistsDlg and
 *      ShowFileLockedDlg.
 *
 *@@added V0.9.0 (99-10-26) [umoeller]
 */

HWND gshrLoadConfirmDlg(HWND hwndOwner,
                        ULONG ulDlgID,
                        PFNWP pfnwp,
                        PGUIFILEERROR pfi)
{
    HWND hwndConfirm = gshrLoadDlg(HWND_DESKTOP, hwndOwner,
                                   pfnwp, // fnwpFileExistsOrLocked,
                                   NULLHANDLE,
                                   ulDlgID,          // variable
                                   NULL);
    gshrSetControlsFont(hwndConfirm, 0, 1003);

    // now set fields in dialog
    // 1)   existing file
    CHAR szTemp[300];
    strhFileDate(szTemp, &(pfi->fs3Existing.fdateLastWrite), G_ulDateFormat, G_szDateSep[0]);
    WinSetDlgItemText(hwndConfirm, ID_WIDI_EXISTS_DATEOLD,
                    szTemp);
    strhFileTime(szTemp, &(pfi->fs3Existing.ftimeLastWrite), G_ulTimeFormat, G_szTimeSep[0]);
    WinSetDlgItemText(hwndConfirm, ID_WIDI_EXISTS_TIMEOLD,
                    szTemp);

    WinSetDlgItemText(hwndConfirm, ID_WIDI_EXISTS_SIZEOLD,
                    strhThousandsULong(szTemp, pfi->fs3Existing.cbFile, G_szThousand[0]));

    // 2)   file in archive
    strhFileDate(szTemp, &pfi->fDateHeader, G_ulDateFormat, G_szDateSep[0]);
    WinSetDlgItemText(hwndConfirm, ID_WIDI_EXISTS_DATENEW,
                    szTemp);
    strhFileTime(szTemp, &pfi->fTimeHeader, G_ulTimeFormat, G_szTimeSep[0]);
    WinSetDlgItemText(hwndConfirm, ID_WIDI_EXISTS_TIMENEW,
                    szTemp);

    WinSetDlgItemText(hwndConfirm, ID_WIDI_EXISTS_SIZENEW,
                      strhThousandsULong(szTemp, pfi->pwifh->origsize, G_szThousand[0]));

    // 3)  issue warning text?
    HWND hwndInfoText = WinWindowFromID(hwndConfirm, ID_WIDI_EXISTS_INFOTEXT);
    if (pfi->iComp < 0)
    {
        WinSetWindowText(hwndInfoText,
                         G_NLSStrings.pszWarningExistingNewer);
                                // "Warning: The existing file is newer!"
        WinShowWindow(hwndInfoText, TRUE);
    }
    else
        WinShowWindow(hwndInfoText, FALSE);

    return (hwndConfirm);
}

/*
 *@@ gshrDisplayHelp:
 *
 *      If ulHelpPanel == 0, the "Using Help" panel
 *      is displayed.
 *
 *@@added V0.9.4 (2000-07-26) [umoeller]
 */

BOOL gshrDisplayHelp(ULONG ulHelpPanel)
{
    return (WinSendMsg(G_hwndHelpInstance,
                       HM_DISPLAY_HELP,
                       (MPARAM)ulHelpPanel,
                       (MPARAM)(    (ulHelpPanel != 0)
                                    ? HM_RESOURCEID
                                    : 0))
                == 0);
}

/*
 *@@ gshrProcessDefMenuItems:
 *      common message handler for all window procs
 *      which handle the items from the WarpIN main
 *      menu plus help messages.
 *
 *      This returns TRUE if the item was processed,
 *      FALSE otherwise.
 *
 *@@added V0.9.0 (99-10-25) [umoeller]
 *@@changed V0.9.3 (2000-05-11) [umoeller]: "Cancel" didn't close main window; fixed
 *@@changed V0.9.4 (2000-07-26) [umoeller]: added "default dirs" settings
 *@@changed V0.9.4 (2000-07-26) [umoeller]: changed prototype, added help handling
 */

BOOL gshrDefWindowProc(HWND hwnd,             // in: from msg proc
                       ULONG msg,             // in: from msg proc
                       MPARAM mp1,
                       MPARAM mp2,
                       ULONG ulDefHelpPanel)  // in: help
{
    BOOL brc = TRUE;        // processed

    switch (msg)
    {
        case WM_COMMAND:
        {
            SHORT sCommand = SHORT1FROMMP(mp1);
            switch (sCommand)
            {
                /*
                 * ID_WIMI_PREFERENCES:
                 *      "Confirmations" settings menu item
                 */

                case ID_WIMI_PREFERENCES:
                    gshrPreferencesDlg(hwnd);
                break;

                /*
                 * ID_WIMI_DEFAULTPATHS:
                 *      "default directories" settings menu item
                 */

                case ID_WIMI_DEFAULTPATHS:
                    gshrDefaultPathsDlg(hwnd);
                break;

                case ID_WIMI_HELPINDEX:
                break;

                case ID_WIMI_HELPGENERAL:
                    gshrDisplayHelp(ulDefHelpPanel);
                break;

                case ID_WIMI_HELPUSINGHELP:
                    gshrDisplayHelp(0);
                break;

                case ID_WIMI_HELPPRODUCTINFO:
                    guiProductInfo(hwnd, FALSE);
                break;

                case ID_WIMI_EXIT:
                    // exit:
                    if (wpiMessageBox(100,
                                      172,  // "Sure you wanna cancel?"
                                      MB_ICONQUESTION | MB_YESNO | MB_MOVEABLE)
                            == MBID_YES)
                        WinPostMsg(hwnd, WM_QUIT, 0, 0);
                break;

                default:
                    brc = FALSE;        // not processed
            } // switch (sCommand)
        break; } // case WM_COMMAND:

        case WM_HELP:
            gshrDisplayHelp(ulDefHelpPanel);
        break;

        default:
            brc = FALSE;
    }

    return (brc);
}

/*
 *@@ guiSleep:
 *
 *      Required callback.
 *
 *@@added V0.9.2 (2000-03-05) [umoeller]
 */

VOID guiSleep(ULONG ulSleep)    // in: sleep time in milliseconds
{
    ULONG ul = 0;
    QMSG qmsg;
    for (ul = 0;
         ul < (ulSleep / 50);
         ul++)
    {
        DosSleep(50);
        while (WinPeekMsg(G_WpiGlobals.habThread1,
                          &qmsg, 0, 0, 0,
                          PM_REMOVE))
            WinDispatchMsg(G_WpiGlobals.habThread1, &qmsg);

    }
}

