
/*
 *@@sourcefile wpi_configure.cpp:
 *      this has the WarpIN system configuration stuff.
 *      See wpiConfigure.
 *
 *      This has been separated from warpin.cpp.
 *
 *@@added V0.9.1 (2000-02-06) [umoeller]
 */

/*      This file Copyright (C) 1998-2000 Ulrich Mller.
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, in version 2 as it comes in the COPYING
 *      file of this distribution.
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 */

#define OS2EMX_PLAIN_CHAR
    // this is needed for "os2emx.h"; if this is defined,
    // emx will define PSZ as _signed_ char, otherwise
    // as unsigned char

#define INCL_DOS
#define INCL_DOSERRORS
#define INCL_WINSHELLDATA
#define INCL_WINWORKPLACE
#include <os2.h>

#include <stdlib.h>
#ifdef __IBMCPP__
    #include <direct.h>
#endif
#include <string.h>
#include <setjmp.h>             // needed for except.h
#include <assert.h>             // needed for except.h

#include <ctype.h>

#ifdef __IBMCPP__
    #include <terminat.h>
#endif

#include "setup.h"              // was missing (99-12-23) [umoeller]

// include's from helpers
#include "helpers\configsys.h"
#include "helpers\dosh.h"
#include "helpers\procstat.h"
#include "helpers\stringh.h"
#include "helpers\threads.h"
#include "helpers\xstring.h"

// back-end includes
#include "wiarchive\wiarchive.h"

// base includes (99-11-07) [umoeller]
#include "base\bs_string.h"
#include "base\bs_errors.h"
#include "base\bs_logger.h"
#include "base\bs_config.h"
#include "base\bs_config_impl.h"
#include "base\bs_script.h"

// front-end includes
#include "frontend\fe_base.h"
#include "frontend\fe_archive.h"
#include "frontend\fe_database.h"
#include "frontend\fe_job.h"
#include "frontend\fe_package.h"
#include "frontend\fe_package_arc.h"

#include "frontend\warpin.h"
#include "frontend\calbacks.h"

#include "helpers\except.h"     // must be at the bottom because wiarchive includes stdio

#pragma hdrstop

/* ******************************************************************
 *
 *  Global variables
 *
 ********************************************************************/

extern WPIGLOBALS      G_WpiGlobals;
            // in warpin.cpp

extern FEDatabase*     G_pDatabase;
            // in warpin.cpp

/* ******************************************************************
 *
 *  Killing processes
 *
 ********************************************************************/

/*
 *@@ wpiKillOneProcess:
 *      checks the process list for the specified process
 *      and prompts for whether it should be killed.
 *
 *      Returns TRUE if the process list should be checked
 *      again.
 *
 *      Gets called in two situations:
 *
 *      -- from wpiKillProcesses, before the Install thread
 *         is started;
 *
 *      -- from guiStartRemove, before the Database thread
 *         starts removing packages.
 *
 *@@added V0.9.4 (2000-07-01) [umoeller]
 *@@changed V0.9.6 (2000-10-26) [umoeller]: added exception handling
 *@@changed V0.9.9 (2001-03-30) [umoeller]: added pLogFile
 *@@changed V0.9.10 (2001-04-08) [umoeller]: added DosQProcStat check
 *@@changed V0.9.11 (2001-04-25) [umoeller]: fixed infinite questioning if DosQProcStat failed
 */

BOOL wpiKillOneProcess(BSKillProcess *pKill,
                       BSFileLogger *pLogFile)   // in: log file or NULL
{
    BOOL            fReloop = FALSE;
    PQPROCSTAT16    pInfo = NULL;
    PSZ             apszTable[2];
    CHAR            szPID[30];

    apszTable[0] = (PSZ)pKill->_pszKillProcess;

    TRY_LOUD(excpt1)
    {
        APIRET arc;
        if (arc = prc16GetInfo(&pInfo))
        {
            CHAR sz[20];
            sprintf(sz, "%d", arc);
            apszTable[0] = sz;
            apszTable[1] = (PSZ)pKill->_pszKillProcess;

            // DosQProcStat failed: V0.9.10 (2001-04-08) [umoeller]
            if (wpiMessageBox(108, // warning
                              205, // "DosQProcStat failed, retry"
                              MB_ICONEXCLAMATION | MB_OKCANCEL,
                              apszTable, 2)
                    == DID_CANCEL)
                throw BSCancelExcpt();

            // "OK" pressed: try again
            // fReloop = TRUE;
            // noooo... V0.9.11 (2001-04-25) [umoeller]
            fReloop = FALSE;
        }
        else
        {
            // check if that process is running
            PQPROCESS16 pProcess = prc16FindProcessFromName(pInfo,
                                                            pKill->_pszKillProcess);
            if (pProcess)
            {
                // running: prompt
                sprintf(szPID, "0x%lX", pProcess->usPID);
                apszTable[1] = szPID;
                ULONG mbrc = wpiMessageBox(108, // warning
                                           180, // "kill these programs first"
                                           MB_ICONEXCLAMATION | MB_YESNOCANCEL,
                                           apszTable, 2);
                switch (mbrc)
                {
                    case MBID_YES:
                        // kill process
                        DosKillProcess(DKP_PROCESS, pProcess->usPID);

                        // sleep a second
                        guiSleep(1000);
                        fReloop = TRUE;
                    break;

                    case MBID_NO:
                        // check again
                        fReloop = TRUE;
                    break;

                    case MBID_CANCEL:
                        throw BSCancelExcpt();
                }
            }
        }
    }
    CATCH(excpt1)
    {
        ULONG mbrc = wpiMessageBox(108, // warning
                                   196, // "exception killing %1"
                                   MB_ICONEXCLAMATION | MB_OK,
                                   apszTable, 1);
        fReloop = FALSE;
    } END_CATCH();

    if (pInfo)
        prc16FreeInfo(pInfo);

    return (fReloop);
}

/*
 *@@ wpiKillProcesses:
 *      this prompts the user for each process which needs to
 *      be killed by going thru all packages which are to be
 *      installed and checking the KILLPROCESS attributes of
 *      each. If such an attribute is found and that process
 *      is currently running (which is checked for here, by
 *      calling prc32FindProcessFromName), the user is offered
 *      to have the process killed.
 *
 *      Gets called from EnterArchiveMode before the Install
 *      thread is started. Calls wpiKillOneProcess in turn.
 *
 *@@added V0.9.2 (2000-03-05) [umoeller]
 *@@changed V0.9.3 (2000-05-05) [umoeller]: switched to 16-bit DosQProcStat
 *@@changed V0.9.9 (2001-03-30) [umoeller]: added pLogFile
 */

ULONG wpiKillProcesses(BSFileLogger *pLogFile) // in: log file or NULL
{
    ULONG   ulrc = 0;

#ifndef __EMX__
    // go thru the list of all packages that were installed
    list<FEJobBase*>::iterator JobStart = G_WpiGlobals.AllJobsList.begin(),
                               JobEnd = G_WpiGlobals.AllJobsList.end();
    for (; JobStart != JobEnd; JobStart++)
    {
        FEJobBase *pTemp = *JobStart;
        FEInstallJob *pJobThis = pTemp->IsInstallJob();
        if (pJobThis)
        {
            if (pJobThis->Is2BeInstalled())
            {
                FEArcPackagePck *pPackageThis = (FEArcPackagePck*)(pJobThis->_pArcPackage);

                // go thru the list of all config objects; this
                // list holds all objects as base objects, so
                // we need to type check
                list<BSConfigBase*>::iterator
                        CfgStart = pPackageThis->_pDecl->_listConfigObjects.begin(),
                        CfgEnd = pPackageThis->_pDecl->_listConfigObjects.end();
                for (; CfgStart != CfgEnd; CfgStart++)
                {
                    // check if this is our class
                    BSKillProcess *pKill = (**CfgStart).IsKillProcess();
                    if (!pKill)
                        // not the class we need: take next then
                        continue;

                    BOOL fReloop = FALSE;

                    do
                    {
                        // OK, we got a KILLPROCESS attribute:
                        fReloop = wpiKillOneProcess(pKill,
                                                    pLogFile);
                    } while (fReloop);
                }
            }
        }
    }
#endif

    return (ulrc);
}

/* ******************************************************************
 *
 *  System configuration
 *
 ********************************************************************/

/*
 *@@ wpiConfigSys:
 *      this changes CONFIG.SYS by going thru the list
 *      of packages in WPIGLOBALS which were installed,
 *      using the BSConfigSys class (bs_config.cpp).
 *
 *      Each FEArcPackagePck has a list of BSConfigSys instances
 *      which at this point have the necessary information
 *      (set up at initialization by FEArcPackagePck::ParsePckAttribsConfigSys).
 *
 *      This resolves macros.
 *
 *      Returns FALSE if any error occured.
 *
 *      Gets called from wpiConfigure.
 *
 *@@changed V0.9.0 (99-11-01) [umoeller]: adjusted for FEInstallJob
 *@@changed V0.9.0 (99-11-01) [umoeller]: added "global install status" window
 *@@changed V0.9.3 (2000-05-12) [umoeller]: added pszBackup
 *@@changed V0.9.6 (2000-10-27) [umoeller]: made CONFIG.SYS backup part of WPIGLOBALS
 *@@changed V0.9.9 (2001-03-30) [umoeller]: added pLogFile
 */

BOOL wpiConfigSys(BSFileLogger *pLogFile) // in: log file or NULL
{
    BOOL    brc = TRUE;

    if (G_WpiGlobals.ulAllowConfig & PCK_CONFIGSYS)
    {
        guiUpdateGlobalStatus(GSTAT_CONFIGSYS);

        // create instance of BSConfigSys; this will load CONFIG.SYS
        BSConfigSys* pConfigSys = new BSConfigSys;

        // go thru the list of all packages that were installed
        list<FEJobBase*>::iterator JobStart = G_WpiGlobals.AllJobsList.begin(),
                                   JobEnd = G_WpiGlobals.AllJobsList.end();
        for (; JobStart != JobEnd; JobStart++)
        {
            FEJobBase *pTemp = *JobStart;
            FEInstallJob *pJobThis = pTemp->IsInstallJob();
            if (pJobThis)
            {
                if (pJobThis->Is2BeInstalled())
                {
                    FEArcPackagePck *pPackageThis = (FEArcPackagePck*)(pJobThis->_pArcPackage);
                    // now go thru the CONFIG.SYS manipulators in the
                    // FEPackage and apply them to the BSConfigSys object;
                    // there may be none of these
                    /* list<BSCfgSysManip*>::iterator ManipFirst = pPackageThis->_listCfgSysManips.begin(),
                                                 ManipEnd = pPackageThis->_listCfgSysManips.end(); */

                    // go thru the list of all config objects; this
                    // list holds all objects as base objects, so
                    // we need to type check
                    list<BSConfigBase*>::iterator
                            CfgStart = pPackageThis->_pDecl->_listConfigObjects.begin(),
                            CfgEnd = pPackageThis->_pDecl->_listConfigObjects.end();
                    for (; CfgStart != CfgEnd; CfgStart++)
                    {
                        // check if this is our class
                        BSCfgSysManip *pManip = (**CfgStart).IsCfgSysManip();
                        if (!pManip)
                            // not the class we need: take next then
                            continue;

                        // resolve macros
                        wpiResolveMacros(&pManip->_ConfigManip.pszNewLine);
                        // apply to CONFIG.SYS text
                        try
                        {
                            pConfigSys->Manipulate(*pManip,
                                                        // the manipulator
                                                   &pPackageThis->_logCfgSysDone,
                                                        // CONFIG.SYS logger for database
                                                   pLogFile);
                                // this may throw BSConfigExcpt
                            G_WpiGlobals.fConfigSysChanged = TRUE;
                        }
                        catch(BSConfigExcpt&)
                        {
                            wpiMessageBox(102,
                                          124, // "error"
                                          MB_ICONEXCLAMATION | MB_OK);
                            brc = FALSE;
                        }
                        catch(...)
                        {
                            throw;      // rethrow
                        }
                    }
                }
            }
        }

        // rewrite CONFIG.SYS
        if (pConfigSys->Flush(G_WpiGlobals.szConfigSysBackup,
                              pLogFile) == 0)
            G_WpiGlobals.fConfigSysChanged = TRUE;

        delete pConfigSys;
    } // end if (G_WpiGlobals.ulDoConfiguration | PCK_CONFIGSYS)

    return (brc);
}

/*
 *@@ wpiWPSClasses:
 *      this registers and replaces WPS classes by going thru
 *      the list of packages in WPIGLOBALS which were installed.
 *
 *      Each FEArcPackagePck has a list of BSRegisterClass instances
 *      and another one of BSReplaceClass instances,
 *      which at this point have the necessary information
 *      (set up at initialization by FEArcPackagePck::ParsePckAttribsWPSClasses).
 *
 *      This invokes BSRegisterClass::Register for each
 *      class to be registered, after having replaced macros.
 *
 *      This resolves macros.
 *
 *      Returns FALSE if any error occured.
 *
 *      Gets called from wpiConfigure.
 *
 *@@changed V0.9.0 (99-11-01) [umoeller]: adjusted for FEInstallJob
 *@@changed V0.9.0 (99-11-01) [umoeller]: added "global install status" window
 *@@changed V0.9.1 (2000-01-06) [umoeller]: "already registered" prompt now only comes up if DLL is different
 *@@changed V0.9.9 (2001-03-30) [umoeller]: added pLogFile
 */

BOOL wpiWPSClasses(BSFileLogger *pLogFile) // in: log file or NULL
{
    BOOL    brc = TRUE;

    if (G_WpiGlobals.ulAllowConfig & PCK_WPSCLASSES)
    {
        guiUpdateGlobalStatus(GSTAT_WPSCLASSES);

        // go thru the list of all packages that were installed
        list<FEJobBase*>::iterator JobStart = G_WpiGlobals.AllJobsList.begin(),
                                   JobEnd = G_WpiGlobals.AllJobsList.end();
        for (; JobStart != JobEnd; JobStart++)
        {
            FEJobBase *pTemp = *JobStart;
            FEInstallJob *pJobThis = pTemp->IsInstallJob();
            if (pJobThis)
            {

                if (pJobThis->Is2BeInstalled())
                {
                    FEArcPackagePck *pPackageThis = (FEArcPackagePck*)(pJobThis->_pArcPackage);
                    // now go thru the BSRegisterClass objects in the list in
                    // FEPackage and invoke Register() on them;
                    // there may be none of these

                    // go thru the list of all config objects; this
                    // list holds all objects as base objects, so
                    // we need to type check
                    list<BSConfigBase*>::iterator
                            CfgStart = pPackageThis->_pDecl->_listConfigObjects.begin(),
                            CfgEnd = pPackageThis->_pDecl->_listConfigObjects.end();
                    for (; CfgStart != CfgEnd; CfgStart++)
                    {
                        // check if this is our class
                        BSRegisterClass *pRegThis = (**CfgStart).IsRegisterClass();
                        if (!pRegThis)
                            // not the class we need: take next then
                            continue;

                        // resolve macros
                        wpiResolveMacros(&(pRegThis->_pszModule));

                        BOOL    fRetry = FALSE;
                        BOOL    fReplace = FALSE;

                        do {
                            fRetry = FALSE;

                            try
                            {
                                pRegThis->Register(fReplace, // initially FALSE
                                                   &pPackageThis->_logRegisterClassesDone,
                                                        // classes logger for database
                                                   pLogFile);
                                G_WpiGlobals.fWPSClassesChanged = TRUE;
                            }
                            catch(BSConfigExcpt& X)
                            {
                                brc = FALSE;
                                // CHAR    szError[1000] = "unknown";
                                PSZ     apsz[2];
                                ULONG   ulRplCount = 0;
                                ULONG   ulMsg = 0;
                                ULONG   ulFlags = MB_ICONEXCLAMATION | MB_OK;
                                PSZ     pszSysError = 0;

                                switch(X._iErrorCode)
                                {
                                    case REGEXCPT_QUERYCLASSLIST:
                                        ulMsg = 135; // error querying WPS class list
                                    break;

                                    case REGEXCPT_ALREADYREGISTERED:
                                        // BSRegisterClass::Register(FALSE) has determined
                                        // that the class is already registered;
                                        // BSConfigExcpt.pszSubstring has the currently
                                        // registered DLL; if the two are different,
                                        // ask user
                                        if (stricmp(pRegThis->_pszModule, // new DLL
                                                    X._strDescription.c_str()) // registered DLL
                                               != 0)
                                        {
                                            // different:
                                            apsz[0] = pRegThis->_pszClassName;
                                            apsz[1] = (PSZ)X._strDescription.c_str();
                                            ulRplCount = 2;
                                            ulMsg = 136; // The class "%1" is already registered with the DLL "%2".
                                            ulFlags = MB_ICONQUESTION | MB_YESNO;
                                        }
                                        else
                                        {
                                            // the two are the same:
                                            fRetry = TRUE;
                                            fReplace = TRUE;
                                        }
                                    break;

                                    case REGEXCPT_REGISTER:
                                    {
                                        // BSRegisterClass::Register failed;
                                        // BSConfigExcpt.iData has the APIRET of winhRegisterClass
                                        pszSysError = doshQuerySysErrorMsg(X._iData);
                                        apsz[0] = pRegThis->_pszClassName;
                                        apsz[1] = pszSysError;
                                        ulRplCount = 2;
                                        ulMsg = 137; // The class "%1" could not be registered. The following error was reported:
                                    break; }
                                } // end switch

                                if (ulMsg)
                                    if (wpiMessageBox(138,  // "WarpIN: Class Registering Error"
                                                      ulMsg,
                                                      ulFlags,
                                                      apsz, ulRplCount)
                                            == MBID_YES)
                                    {
                                        fRetry = TRUE;
                                        fReplace = TRUE;
                                    }

                                if (pszSysError)
                                    free(pszSysError);
                            }
                        } while (fRetry);
                    }

                    // now go thru the BSReplaceClass objects in the list in
                    // FEPackage and invoke Replace() on them;
                    // there may be none of these

                    // go thru the list of all config objects; this
                    // list holds all objects as base objects, so
                    // we need to type check
                    // list<BSConfigBase*>::iterator
                            CfgStart = pPackageThis->_pDecl->_listConfigObjects.begin(),
                            CfgEnd = pPackageThis->_pDecl->_listConfigObjects.end();
                    for (; CfgStart != CfgEnd; CfgStart++)
                    {
                        // check if this is our class
                        BSReplaceClass *pReplThis = (**CfgStart).IsReplaceClass();
                        if (!pReplThis)
                            // not the class we need: take next then
                            continue;

                        try
                        {
                            pReplThis->Replace(&pPackageThis->_logReplaceClassDone,
                                                    // classes logger for database
                                               pLogFile);
                            G_WpiGlobals.fWPSClassesChanged = TRUE;
                        }
                        catch(BSConfigExcpt& X)
                        {
                            brc = FALSE;
                            PSZ     apsz[2];
                            ULONG   ulRplCount = 0,
                                    ulMsg = 142;    // unknown error

                            switch(X._iErrorCode)
                            {
                                case REGEXCPT_REPLACE:
                                                // error querying WPS class list
                                    apsz[0] = pReplThis->_pszOldClassName;
                                    apsz[1] = pReplThis->_pszNewClassName;
                                    ulRplCount = 2;
                                    ulMsg = 140;    // Error replacing WPS class "%1" with "%2".
                                break;
                            } // end switch

                            wpiMessageBox(139,   // WarpIN: Class Replacement Error
                                          ulMsg,
                                          MB_ICONEXCLAMATION | MB_OK,
                                          apsz, ulRplCount);
                        }
                    }
                } // end if (ppiThis->ulSelection)
            }
        }
    } // end if (G_WpiGlobals.ulDoConfiguration | PCK_WPSCLASSES)

    return (brc);
}

/*
 *@@ wpiCreateObjects:
 *      this creates WPS objects by going thru the list of
 *      packages in WPIGLOBALS which were installed.
 *
 *      Each FEArcPackagePck has a list of BSCreateObject instances
 *      which at this point have the necessary information
 *      (set up at initialization by FEArcPackagePck::ParsePckAttribsWPSObjects).
 *
 *      This invokes BSCreateWPSObject::CreateObject for each
 *      object to be created, which in turn stores the
 *      HOBJECT in its instance data, after having replaced macros.
 *
 *      This resolves macros.
 *
 *      Returns FALSE if any error occured.
 *
 *      Gets called from wpiConfigure.
 *
 *@@changed V0.9.0 (99-11-01) [umoeller]: adjusted for FEInstallJob
 *@@changed V0.9.0 (99-11-01) [umoeller]: added "global install status" window
 *@@changed V0.9.9 (2001-03-30) [umoeller]: added pLogFile
 */

BOOL wpiCreateObjects(BSFileLogger *pLogFile) // in: log file or NULL
{
    BOOL    brc = TRUE;

    if (G_WpiGlobals.ulAllowConfig & PCK_WPSOBJECTS)
    {
        guiUpdateGlobalStatus(GSTAT_WPSOBJECTS);

        // go thru the list of all packages that were installed
        list<FEJobBase*>::iterator JobStart = G_WpiGlobals.AllJobsList.begin(),
                                   JobEnd = G_WpiGlobals.AllJobsList.end();
        for (; JobStart != JobEnd; JobStart++)
        {
            FEJobBase *pTemp = *JobStart;
            FEInstallJob *pJobThis = pTemp->IsInstallJob();
            if (pJobThis)
            {

                if (pJobThis->Is2BeInstalled())
                {
                    FEArcPackagePck *pPackageThis = (FEArcPackagePck*)(pJobThis->_pArcPackage);
                    // now go thru the CreateWPSObjects objects in the list in
                    // FEPackage and invoke CreateObject() on them;
                    // there may be none of these

                    // go thru the list of all config objects; this
                    // list holds all objects as base objects, so
                    // we need to type check
                    list<BSConfigBase*>::iterator
                            CfgStart = pPackageThis->_pDecl->_listConfigObjects.begin(),
                            CfgEnd = pPackageThis->_pDecl->_listConfigObjects.end();
                    for (; CfgStart != CfgEnd; CfgStart++)
                    {
                        // check if this is our class
                        BSCreateWPSObject *pcroThis = (**CfgStart).IsCreateWPSObject();
                        if (!pcroThis)
                            // not the class we need: take next then
                            continue;

                        // resolve macros
                        wpiResolveMacros(&(pcroThis->_pszSetupString));
                        wpiResolveMacros(&(pcroThis->_pszLocation));

                        try
                        {
                            pcroThis->CreateObject(&pPackageThis->_logWPSObjectsDone,
                                                // WPS objects logger for database
                                                   pLogFile);
                                    // this may throw BSConfigExcpt
                            G_WpiGlobals.fWPSObjectsCreated = TRUE;
                        }
                        catch(BSConfigExcpt&)
                        {
                            // error occured:
                            PSZ apsz[4] =  {
                                              pcroThis->_pszTitle,
                                              pcroThis->_pszClassName,
                                              pcroThis->_pszLocation,
                                              pcroThis->_pszSetupString
                                           };
                            brc = FALSE;
                            wpiMessageBox(102, // "WarpIN: Error"
                                          141,   // "WPS object could not be created"
                                          MB_ICONEXCLAMATION | MB_OK,
                                          apsz, 4);
                        }
                    }
                }
            }
        } // end for (; JobStart != JobEnd; JobStart++)


    } // end if (G_WpiGlobals.ulDoConfiguration | PCK_WPSOBJECTS)

    return (brc);
}

/*
 *@@ wpiWriteProfiles:
 *      this calls the executables specified by the WRITEPROFILE
 *      tags by going thru the list of packages in WPIGLOBALS
 *      which were installed.
 *
 *      Each FEArcPackagePck has a list of BSWriteProfile instances
 *      which at this point have the necessary information
 *      (set up at initialization by FEArcPackagePck::ParsePckAttribsProfiles).
 *
 *      This resolves macros.
 *
 *      Returns FALSE if any error occured.
 *
 *      Gets called from wpiConfigure.
 *
 *@@added V0.9.1 (2000-02-07) [umoeller]
 *@@changed V0.9.9 (2001-03-30) [umoeller]: added pLogFile
 */

BOOL wpiWriteProfiles(BSFileLogger *pLogFile) // in: log file or NULL
{
    BOOL    brc = TRUE;

    // if (G_WpiGlobals.ulAllowConfig & PCK_EXECUTES)
    {
        // guiUpdateGlobalStatus(GSTAT_WPSOBJECTS);

        // go thru the list of all packages that were installed
        list<FEJobBase*>::iterator JobStart = G_WpiGlobals.AllJobsList.begin(),
                                   JobEnd = G_WpiGlobals.AllJobsList.end();
        for (; JobStart != JobEnd; JobStart++)
        {
            FEJobBase *pTemp = *JobStart;
            FEInstallJob *pJobThis = pTemp->IsInstallJob();
            if (pJobThis)
            {
                if (pJobThis->Is2BeInstalled())
                {
                    FEArcPackagePck *pPackageThis = (FEArcPackagePck*)(pJobThis->_pArcPackage);
                    // now go thru the BSWriteProfile objects in the list in
                    // FEPackage and invoke Write() on them;
                    // there may be none of these

                    // go thru the list of all config objects; this
                    // list holds all objects as base objects, so
                    // we need to type check
                    list<BSConfigBase*>::iterator
                            CfgStart = pPackageThis->_pDecl->_listConfigObjects.begin(),
                            CfgEnd = pPackageThis->_pDecl->_listConfigObjects.end();
                    for (; CfgStart != CfgEnd; CfgStart++)
                    {
                        // check if this is our class
                        BSWriteProfile *pPrfThis = (**CfgStart).IsWriteProfile();
                        if (!pPrfThis)
                            // not the class we need: take next then
                            continue;

                        // resolve macros
                        wpiResolveMacros(&(pPrfThis->_pszProfile));
                        wpiResolveMacros(&(pPrfThis->_pszWriteString));

                        try
                        {
                            pPrfThis->Write(G_WpiGlobals.habThread1,
                                            &pPackageThis->_logWritePrfsDone,
                                            pLogFile);
                                    // this may throw BSConfigExcpt

                            /* if (ulConfigThis & PCK_CONFIGSYS)
                                G_WpiGlobals.fConfigSysChanged = TRUE;
                            if (ulConfigThis & PCK_WPSCLASSES)
                                G_WpiGlobals.fWPSObjectsCreated = TRUE;
                            if (ulConfigThis & PCK_WPSOBJECTS)
                                G_WpiGlobals.fWPSClassesChanged = TRUE; */
                        }
                        catch(BSConfigExcpt& X)
                        {
                            brc = FALSE;
                            wpiMessageBox(102, // "WarpIN: Error"
                                          178,   // "error writing to profile"
                                          MB_ICONEXCLAMATION | MB_OK,
                                          0, 0);
                        }
                    }
                }
            }
        } // end for (; JobStart != JobEnd; JobStart++)
    } // end if (G_WpiGlobals.ulDoConfiguration | PCK_EXECUTES)

    return (brc);
}

/*
 *@@ wpiExecutes:
 *      this calls the executables specified by the EXECUTABLE
 *      tags by going thru the list of packages in WPIGLOBALS
 *      which were installed.
 *
 *      Each FEArcPackagePck has a list of BSExecute instances
 *      which at this point have the necessary information
 *      (set up at initialization by FEArcPackagePck::ParsePckAttribsExecutes).
 *
 *      This resolves macros.
 *
 *      Returns FALSE if any error occured.
 *
 *      Gets called from wpiConfigure.
 *
 *@@added V0.9.1 (2000-02-07) [umoeller]
 *@@changed V0.9.3 (2000-04-08) [umoeller]: G_WpiGlobals.ulAllowConfig wasn't always set properly, so we ignore this
 *@@changed V0.9.9 (2001-03-27) [umoeller]: reported the wrong config flags, fixed
 *@@changed V0.9.9 (2001-03-30) [umoeller]: added pLogFile
 */

BOOL wpiExecutes(BSFileLogger *pLogFile)        // in: log file or NULL
{
    BOOL    brc = TRUE;

    // go thru the list of all packages that were installed
    list<FEJobBase*>::iterator JobStart = G_WpiGlobals.AllJobsList.begin(),
                               JobEnd = G_WpiGlobals.AllJobsList.end();
    for (; JobStart != JobEnd; JobStart++)
    {
        FEJobBase *pTemp = *JobStart;
        FEInstallJob *pJobThis = pTemp->IsInstallJob();
        if (pJobThis)
        {
            if (pJobThis->Is2BeInstalled())
            {
                FEArcPackagePck *pPackageThis = (FEArcPackagePck*)(pJobThis->_pArcPackage);
                // now go thru the BSExecute objects in the list in
                // FEPackage and invoke CreateObject() on them;
                // there may be none of these

                // go thru the list of all config objects; this
                // list holds all objects as base objects, so
                // we need to type check
                list<BSConfigBase*>::iterator
                        CfgStart = pPackageThis->_pDecl->_listConfigObjects.begin(),
                        CfgEnd = pPackageThis->_pDecl->_listConfigObjects.end();
                for (; CfgStart != CfgEnd; CfgStart++)
                {
                    // check if this is our class
                    BSExecute *pExecThis = (**CfgStart).IsExecute();
                    if (!pExecThis)
                        // not the class we need: take next then
                        continue;

                    ULONG       ulConfigThis = 0;
                    BOOL        fStartAllowed = TRUE;
                    // get execute's configure flags and check
                    // if they're allowed by the user
                    if (pExecThis->_ulExecType & CFGT_CFGSYS)
                    {
                        if (G_WpiGlobals.ulAllowConfig & PCK_CONFIGSYS)
                            ulConfigThis |= PCK_CONFIGSYS;
                        else
                            // not allowed:
                            fStartAllowed = FALSE;
                    }
                    if (pExecThis->_ulExecType & CFGT_REGISTERCLASS)
                    {
                        if (G_WpiGlobals.ulAllowConfig & PCK_WPSCLASSES)
                            ulConfigThis |= PCK_WPSCLASSES;
                        else
                            // not allowed:
                            fStartAllowed = FALSE;
                    }
                    if (pExecThis->_ulExecType & CFGT_CREATEOBJECT)
                    {
                        if (G_WpiGlobals.ulAllowConfig & PCK_WPSOBJECTS)
                            ulConfigThis |= PCK_WPSOBJECTS;
                        else
                            // not allowed:
                            fStartAllowed = FALSE;
                    }
                    // ulConfigThis is still 0 if no flags exist
                    // in the executable

                    if (fStartAllowed)
                    {
                        // resolve macros
                        wpiResolveMacros(&(pExecThis->_pszExecutable));
                        wpiResolveMacros(&(pExecThis->_pszParams));

                        if (ulConfigThis & PCK_WPSOBJECTS)
                            guiUpdateGlobalStatus(GSTAT_WPSOBJECTS);
                        else if (ulConfigThis & PCK_WPSCLASSES)
                            guiUpdateGlobalStatus(GSTAT_WPSCLASSES);
                        else if (ulConfigThis & PCK_CONFIGSYS)
                            guiUpdateGlobalStatus(GSTAT_CONFIGSYS);

                        // use GUI callback for this...
                        // we must not block PM thread 1!
                        guiExecute(pExecThis,
                                   pLogFile);

                        if (ulConfigThis & PCK_CONFIGSYS)
                            G_WpiGlobals.fConfigSysChanged = TRUE;
                        // fixed the following V0.9.9 (2001-03-27) [umoeller]
                        if (ulConfigThis & PCK_WPSCLASSES)
                            G_WpiGlobals.fWPSClassesChanged = TRUE;
                        if (ulConfigThis & PCK_WPSOBJECTS)
                            G_WpiGlobals.fWPSObjectsCreated = TRUE;
                    }
                }
            }
        }
    } // end for (; JobStart != JobEnd; JobStart++)

    return (brc);
}

/*
 *@@ wpiDeExecutes:
 *      just runs through al BSDeExecute instances to resolve
 *      the macros, so that these can be safely stored in the
 *      database.
 *
 *@@added V0.9.9 (2001-03-27) [umoeller]
 *@@changed V0.9.9 (2001-03-30) [umoeller]: added pLogFile
 */

BOOL wpiDeExecutes(BSFileLogger *pLogFile) // in: log file or NULL&ouml;
{
    BOOL    brc = TRUE;

    // go thru the list of all packages that were installed
    list<FEJobBase*>::iterator JobStart = G_WpiGlobals.AllJobsList.begin(),
                               JobEnd = G_WpiGlobals.AllJobsList.end();
    for (; JobStart != JobEnd; JobStart++)
    {
        FEJobBase *pTemp = *JobStart;
        FEInstallJob *pJobThis = pTemp->IsInstallJob();
        if (pJobThis)
        {
            if (pJobThis->Is2BeInstalled())
            {
                FEArcPackagePck *pPackageThis = (FEArcPackagePck*)(pJobThis->_pArcPackage);
                // now go thru the BSDeExecute objects in the list in
                // FEPackage

                // go thru the list of all config objects; this
                // list holds all objects as base objects, so
                // we need to type check
                list<BSConfigBase*>::iterator
                        CfgStart = pPackageThis->_pDecl->_listConfigObjects.begin(),
                        CfgEnd = pPackageThis->_pDecl->_listConfigObjects.end();
                for (; CfgStart != CfgEnd; CfgStart++)
                {
                    // check if this is our class
                    BSDeExecute *pExecThis = (**CfgStart).IsDeExecute();
                    if (!pExecThis)
                        // not the class we need: take next then
                        continue;

                    // resolve macros and store the resolved thingie
                    wpiResolveMacros(&(pExecThis->_pszExecutable));
                    pPackageThis->_logDeExecutesResolved.Append(pExecThis->_pszExecutable);

                    if (pExecThis->_pszParams)
                    {
                        wpiResolveMacros(&(pExecThis->_pszParams));
                        pPackageThis->_logDeExecutesResolved.Append("|");
                        pPackageThis->_logDeExecutesResolved.Append(pExecThis->_pszExecutable);
                    }
                }
            }
        }
    } // end for (; JobStart != JobEnd; JobStart++)

    return (brc);
}

/*
 *@@ wpiConfigure:
 *      this gets called from EnterArchiveMode to
 *      configure the system after all files have
 *      been unpacked.
 *
 *      This calls wpiConfigSys, wpiWPSClasses,
 *      and wpiCreateObjects (in this order) if
 *      required by the archive being installed.
 *
 *      This is running on thread 1. As a result, we
 *      block the PM input queue while this is going on.
 *      We probably should do this while creating WPS
 *      objects etc. in the first place. ;-)
 *
 *@@changed V0.9.3 (2000-05-12) [umoeller]: added pszBackup
 *@@changed V0.9.5 (2000-08-26) [umoeller]: executes wasn't executed if ulAllowConfig was 0
 *@@changed V0.9.6 (2000-10-27) [umoeller]: made CONFIG.SYS backup part of WPIGLOBALS
 *@@changed V0.9.9 (2001-03-30) [umoeller]: added pLogFile
 */

BOOL wpiConfigure(BSFileLogger *pLogFile) // in: log file or NULL
{
    BOOL        brc = TRUE;

    if (G_WpiGlobals.ulAllowConfig)
    {
        // we have something to configure at all:
        wpiConfigSys(pLogFile);
        wpiWPSClasses(pLogFile);
        wpiCreateObjects(pLogFile);
        wpiWriteProfiles(pLogFile);
    }

    // run thru executes in any case
    // this checks G_WpiGlobals.ulAllowConfig for every exec
    wpiExecutes(pLogFile);

    // and run thru de-executes, just to resolve the macros
    // V0.9.9 (2001-03-27) [umoeller]
    wpiDeExecutes(pLogFile);

    return (brc);
}


