
/*
 *@@sourcefile wpi_helpers.cpp:
 *      this has various helpers used throughout the WarpIN code.
 *
 *      This has been separated from warpin.cpp.
 *
 *@@added V0.9.1 (2000-02-06) [umoeller]
 */

/*      This file Copyright (C) 1998-2000 Ulrich Mller.
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, in version 2 as it comes in the COPYING
 *      file of this distribution.
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 */

#define OS2EMX_PLAIN_CHAR
    // this is needed for "os2emx.h"; if this is defined,
    // emx will define PSZ as _signed_ char, otherwise
    // as unsigned char

#define INCL_DOS
#define INCL_DOSERRORS
#define INCL_WINSHELLDATA
#define INCL_WINWORKPLACE
#include <os2.h>

#include <stdlib.h>
#ifdef __IBMCPP__
    #include <direct.h>
#endif
#include <string.h>
#include <setjmp.h>             // needed for except.h
#include <assert.h>             // needed for except.h

#include <ctype.h>

#ifdef __IBMCPP__
    #include <terminat.h>
#endif

#include "setup.h"              // was missing (99-12-23) [umoeller]

// include's from helpers
#include "helpers\dosh.h"
#include "helpers\stringh.h"
#include "helpers\threads.h"
#include "helpers\xstring.h"

// back-end includes
#include "wiarchive\wiarchive.h"

// base includes (99-11-07) [umoeller]
#include "base\bs_string.h"
#include "base\bs_errors.h"
#include "base\bs_logger.h"
#include "base\bs_config.h"
#include "base\bs_script.h"

// front-end includes
#include "frontend\fe_base.h"
#include "frontend\fe_archive.h"
#include "frontend\fe_database.h"
#include "frontend\fe_job.h"
#include "frontend\fe_package.h"
#include "frontend\fe_package_arc.h"
#include "frontend\fe_package_db.h"

#include "frontend\warpin.h"
#include "frontend\calbacks.h"

#include "helpers\except.h"     // must be at the bottom because wiarchive includes stdio

#pragma hdrstop

/* ******************************************************************
 *
 *  Global variables
 *
 ********************************************************************/

extern WPIGLOBALS      G_WpiGlobals;
            // in warpin.cpp

extern FEDatabase*     G_pDatabase;
            // in warpin.cpp

/* ******************************************************************
 *
 *  Helper functions
 *
 ********************************************************************/

/*
 *@@ wpiMessageBox:
 *      this displays a message box using
 *      message IDs for the title and the
 *      message.
 *
 *@@added V0.9.0 (99-11-01) [umoeller]
 */

ULONG wpiMessageBox(ULONG ulTitleMsg, // in: TMF msg ID for title
                    ULONG ulMsgMsg,   // in: TMF msg ID for message
                    ULONG ulFlags, // in: as in WinMessageBox (MB_OK etc.)
                    PCHAR *pTable,  // in: string replacement table for ulMsgMsg (wpiGetMessage), defaults to NULL
                    ULONG cTable)   // in: string replacement count, defaults to 0
{
    BSString    strTitle, strMsg;
    wpiGetMessage(strTitle,
                  ulTitleMsg);
    wpiGetMessage(strMsg,
                  ulMsgMsg,
                  pTable, cTable);
    return (guiShowMessage(strTitle, strMsg, ulFlags));
}

/*
 *@@ wpiResolveMacros:
 *      this resolves the macros that many script tag attributes support.
 *
 *      Specify the address of the PSZ to be resolved in ppszString.
 *      If any macros are resolved, *ppszString is free()'d and
 *      reset to a newly allocated string with the resolved macros.
 *
 *      This returns the number of macros resolved or 0 if no macros
 *      were found. In that case, ppszString is not changed.
 *
 *      The following macros can be resolved:
 *
 *      --  "?:\":      "?" is replaced with the OS/2 boot drive letter.
 *      --  "$(xxx)" (with "xxx" being a decimal number):
 *                      "xxx" is replaced with the target path of the package
 *                      with the index "xxx".
 *      --  "$(vendor\application\package)" is replaced with the target path
 *                      of the specified package, which must be installed.
 *      --  "$(env)" (with "env" being an environment variable):
 *                      "env" is replaced with the value of an environment variable.
 *                      If that variable is not found, the user is prompted to
 *                      enter one.
 */

ULONG wpiResolveMacros(char **ppsz)
{
    ULONG ulrc = 0;
    BSString strTemp = *ppsz;
    ulrc = wpiResolveMacros(strTemp);
    free(*ppsz);
    *ppsz = strdup(strTemp.c_str());

    return (ulrc);
}

/*
 *@@ wpiResolveMacros:
 *      overloaded for BSString.
 *
 *@@added V0.9.2 (2000-03-11) [umoeller]
 *@@changed V0.9.3 (2000-04-28) [umoeller]: finally made "?:\" stuff work
 *@@changed V0.9.3 (2000-06-04) [umoeller]: added package ID support
 *@@changed V0.9.6 (2000-11-01) [umoeller]: added UNIX slash support
 *@@changed V0.9.9 (2001-04-06) [umoeller]: fixed broken ?:\
 */

ULONG wpiResolveMacros(BSString &str)
{
    ULONG ulrc = 0;

    PSZ     pMacroBegin = NULL;

    if (str.size() > 2)         // was 3, fixed; this broke ?:\ macros
                                // V0.9.9 (2001-04-06) [umoeller]
    {
        // boot drive letter?
        if (memcmp(str.c_str(), "?:\\", 3) == 0)
            *((CHAR*)str.c_str()) = doshQueryBootDrive();

        // environment variables or package paths
        while ((pMacroBegin = strstr(str.c_str(), "$(")))
        {
            // macro found:
            PSZ     pEndOfMacro = strchr(pMacroBegin, ')');
            if (pEndOfMacro)
            {
                // extract stuff from "$(" to ")"; this will be replaced
                PSZ     pszToReplace = strhSubstr(pMacroBegin,
                                                  pEndOfMacro+1); // include ')'
                // extract stuff in brackets (between "$(" and ")")
                PSZ     pszInBrackets1 = strhSubstr(pMacroBegin + 2,
                                                    pEndOfMacro);  // exclude ')'

                BOOL    fResolved = FALSE;

                PSZ pInBrackets2 = pszInBrackets1;
                BOOL fUnixSlashes = FALSE;
                // check if first char is '/' --> UNIX slashes
                if (*pInBrackets2 == '/')
                {
                    fUnixSlashes = TRUE;
                    // skip that char
                    pInBrackets2++;
                }

                // do we have a '\'? --> package path from database
                if (strchr(pInBrackets2, '\\'))
                {
                    // yes: it's a package ID
                    FEPackageID PckIDThis(pInBrackets2, __FILE__, __LINE__,
                                          TRUE);    // allow short format!
                    const FEDBPackage *pDBPackage
                            = G_pDatabase->FindDBPackage(PckIDThis); /* ._pszAuthor,
                                                         PckIDThis._pszApplication,
                                                         PckIDThis._pszPackageName); */
                    if (pDBPackage)
                    {
                        // DB package found:

                        string strTargetPath(pDBPackage->QueryTargetPath());

                        if (fUnixSlashes)
                        {
                            // UNIX slashes: convert all "\" to "/"
                            ULONG ulPos = 0;
                            while (strTargetPath._find_replace('\\', '/', &ulPos)
                                    != string::npos)
                                ;
                        }

                        ULONG ulPos = 0;
                        str._find_replace(pszToReplace,     // search string
                                          strTargetPath,    // replacement string
                                          &ulPos);

                        ulrc++;     // replacement count

                        fResolved = TRUE;
                    }
                    else
                    {
                        CHAR szID[2000];
                        sprintf(szID, "%s\\%s\\%s", PckIDThis._strAuthor.c_str(),
                                                    PckIDThis._strApplication.c_str(),
                                                    PckIDThis._strPackageName.c_str());
                        PSZ pszID = szID;
                        throw FEFatalErrorExcpt(189,
                                                &pszID,
                                                1);
                    }
                }

                // not found yet?
                if (!fResolved)
                {
                    // all decimals? --> package number in archive
                    if (strhIsDecimal(pInBrackets2))
                    {
                        // OK, we appear to have a package index:
                        ULONG   ulIndex = 0;
                        sscanf(pInBrackets2, "%d", &ulIndex);
                        // find that package in G_WpiGlobals
                        list<FEJobBase*>::iterator JobStart = G_WpiGlobals.AllJobsList.begin(),
                                                   JobEnd = G_WpiGlobals.AllJobsList.end();
                        for (; JobStart != JobEnd; JobStart++)
                        {
                            FEJobBase *pTemp = *JobStart;
                            FEInstallJob *pJobThis = pTemp->IsInstallJob();
                            if (pJobThis)
                            {
                                const FEArcPackageBase *ppiThis = pJobThis->_pArcPackage;

                                if (ppiThis->_PackHeader.number == ulIndex)
                                {
                                    string strTargetPath = pJobThis->QueryTargetPath();
                                    ULONG ulPos = 0;

                                    if (fUnixSlashes)
                                    {
                                        // UNIX slashes: convert all "\" to "/"
                                        while (strTargetPath._find_replace('\\', '/', &ulPos)
                                                    != string::npos)
                                            ;
                                        ulPos = 0;
                                    }

                                    // replace macro with target path
                                    str._find_replace(pszToReplace,     // search string
                                                      strTargetPath,   // replacement string
                                                      &ulPos);

                                    fResolved = TRUE;

                                    ulrc++;     // replacement count

                                    break;  // for PckStart
                                }
                            }
                        }
                    }
                }

                if (!fResolved)
                {
                    // finally, attempt to resolve an environment variable
                    PSZ pszValue = getenv(pszInBrackets1);
                    if (!pszValue)
                    {
                        // variable not defined:
                        // check if we have a VARPROMPT for this
                        // const char *pcszDescription = 0;
                        BSVarPrompt VarPrompt(pszInBrackets1);
                        BSVarPromptsMap::iterator iPrompt
                            = BSScriptBase::VarPromptsMap.find(BSString(pszInBrackets1));
                        if (iPrompt != BSScriptBase::VarPromptsMap.end())
                            // VARPROMPT found:
                            VarPrompt = (*iPrompt).second;
                        // now prompt
                        pszValue = guiVarPrompt(VarPrompt);
                                // can be NULL
                    }

                    // replace macro with value
                    ULONG ulPos = 0;
                    str._find_replace(pszToReplace,              // search string
                                      pszValue,                  // replacement string
                                      &ulPos);

                    ulrc++;     // replacement count
                }

                free(pszToReplace);
                free(pszInBrackets1);
            } // end if (pEndOfMacro)
            else
                break;

            // OK, search next macro;
            // since the macro search string has been replaced
            // with real text, we can safely search the whole
            // buffer again
            // (and we must, because pszCurrent points to an
            // all new buffer)

        } // end while (pMacro = strstr(*ppszString, "$(");)
    }

    return (ulrc);
}

/*
 *@@ wpiCTime2FTime:
 *      useful helper function for converting
 *      the C runtime "time_t" format to the
 *      OS/2 FDATE and FTIME format.
 */

void wpiCTime2FTime(time_t *pCTime,     // in: C runtime time_t format
                    FDATE *pfDate,      // out: OS/2 FDATE format
                    FTIME *pfTime)      // out: OS/2 FTIME format
{
    struct tm* pTimeBuf = localtime(pCTime);
    pfDate->year      = (pTimeBuf->tm_year + 1900) - 1980;
    pfDate->month     = (pTimeBuf->tm_mon + 1);
    pfDate->day       = (pTimeBuf->tm_mday);
    pfTime->hours     = (pTimeBuf->tm_hour);
    pfTime->minutes   = (pTimeBuf->tm_min);
    pfTime->twosecs   = (pTimeBuf->tm_sec) / 2;
}

/*
 *@@ wpiCompareFileDates:
 *      this helper compares the last write date
 *      as found in PFILESTATUS3 (which should have
 *      been filled using DosQueryPathInfo on an
 *      existing file) to the last write date in
 *      the specified WIFileHeader.
 *
 *      The header file's last write date will be
 *      written in the two specified buffers in
 *      OS/2 date/time format. These buffers _must_
 *      be specified (or this will crash).
 *
 *      Returns:
 *          // < 0:  existing file is newer
 *          // == 0: existing file is same write date
 *          // > 0:  existing file is older
 */

int wpiCompareFileDates(WIFileHeader* pHeader,      // in: file header to compare
                        FILESTATUS3 *pfs3,          // in: actual file to compare
                        FDATE *pfdateLastWrite,     // out: last-write date of pHeader in OS/2 format
                        FTIME *pftimeLastWrite)     // out: last-write time of pHeader in OS/2 format
{
    // convert file header's time information
    wpiCTime2FTime(&pHeader->lastwrite,
                   pfdateLastWrite,
                   pftimeLastWrite);

    // and compare to the actual file's information
    static const PSZ pszFormatTimestamp = "%4u%02u%02u%02u%02u%02u";
                                        // >  M   D   H   M   2S

    CHAR szHeaderStamp[30],
         szExistingStamp[30];

    // create timestamps
    sprintf(szHeaderStamp,
            pszFormatTimestamp,
            pfdateLastWrite->year + 1980,
            pfdateLastWrite->month,
            pfdateLastWrite->day,
            pftimeLastWrite->hours,
            pftimeLastWrite->minutes,
            pftimeLastWrite->twosecs * 2);
    sprintf(szExistingStamp,
            pszFormatTimestamp,
            pfs3->fdateLastWrite.year + 1980,
            pfs3->fdateLastWrite.month,
            pfs3->fdateLastWrite.day,
            pfs3->ftimeLastWrite.hours,
            pfs3->ftimeLastWrite.minutes,
            pfs3->ftimeLastWrite.twosecs * 2);

    return (strcmp(szHeaderStamp, szExistingStamp));
}

/*
 *@@ PrepareGUIFILEERROR:
 *      helper func to fill a FILEEXISTS structure
 *      with file information.
 *
 *@@added V0.9.0 (99-10-26) [umoeller]
 */

VOID wpiPrepareGUIFILEERROR(PGUIFILEERROR pfi,          // out: file-exists info
                            const char *pcszBasePath,   // in: if != NULL, base path;
                                                        // if NULL, we get the curdir
                            WIFileHeader* pwifh)        // in: file header from archive
{
    if (!pcszBasePath)
        // get file name (in current working directory)
        _getcwd(pfi->szFilename, sizeof(pfi->szFilename));
    else
        strcpy(pfi->szFilename, pcszBasePath);

    if (pfi->szFilename[strlen(pfi->szFilename) - 1] != '\\')
        strcat(pfi->szFilename, "\\");
    strcat(pfi->szFilename, pwifh->name);

    // 1) get file info for file on disk
    //    (current directory)
    DosQueryPathInfo(pfi->szFilename,
                     FIL_STANDARD,
                     &pfi->fs3Existing,
                     sizeof(pfi->fs3Existing));

    // 2) info for file in archive:
    // WIFileHeader has two time_t fields which we need
    // to convert to human-readable format first.
    pfi->iComp = wpiCompareFileDates(pwifh,
                                     &pfi->fs3Existing,
                                     &pfi->fDateHeader,
                                     &pfi->fTimeHeader);
        // this returns:
        // < 0:  existing file is newer
        // == 0: existing file is same write date
        // > 0:  existing file is older
    pfi->pwifh = pwifh;
}

/*
 *@@ wpiFindPackageInArchives:
 *      this goes thru all currently open archives
 *      to find a package with the specified ID.
 *      See FEPackage::IsInArchive for the parameters,
 *      which gets called for all currently open
 *      archives by this helper function.
 *
 *@@added V0.9.1 (2000-01-08) [umoeller]
 */

FEArcPackagePck* wpiFindPackageInArchives(const char *pszID,
                                          PULONG pulComp)
{
    FEArcPackagePck *pArcReturn = 0;

    list<FEArchive*>::iterator ArcStart = G_WpiGlobals.ArchivesList.begin(),
                               ArcEnd = G_WpiGlobals.ArchivesList.end();
    for (; ArcStart != ArcEnd; ArcStart++)
    {
        FEArchive *pArcThis = *ArcStart;

        if ( (pArcReturn = pArcThis->IsInArchive(pszID, pulComp)) )
            break;
    }

    return (pArcReturn);
}

/*
 *@@ wpiFindInstallJob:
 *      since there is no reverse linkage from "package"
 *      packages to install jobs, this helper function
 *      can be used to find the install job for a
 *      given "package" package.
 *
 *      Returns NULL if not found.
 *
 *@@added V0.9.1 (2000-01-08) [umoeller]
 */

FEInstallJob* wpiFindInstallJob(const FEArcPackagePck *pArcPackage)
{
    list<FEJobBase*>::iterator JobStart = G_WpiGlobals.AllJobsList.begin(),
                               JobEnd = G_WpiGlobals.AllJobsList.end();
    for (; JobStart != JobEnd; JobStart++)
    {
        FEJobBase *pTemp = *JobStart;
        FEInstallJob *pJobThis = pTemp->IsInstallJob();
        if (pJobThis)
        {
            if (pJobThis->_pArcPackage == pArcPackage)
            {
                return (pJobThis);
            }
        }
    }

    return (NULL);
}

/*
 *@@ wpiFindInstallJob:
 *      second version of wpiFindInstallJob which
 *      takes a five- or six-part package ID as
 *      input instead.
 *
 *      Also returns NULL if not found.
 *
 *@@added V0.9.1 (2000-01-08) [umoeller]
 */

FEInstallJob* wpiFindInstallJob(const char *pszID)
{
    FEInstallJob    *pJobReturn = NULL;
    ULONG           ulComp = 0;
    // 1) find package with that ID in open archives
    FEArcPackagePck *pPckInArc = wpiFindPackageInArchives(pszID,
                                                          &ulComp);
    if (pPckInArc)
    {
        // found: check all deinstall jobs whether they
        // match that package
        list<FEJobBase*>::iterator JobStart = G_WpiGlobals.AllJobsList.begin(),
                                   JobEnd = G_WpiGlobals.AllJobsList.end();
        for (; JobStart != JobEnd; JobStart++)
        {
            FEJobBase *pTemp = *JobStart;
            FEInstallJob *pJobThis = pTemp->IsInstallJob();
            if (pJobThis)
            {
                if (pJobThis->_pArcPackage == pPckInArc)
                {
                    // found: return that
                    pJobReturn = pJobThis;
                    break;
                }
            }
        }
    }

    return (pJobReturn);
}

/*
 *@@ wpiFindGroupJob:
 *      since there is no reverse linkage from group
 *      packages to group jobs, this helper function
 *      can be used to find the group job for a group's
 *      member package.
 *
 *      Returns NULL if not found.
 *
 *@@added V0.9.0 (99-11-06) [umoeller]
 */

FEGroupJob* wpiFindGroupJob(const FEArcPackageGroup *pGroupPackage)
{
    list<FEJobBase*>::iterator JobStart2 = G_WpiGlobals.AllJobsList.begin(),
                               JobEnd2 = G_WpiGlobals.AllJobsList.end();
    for (; JobStart2 != JobEnd2; JobStart2++)
    {
        FEJobBase *pTemp = *JobStart2;
        FEGroupJob *pJobThis2 = pTemp->IsGroupJob();
        if (pJobThis2)
            if (pJobThis2->_pGroupPackage == pGroupPackage)
            {
                return (pJobThis2);
            }
    }

    return (NULL);
}

/*
 *@@ wpiIs2BeRemoved:
 *
 *@@added V0.9.1 (2000-01-08) [umoeller]
 */

FEInstallJob* wpiIs2BeRemoved(const char *pszID)
{
    FEInstallJob *pJob = wpiFindInstallJob(pszID);
    if (pJob)
        if (pJob->Is2BeRemoved())
            return (pJob);

    return (NULL);
}

/*
 *@@ wpiCheckJobs:
 *      helper function which finds out what
 *      needs to be done.
 *
 *      This returns the no. of packages which
 *      are selected for installation or
 *      deinstallation as well as the configuration
 *      which needs to be done.
 *
 *      The config data is the sum of all packages,
 *      masked by WPIGLOBALS.ulDoConfiguration.
 *
 *      Any of the pointers can be NULL if you don't
 *      need that info.
 *
 *      NOTE: This function does _not_ initialize
 *      the ULONGs. Set these to 0 yourself before
 *      calling this.
 *
 *@@added V0.9.0 (99-11-02) [umoeller]
 */

VOID wpiCheckJobs(PULONG pulToAdd,      // out: packages to install
                  PULONG pulToRemove,   // out: packages to remove
                  PULONG pulConfig)     // out: CONFIG_* flags
{
    list<FEJobBase*>::iterator JobStart = G_WpiGlobals.AllJobsList.begin(),
                               JobEnd = G_WpiGlobals.AllJobsList.end();
    for (; JobStart != JobEnd; JobStart++)
    {
        FEJobBase *pTemp = *JobStart;
        FEInstallJob *pJobThis = pTemp->IsInstallJob();
        if (pJobThis)
        {
            if (pJobThis->Is2BeInstalled())
            {
                if (pulToAdd)
                    (*pulToAdd)++;
                if (pulConfig)
                    (*pulConfig) |= ((pJobThis->_pArcPackage->_pDecl->_ulConfigData)
                                     & G_WpiGlobals.ulAllowConfig);
            }
            else if (pJobThis->Is2BeRemoved())
                if (pulToRemove)
                    (*pulToRemove)++;
        }
    }
}

/*
 *@@ wpiCheckDependencies:
 *      this goes thru the install jobs list to
 *      check if all package requirements are met.
 *      This prompts for all missing packages:
 *
 *      -- if a missing package is in the archive,
 *         the user is prompted whether it should
 *         be selected;
 *
 *      -- if it's not, an error message is displayed
 *         that the package could not be found.
 *
 *      This returns TRUE only if all requirements
 *      are met.
 *
 *      This gets called automatically from wpiVerifySelections,
 *      i.e. when the "Container" page is left, so if FALSE
 *      is returned, the Container page is not left.
 *
 *      Throws:
 *      -- FEFatalErrorExcpt.
 *
 *@@added V0.9.1 (2000-01-08) [umoeller]
 *@@changed V0.9.6 (2000-11-09) [umoeller]: version checks were missing, fixed.
 */

BOOL wpiCheckDependencies()
{
    BOOL    rc = TRUE;
    BOOL    fCheckAgain = FALSE;

    do // this loops if any package selections are changed
    {
        fCheckAgain = FALSE;

        // results:
        BSString strMissing;
                // string of packages which are required by archive,
                // but not accessible (error)
        list<FEJobBase*> listJobsRequiredByArchive;
                // list of archive packages which are required by other
                // packages in the archive
        BSString strRequiredByArchive;
                // string list for that
        list<FEJobBase*> listJobsRequiredByDatabase;
                // list of archive packages which are required by the
                // database
        BSString strRequiredByDatabase;
                // string list for that

        // go thru all jobs
        list<FEJobBase*>::iterator JobStart = G_WpiGlobals.AllJobsList.begin(),
                                   JobEnd = G_WpiGlobals.AllJobsList.end();
        for (; JobStart != JobEnd; JobStart++)
        {
            FEJobBase *pTemp = *JobStart;
            FEInstallJob *pJobThis = pTemp->IsInstallJob();
            if (pJobThis)
            {
                // 1) if package has been selected for installation,
                //    check if requirements are met for that package

                if (pJobThis->Is2BeInstalled())
                {
                    // job is to be installed (selected):
                    // check required packages for that job
                    const FEPackageBase *pPckThis = pJobThis->_pArcPackage;
                    // decode "requires" logger strings
                    PSZ     pszLogStart = pPckThis->_logRequiresIDs._pabLogString,
                            pszRequiresIDThis = pszLogStart;

                    while (pszRequiresIDThis
                           < (pszLogStart + pPckThis->_logRequiresIDs._cbLogString))
                    {
                        ULONG   cbLogThis = strlen(pszRequiresIDThis);
                        ULONG   ulComp = 0;
                        BOOL    fNeeds2BeSelected = FALSE;

                        // pszRequiresIDThis now has the ID of the package
                        // which is required:
                        // first check if it's in the database maybe

                        FEPackageID RequiredPckIDThis(pszRequiresIDThis, __FILE__, __LINE__,
                                                      FALSE);    // don't allow short format
                        FEPackageID *_pPackageIDInstalled
                                = G_pDatabase->FindPackageID(RequiredPckIDThis._strAuthor.c_str(),
                                                             RequiredPckIDThis._strApplication.c_str(),
                                                             RequiredPckIDThis._strPackageName.c_str());
                        if (!_pPackageIDInstalled)
                            // required, but not installed:
                            fNeeds2BeSelected = TRUE;
                        else
                        {
                            // installed: check if maybe it's selected for
                            // de-installation, which ain't good either
                            if (wpiIs2BeRemoved(pszRequiresIDThis))
                                fNeeds2BeSelected = TRUE;
                            else
                            {
                                // check if installed version level is sufficient
                                // V0.9.6 (2000-11-06) [umoeller]
                                if (    (_pPackageIDInstalled->Compare(RequiredPckIDThis)
                                               & IDCOMP_VERSIONMASK)
                                        == IDCOMP_THISOLDER
                                   )
                                {
                                    // installed package is too old:
                                    fNeeds2BeSelected = TRUE;
                                }
                            }
                        }

                        if (fNeeds2BeSelected)
                        {
                            // not installed or too old:
                            FEArcPackagePck *pPckInArc = wpiFindPackageInArchives(pszRequiresIDThis,
                                                                                  &ulComp);

                            if (pPckInArc)
                            {
                                // required package is in archive:
                                // find job for that
                                FEInstallJob *pJob2BeSelected = wpiFindInstallJob(pPckInArc);
                                if (!pJob2BeSelected)
                                    throw FEFatalErrorExcpt("job not found");

                                // is job selected?
                                if (!pJob2BeSelected->Is2BeInstalled())
                                {
                                    // no: add to list
                                    if (strRequiredByArchive())
                                        strRequiredByArchive += "\n";
                                    strRequiredByArchive += "    Package \""
                                                      + BSString(pJobThis->_pArcPackage->QueryTitle())
                                                      + "\" requires \"" +
                                                      + pPckInArc->QueryTitle()
                                                      + "\"";
                                    // store the job of the missing package
                                    // in the list of items to be selected
                                    listJobsRequiredByArchive.push_back(pJob2BeSelected);
                                }
                            }
                            else
                            {
                                // required package is NOT in archive:
                                // this will cause an error below
                                if (strMissing())
                                    strMissing += "\n";
                                strMissing += "    Package \""
                                              + BSString(pJobThis->_pArcPackage->QueryTitle())
                                              + "\" requires \"" +
                                              + pszRequiresIDThis
                                              + "\"";
                            }
                        } // end if (!_pPackageIDInstalled)

                        pszRequiresIDThis += cbLogThis + 1;    // go beyond null byte
                    } // end while (pszRequiresIDThis < pszLogStart + pPckThis->_logRequiresIDs._cbLogString)
                } // end if (pJobThis->Is2BeInstalled())

                // 2) if job has been selected for de-installation,
                //    check if maybe some other package requires this

                else if (pJobThis->Is2BeRemoved())
                {
                    // check if a package in the database
                    // requires package which is to be deinstalled
                    FEPackageID PckIDThis(pJobThis->_pArcPackage, __FILE__, __LINE__,
                                          FALSE);    // don't allow short format

                    // enumerate all packages in the database
                    PSZ     pszDBPckIDThis = G_pDatabase->EnumPackages(NULL);
                    while (pszDBPckIDThis)
                    {
                        // get database package from that ID
                        FEPackageID DBPckIDThis(pszDBPckIDThis, __FILE__, __LINE__,
                                                FALSE);    // don't allow short format
                        const FEDBPackage *pDBPckThis
                            = G_pDatabase->FindDBPackage(DBPckIDThis); /* ._pszAuthor,
                                                       DBPckIDThis._pszApplication,
                                                       DBPckIDThis._pszPackageName); */
                        if (!pDBPckThis)
                            throw FEFatalErrorExcpt("database package not found");

                        // decode "requires" logger strings for that DBPackage
                        PSZ     pszLogStart = pDBPckThis->_logRequiresIDs._pabLogString,
                                pszDBRequiresIDThis = pszLogStart;
                        // int     iCount = 0;

                        while (pszDBRequiresIDThis < pszLogStart + pDBPckThis->_logRequiresIDs._cbLogString)
                        {
                            ULONG   cbLogThis = strlen(pszDBRequiresIDThis);

                            // pszDBRequiresIDThis now has a requirement for the DB package;
                            // check if that ID is the same as that of the job which is
                            // to be removed
                            FEPackageID DBRequiresIDThis(pszDBRequiresIDThis, __FILE__, __LINE__,
                                                         FALSE);    // don't allow short format
                            if (DBRequiresIDThis.Compare(PckIDThis))
                            {
                                // matches:
                                // check if the DB package which requires that package
                                // has also been selected for removal
                                if (!wpiIs2BeRemoved(pszDBPckIDThis))
                                {
                                    // no: this means that
                                    // required package is to be removed:
                                    FEInstallJob *pJob2BeRemoved = wpiIs2BeRemoved(pszDBRequiresIDThis);

                                    FEInstallJob *pRemovalJobInArchive = wpiFindInstallJob(pszDBPckIDThis);
                                    // check if the requiring database package
                                    // is also in the archive
                                    if (pRemovalJobInArchive)
                                    {
                                        // yes: add this to the regular strRequiredByArchive
                                        // strings
                                        if (strRequiredByArchive())
                                            strRequiredByArchive += "\n";
                                        strRequiredByArchive += "    Package \""
                                                          + BSString(pRemovalJobInArchive->_pArcPackage->QueryTitle())
                                                          + "\" requires \""
                                                          + BSString(pJob2BeRemoved->_pArcPackage->QueryTitle())
                                                          + "\"";
                                        // store the job of the missing package
                                        // in the list of items to be selected
                                        listJobsRequiredByArchive.push_back(pJob2BeRemoved);
                                    }
                                    else
                                    {
                                        // not in archive:
                                        // add to "required by database" strings
                                        if (strRequiredByDatabase())
                                            strRequiredByDatabase += "\n";
                                        strRequiredByDatabase += "    Package \""
                                                          + BSString(pJob2BeRemoved->_pArcPackage->QueryTitle())
                                                          + "\" is required by database package \"" +
                                                          + pszDBPckIDThis
                                                          + "\"";
                                        // store the job of the missing package
                                        // in the list of items to be selected
                                        listJobsRequiredByDatabase.push_back(pJob2BeRemoved);
                                    }
                                }
                            }

                            pszDBRequiresIDThis += cbLogThis + 1;    // go beyond null byte
                        }

                        // next package in the database
                        PSZ pszPckIDNext = G_pDatabase->EnumPackages(pszDBPckIDThis);
                        free(pszDBPckIDThis);
                        pszDBPckIDThis = pszPckIDNext;
                    }
                }
            }
        } // end for (; JobStart != JobEnd; JobStart++)

        // 3) report results

        // check missing packages first; this is an error
        if (strMissing())
        {
            BSString strMsg("There are packages in the archive which require other packages to be installed:\n");
            strMsg += strMissing
                      + "\n\nThose packages were not found in the archive. WarpIN cannot continue.";
            guiShowMessage("WarpIN", strMsg, MB_OK);
            rc = FALSE;
        }

        if (rc)
        {
            // report archive packages which are required within archive
            if (strRequiredByArchive())
            {
                BSString strMsg("There are packages in the archive which require other packages to be installed:\n");
                strMsg += strRequiredByArchive
                          + "\n\nShould WarpIN select these packages for you?";
                if (guiShowMessage("WarpIN", strMsg, MB_YESNO)
                        == MBID_YES)
                {
                    JobStart = listJobsRequiredByArchive.begin();
                    JobEnd = listJobsRequiredByArchive.end();
                    for (; JobStart != JobEnd; JobStart++)
                    {
                        (**JobStart).Select(JOB_INSTALL);
                        fCheckAgain = TRUE; // loop again, because selections have changed
                    }
                }

                // report "errors found"
                rc = FALSE;
            }
        }

        if (rc)
        {
            // report archive packages which are required by database
            if (strRequiredByDatabase())
            {
                BSString strMsg("You have selected packages for removal which are required by packages in the database:\n");
                strMsg += strRequiredByDatabase
                          + "\n\nShould WarpIN deselect removal of these packages for you?";
                if (guiShowMessage("WarpIN", strMsg, MB_YESNO)
                        == MBID_YES)
                {
                    JobStart = listJobsRequiredByDatabase.begin();
                    JobEnd = listJobsRequiredByDatabase.end();
                    for (; JobStart != JobEnd; JobStart++)
                    {
                        (**JobStart).Select(JOB_INSTALL_IGNORE);
                        fCheckAgain = TRUE; // loop again, because selections have changed
                    }
                }

                // report "errors found"
                rc = FALSE;
            }
        }
    } while (fCheckAgain);

    return (rc);
}

/*
 *@@ wpiVerifySelections:
 *      helper function which should get called from
 *      the GUI when leaving the "Container" page to
 *      check if all package selections are valid.
 *
 *      If WARPIN_DISPLAYPAGES=NO, this gets called
 *      from EnterArchiveMode instead.
 *
 *      This calls wpiCheckDependencies in turn.
 *
 *      This also prompts for the directories to be
 *      created, if (fPromptCreateDirs == TRUE).
 *
 *      This returns TRUE if no errors were found
 *      and the GUI should continue, or FALSE
 *      if the GUI should stay on the "Container"
 *      page.
 *
 *@@changed V0.9.0 (99-11-01) [umoeller]: this was guiCheckPackages
 *@@changed V0.9.0 (99-11-04) [umoeller]: complained about missing directories even with deselected packages; fixed
 *@@changed V0.9.1 (2000-01-08) [umoeller]: added call to wpiCheckDependencies
 *@@changed V0.9.1 (2000-01-08) [umoeller]: fixed duplicate directory reports
 */

BOOL wpiVerifySelections(BOOL fPromptCreateDirs)
{
    list<BSString> listInvalidDirs,
                   listMissingDirs;

    BOOL    fInvalidDirsFound = FALSE;
    BOOL    fPackagesToAdd = FALSE,
            fPackagesToRemove = FALSE,
            fDirsMissing = FALSE,
            fAnyDowngrades = FALSE;

    if (!wpiCheckDependencies())
        return (FALSE);

    list<FEJobBase*>::iterator JobStart = G_WpiGlobals.AllJobsList.begin(),
                               JobEnd = G_WpiGlobals.AllJobsList.end();
    for (; JobStart != JobEnd; JobStart++)
    {
        FEJobBase *pTemp = *JobStart;
        FEInstallJob *pJobThis = pTemp->IsInstallJob();
        if (pJobThis)
        {
            if (pJobThis->Is2BeInstalled())
            {
                fPackagesToAdd = TRUE;

                BOOL    fInvalidThis = FALSE;
                BSString strTargetPath = pJobThis->QueryTargetPath();

                // check target paths
                if (strTargetPath.size() < 3) // V0.9.9 (2001-03-23) [umoeller]
                    fInvalidThis = TRUE;
                else if (strTargetPath[1] != ':')
                    fInvalidThis = TRUE;
                else if (strTargetPath[2] != '\\')
                    fInvalidThis = TRUE;
                else if (doshIsValidFileName(strTargetPath.c_str(), TRUE))
                    fInvalidThis = TRUE;

                if (fInvalidThis)
                {
                    listInvalidDirs.push_back(strTargetPath);
                    fInvalidDirsFound = TRUE;
                }
                else if (fPromptCreateDirs)
                {
                    // dir OK, but missing:
                    if (!doshQueryDirExist((PSZ)(pJobThis->QueryTargetPath())))
                    {
                        listMissingDirs.push_back(pJobThis->QueryTargetPath());
                        fDirsMissing = TRUE;
                    }
                }

                if (pJobThis->_ulInstallStatus == INSTALLED_IS_NEWER)
                    fAnyDowngrades = TRUE;
            }
            else if (pJobThis->Is2BeRemoved())
                fPackagesToRemove = TRUE;
        }
    } // end for

    if (fAnyDowngrades)
    {
        // selected packages are old than the installed versions:
        if (wpiMessageBox(108,       // "Warning"
                          187,       // "downgrading!"
                          MB_ICONEXCLAMATION | MB_YESNO | MB_MOVEABLE)
                != MBID_YES)
            // report error
            return (FALSE);

    }

    if ( (!fPackagesToAdd) && (!fPackagesToRemove) )
    {
        // no packages selected for installation/deinstallation:
        // this doesn't make any sense
        wpiMessageBox(102,       // "Error"
                      126,       // "no packages"
                      MB_ICONEXCLAMATION | MB_OK | MB_MOVEABLE);
        // report error
        return (FALSE);
    }

    if (fInvalidDirsFound)
    {
        // remove duplicate entries
        listInvalidDirs.unique();

        // compose message
        BSString strInvalidDirs;
        list<BSString>::iterator StrStart = listInvalidDirs.begin(),
                                 StrEnd = listInvalidDirs.end();
        for (; StrStart != StrEnd; StrStart++)
        {
            strInvalidDirs += "\n    " + *StrStart;
        }

        PSZ pTable = (PSZ)strInvalidDirs.c_str();
        wpiMessageBox(114,    // "invalid directories"
                      115,
                      MB_ICONEXCLAMATION | MB_OK | MB_MOVEABLE,
                      &pTable, 1);
        // report error
        return (FALSE);
    }

    // check if disk space on target drives is sufficient
    // by evaluating the global flag which is maintained
    // by the "Drive info" window (fnwpDriveInfo)
    if (!G_WpiGlobals.fPlentyOfSpace)
        if (wpiMessageBox(108,   // "WarpIN: Warning"
                          110,   // "not enough space"
                          MB_ICONEXCLAMATION | MB_YESNO | MB_MOVEABLE)
                != MBID_YES)
            // report error
            return (FALSE);

    if (fDirsMissing)
    {
        // remove duplicate entries
        listMissingDirs.unique();

        // compose message
        BSString strMissingDirs;
        list<BSString>::iterator StrStart = listMissingDirs.begin(),
                                 StrEnd = listMissingDirs.end();
        for (; StrStart != StrEnd; StrStart++)
        {
            strMissingDirs += "\n    " + *StrStart;
        }

        PSZ pTable = (PSZ)strMissingDirs.c_str();
        if (wpiMessageBox(116,    // "create directories"
                          117,
                          MB_ICONEXCLAMATION | MB_YESNO | MB_MOVEABLE,
                          &pTable, 1)
                    != MBID_YES)
            // report error
            return (FALSE);
    }

    return (TRUE);
}

/*
 *@@ TEMPFILETEMP:
 *
 *@@added V0.9.9 (2001-04-04) [umoeller]
 */

typedef struct _TEMPFILETEMP
{
    const char  *pcszFile2Check;
    BOOL        fFound;
    ULONG       ulError;
} TEMPFILETEMP, *PTEMPFILETEMP;

/*
 *@@ TempFileCallback:
 *      WIArchive callback for unpack while
 *      wpiExtractTempFile is running.
 *
 *@@added V0.9.9 (2001-04-04) [umoeller]
 */

int TempFileCallback(short mode,         // in: CBM_* flags def'd in wiarchive.h
                     short s,            // in: percentage of current file
                     WIFileHeader* pwifh, // in: file header of current file
                     void *pvUser)      // in: user param; we use the single file name here
{
    int rc = CBRC_ABORT;

    PTEMPFILETEMP ptft = (PTEMPFILETEMP)pvUser;

    switch (mode)
    {
        case CBM_NEXTFILE:
        {
            if (ptft->fFound)
                // found the file on the last loop:
                // stop now
                rc = CBRC_ABORT;
            else if (stricmp(pwifh->name,
                             ptft->pcszFile2Check))
                // no match:
                rc = CBRC_SKIP;
            else
            {
                // only for this one file:
                rc = CBRC_PROCEED;
                // report "abort" on the next file
                ptft->fFound = TRUE;
            }
        break; }

        case CBM_PERCENTAGE:
            rc = CBRC_PROCEED;
        break;

        case CBM_ERR_WRITE:
        case CBM_ERR_READ:
        case CBM_ERR_ZLIB:
        default:
            ptft->ulError = mode;
            rc = CBRC_ABORT;
    }

    return (rc);
}

/*
 *@@ wpiExtractTempFile:
 *      attempts to extract the single file with the
 *      given file name in the given package into
 *      the system's %TEMP% directory.
 *
 *      Returns:
 *
 *      --  NO_ERROR: file was extracted, and pstrTempFile
 *          name receives the full path of the temp file
 *          so the caller can play with it.
 *
 *      --  ERROR_FILE_NOT_FOUND: file doesn't exist in
 *          the specified package.
 *
 *      --  ERROR_INVALID_DATA: cannot write to disk, or
 *          archive is broken.
 *
 *@@added V0.9.9 (2001-04-04) [umoeller]
 *@@changed V0.9.13 (2001-06-21) [umoeller]: fixed bad DosQueryCurrentDir, thanks Yuri Dario
 */

APIRET wpiExtractTempFile(ULONG ulPckIndex,     // in: package to extract from
                          const char *pcszFileInPck, // in: file name in pck
                          BSString *pstrTempFileName)   // out: temporary file created
{
    APIRET arc = NO_ERROR;

    // create temp dir... the backend is too dumb to
    // allow us to specify an output file name, so
    // we first create

    CHAR    szDir[CCHMAXPATH];

    if (!(arc = doshCreateTempFileName(szDir,
                                       NULL,        // $(TEMP)
                                       "wpi",       // prefix
                                       "tmp")))     // extension
    {
        if (!(arc = DosCreateDir(szDir,
                                 NULL)))
        {
            CHAR    szStartDir[CCHMAXPATH];
            ULONG   cbStartDir = sizeof(szStartDir);        // V0.9.13 (2001-06-21) [umoeller]
            DosQueryCurrentDir(0, szStartDir, &cbStartDir);

            if (!(arc = doshSetCurrentDir(szDir)))
            {
                TEMPFILETEMP tft = {
                                        pcszFileInPck,
                                        FALSE,      // not found
                                        0           // no error yet
                                   };
                G_WpiGlobals.pFirstArchive->_Arc.setCallbackFunc(TempFileCallback,
                                                                 // user param: file name 2 check
                                                                 (VOID*)&tft);

                G_WpiGlobals.pFirstArchive->_Arc.unpack(ulPckIndex);

                if (!tft.fFound)
                    arc = ERROR_FILE_NOT_FOUND;
                else
                    if (tft.ulError)
                        arc = ERROR_INVALID_DATA;
                    else
                    {
                        // else: OK, we got the file. Cute...
                        // create another temp file name in
                        // the parent directory and move the
                        // file there
                        string  strTempTemp;
                        CHAR    szTempFinal[CCHMAXPATH];
                        strTempTemp = szDir;
                        if (strTempTemp[strTempTemp.length() - 1] != '\\')
                            strTempTemp += '\\';
                        strTempTemp += pcszFileInPck;

                        if (!(arc = doshCreateTempFileName(szTempFinal,
                                                           NULL,        // $(TEMP)
                                                           "wpi",       // prefix
                                                           "tmp")))     // extension
                        {
                            // move to parent now
                            arc = DosMove(strTempTemp.c_str(),
                                          szTempFinal);
                            if (!arc)
                            {
                                // move to parent directory
                                DosSetCurrentDir("..");
                                // remove the empty dir
                                DosDeleteDir(szDir);

                                // give the file name to the parent,
                                // if he wants us to
                                if (pstrTempFileName)
                                    pstrTempFileName->assign(szTempFinal);

                                // store the file in the list of
                                // temp files to be deleted on exit
                                G_WpiGlobals.strTempFiles += szTempFinal;
                                G_WpiGlobals.strTempFiles += "|";
                            }
                        }
                    }
            }

            // reset directory
            doshSetCurrentDir(szStartDir);
        }
    }

    return (arc);
}

/*
 *@@ wpiCreateCIDFile:
 *      this creates a CID file according to the current
 *      global and package data. pszFilename must contain
 *      the full pathname of where the CID file should be
 *      created. This will be a plain OS/2 batch (.CMD)
 *      file and should have the .CMD extension.
 *
 *      This gets called from the GUI when the "Create CID file"
 *      button is pressed.
 *
 *@@changed V0.9.6 (2000-11-23) [umoeller]: fixed WARPIN_SELECTPACKAGE
 */

BOOL wpiCreateCIDFile(const char *pcszCmdName,      // in: .cmd name (q'fied)
                      const char *pcszArcName)      // in: .wpi name (q'fied)
{
    BOOL brc = FALSE;

    FILE* CIDFile = fopen(pcszCmdName, "w");  // text file for writing, replace;
    if (CIDFile)
    {
        fprintf(CIDFile, "REM WarpIN CID file\n");
        DATETIME DT;
        DosGetDateTime(&DT);
        fprintf(CIDFile,
                "REM Created %04d-%02d-%02d, %02d:%02d:%02d\n",
                DT.year, DT.month, DT.day,
                DT.hours, DT.minutes, DT.seconds);

        fprintf(CIDFile, "\nREM     If the following is set to NO, WarpIN will not display pages,\n"
                         "REM     but jump to the installation directly.\n\n"
                         "SET WARPIN_DISPLAYPAGES=YES\n");

        fprintf(CIDFile, "\nREM     Here come the packages:\n\n");

        list<FEJobBase*>::iterator JobStart = G_WpiGlobals.AllJobsList.begin(),
                                   JobEnd = G_WpiGlobals.AllJobsList.end();
        for (; JobStart != JobEnd; JobStart++)
        {
            FEJobBase *pTemp = *JobStart;
            // check if this is an install job (not group job)
            FEInstallJob *pJobThis = pTemp->IsInstallJob();
            if (pJobThis)
            {
                const FEArcPackageBase *pPackageThis = pJobThis->_pArcPackage;

                PSZ pszValue = NULL;

                // V0.9.6 (2000-11-23) [umoeller]
                switch (pJobThis->QueryJobID())
                {
                    case JOB_INSTALL_IGNORE:
                        pszValue = "IGNORE";
                    break;

                    case JOB_INSTALL:
                        pszValue = "INSTALL";
                    break;

                    case JOB_DEINSTALL:
                        pszValue = "DEINSTALL";
                    break;

                    // we can't get JOB_GROUP_MIXED because this
                    // must not be a group job
                }

                if (pszValue)
                {
                    fprintf(CIDFile,
                            "SET WARPIN_SELECTPACKAGE%d=%s\n",
                            pPackageThis->_PackHeader.number,
                            pszValue);

                    // target path
                    fprintf(CIDFile,
                            "SET WARPIN_TARGETPATH%d=%s\n",
                            pPackageThis->_PackHeader.number,
                            pJobThis->QueryTargetPath());
                }
            }
        }

        fprintf(CIDFile, "\nREM     Here comes configuration: either YES or NO\n\n");
        fprintf(CIDFile,
                "SET WARPIN_UPDATECONFIGSYS=%s\n",
                ((G_WpiGlobals.ulAllowConfig & PCK_CONFIGSYS) != 0)
                      ? "YES"
                      : "NO");
        fprintf(CIDFile,
                "SET WARPIN_INSTALLWPSCLASSES=%s\n",
                ((G_WpiGlobals.ulAllowConfig & PCK_WPSCLASSES) != 0)
                      ? "YES"
                      : "NO");
        fprintf(CIDFile,
                "SET WARPIN_CREATEWPSOBJECTS=%s\n",
                ((G_WpiGlobals.ulAllowConfig & PCK_WPSOBJECTS) != 0)
                      ? "YES"
                      : "NO");

        fprintf(CIDFile, "\nREM     Set the following to any of PROMPT, SKIP, or OVERWRITE:\n\n");
        fprintf(CIDFile,
                "SET WARPIN_IFSAMEDATE=%s\n",
                (G_WpiGlobals.ulIfSameDate == FE_PROMPT) ? "PROMPT"
                : (G_WpiGlobals.ulIfSameDate == FE_SKIP) ? "SKIP"
                : "OVERWRITE");
        fprintf(CIDFile,
                "SET WARPIN_IFEXISTINGOLDER=%s\n",
                (G_WpiGlobals.ulIfExistingOlder == FE_PROMPT) ? "PROMPT"
                : (G_WpiGlobals.ulIfExistingOlder == FE_SKIP) ? "SKIP"
                : "OVERWRITE");
        fprintf(CIDFile,
                "SET WARPIN_IFEXISTINGNEWER=%s\n",
                (G_WpiGlobals.ulIfExistingNewer == FE_PROMPT) ? "PROMPT"
                : (G_WpiGlobals.ulIfExistingNewer == FE_SKIP) ? "SKIP"
                : "OVERWRITE");

        fprintf(CIDFile, "\nREM     Full path specification of install log file:\n\n");
        fprintf(CIDFile,
                "SET WARPIN_LOGFILEPATH=%s\n",
                G_WpiGlobals.szLogFilename);

        fprintf(CIDFile, "\nREM     Call WarpIN with the archive now:\n\n");
        fprintf(CIDFile,
                "SET BEGINLIBPATH=%s\n",
                G_WpiGlobals.szWarpINPath);
        fprintf(CIDFile,
                "%s %s\n\n\n",
                G_WpiGlobals.szWarpINFile,
                pcszArcName);

        fclose(CIDFile);
        brc = TRUE;
    }
    return (brc);
}


