
/*
 *@@sourcefile wpi_install_thread.cpp:
 *      this holds the WarpIN Install thread which runs while
 *      WarpIN is unpacking files. It's the install thread
 *      which calls the back-end unpack functions.
 *
 *      The Install thread is started simply by calling
 *      wpiStartInstallThread. From then on, the GUI receives
 *      lots of callbacks. See wpiInstallThread.
 *
 *      This has been separated from warpin.cpp.
 *
 *@@added V0.9.1 (2000-02-06) [umoeller]
 *
 *@@header "frontend\warpin.h"
 *@@header "base\bs_errors.h"
 *@@header "frontend\calbacks.h"
 */

/*      This file Copyright (C) 1998-2000 Ulrich Mller.
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, in version 2 as it comes in the COPYING
 *      file of this distribution.
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 */

#define OS2EMX_PLAIN_CHAR
    // this is needed for "os2emx.h"; if this is defined,
    // emx will define PSZ as _signed_ char, otherwise
    // as unsigned char

#define INCL_DOS
#define INCL_DOSERRORS
#define INCL_WINSHELLDATA
#define INCL_WINWORKPLACE
#include <os2.h>

#include <stdlib.h>
#ifdef __IBMCPP__
    #include <direct.h>
#endif
#include <string.h>
#include <setjmp.h>             // needed for except.h
#include <assert.h>             // needed for except.h

#include <ctype.h>

#ifdef __IBMCPP__
    #include <terminat.h>
#endif

#include "setup.h"              // was missing (99-12-23) [umoeller]

// include's from helpers
#include "helpers\dosh.h"
#include "helpers\stringh.h"
#include "helpers\threads.h"
#include "helpers\xstring.h"

// back-end includes
#include "wiarchive\wiarchive.h"

// base includes (99-11-07) [umoeller]
#include "base\bs_string.h"
#include "base\bs_errors.h"
#include "base\bs_logger.h"

// front-end includes
#include "frontend\fe_base.h"
#include "frontend\fe_archive.h"
#include "frontend\fe_job.h"

#include "frontend\warpin.h"
#include "frontend\calbacks.h"

#include "helpers\except.h"     // must be at the bottom because wiarchive includes stdio

#pragma hdrstop

/* ******************************************************************
 *
 *  Global variables
 *
 ********************************************************************/

extern WPIGLOBALS   G_WpiGlobals;
    // in warpin.cpp

ULONG           G_ulCurrentFile = 0,
                G_ulFilesTotal = 0,
                G_ulCurrentPackage = 0,
                G_ulPackagesTotal = 0;
double          G_dTotalBytesToDo = 0,
                G_dBytesDone = 0,
                G_dBytesThisPackage = 0,
                G_dBytesOfLastFile = 0;
char            G_szLastFile[300] = "";

BSFileLogger    *G_pLogFile = NULL;

// current package being unpacked
// FEArcPackagePck *G_pCurrentArcPck = NULL;
FEInstallJob    *G_pCurrentInstallJob = NULL;

/*
 *@@ LogRC:
 *
 *@@added V0.9.9 (2001-03-30) [umoeller]
 */

VOID LogRC(BSFileLogger *pLogFile,
           int rc)
{
    switch (rc)
    {
        case CBRC_ABORT:
            pLogFile->Append("  abort requested!\n");
        break;

        case CBRC_SKIP:
            pLogFile->Append("  skipping existing\n");
        break;

        case CBRC_PROCEED:
            pLogFile->Append("  overwriting existing\n");
        break;

        case CBRC_RETRY:
            pLogFile->Append("  retrying\n");
        break;

        case CBRC_UNLOCK:
            pLogFile->Append("  unlocking\n");
        break;

        case CBRC_DEFER:
            pLogFile->Append("  deferring through CONFIG.SYS\n");
        break;

        default:
            pLogFile->Append("  unknown user return code %d\n", rc);
        break;
    }
}

/*
 *@@ WICallback:
 *      this is the callback function which was given to the
 *      WIArchive class in wpiInstallThread. As a result,
 *      this runs on the Install thread whenever the back end
 *      reports anything to us during execution of the
 *      WIArchive::unpack method.
 *
 *      This includes progress of the installation and all
 *      kinds of errors. This func is therefore pretty
 *      complex. Here we analyze the parameters and call the
 *      GUI callbacks accordingly.
 */

int  WICallback(short mode,         // in: CBM_* flags def'd in wiarchive.h
                short s,            // in: percentage of current file
                WIFileHeader* pwifh, // in: file header of current file
                void *pvUser)       // in: user param (NULL presently)
{
    int rc = CBRC_ABORT;

    switch (mode)
    {
        /*
         * CBM_NEXTFILE:
         *      this comes just before the WIArchive class attempts to open
         *      a new output file for writing. "param" is always 0, "pwifh"
         *      points to the file header which is about to be opened.
         *      This allows the front-end to do two things:
         *
         *      a)   update the "current file" display by calling
         *           guiUpdateFile();
         *
         *      b)   check for whether that file exists already:
         *           -- if not, we return CBRC_PROCEED;
         *           -- if it does exist, we compare the existing date
         *              with the one in the archive and call guiFileExists()
         *              if G_WpiGlobals says we should prompt. In any case,
         *              we need to return CBRC_PROCEED (overwrite) or
         *              CBRC_SKIP.
         */

        case CBM_NEXTFILE:
        {
            G_ulCurrentFile++;

            // store the new current file for later
            strcpy(G_szLastFile, pwifh->name);

            // increase the bytes we've done
            G_dBytesDone += G_dBytesOfLastFile;
            // remember the size of this file for later,
            // so we can calculate the total percentage
            G_dBytesOfLastFile = pwifh->origsize;

            // update logger
            if (G_pLogFile)
                G_pLogFile->Append("Unpacking \"%s\"\n", G_szLastFile);

            // update display
            guiUpdateFile(pwifh, G_ulCurrentFile, G_ulFilesTotal);

            rc = CBRC_PROCEED;

            // check for whether the file exists
            FILE *TestFile = fopen (pwifh->name, "rb");
            if (TestFile != NULL)
            {
                // file exists:

                fseek(TestFile, 0, SEEK_END);
                if (ftell(TestFile) == 0)
                {
                    if (G_pLogFile)
                        G_pLogFile->Append("  overwriting existing 0-byte file\n");
                }
                else
                {
                    // file exists and has more than 0 bytes:
                    GUIFILEERROR fi;
                    wpiPrepareGUIFILEERROR(&fi,
                                           NULL,        // get current dir
                                           pwifh);

                    // now check WPIGLOBALS for confirmation settings
                    BOOL    fPrompt = FALSE;
                    PULONG  pulQuery = 0;
                    if (fi.iComp < 0)
                        // existing is newer:
                        pulQuery = &G_WpiGlobals.ulIfExistingNewer;
                    else if (fi.iComp == 0)
                        // same date:
                        pulQuery = &G_WpiGlobals.ulIfSameDate;
                    else
                        // existing is older:
                        pulQuery = &G_WpiGlobals.ulIfExistingOlder;

                    switch (*pulQuery)
                    {
                        // 0: prompt
                        // 1: skip
                        // 2: overwrite
                        case 0: fPrompt = TRUE; break;
                        case 1: rc = CBRC_SKIP; break;
                        case 2: rc = CBRC_PROCEED; break;
                    }

                    if (fPrompt)
                    {
                        if (G_pLogFile)
                            G_pLogFile->Append("  file exists, prompting user\n");

                        // OK, we should prompt the user:
                        rc = guiFileExists(&fi);
                                // this returns:
                                //      ERROR_PROCEED    replace this file
                                //      ERROR_SKIP       skip this file
                    }

                    if (G_pLogFile)
                        LogRC(G_pLogFile, rc);
                }
                fclose(TestFile);
            }

            // store the new current file in the
            // current package's PSZ file list for
            // the global database later
            if (G_pCurrentInstallJob)
                G_pCurrentInstallJob->AddFileHeader(pwifh);

        break; }

        /*
         * CBM_PERCENTAGE:
         *      in this case, "param" contains the percentage of the current
         *      file being processed, and pwifh the current file header.
         *      The return value doesn't matter.
         */

        case CBM_PERCENTAGE:
        {
            // calculate the bytes we've really done so far
            double dBytesDone2 = G_dBytesDone + ((pwifh->origsize * s) / 100);
            // calculate total percentage
            // SHORT sPercent = (SHORT)((dBytesDone2 / dTotalBytesToDo) * 100);
            // call percentage callback (for progress bar)
            guiUpdateBytes(dBytesDone2, G_dTotalBytesToDo);
        break; }

        /*
         * CBM_ERR_WRITE:
         *      error writing into output file; this is most probable when the
         *      target disk is full.
         *      "param" then has one of the CBREC_* values, which identify
         *      whether the error is recoverable. Depending on that value,
         *      we may return one of the CBRC_* values.
         */

        case CBM_ERR_WRITE:
        {
            APIRET  arc;

            // first check if maybe the write error was due
            // to a full disk, because in that case, it's
            // not ERROR_DISK_FULL which is returned, so
            // we better do this
            ULONG   ulDisk, ulMap;
            DosQueryCurrentDisk(&ulDisk, &ulMap);
            double d = 0;
            arc = doshQueryDiskFree(ulDisk, &d);
            if (arc == NO_ERROR)
                if (d < pwifh->origsize)
                    arc = ERROR_DISK_FULL;

            if (arc != ERROR_DISK_FULL)
            {
                // get the DOS error number; since the back end
                // is OS-independent, we attempt to recreate the
                // error by opening the file for writing (99-10-26) [umoeller]
                CHAR    szFailingFile[2*CCHMAXPATH];
                _getcwd(szFailingFile, sizeof(szFailingFile));
                strcat(szFailingFile, "\\");
                strcat(szFailingFile, pwifh->name);

                HFILE       hfFailing = 0;
                ULONG       ulAction = 0;
                arc = DosOpen(szFailingFile,
                              &hfFailing,
                              &ulAction,
                              0,             // new file size
                              FILE_NORMAL,
                              OPEN_ACTION_REPLACE_IF_EXISTS,
                              OPEN_FLAGS_FAIL_ON_ERROR
                                 | OPEN_SHARE_DENYREADWRITE
                                 | OPEN_ACCESS_READWRITE,
                              NULL);     // EAs;
                if (arc == NO_ERROR)
                {
                    // shouldn't happen, but just to make sure
                    DosClose(hfFailing);
                    // the GUI needs an error code, so we invent one
                    arc = ERROR_WRITE_FAULT;
                }
            }

            if (arc == ERROR_SHARING_VIOLATION)
            {
                // "file in use by another process":

                if (G_pLogFile)
                    G_pLogFile->Append("  file is locked\n");

                // use special callback for this
                GUIFILEERROR fi;
                wpiPrepareGUIFILEERROR(&fi,
                                       NULL,        // get current dir
                                       pwifh);
                rc = guiFileLockedError(&fi,
                                        arc);

                if (G_pLogFile)
                    LogRC(G_pLogFile, rc);

                switch (rc)
                {
                    case CBRC_SKIP:
                        // ignore the error, go on (skip the file).
                        rc = CBRC_SKIP;
                    break;

                    case CBRC_RETRY:
                        // have the back end retry
                        rc = CBRC_RETRY;
                    break;

                    case CBRC_UNLOCK:
                        // unlock the file:
                        #ifndef __EMX__
                            // we use the undocumented DosReplaceModule API.
                            DosReplaceModule(fi.szFilename,     // old module
                                             NULL,              // new module
                                             NULL);             // backup module
                        #endif
                        // and have the back end retry now
                        rc = CBRC_RETRY;
                    break;

                    case CBRC_DEFER:
                        // have the install thread defer processing thru CONFIG.SYS
                        DosBeep(2000, 1000);
                        rc = CBRC_SKIP; break;
                }
            }
            else
            {
                if (G_pLogFile)
                    G_pLogFile->Append("  write error %d occured\n", arc);

                // error other than "locked":
                rc = guiFileError(G_pCurrentInstallJob, // G_pCurrentArcPck,
                                  pwifh,
                                  ECLS_DOS_WRITE,
                                  arc,
                                  s); // CBREC_* param passed from WIArchive
                if (G_pLogFile)
                    LogRC(G_pLogFile, rc);

                // if the user presses "Cancel" there,
                // that function will terminate everthing
            }
        break; }

        /*
         * CBM_ERR_READ:
         *      error reading from input file; maybe the download is corrupt.
         *      "param" then has one of the CBREC_* values below, which identify
         *      whether the error is recoverable. Depending on that value,
         *      we may return one of the CBRC_* values.
         */

        case CBM_ERR_READ:
        {
            if (G_pLogFile)
                G_pLogFile->Append("  read error occured\n");

            rc = guiFileError(G_pCurrentInstallJob, // G_pCurrentArcPck,
                              pwifh,
                              ECLS_DOS_READ,
                              #ifdef __IBMCPP_
                                  _doserrno,
                              #else
                                  errno,
                              #endif
                              s); // CBREC_* param passed from WIArchive
                // if the user presses "Cancel" there,
                // that function will terminate everthing

            if (G_pLogFile)
                LogRC(G_pLogFile, rc);
        break; }

        /*
         * CBM_ERR_ZLIB:
         *      error somewhere in the zlib decompression part of WIArchive;
         *      this usually signifies that the archive is broken. We then
         *      terminate installation, since there's no way to recover.
         *      "param" then contains one of the ZLIBERR_* flags below.
         */

        case CBM_ERR_ZLIB:
        {
            if (G_pLogFile)
                G_pLogFile->Append("  ZLIB error occured\n");

            rc = guiFileError(G_pCurrentInstallJob, // G_pCurrentArcPck,
                              NULL,              // WIFileHeader, unknown
                              ECLS_DECOMP,
                              s,
                              CBREC_CANCEL);
                            // and stop!
        break; }

        /*
         * default:
         *      some other error occured. This is probably
         *      just as serious, so we'll stop.
         */

        default:
        {
            if (G_pLogFile)
                G_pLogFile->Append("  unknown error occured\n");

            rc = guiFileError(G_pCurrentInstallJob, // G_pCurrentArcPck,
                              NULL,
                              ECLS_UNKNOWN,
                              s,
                              CBREC_CANCEL);
                            // and stop!
        }
    }

    return (rc);
}

/*
 *@@ wpiInstallThread:
 *      thread function for the Install thread.
 *
 *      This gets started by wpiStartInstallThread and
 *      does the actual unpacking of the archive
 *      packages.
 *
 *      This is pretty short, because it only creates
 *      the target directories and then calls
 *      WIArchive::unpack, which will in turn call
 *      WICallback above.
 *
 *@@changed V0.9.0 (99-11-01) [umoeller]: adjusted for FEInstallJob
 *@@changed V0.9.9 (2001-03-30) [umoeller]: added log file
 *@@changed V0.9.12 (2001-05-31) [umoeller]: fixed log file crash on exit
 *@@changed V0.9.13 (2001-06-21) [umoeller]: fixed bad DosQueryCurrentDir, thanks Yuri Dario
 */

void _Optlink wpiInstallThread(PTHREADINFO pti)
{
    BSFileLogger *pLogFile = (BSFileLogger*)pti->ulData;

    // install the LOUD exception handler (except.h)
    TRY_LOUD(excpt1)
    {
        if (pLogFile)
            pLogFile->Append("Install thread started.\n");

        // first go thru the list of packages which are to
        // be installed to find out the total bytes
        // that will be installed
        // changed (99-10-31) [umoeller]
        list<FEJobBase*>::iterator JobStart = G_WpiGlobals.AllJobsList.begin(),
                                   JobEnd = G_WpiGlobals.AllJobsList.end();
        for (; JobStart != JobEnd; JobStart++)
        {
            FEJobBase *pTemp = *JobStart;
            FEInstallJob *pJobThis = pTemp->IsInstallJob();
            if (pJobThis)
            {
                if (pJobThis->Is2BeInstalled())
                {
                    G_ulPackagesTotal++;
                    G_ulFilesTotal += pJobThis->QueryPckFilesCount();
                    G_dTotalBytesToDo += pJobThis->QueryPckOrigSize();
                }
            }
        }

        // set logger for callbacks
        G_pLogFile = pLogFile;      // can be NULL

        if (    (G_ulPackagesTotal)
             && (G_dTotalBytesToDo)
           )
        {
            // alright, now let's go:
            // get current directory, which we need to switch
            // back to later
            CHAR    szStartDir[CCHMAXPATH];
            ULONG   cbStartDir = sizeof(szStartDir);        // fixed V0.9.13 (2001-06-21) [umoeller]
            DosQueryCurrentDir(0, szStartDir, &cbStartDir);

            if (pLogFile)
            {
                CHAR sz[50];
                pLogFile->Append("%d packages to install, %s uncompressed bytes total.\n",
                                G_ulPackagesTotal,
                                strhThousandsDouble(sz,
                                                    G_dTotalBytesToDo,
                                                    '.'));
            }

            // go for installing the packages!
            JobStart = G_WpiGlobals.AllJobsList.begin();
            JobEnd = G_WpiGlobals.AllJobsList.end();
            for (; JobStart != JobEnd; JobStart++)
            {
                FEJobBase *pTemp = *JobStart;
                FEInstallJob *pJobThis = pTemp->IsInstallJob();
                if (pJobThis)
                {
                    // to be installed, package, and selected?
                    if (pJobThis->Is2BeInstalled())
                    {
                        FEArcPackagePck *pPackageThis = (FEArcPackagePck*)(pJobThis->_pArcPackage);

                        // set global variables for this package
                        G_ulCurrentPackage++;
                        G_dBytesThisPackage = pJobThis->QueryPckOrigSize();

                        if (pLogFile)
                        {
                            CHAR sz[50];
                            pLogFile->Append("Unpacking files in package \"%s\".\n",
                                             pJobThis->QueryPckPackageID());
                            pLogFile->IncIndent(4);
                            pLogFile->Append("Target path: \"%s\".\n",
                                             pJobThis->QueryTargetPath());
                            pLogFile->Append("Uncompressed bytes: %s.\n",
                                             strhThousandsDouble(sz,
                                                                 G_dBytesThisPackage,
                                                                 '.'));
                        }

                        // update gui status
                        guiUpdatePackage(pJobThis,
                                         G_ulCurrentPackage,
                                         G_ulPackagesTotal);

                        // create target directory
                        APIRET arc = NO_ERROR;

                        do
                        {
                            BSString strTargetPath = pJobThis->QueryTargetPath();
                            // create path and change dir to it
                            arc = doshCreatePath(strTargetPath.c_str(),
                                                 FALSE);        // not hidden
                            if (arc == NO_ERROR)
                                arc = doshSetCurrentDir(strTargetPath.c_str());

                            if (arc == NO_ERROR)
                            {
                                // set global variable to current package
                                // G_pCurrentArcPck = pPackageThis;
                                G_pCurrentInstallJob = pJobThis;

                                // UNPACK THIS PACKAGE;
                                // this will keep calling the callback above
                                pJobThis->Unpack();
                            }
                            else
                            {
                                if (pLogFile)
                                    pLogFile->Append("Error creating directory \"%s\".\n",
                                                     strTargetPath.c_str());

                                guiFileError(G_pCurrentInstallJob, // G_pCurrentArcPck,
                                             (PSZ)strTargetPath.c_str(),
                                             ECLS_DOS_CREATEDIR,
                                             arc,
                                             CBREC_RETRYCANCEL);
                                    // if the user presses "Cancel" there,
                                    // that will terminate everthing
                            }
                        } while (arc != NO_ERROR);

                        if (pLogFile)
                        {
                            pLogFile->IncIndent(-4);
                            pLogFile->Append("Done unpacking files in package \"%s\".\n",
                                             pJobThis->QueryPckPackageID());
                        }

                    } // end if ((**PackageStart).ulType == PCK_PCK)
                }
            } // end for (; PackageStart != PackageEnd; PackageStart++)

            // finally, set percentage to 100
            guiUpdateBytes(G_dTotalBytesToDo,
                           G_dTotalBytesToDo);
            DosSleep(300);
        }

        G_WpiGlobals.fInstallComplete = TRUE;

        if (pLogFile)
            pLogFile->Append("Install thread is exiting.\n");

        // PM function to clean up
        // (moved this below pLogFile->Append, or we'll crash)
        // V0.9.12 (2001-05-31) [umoeller]
        guiDoneWithInstall();
    }
    CATCH (excpt1)
    {
        // exception occured:
        guiFileError(G_pCurrentInstallJob, // G_pCurrentArcPck,
                     "An exception occured in the WarpIn install thread. "
                     "A file named \"WPITRAP.LOG\" was created in the root "
                     "directory of your boot drive, which contains more information.",
                     ECLS_EXCEPTION, 0,
                     CBREC_CANCEL); // terminate only
    } END_CATCH();

}


