/* $Id: wic.cpp,v 1.12 2001/07/06 16:53:09 umoeller Exp $ */


/*
 *@@sourcefile wic.cpp:
 *      this is the command-line WarpIn Archive Creator, "wic.exe".
 *
 *      This file Copyright (C) 1998-2000 Ulrich Mller.
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, in version 2 as it comes in the COPYING
 *      file of this distribution.
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 */

#define OS2EMX_PLAIN_CHAR
    // this is needed for "os2emx.h"; if this is defined,
    // emx will define PSZ as _signed_ char, otherwise
    // as unsigned char

#define INCL_DOS
#define INCL_DOSERRORS
#include <os2.h>

#include <io.h>
#include <signal.h>             // added V0.9.12 (2001-06-11) [bird]

#ifdef __IBMCPP__
    #include <direct.h>
#endif

#include "setup.h"              // added V0.9.2 (2000-03-15) [umoeller]

#include "helpers\ansiscrn.h"
#include "helpers\dosh.h"
#include "helpers\stringh.h"

#include "bldlevel.h"

#include "wiarchive\wiarchive.h"

#pragma hdrstop

class BSCancelExcpt                     //bird
{                                       //bird
public:                                 //bird
    int iErrno;                         //bird
    BSCancelExcpt(int iErrno)           //bird
        : iErrno(iErrno)                //bird
    {}                                  //bird
};                                      //bird

/*
 *@@ PackageMaskSpec:
 *      structure created for each package passed
 *      on the command line in "add" mode.
 *
 *      These things are created in main() and passed to AddMode().
 */

struct PackageMaskSpec
{
    LONG        lPackage;           // package no. to add to (as on cmd line)
    PSZ         pszSubdir;          // if != NULL: directory where to search
                                    // for files ("-c" switch)
    BOOL        fRecurse;           // if TRUE: recurse into subdirectories ("-r" switch)
    list<PSZ>   listFileMasks;      // list of file masks to add to package, as
                                    // specified on command line
    ULONG       cFileMasks;         // count of file masks in listFileMasks

    // constructor
    PackageMaskSpec()
    {
        lPackage = 0;
        pszSubdir = 0;
        fRecurse = FALSE;
        cFileMasks = 0;
    }
};

/* ******************************************************************
 *
 *  Global variables
 *
 ********************************************************************/

char        G_szFilestem[256] = "";  // filestem given on the cmdline
char        G_szArcFile[256],
            G_szStubFile[256],
            G_szScriptFile[256];

list<PackageMaskSpec*> G_listPackageMaskSpecs;
    // list of files to work on in 'Add' mode
ULONG       G_ulPackageMaskSpecsCount = 0;
    // item count on that list

// verbosity level: 1 means "-v"
ULONG       G_ulVerbosity = 0;

/* ******************************************************************
 *
 *  Miscellaneous
 *
 ********************************************************************/

/*
 * ExplainParams:
 *      say hello if the params aren't right.
 *
 *@@changed V0.9.6 (2000-10-27) [umoeller]: added "-x", "-X" modes
 *@@changed V0.9.13 (2001-06-09)    [bird]: added "@paramfile".
 */

void ExplainParams(void)
{
    ANSI_fg_color(B_WHITE);
    printf("wic V"BLDLEVEL_VERSION" "__DATE__" - WarpIn archive creation and maintenance\n");
    ANSI_fg_color(WHITE);
    printf("(C) 1998-2001 Jens Bckman, Ulrich Mller\n");
    printf("Published under the GNU General Public Licence (see the COPYING file).\n");
    ANSI_fg_color(B_WHITE);
    printf("Syntax: ");
    ANSI_fg_color(WHITE);
    printf("wic <arc> -<mode> [-v] <params> [@paramfile]\n"
           "with <arc> being the archive filename to work on (default extension: .WPI).\n"
           "'-v' switches to 'verbose' mode with more output for testing.\n"
           "The different <mode>'s are:\n");
    ANSI_fg_color(B_WHITE);
    printf("Add files (-a):");
    ANSI_fg_color(WHITE);
                    printf(" wic <arc> -a {<pckno> [-r]  [-u<exe>] [-c<dir>] <mask>...}...\n"
           "    adds the files in <mask> to decimal <pckno> in filename <arc>.\n"
           "    '-r' will recurse into subdirectories for that one <pckno> only.\n"
           "    '-u<exe>' will use <exe> as stub for self-extracting archive.\n"
           "    '-c<dir>' will change to <dir> while collecting files for <pckno>.\n"
           "    You can specify several packages and/or several filemasks.\n"
           "    Type \"wic -ha\" for samples.\n");
    ANSI_fg_color(B_WHITE);
    printf("List packages (-l):");
    ANSI_fg_color(WHITE);
                        printf(" wic <arc> -l [<pckno>]\n"
           "    list the files in <pckno> in <arc>.\n"
           "    If <pckno> is not specified, an archive summary is displayed.\n");
    ANSI_fg_color(B_WHITE);
    printf("Set installation script (-s):");
    ANSI_fg_color(WHITE);
                                  printf(" wic <arc> -s <file>\n"
           "    sets <file> as the install script for <arc>.wpi. This can be\n"
           "    combined with 'Add' mode, e.g.:\n"
           "        wic test -s test.wis -a 1 *.cpp *.h 2 *.exe\n");
    ANSI_fg_color(B_WHITE);
    printf("Extract files (-x):");
    ANSI_fg_color(WHITE);
                        printf(" wic <arc> -x {[<pckno>] [<fmask>]}...\n"
           "    extracts files from <pckno> (def.: all) according to <fmask> (def.: *).\n");
    ANSI_fg_color(B_WHITE);
    printf("Extract script (-X):");
    ANSI_fg_color(WHITE);
                        printf(" wic <arc> -X [<filename>]\n");
    exit(1);
}

/*
 *@@ ExplainArc:
 *      called upon "wic -ha" for explaining "add mode" further.
 *
 *@@added V0.9.1 (2000-02-05) [umoeller]
 */

VOID ExplainArc(CHAR cMode)     // can be 'h'
{
    switch (cMode)
    {
        case 'a':
            ANSI_fg_color(B_WHITE);
            printf("Examples for archive mode (type \"wic -h\" to get the main help):\n");
            ANSI_fg_color(WHITE);
            printf(
                   "    wic test -a 1 *.exe\n"
                   "        adds all *.exe files in the current directory to package 1.\n"
                   "    wic test -a 1 *.cpp *.h 2 *.exe\n"
                   "        adds all *.cpp and *.h files in the current directory to package 1\n"
                   "        and all *.exe files in the current directory to package 2.\n"
                   "    wic test -a 1 -r -csubdir1 * 2 -r *.inf\n"
                   "        changes to subdirectory 'subdir1' first and adds all files in that\n"
                   "        directory and subdirectories to package 1. After that, all *.inf\n"
                   "        files in the current (previous) directory and its subdirectories\n"
                   "        are added to package 2.\n"
                  );
        break;

        default:
            ExplainParams();
    }

    exit(1);
}

/*
 * PrintFilesCallback:
 *      callback for WIArchive::for_all_files
 *      when enumerating files in "list mode".
 *
 *@@changed V0.9.6 (2000-11-23) [umoeller]: fixed divide-by-0 with 0-byte files
 */

int PrintFilesCallback(WIFileHeader* file,
                        unsigned long ulNotUsedHere)
{
    CHAR    szOriginal[20],
            szCompressed[20];

    printf(" %-40s %10s %10s   %3d%%\n",
                file->name,
                strhThousandsULong(szOriginal,
                                   file->origsize,
                                   '.'),
                strhThousandsULong(szCompressed,
                                   file->compsize,
                                   '.'),
                (file->origsize)    // V0.9.6 (2000-11-23) [umoeller]
                    ? 100 - ( (file->compsize * 100) / file->origsize)
                    : 0);

    return (0);
}

/* ******************************************************************
 *
 *  "Add" Mode
 *
 ********************************************************************/

/*
 * WICallbackAddMode:
 *      this is the general callback from the WIArchive class.
 *      Gets called while we're packing the files for progress
 *      display.
 *
 *@@changed V0.9.6 (2000-11-23) [umoeller]: fixed divide-by-0 with 0-byte files
 */

int WICallbackAddMode(short mode,
                      short s,
                      WIFileHeader *pwifh,
                      void *pvUser)
{
    static char     szLastFile[300] = "";
    static int      iLastPackage = -1;

    if (mode == CBM_PERCENTAGE)
    {
        if (iLastPackage != pwifh->package)
        {
            printf("Now going for package %d...\n", pwifh->package);
            iLastPackage = pwifh->package;
        }
        // s has the percentage
        if (strcmp(szLastFile, pwifh->name) != 0)
            // new file:
            strcpy(szLastFile, pwifh->name);
        else
            // go back in same line
            printf ("\r");
        printf ("Compressing %s (%03d%%)", pwifh->name, s);
        fflush(stdout);
    }
    else if (mode == CBM_UPDATING)
    {
        // ANSI: go right 60 chars
        printf("\r%c[%dC",
                    27,     // ESC
                    60);
        printf("%02d%% compression \n",
                (pwifh->origsize)   // V0.9.6 (2000-11-23) [umoeller]: avoid divide-by-0
                    ? 100 - ((pwifh->compsize * 100) / pwifh->origsize)
                    : 0);
        fflush(stdout);
    }
    else
    {
        printf("\nwic error: backend reported error %d. Terminating.\n", mode);
    }
    return (CBRC_ABORT);
}

/*
 *@@ AddFileMaskToPackage:
 *      this actually processes one file mask, as specified
 *      in PackageMaskSpec, and calls WIArchive.add for each file which was
 *      found. This changes to subdirectories and eventually
 *      recurses, if desired.
 *
 *      Gets called by AddMode().
 *
 *      Throws:
 *      -- BSCancelExcpt.
 *
 *@@added V0.9.0 (2000-01-03) [umoeller]
 *@@changed V0.9.1 (2000-02-07) [umoeller]: added "-c" subdir support
 *@@changed V0.9.9 (2001-04-06) [umoeller]: fixed root dir
 *@@changed V0.9.12 (2001-06-11) [umoeller]: various adjustments for error codes
 */

APIRET AddFileMaskToPackage(WIArchive *pArc,        // in: archive to append files to
                            PackageMaskSpec *pPackageMaskSpecThis, // in: spec what to append
                            PSZ pszFileMaskThis,    // in: current file mask
                            ULONG ulRecursionLevel) // initially 0
{
    APIRET arc = NO_ERROR;

    CHAR    szCurDirSaved[2*CCHMAXPATH] = "",
            szCurDir2Collect[2*CCHMAXPATH] = "";

    // change to subdir?
    if (ulRecursionLevel == 0)
        if (pPackageMaskSpecThis->pszSubdir)
        {
            printf("  changing to subdir \"%s\"...", pPackageMaskSpecThis->pszSubdir);
            // yes: get current directory (for switching back later)
            _getcwd(szCurDirSaved, sizeof(szCurDirSaved));
            int rc = doshSetCurrentDir(pPackageMaskSpecThis->pszSubdir);  // V0.9.12 (2001-06-11) [umoeller]
            if (rc)
            {
                // error:
                printf("\nwic error: Unable to change to subdir \"%s\".",
                       pPackageMaskSpecThis->pszSubdir);
                printf("\n           Current directory is: \"%s\".\n", szCurDirSaved);
                throw BSCancelExcpt(rc);
            }
            else
                printf(" OK.\n");
        }

    _getcwd(szCurDir2Collect, sizeof(szCurDir2Collect));

    // indent
    ULONG ul0;
    for (ul0 = 0; ul0 < ulRecursionLevel; ul0++)
        printf("  ");
    printf("  adding \"%s\" to package %d",
           pszFileMaskThis,
           pPackageMaskSpecThis->lPackage);
    if (pPackageMaskSpecThis->fRecurse)
        printf(" recursively");
    fflush(stdout);

    // DosFindFirst returns the file names only, so if the
    // user specified relative paths on the command line,
    // we'll need to manually add those again when adding
    // files to the archive.
    CHAR    szDrive[_MAX_DRIVE],        // "C:"
            szDir[_MAX_DIR],            // "\test\"
            szFilestem[_MAX_FNAME],     // "*"
            szExt[_MAX_EXT];            // ".obj"
    _splitpath(pszFileMaskThis, szDrive, szDir, szFilestem, szExt);

    // now check if this is valid
    if (szDrive[0] != 0)
    {
        // drive specified: no!
        printf("\nwic: Error in file mask \"%s\".\n"
               "You cannot store absolute drive specifications in an archive because this would\n"
               "require the user to have the same drive letters.\n",
               pszFileMaskThis);
        throw BSCancelExcpt(-1);
    }
    // check if first part of the path is a backslash; that's absolute too
    if (szDir[0] == '\\')
    {
        printf("\nwic: Error in file mask \"%s\".\n"
               "You cannot store absolute path specifications in an archive because this would\n"
               "prevent the user from installing to any directory.\n"
               "If you specify paths in file masks, use relative specifications only.\n",
               pszFileMaskThis);
        throw BSCancelExcpt(-1);
    }

    HDIR          hdirFindHandle = HDIR_CREATE;
    FILEFINDBUF3  ffb3           = {0};      // Returned from FindFirst/Next
    ULONG         ulResultBufLen = sizeof(FILEFINDBUF3);
    ULONG         ulFindCount    = 1;        // Look for 1 file at a time

    BOOL          fAnythingFound = FALSE;

    // 1) find files

    arc = DosFindFirst(pszFileMaskThis,      // PSZ file pattern
                       &hdirFindHandle,      // directory search handle
                       FILE_NORMAL,          // search attribute
                       &ffb3,                // result buffer
                       ulResultBufLen,       // result buffer length
                       &ulFindCount,         // number of entries to find
                       FIL_STANDARD);        // return level 1 file info

    if (arc == NO_ERROR)
    {
        // keep finding the next file until there are no more files
        while (arc != ERROR_NO_MORE_FILES)
        {
            // PSZ pszFileNameInArc;
            if (    // skip directory entries
                    (strcmp(ffb3.achName, ".") != 0)
                 && (strcmp(ffb3.achName, "..") != 0)
                    // skip the archive file being created
                 && (strcmp(ffb3.achName, G_szArcFile) != 0)
               )
            {
                CHAR    szNameInArchive[2*CCHMAXPATH],
                        szNameFullPath[3*CCHMAXPATH];
                sprintf(szNameInArchive, "%s%s",
                        szDir,
                        ffb3.achName);

                // fixed root dir V0.9.9 (2001-04-06) [umoeller]
                strcpy(szNameFullPath, szCurDir2Collect);
                if (szNameFullPath[strlen(szNameFullPath) - 1] != '\\')
                    strcat(szNameFullPath, "\\");
                strcat(szNameFullPath, szNameInArchive);

                if (G_ulVerbosity)
                {
                    // indent
                    printf("\n");
                    for (ul0 = 0; ul0 < ulRecursionLevel; ul0++)
                        printf("  ");
                    printf("    adding \"%s\" -> \"%s\"",
                           szNameFullPath,
                           szNameInArchive);
                }
                else
                    printf(".");
                fflush(stdout);

                pArc->add(szNameFullPath,       // real file name
                          szNameInArchive,      // file name in archive
                          pPackageMaskSpecThis->lPackage); // package no.
                fAnythingFound = TRUE;                                                              //bird
            }

            ulFindCount = 1;                    // reset find count
            arc = DosFindNext(hdirFindHandle,   // directory handle
                             &ffb3,             // result buffer
                             ulResultBufLen,    // result buffer size
                             &ulFindCount);     // number of entries to find

        } // endwhile

        arc = DosFindClose(hdirFindHandle);     // close our find handle
    }

    printf("\n");

    // 2) recurse for subdirectories

    if (pPackageMaskSpecThis->fRecurse)
    {
        hdirFindHandle = HDIR_CREATE;
        ulResultBufLen = sizeof(FILEFINDBUF3);
        ulFindCount    = 1;        // look for 1 file at a time

        CHAR    szDirsMask[CCHMAXPATH];
        sprintf(szDirsMask,
                "%s%s*",
                szDrive, szDir);

        // recursion enabled:
        // search again, for subdirs this time
        arc = DosFindFirst(szDirsMask,          // file pattern
                           &hdirFindHandle,     // directory search handle
                           MUST_HAVE_DIRECTORY, // search attribute
                           &ffb3,               // result buffer
                           ulResultBufLen,      // result buffer length
                           &ulFindCount,        // number of entries to find
                           FIL_STANDARD);       // return level 1 file info

        if (arc == NO_ERROR)
        {
            fAnythingFound = TRUE;

            // keep finding the next file until there are no more files
            while (arc != ERROR_NO_MORE_FILES)
            {
                if (    // skip directory entries
                        (strcmp(ffb3.achName, ".") != 0)
                     && (strcmp(ffb3.achName, "..") != 0)
                   )
                {
                    // valid subdirectory found:
                    CHAR szRecursionFileMask[CCHMAXPATH];
                    sprintf(szRecursionFileMask,
                            "%s%s%s\\%s%s",
                            szDrive, szDir, ffb3.achName, szFilestem, szExt);

                    // RECURSE!!
                    arc = AddFileMaskToPackage(pArc,
                                               pPackageMaskSpecThis,
                                               szRecursionFileMask,
                                               ulRecursionLevel + 1);
                }

                ulFindCount = 1;                    // reset find count
                arc = DosFindNext(hdirFindHandle,   // directory handle
                                 &ffb3,             // result buffer
                                 ulResultBufLen,    // result buffer size
                                 &ulFindCount);     // number of entries to find
            } // endwhile

            arc = DosFindClose(hdirFindHandle);     // close our find handle
        }
    }

    if (!fAnythingFound)
    {
        printf("\nwic error: No files for search mask \"%s\" found.\n",
                pszFileMaskThis);
        if (arc)                                                                                    //bird
            printf("OS/2 reported SYS%04d: \"%s\"\n", arc, doshQuerySysErrorMsg(arc));
        printf("wic warning: archive \"%s\" is invalid now.\n", G_szArcFile);
        throw BSCancelExcpt(ERROR_FILE_NOT_FOUND);                                                  //bird
    }

    // did we change to a subdir?
    if (ulRecursionLevel == 0)
        if (pPackageMaskSpecThis->pszSubdir)
        {
            int rc = doshSetCurrentDir(szCurDirSaved);                                              //bird
            if (rc)                                                                                 //bird
            {
                // error:
                printf("\nwic error: Unable to change to subdir \"%s\".",
                       pPackageMaskSpecThis->pszSubdir);
                printf("\n           Current directory is: \"%s\".\n", szCurDirSaved);
                throw BSCancelExcpt(rc);                                                            //bird
            }
        }

    return (arc);
}

/*
 *@@ AddMode:
 *      this gets called from main() if "-a" was
 *      specified ("add" mode). At this point,
 *      listPackageMaskSpecs has a list of specifications
 *      to add.
 *
 *      This function is responsible for adding all
 *      files to the archive and setting the script.
 *
 *      Throws:
 *      -- BSCancelExcpt.
 *
 *@@changed V0.9.13 (2001-06-04) [umoeller]: changed header field values.
 */

VOID AddMode(VOID)
{
    WIArchive   Arc;
    WIArcHeader ArcHeader;

    if (access(G_szArcFile, 0) == 0)
        printf("Archive \"%s\" exists and will be updated.\n",
                G_szArcFile);

    if (Arc.open(G_szArcFile, 1))
    {
        printf("wic: Error reading/creating archive \"%s\".\n", G_szArcFile);
        throw BSCancelExcpt(-1);
    }

    if (G_szStubFile[0])
       Arc.setStubFile(G_szStubFile);

    /* strcpy(ArcHeader.name_app, "Test application");
    strcpy(ArcHeader.name_dev, "Ulrich Mller"); */
    // replaced this V0.9.13 (2001-06-04) [umoeller]
    strcpy(ArcHeader.name_app, "reserved");
    strcpy(ArcHeader.name_dev, "reserved");
    strcpy(ArcHeader.inet_address, "N/A");
    ArcHeader.rev_app = 1;
    Arc.setArcHeader(ArcHeader);

    APIRET arc = NO_ERROR;

    if (G_ulPackageMaskSpecsCount)
    {
        printf("Collecting files:\n");
        fflush(stdout);

        // go thru the packages which were specified on the command line;
        // each of these PackageMaskSpec structures has in turn a list
        // of file masks to be added.
        // This list might be empty if only a script is to be added.
        list<PackageMaskSpec*>::iterator PckStart = G_listPackageMaskSpecs.begin(),
                                         PckEnd = G_listPackageMaskSpecs.end();
        for (; PckStart != PckEnd; PckStart++)
        {
            PackageMaskSpec* pPackageMaskSpecThis = *PckStart;

            char szPackageName[20];
            sprintf(szPackageName, "Pck%03d", pPackageMaskSpecThis->lPackage);
            Arc.setPackage(pPackageMaskSpecThis->lPackage, szPackageName);

            // go thru list of file masks for this package
            list<PSZ>::iterator FileMaskStart = pPackageMaskSpecThis->listFileMasks.begin(),
                                FileMaskEnd = pPackageMaskSpecThis->listFileMasks.end();

            for (; FileMaskStart != FileMaskEnd; FileMaskStart++)
            {
                PSZ pszFileMaskThis = *FileMaskStart;

                arc = AddFileMaskToPackage(&Arc,
                                           pPackageMaskSpecThis,
                                           pszFileMaskThis,
                                           0);
                if (arc != NO_ERROR)
                    break;
            } // end for FileMaskStart
        } // end for PckStart...
    } // end if (ulPackageMaskSpecsCount)

    if (arc == NO_ERROR)
        if (G_szScriptFile[0] != 0)
        {
            PSZ         pszScriptBuf = NULL;

            arc = doshLoadTextFile(G_szScriptFile, &pszScriptBuf);
            if (arc != NO_ERROR)
            {
                printf("Script file %s.wis not found. Terminating.\n", G_szFilestem);
                throw BSCancelExcpt(ERROR_FILE_NOT_FOUND);
            }
            printf("Script \"%s\".wis successfully read, %d bytes.\n",
                    G_szFilestem,
                    strlen(pszScriptBuf));
            printf("Writing script to archive... ");
            fflush(stdout);
            Arc.setScript(pszScriptBuf);
            printf("OK\n");
        }

    Arc.setCallbackFunc(WICallbackAddMode, NULL);

    Arc.close();

    printf("Done.\n");
}

/* ******************************************************************
 *
 *  "List" Mode
 *
 ********************************************************************/

/*
 *@@ ListMode:
 *
 *      Throws:
 *      -- BSCancelExcpt.
 *
 *@@changed V0.9.6 (2000-11-23) [umoeller]: fixed divide-by-0 with 0-byte files
 */

void ListMode(LONG lListPackage)
{
    WIArchive   Arc;

    CHAR szOriginal[30],
         szCompressed[30];

    if (Arc.open(G_szArcFile, 0))
    {
        printf("wic error: Unable to open \"%s\" for reading.\n", G_szArcFile);
        throw BSCancelExcpt(ERROR_OPEN_FAILED);
    }

    if (lListPackage == 0)
    {
        // "list all packages" mode:
        ULONG ulPckCount = 0,
              ulTotalFiles = 0,
              ulTotalCompressed = 0,
              ulTotalOriginal = 0;
        list<WIPackHeader*> *pPackageList = Arc.getPackList();
        list<WIPackHeader*>::iterator pckStart, pckEnd;
        pckStart = pPackageList->begin();
        pckEnd = pPackageList->end();
        for (; pckStart != pckEnd; pckStart++)
        {
            WIPackHeader* pHeaderThis = *pckStart;

            printf("Package %d\n", pHeaderThis->number);
            printf("  %4d files, size: %10s bytes original\n"
                   "                    %10s bytes compressed (%d%% compression)\n",
                    pHeaderThis->files,
                    strhThousandsULong(szOriginal,
                                       pHeaderThis->origsize,
                                       '.'),
                    strhThousandsULong(szCompressed,
                                       pHeaderThis->compsize,
                                       '.'),
                    (pHeaderThis->origsize) // V0.9.6 (2000-11-23) [umoeller]
                        ? 100 - ( (pHeaderThis->compsize * 100 ) / pHeaderThis->origsize)
                        : 0
                );

            ulTotalFiles += pHeaderThis->files;
            ulTotalCompressed += pHeaderThis->compsize;
            ulTotalOriginal += pHeaderThis->origsize;
            ulPckCount++;
        }
        if (ulTotalFiles == 0)
            // otherwise we'd get a division by zero below
            printf("No packages found.\n");
        else
            printf("Summary: %d package(s), %d files total\n"
                   "    %10s bytes original\n"
                   "    %10s bytes compressed (%d%% compression)\n",
                    ulPckCount,
                    ulTotalFiles,
                    strhThousandsULong(szOriginal,
                                       ulTotalOriginal,
                                       '.'),
                    strhThousandsULong(szCompressed,
                                       ulTotalCompressed,
                                       '.'),
                    (ulTotalOriginal)   // V0.9.6 (2000-11-23) [umoeller]
                        ? 100 - ( (ulTotalCompressed * 100 ) / ulTotalOriginal )
                        : 0
                );
    }
    else
    {
        // list one package only:
        // enumerate through package headers
        BOOL fFound = FALSE;
        list<WIPackHeader*> *pPackageList = Arc.getPackList();
        list<WIPackHeader*>::iterator pckStart = pPackageList->begin(),
                                      pckEnd = pPackageList->end();
        WIPackHeader* pPckThis = NULL;

        for (; pckStart != pckEnd; pckStart++)
        {
            pPckThis = *pckStart;
            if (pPckThis->number == lListPackage)
            {
                fFound = TRUE;
                break;
            }
        }

        if (fFound)
        {
            // pPckThis has found package
            printf("Listing files in package %d\n", lListPackage);
            printf(" %-40s %10s %10s   compression\n",
                        "filename",
                        "origsize",
                        "compsize");
            printf(" ----------------------------------------------------------------------------\n");
            Arc.forAllFiles(pPckThis, PrintFilesCallback, 0);

            printf("Package %d summary:\n", pPckThis->number);
            printf("  %4d files, size: %10s bytes original\n"
                   "                    %10s bytes compressed (%d%% compression)\n",
                    pPckThis->files,
                    strhThousandsULong(szOriginal,
                                       pPckThis->origsize,
                                       '.'),
                    strhThousandsULong(szCompressed,
                                       pPckThis->compsize,
                                       '.'),
                    (pPckThis->origsize)    // V0.9.6 (2000-11-23) [umoeller]
                        ? 100 - ( (pPckThis->compsize * 100 ) / pPckThis->origsize )
                        : 0
                );
        }
        else
        {
            printf("wic error: Package %d does not exist in archive \"%s\".\n",
                    lListPackage,
                    G_szArcFile);
            throw BSCancelExcpt(-1);
        }
    }
    Arc.close();
}

/* ******************************************************************
 *
 *  "Extract" Mode
 *
 ********************************************************************/

ULONG   G_ulCurrentFile = 0;

/*
 * WICallbackExtractMode:
 *      this is the general callback from the WIArchive class.
 *      Gets called while we're extracting the files for progress
 *      display.
 */

int WICallbackExtractMode(short mode,
                          short s,
                          WIFileHeader *pwifh,
                          void *pvUser)
{
    int rc = CBRC_ABORT;

    PackageMaskSpec* pPackageMaskSpecThis = (PackageMaskSpec*)pvUser;

    static char szLastFile[CCHMAXPATH] = "";

    switch (mode)
    {
        case CBM_NEXTFILE:
            if (szLastFile[0])
                // not first file:
                printf("\r  %s (100%%)\n", szLastFile);

            // ### check for whether file exists

            rc = CBRC_PROCEED;

            if (pwifh)
            {
                strcpy(szLastFile, pwifh->name);
                G_ulCurrentFile++;

                if (    pPackageMaskSpecThis
                     && pPackageMaskSpecThis->cFileMasks
                   )
                {
                    // we have file mask:
                    // apply filters
                    rc = CBRC_SKIP;

                    list<PSZ>::iterator MaskStart = pPackageMaskSpecThis->listFileMasks.begin(),
                                        MaskEnd = pPackageMaskSpecThis->listFileMasks.end();
                    for (; MaskStart != MaskEnd; MaskStart++)
                    {
                        PSZ pszMask = *MaskStart;
                        if (strhMatchOS2((const unsigned char *)pszMask,
                                         (const unsigned char *)pwifh->name))
                        {
                            rc = CBRC_PROCEED;
                            break;
                        }
                    }

                    if (rc != CBRC_PROCEED)
                    {
                        printf("  skipping %s\n", pwifh->name);
                        szLastFile[0] = '\0';
                    }
                }
            }

        break;

        case CBM_PERCENTAGE:
            printf("\r  %s (%03d%%)", pwifh->name, (ULONG)s);
            fflush(stdout);
            rc = CBRC_PROCEED;
        break;

        default:
            printf("Error.\n");
        break;
    }

    return (rc);
}

/*
 *@@ ExtractMode:
 *
 *      Throws:
 *      -- BSCancelExcpt.
 *
 *@@added V0.9.6 (2000-10-27) [umoeller]
 */

void ExtractMode(VOID)
{
    WIArchive   Arc;

    if (Arc.open(G_szArcFile, 0))
    {
        printf("wic error: Unable to open \"%s\" for reading.\n", G_szArcFile);
        throw BSCancelExcpt(ERROR_OPEN_FAILED);
    }

    Arc.setCallbackFunc(WICallbackExtractMode, NULL);

    if (!G_ulPackageMaskSpecsCount)
    {
        // no packages specified: extract all packages,
        // so create package specs for all packages in the archive first
        // for the loop below
        ULONG ulPckCount = 0;
        list<WIPackHeader*> *pPackageList = Arc.getPackList();
        list<WIPackHeader*>::iterator pckStart = pPackageList->begin(),
                                      pckEnd = pPackageList->end();
        for (; pckStart != pckEnd; pckStart++)
        {
            WIPackHeader* pHeaderThis = *pckStart;

            PackageMaskSpec* pPackageMaskSpecThis = new(PackageMaskSpec);
            pPackageMaskSpecThis->lPackage = pHeaderThis->number;
            G_listPackageMaskSpecs.push_back(pPackageMaskSpecThis);
            G_ulPackageMaskSpecsCount++;

            ulPckCount++;
        }
    }

    // now go thru the packages
    list<PackageMaskSpec*>::iterator PckStart = G_listPackageMaskSpecs.begin(),
                                     PckEnd = G_listPackageMaskSpecs.end();
    for (; PckStart != PckEnd; PckStart++)
    {
        PackageMaskSpec* pPackageMaskSpecThis = *PckStart;

        printf("Extracting package %d...\n", pPackageMaskSpecThis->lPackage);

        Arc.setCallbackFunc(WICallbackExtractMode,
                            pPackageMaskSpecThis);

        Arc.unpack(pPackageMaskSpecThis->lPackage);

        WICallbackExtractMode(CBM_NEXTFILE,
                              0,
                              NULL,
                              pPackageMaskSpecThis);

    } // end for PckStart...

    Arc.close();
}

/* ******************************************************************
 *
 *  "Extract Script" Mode
 *
 ********************************************************************/

/*
 *@@ ExtractScriptMode:
 *
 *      Throws:
 *      -- BSCancelExcpt.
 *
 *@@added V0.9.6 (2000-10-27) [umoeller]
 */

void ExtractScriptMode(VOID)
{
    WIArchive   Arc;

    if (G_szScriptFile[0] == 0)
    {
        // no script filename given:
        // use archive file with different extension
        CHAR    szDrive[_MAX_DRIVE],        // "C:"
                szDir[_MAX_DIR],            // "\test\"
                szFilestem[_MAX_FNAME],     // "*"
                szExt[_MAX_EXT];            // ".wis"
        _splitpath(G_szArcFile, szDrive, szDir, szFilestem, szExt);
        strcpy(szExt, ".wis");
        // strip directory information of source .WPI file;
        // write output to current directory
        sprintf(G_szScriptFile, "%s%s", szFilestem, szExt);
    }

    printf("wic: Writing script to text file \"%s\"... ", G_szScriptFile);
    fflush(stdout);

    if (Arc.open(G_szArcFile, 0))
    {
        printf("wic error: Unable to open \"%s\" for reading.\n", G_szArcFile);
        throw BSCancelExcpt(ERROR_OPEN_FAILED);
    }

    // get the installation profile from the archive
    const char *pcszProfileTemp = Arc.getScript();
    if (pcszProfileTemp == NULL)
    {
        printf("wic error: Cannot read script in archive \"%s\".\n", G_szArcFile);
        throw BSCancelExcpt(-1);
    }

    // OK, we got the script.... open a text file for writing
    ULONG ulWritten = 0;
    APIRET arc = doshWriteTextFile(G_szScriptFile,
                                   pcszProfileTemp,
                                   &ulWritten,
                                   NULL);          // no backup
    if (arc != NO_ERROR)
    {
        printf("wic error: Error %d occured writing script to target file \"%s\".\n",
                arc,
                G_szScriptFile);
    }
    else
        printf("Done, wrote %d bytes.\n", ulWritten);

    Arc.close();
}

/* ******************************************************************
 *
 *  SigHandler()
 *
 ********************************************************************/

/*
 *@@ SigHandler:
 *      signalhandler which exits the application
 *      with an non-zero exit code.
 *
 *@@added V0.9.13 (2001-06-09)    [bird]:   Initial coding.
 */

void SigHandler(int sig)
{
    flushall();
    switch (sig)
    {
        case SIGBREAK:
            printf("\nSIGBREAK\n");
            exit(-1);
            break;
        case SIGINT:
            printf("\nSIGINT\n");
            exit(-1);
            break;
        case SIGTERM:
            printf("\nSIGTERM\n");
            exit(-1);
            break;
    }
}

/* ******************************************************************
 *
 *  main()
 *
 ********************************************************************/

// mode options
#define MODE_ADD            1
// #define MODE_DELETE         2
#define MODE_LIST           3
#define MODE_SETSCRIPT      4
#define MODE_EXTRACT        5
#define MODE_EXTRACTSCRIPT  6

/*
 *@@ main:
 *      program entry point. Parses parameters and calls
 *      AddMode or ListMode accordingly.
 *
 *@@changed V0.9.1 (2000-02-07) [umoeller]: added "-c" subdir support
 *@@changed V0.9.6 (2000-10-27) [umoeller]: added "-x", "-X" modes
 *@@changed V0.9.13 (2001-06-09)    [bird]: added "@paramfile".
 */

int main(int argc, char *argv[])
{
    PackageMaskSpec*   pPackageMaskSpecThis = 0;
    BOOL        fContinue = TRUE;
    ULONG       ulNextMode = 0;
    LONG        lListPackage = 0;       // default: list all packages
    int         rc;

    // install signal handlers V0.9.12 (2001-06-09) [bird]
    signal(SIGBREAK, (_SigFunc)SigHandler);
    signal(SIGINT,   (_SigFunc)SigHandler);
    signal(SIGABRT,  (_SigFunc)SigHandler);

    // parse parameters on cmd line
    if (argc == 1)
        // no parameters:
        ExplainParams();

    if (argc > 1)
    {
        int i = 0;
        while (     (i++ < argc-1)
                 && (fContinue)
              )
        {
            if (argv[i][0] == '-')
            {
                ULONG i2;
                for (i2 = 1; i2 < strlen(argv[i]); i2++)
                {
                    BOOL fNextArg = FALSE;

                    switch (argv[i][i2])
                    {
                        case 'a':
                            if (ulNextMode != 0)
                                if (ulNextMode != MODE_SETSCRIPT)
                                {
                                    printf("'Add' mode can only be combined with 'Script' mode.\n");
                                    fContinue = FALSE;
                                }
                                else
                                    // script mode before
                                    if (G_szScriptFile[0] == 0)
                                    {
                                        printf("No script specified for 'Script' mode.");
                                        fContinue = FALSE;
                                    }
                            ulNextMode = MODE_ADD;
                        break;

                        case 'l':
                            if (ulNextMode != 0)
                            {
                                printf("'List' mode cannot be combined with other modes.\n");
                                fContinue = FALSE;
                            }
                            ulNextMode = MODE_LIST;
                        break;

/*                         case 'd':
                            if (ulNextMode != 0)
                            {
                                printf("'Delete' mode cannot be combined with other modes.\n");
                                fContinue = FALSE;
                            }
                            ulNextMode = MODE_DELETE;
                        break; */

                        case 's':
                            if (ulNextMode != 0)
                                if (ulNextMode != MODE_ADD)
                                {
                                    printf("'Script' mode can only be combined with 'Add' mode.\n");
                                    fContinue = FALSE;
                                }
                            ulNextMode = MODE_SETSCRIPT;
                        break;

                        case 'x':
                            if (ulNextMode != 0)
                            {
                                printf("'Extract files' mode cannot be combined with other modes.\n");
                                fContinue = FALSE;
                            }
                            ulNextMode = MODE_EXTRACT;
                        break;

                        case 'X':
                            if (ulNextMode != 0)
                            {
                                printf("'Extract script' mode cannot be combined with other modes.\n");
                                fContinue = FALSE;
                            }
                            ulNextMode = MODE_EXTRACTSCRIPT;
                        break;

                        case 'r':
                            // "recursive" on:
                            // add-mode only
                            if (ulNextMode != MODE_ADD)
                            {
                                printf("wic error: \"-r\" can only be used in 'Add' mode.\n");
                                fContinue = FALSE;
                            }
                            // this can only appear after a package number
                            else if (!pPackageMaskSpecThis)
                            {
                                printf("wic error: \"-r\" for recursion needs a package number first.\n");
                                fContinue = FALSE;
                            }
                            else
                                pPackageMaskSpecThis->fRecurse = TRUE;
                        break;

                        case 'c':
                            // change to subdir:
                            // add-mode only
                            if (ulNextMode != MODE_ADD)
                            {
                                printf("wic error: \"-c\" can only be used in 'Add' mode.\n");
                                fContinue = FALSE;
                            }
                            // this can only appear after a package number
                            if (!pPackageMaskSpecThis)
                            {
                                printf("wic error: \"-c\" for subdirs needs a package number first.\n");
                                fContinue = FALSE;
                            }
                            else
                            {
                                pPackageMaskSpecThis->pszSubdir = strdup(&argv[i][i2+1]);
                                fNextArg = TRUE;
                            }
                        break;

                        case 'u':
                            strcpy( G_szStubFile, strdup(&argv[i][i2+1]));
                            fNextArg = TRUE;
                        break;

                        case 'v':
                            G_ulVerbosity = 1;
                        break;

                        case 'h':
                            // "help":
                            ExplainArc(argv[i][i2+1]);      // next character, e.g. 'a'

                        default:  // unknown parameter
                            printf("wic error: unknown parameter \"-%c\".\n", argv[i][i2]);
                            fContinue = FALSE;
                        break;
                    }

                    if (fNextArg)
                        break; // for (i2 = 1; i2 < strlen(argv[i]); i2++)

                } // end for (i2 = 1; i2 < strlen(argv[i]); i2++)
            } // end if (argv[i][0] == '-')
            else if (argv[i][0] == '@') // V0.9.12 (2001-06-09) [bird]
            {   /*
                 * Parameter file (debugger parameter length restrictions led to this):
                 *    Create a textbuffer of the entrie file.
                 *    Parse the file and create a new parameter vector.
                 *    Set argv to the new parameter vector, argi to 0 and argc to
                 *      the parameter count.
                 *    Restrictions: Parameters enclosed in "" is not implemented.
                 *                  Stupid users is allowed to loop parameter files.
                 */
                char *pszBuffer = NULL;
                FILE *phFile = fopen(&argv[i][1], "rb");
                if (phFile != NULL)
                {
                    fseek(phFile, 0, SEEK_END);
                    signed long cbFile = ftell(phFile);
                    if (cbFile >= 0)
                    {
                        pszBuffer = (char*)malloc(cbFile + 1);
                        if (pszBuffer != NULL)
                        {
                            memset(pszBuffer, 0, cbFile + 1);
                            fseek(phFile, 0, SEEK_SET);
                            if (cbFile > 0 && fread(pszBuffer, 1, cbFile, phFile) == 0)
                            {   /* failed! */
                                free(pszBuffer);
                                pszBuffer = NULL;
                            }
                        }
                        else
                            printf("Ooops! out of memory. (line=%d)\n", __LINE__);
                    }
                    fclose(phFile);
                }

                if (pszBuffer != NULL)
                {
                    char ** papszArgs = NULL;
                    char *  psz = pszBuffer;
                    int     iArg = 0;

                    while (*psz != '\0')
                    {
                        /* find end of parameter word */
                        char *pszEnd = psz + 1;
                        char  ch = *pszEnd;
                        while (ch != ' ' && ch != '\t' && ch != '\n' && ch != '\r' && ch != '\0')
                            ch = *++pszEnd;

                        /* allocate more arg array space? */
                        if ((iArg % 512) == 0)
                        {
                            papszArgs = (char**)realloc(papszArgs, sizeof(char*) * 512);
                            if (papszArgs == NULL)
                            {
                                printf("Ooops! out of memory. (line=%d)\n", __LINE__);
                                fContinue = FALSE;
                                break;
                            }
                        }
                        *pszEnd = '\0';
                        papszArgs[iArg++] = psz;

                        /* next */
                        psz = pszEnd + 1;
                        ch = *psz;
                        while (ch == ' ' || ch == '\t' || ch == '\n' || ch == '\r')
                            ch = *++psz;
                    }

                    /*
                     * Add the remaining args.
                     */
                    for (i ;i < argc; i++)
                    {
                        /* allocate more arg array space? */
                        if ((iArg % 512) == 0)
                        {
                            papszArgs = (char**)realloc(papszArgs, sizeof(char*) * 512);
                            if (papszArgs == NULL)
                            {
                                printf("Ooops! out of memory. (line=%d)\n", __LINE__);
                                fContinue = FALSE;
                                break;
                            }
                        }
                        papszArgs[iArg++] = argv[i];
                    }

                    /*
                     * Install the new argument array and start processing it.
                     */
                    i = -1;
                    argc = iArg;
                    argv = papszArgs;
                }
                else
                {
                    printf("Could not open parameter file '%s'.\n", &argv[i][1]);
                    fContinue = FALSE;
                }
            } // end if (argv[i][0] == '@') V0.9.12 (2001-06-09) [bird]
            else
            {
                // no option ("-"): seems to be file
                if (G_szFilestem[0] == 0)
                    // first non-option: seems to be archive name (all modes)
                    strcpy(G_szFilestem, argv[i]);
                else
                    // archive name already copied:
                    // with script modes, this must be the script name then
                    if (    (ulNextMode == MODE_SETSCRIPT)
                         || (ulNextMode == MODE_EXTRACTSCRIPT)
                       )
                    {
                        if (G_szScriptFile[0] == 0)
                            strcpy(G_szScriptFile, argv[i]);
                        else
                        {
                            printf("Too many arguments after script name.");
                            fContinue = FALSE;
                        }
                    }
                    // with add or extract modes, this can be a package no.
                    else if (   (ulNextMode == MODE_ADD)
                             || (ulNextMode == MODE_EXTRACT)
                            )
                    {
                        if (strhIsDecimal(argv[i]))
                        {
                            // argument is decimal: seems to be package number;
                            // save the previous stuff then
                            if (pPackageMaskSpecThis)
                            {
                                G_listPackageMaskSpecs.push_back(pPackageMaskSpecThis);
                                G_ulPackageMaskSpecsCount++;
                            }
                            pPackageMaskSpecThis = new(PackageMaskSpec);

                            LONG lPackage = 0;
                            sscanf(argv[i], "%d", &lPackage);
                            if (lPackage == 0)
                            {
                                printf("wic error: You have specified an invalid package number.\n");
                                fContinue = FALSE;
                            }
                            pPackageMaskSpecThis->lPackage = lPackage;
                        }
                        else
                        {
                            // non-decimal: seems to be file mask;
                            // in add mode, this can only appear after a package number
                            if ((!pPackageMaskSpecThis) && (ulNextMode == MODE_ADD))
                            {
                                printf("wic error: You need to specify a package number first.\n");
                                fContinue = FALSE;
                            }
                            else
                            {
                                pPackageMaskSpecThis->listFileMasks.push_back(strdup(argv[i]));
                                pPackageMaskSpecThis->cFileMasks++;
                            }
                        }
                    } // end if (ulNextMode == MODE_ADD)
                    else if (ulNextMode == MODE_LIST)
                        if (strhIsDecimal(argv[i]))
                            sscanf(argv[i], "%d", &lListPackage);
                        else
                        {
                            printf("wic error: Invalid parameter for 'list' mode.\n");
                            fContinue = FALSE;
                        }
            } // end else if (argv[i][0] == '-')
        } // end while ((i++ < argc-1) && (fContinue))
    } // end if (argc > 1)

    if (G_szFilestem[0] == 0)
    {
        printf("wic error: You have not specified an archive filename.\n");
        fContinue = FALSE;
    }

    if (ulNextMode == 0)
    {
        printf("wic error: You have not specified a valid operation mode.\n");
        fContinue = FALSE;
    }

    if (ulNextMode == MODE_ADD) // checks for add mode only, extract can be used without this
    {
        if (pPackageMaskSpecThis == 0)
        {
            printf("wic error: No package number specified for \"add\" mode.\n");
            fContinue = FALSE;
        }
        else if (pPackageMaskSpecThis->cFileMasks == 0)
        {
            printf("wic error: No file mask specified for \"add\" mode.\n");
            fContinue = FALSE;
        }
    }

    if ( (ulNextMode == MODE_SETSCRIPT) && (G_szScriptFile[0] == 0) )
    {
        printf("wic error: No script specified for \"script\" mode.\n");
        fContinue = FALSE;
    }

    if (!fContinue)
    {
        printf("Type \"wic -h\" for help.\n");
        exit(2);
    }

    if (pPackageMaskSpecThis)
    {
        G_listPackageMaskSpecs.push_back(pPackageMaskSpecThis);
        G_ulPackageMaskSpecsCount++;
    }

    try
    {
        if (strchr(G_szFilestem, '.') == NULL)
            sprintf(G_szArcFile, "%s.wpi", G_szFilestem);
        else
            strcpy(G_szArcFile, G_szFilestem);

        /*
         * MODE_SETSCRIPT only:
         *
         */
        if (ulNextMode == MODE_SETSCRIPT)
            ulNextMode = MODE_ADD;

        /*
         * MODE_ADD:
         *
         */

        if (ulNextMode == MODE_ADD)
        {
            AddMode();
        }

        /*
         * MODE_DELETE:
         *
         */

        /* else if (ulNextMode == MODE_DELETE)
        {
            printf("Delete mode not implemented yet.\n");
            throw BSCancelExcpt(-1);
        } */

        /*
         * MODE_LIST:
         *
         */

        else if (ulNextMode == MODE_LIST)
        {
            ListMode(lListPackage);
        } // end else if (ulNextMode == MODE_LIST)

        /*
         * MODE_EXTRACT:
         *
         */

        else if (ulNextMode == MODE_EXTRACT)
        {
            ExtractMode();
        } // end else if (ulNextMode == MODE_LIST)

        /*
         * MODE_EXTRACTSCRIPT:
         *
         */

        else if (ulNextMode == MODE_EXTRACTSCRIPT)
        {
            ExtractScriptMode();
        } // end else if (ulNextMode == MODE_LIST)

        rc = 0;

    }
    catch (BSCancelExcpt& CancelExcpt)
    {
        printf("Terminating. rc=%d\n", CancelExcpt.iErrno);
        rc = CancelExcpt.iErrno;
    }

    return rc;
}

